@@ -124,11 +124,37 @@ DEFINE_bool(enable_sync_to_latest_event_on_ddls, false, "This configuration is "
     "(if enabled). If this config is enabled, then the flag invalidate_hms_cache_on_ddls "
     "should be disabled");
 
+DEFINE_bool(enable_reload_events, false, "This configuration is used to fire a "
+    "refresh/invalidate table event to the HMS such that other event processors "
+    "(such as other Impala catalogds) that poll HMS notification logs can process "
+    "this event. The default value is false, so impala will not fire this "
+    "event. If enabled, impala will fire this event and other catalogD will process it."
+    "This config only affects the firing of the reload event. Processing of reload "
+    "event will always happen");
+
+DEFINE_string(file_metadata_reload_properties, "EXTERNAL, metadata_location,"
+    "transactional, transactional_properties, TRANSLATED_TO_EXTERNAL, repl.last.id",
+    "This configuration is used to whitelist the table properties that are supposed to"
+    "refresh file metadata when these properties are changed. To skip this optimization,"
+    "set the value to empty string");
+
+DEFINE_bool(enable_skipping_older_events, false, "This configuration is used to skip any"
+    "older events in the event processor based on the lastRefreshEventId on the"
+    "database/table/partition in the cache. All the DML queries that change the metadata"
+    "in the catalogD's cache will update the lastRefreshEventId i.e.., fetch the latest"
+    "available event on HMS and set it on the object. In case the event processor is"
+    "lagging, the older events in event processor queue can be skipped by comparing the"
+    "current event id to that of lastRefreshEventId. The default is set to false to"
+    "disable the optimisation. Set this true to enable skipping the older events and"
+    "quickly catch with the events of HMS");
+
 DECLARE_string(state_store_host);
 DECLARE_int32(state_store_subscriber_port);
 DECLARE_int32(state_store_port);
 DECLARE_string(hostname);
 DECLARE_bool(compact_catalog_topic);
+DECLARE_bool(enable_catalogd_ha);
+DECLARE_bool(force_catalogd_active);
 
 #ifndef NDEBUG
 DECLARE_int32(stress_catalog_startup_delay_ms);
@@ -140,9 +166,11 @@ string CatalogServer::IMPALA_CATALOG_TOPIC = "catalog-update";
 
 const string CATALOG_SERVER_TOPIC_PROCESSING_TIMES =
     "catalog-server.topic-processing-time-s";
-
 const string CATALOG_SERVER_PARTIAL_FETCH_RPC_QUEUE_LEN =
     "catalog.partial-fetch-rpc.queue-len";
+const string CATALOG_ACTIVE_STATUS = "catalog-server.active-status";
+const string CATALOG_HA_NUM_ACTIVE_STATUS_CHANGE =
+    "catalog-server.ha-number-active-status-change";
 
 const string CATALOG_WEB_PAGE = "/catalog";
 const string CATALOG_TEMPLATE = "catalog.tmpl";
@@ -168,7 +196,10 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
   // Executes a TDdlExecRequest and returns details on the result of the operation.
   void ExecDdl(TDdlExecResponse& resp, const TDdlExecRequest& req) override {
     VLOG_RPC << "ExecDdl(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->ExecDdl(req, &resp);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->ExecDdl(req, &resp);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
     TStatus thrift_status;
     status.ToThrift(&thrift_status);
@@ -181,7 +212,10 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
       override {
     VLOG_RPC << "ResetMetadata(): request=" << ThriftDebugString(req);
     DebugActionNoFail(FLAGS_debug_actions, "RESET_METADATA_DELAY");
-    Status status = catalog_server_->catalog()->ResetMetadata(req, &resp);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->ResetMetadata(req, &resp);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
     TStatus thrift_status;
     status.ToThrift(&thrift_status);
@@ -194,7 +228,10 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
   void UpdateCatalog(TUpdateCatalogResponse& resp, const TUpdateCatalogRequest& req)
       override {
     VLOG_RPC << "UpdateCatalog(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->UpdateCatalog(req, &resp);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->UpdateCatalog(req, &resp);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
     TStatus thrift_status;
     status.ToThrift(&thrift_status);
@@ -207,7 +244,10 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
   void GetFunctions(TGetFunctionsResponse& resp, const TGetFunctionsRequest& req)
       override {
     VLOG_RPC << "GetFunctions(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->GetFunctions(req, &resp);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->GetFunctions(req, &resp);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
     TStatus thrift_status;
     status.ToThrift(&thrift_status);
@@ -219,9 +259,15 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
   void GetCatalogObject(TGetCatalogObjectResponse& resp,
       const TGetCatalogObjectRequest& req) override {
     VLOG_RPC << "GetCatalogObject(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->GetCatalogObject(req.object_desc,
-        &resp.catalog_object);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->GetCatalogObject(
+          req.object_desc, &resp.catalog_object);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
+    TStatus thrift_status;
+    status.ToThrift(&thrift_status);
+    resp.__set_status(thrift_status);
     VLOG_RPC << "GetCatalogObject(): response=" << ThriftDebugStringNoThrow(resp);
   }
 
@@ -235,7 +281,10 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
     // so a heavy query workload against a table undergoing a slow refresh doesn't
     // end up taking down the catalog by creating thousands of threads.
     VLOG_RPC << "GetPartialCatalogObject(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->GetPartialCatalogObject(req, &resp);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->GetPartialCatalogObject(req, &resp);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
     TStatus thrift_status;
     status.ToThrift(&thrift_status);
@@ -246,7 +295,10 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
   void GetPartitionStats(TGetPartitionStatsResponse& resp,
       const TGetPartitionStatsRequest& req) override {
     VLOG_RPC << "GetPartitionStats(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->GetPartitionStats(req, &resp);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->GetPartitionStats(req, &resp);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
     TStatus thrift_status;
     status.ToThrift(&thrift_status);
@@ -260,7 +312,10 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
   void PrioritizeLoad(TPrioritizeLoadResponse& resp, const TPrioritizeLoadRequest& req)
       override {
     VLOG_RPC << "PrioritizeLoad(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->PrioritizeLoad(req);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->PrioritizeLoad(req);
+    }
     if (!status.ok()) LOG(ERROR) << status.GetDetail();
     TStatus thrift_status;
     status.ToThrift(&thrift_status);
@@ -271,23 +326,45 @@ class CatalogServiceThriftIf : public CatalogServiceIf {
   void UpdateTableUsage(TUpdateTableUsageResponse& resp,
       const TUpdateTableUsageRequest& req) override {
     VLOG_RPC << "UpdateTableUsage(): request=" << ThriftDebugString(req);
-    Status status = catalog_server_->catalog()->UpdateTableUsage(req);
+    Status status = CheckProtocolVersion(req.protocol_version);
+    if (status.ok()) {
+      status = catalog_server_->catalog()->UpdateTableUsage(req);
+    }
     if (!status.ok()) LOG(WARNING) << status.GetDetail();
+    TStatus thrift_status;
+    status.ToThrift(&thrift_status);
+    resp.__set_status(thrift_status);
+    VLOG_RPC << "UpdateTableUsage(): response.status=" << resp.status;
   }
 
  private:
   CatalogServer* catalog_server_;
+
+  Status CheckProtocolVersion(CatalogServiceVersion::type client_version) {
+    Status status = Status::OK();
+    if (client_version < catalog_server_->GetProtocolVersion()) {
+      status = Status(TErrorCode::CATALOG_INCOMPATIBLE_PROTOCOL, client_version + 1,
+          catalog_server_->GetProtocolVersion() + 1);
+    }
+    return status;
+  }
 };
 
 CatalogServer::CatalogServer(MetricGroup* metrics)
-  : thrift_iface_(new CatalogServiceThriftIf(this)),
+  : protocol_version_(CatalogServiceVersion::V2),
+    thrift_iface_(new CatalogServiceThriftIf(this)),
     thrift_serializer_(FLAGS_compact_catalog_topic), metrics_(metrics),
+    is_active_(!FLAGS_enable_catalogd_ha),
     topic_updates_ready_(false), last_sent_catalog_version_(0L),
     catalog_objects_max_version_(0L) {
   topic_processing_time_metric_ = StatsMetric<double>::CreateAndRegister(metrics,
       CATALOG_SERVER_TOPIC_PROCESSING_TIMES);
   partial_fetch_rpc_queue_len_metric_ =
       metrics->AddGauge(CATALOG_SERVER_PARTIAL_FETCH_RPC_QUEUE_LEN, 0);
+  active_status_metric_ =
+      metrics->AddProperty(CATALOG_ACTIVE_STATUS, !FLAGS_enable_catalogd_ha);
+  num_ha_active_status_change_metric_ =
+      metrics->AddCounter(CATALOG_HA_NUM_ACTIVE_STATUS_CHANGE, 0);
 }
 
 Status CatalogServer::Start() {
@@ -311,9 +388,11 @@ Status CatalogServer::Start() {
   RETURN_IF_ERROR(Thread::Create("catalog-server", "catalog-metrics-refresh-thread",
       &CatalogServer::RefreshMetrics, this, &catalog_metrics_refresh_thread_));
 
-  statestore_subscriber_.reset(new StatestoreSubscriber(
+  active_catalogd_version_checker_.reset(new ActiveCatalogdVersionChecker());
+  statestore_subscriber_.reset(new StatestoreSubscriberCatalog(
      Substitute("catalog-server@$0", TNetworkAddressToString(server_address)),
-     subscriber_address, statestore_address, metrics_));
+     subscriber_address, statestore_address, metrics_, protocol_version_,
+     server_address));
 
   StatestoreSubscriber::UpdateCallback cb =
       bind<void>(mem_fn(&CatalogServer::UpdateCatalogTopicCallback), this, _1, _2);
@@ -329,22 +408,38 @@ Status CatalogServer::Start() {
     status.AddDetail("CatalogService failed to start");
     return status;
   }
+  // Add callback to handle notification of updating catalogd from Statestore.
+  if (FLAGS_enable_catalogd_ha) {
+    StatestoreSubscriber::UpdateCatalogdCallback update_catalogd_cb =
+        bind<void>(mem_fn(&CatalogServer::UpdateActiveCatalogd), this, _1, _2, _3);
+    statestore_subscriber_->AddUpdateCatalogdTopic(update_catalogd_cb);
+  }
+
   RETURN_IF_ERROR(statestore_subscriber_->Start());
+  if (FLAGS_force_catalogd_active && !IsActive()) {
+    // If both catalogd are started with 'force_catalogd_active' as true in short time,
+    // the second election overwrite the first election. The one which registering with
+    // statstore first will be inactive.
+    LOG(WARNING) << "Could not start CatalogD as active instance";
+  }
 
   // Notify the thread to start for the first time.
   {
     lock_guard<mutex> l(catalog_lock_);
-    catalog_update_cv_.NotifyOne();
+    if (is_active_) catalog_update_cv_.NotifyOne();
   }
   return Status::OK();
 }
 
-void CatalogServer::RegisterWebpages(Webserver* webserver) {
+void CatalogServer::RegisterWebpages(Webserver* webserver, bool metrics_only) {
   Webserver::RawUrlCallback healthz_callback =
       [this](const auto& req, auto* data, auto* response) {
         return this->HealthzHandler(req, data, response);
       };
   webserver->RegisterUrlCallback(CATALOG_SERVICE_HEALTH_WEB_PAGE, healthz_callback);
+
+  if (metrics_only) return;
+
   webserver->RegisterUrlCallback(CATALOG_WEB_PAGE, CATALOG_TEMPLATE,
       [this](const auto& args, auto* doc) { this->CatalogUrlCallback(args, doc); }, true);
   webserver->RegisterUrlCallback(CATALOG_OBJECT_WEB_PAGE, CATALOG_OBJECT_TEMPLATE,
@@ -355,7 +450,7 @@ void CatalogServer::RegisterWebpages(Webserver* webserver) {
       false);
   webserver->RegisterUrlCallback(EVENT_WEB_PAGE, EVENT_METRICS_TEMPLATE,
       [this](const auto& args, auto* doc) { this->EventMetricsUrlCallback(args, doc); },
-      false);
+      true);
   webserver->RegisterUrlCallback(CATALOG_OPERATIONS_WEB_PAGE, CATALOG_OPERATIONS_TEMPLATE,
       [this](const auto& args, auto* doc) { this->OperationUsageUrlCallback(args, doc); },
       true);
@@ -370,10 +465,10 @@ void CatalogServer::UpdateCatalogTopicCallback(
   if (topic == incoming_topic_deltas.end()) return;
 
   unique_lock<mutex> l(catalog_lock_, std::try_to_lock);
-  // Return if unable to acquire the catalog_lock_ or if the topic update data is
-  // not yet ready for processing. This indicates the catalog_update_gathering_thread_
-  // is still building a topic update.
-  if (!l || !topic_updates_ready_) return;
+  // Return if unable to acquire the catalog_lock_, or this instance is not active,
+  // or if the topic update data is not yet ready for processing. This indicates the
+  // catalog_update_gathering_thread_ is still building a topic update.
+  if (!l || !is_active_ || !topic_updates_ready_) return;
 
   const TTopicDelta& delta = topic->second;
 
@@ -411,6 +506,58 @@ void CatalogServer::UpdateCatalogTopicCallback(
   catalog_update_cv_.NotifyOne();
 }
 
+void CatalogServer::UpdateActiveCatalogd(bool is_registration_reply,
+    int64_t active_catalogd_version, const TCatalogRegistration& catalogd_registration) {
+  lock_guard<mutex> l(catalog_lock_);
+  if (!active_catalogd_version_checker_->CheckActiveCatalogdVersion(
+          is_registration_reply, active_catalogd_version)) {
+    return;
+  }
+  if (catalogd_registration.address.hostname.empty()
+      || catalogd_registration.address.port == 0) {
+    return;
+  }
+  LOG(INFO) << "Get notification of active catalogd: "
+            << TNetworkAddressToString(catalogd_registration.address);
+  bool is_matching = (catalogd_registration.address.hostname == FLAGS_hostname
+      && catalogd_registration.address.port == FLAGS_catalog_service_port);
+  if (is_matching) {
+    if (!is_active_) {
+      is_active_ = true;
+      active_status_metric_->SetValue(true);
+      num_ha_active_status_change_metric_->Increment(1);
+      // Reset last_sent_catalog_version_ when the catalogd become active. This will
+      // lead to non-delta catalog update for next IMPALA_CATALOG_TOPIC which also
+      // instruct the statestore to clear all entries for the catalog update topic.
+      last_sent_catalog_version_ = 0;
+      // Regenerate Catalog Service ID.
+      catalog_->RegenerateServiceId();
+      // Clear pending topic updates.
+      pending_topic_updates_.clear();
+      // Signal the catalog update gathering thread to start.
+      topic_updates_ready_ = false;
+      catalog_update_cv_.NotifyOne();
+      LOG(INFO) << "This catalogd instance is changed to active status";
+    }
+  } else {
+    if (is_active_) {
+      is_active_ = false;
+      active_status_metric_->SetValue(false);
+      num_ha_active_status_change_metric_->Increment(1);
+      LOG(INFO) << "This catalogd instance is changed to inactive status. "
+                << "Current active catalogd: "
+                << TNetworkAddressToString(catalogd_registration.address)
+                << ", active_catalogd_version: "
+                << active_catalogd_version;
+    }
+  }
+}
+
+bool CatalogServer::IsActive() {
+  lock_guard<mutex> l(catalog_lock_);
+  return is_active_;
+}
+
 [[noreturn]] void CatalogServer::GatherCatalogUpdatesThread() {
   while (true) {
     unique_lock<mutex> unique_lock(catalog_lock_);
@@ -666,6 +813,12 @@ void CatalogServer::EventMetricsUrlCallback(
       event_processor_summary_response.summary.c_str(), document->GetAllocator());
   document->AddMember(
       "event_processor_metrics", event_processor_summary, document->GetAllocator());
+  if (event_processor_summary_response.__isset.error_msg) {
+    Value error_msg(
+        event_processor_summary_response.error_msg.c_str(), document->GetAllocator());
+    document->AddMember(
+        "event_processor_error_msg", error_msg, document->GetAllocator());
+  }
 }
 
 void CatalogServer::CatalogObjectsUrlCallback(const Webserver::WebRequest& req,
@@ -766,7 +919,7 @@ void CatalogServer::TableMetricsUrlCallback(const Webserver::WebRequest& req,
   if (object_name_arg != args.end()) {
     // Parse the object name to extract database and table names
     const string& full_tbl_name = object_name_arg->second;
-    int pos = full_tbl_name.find(".");
+    int pos = full_tbl_name.find('.');
     if (pos == string::npos || pos >= full_tbl_name.size() - 1) {
       stringstream error_msg;
       error_msg << "Invalid table name: " << full_tbl_name;
