@@ -137,10 +137,11 @@ class Catalog {
   /// report.
   Status UpdateTableUsage(const TUpdateTableUsageRequest& req);
 
- private:
-  /// Descriptor of Java Catalog class itself, used to create a new instance.
-  jclass catalog_class_;
+  /// Regenerate Catalog Service ID.
+  /// The function should be called when the CatalogD becomes active.
+  void RegenerateServiceId();
 
+ private:
   jobject catalog_;  // instance of org.apache.impala.service.JniCatalog
   jmethodID update_metastore_id_;  // JniCatalog.updateMetaastore()
   jmethodID exec_ddl_id_;  // JniCatalog.execDdl()
@@ -162,6 +163,7 @@ class Catalog {
   jmethodID prioritize_load_id_; // JniCatalog.prioritizeLoad()
   jmethodID catalog_ctor_;
   jmethodID update_table_usage_id_;
+  jmethodID regenerate_service_id_; // JniCatalog.regenerateServiceId()
 };
 
 }
