@@ -58,6 +58,7 @@
 #include "exprs/math-functions-ir.cc"
 #include "exprs/operators-ir.cc"
 #include "exprs/scalar-expr-ir.cc"
+#include "exprs/scalar-expr-evaluator-ir.cc"
 #include "exprs/string-functions-ir.cc"
 #include "exprs/timestamp-functions-ir.cc"
 #include "exprs/tuple-is-null-predicate-ir.cc"
@@ -69,6 +70,7 @@
 #include "runtime/runtime-filter-ir.cc"
 #include "runtime/sorter-ir.cc"
 #include "runtime/tuple-ir.cc"
+#include "runtime/sorted-run-merger-ir.cc"
 #include "udf/udf-ir.cc"
 #include "util/bloom-filter-ir.cc"
 #include "util/hash-util-ir.cc"
