@@ -169,6 +169,9 @@ DEFINE_int32(stress_catalog_startup_delay_ms, 0, "A stress option that injects e
     "catalogd opens ports or accepts connections. Delay <= 0 injects no delay.");
 DEFINE_int32(stress_disk_read_delay_ms, 0, "A stress option that injects extra delay"
     " in milliseconds when the I/O manager is reading from disk.");
+DEFINE_int32(stress_statestore_startup_delay_ms, 0, "A stress option that injects extra "
+    "delay in milliseconds during the startup of statestore. The delay is before the "
+    "statestored opens ports or accepts connections. Delay <= 0 injects no delay.");
 #endif
 
 DEFINE_string(debug_actions, "", "For testing only. Uses the same format as the debug "
@@ -375,6 +378,52 @@ DEFINE_bool(pull_table_types_and_comments, false,
     "catalogd-only flag. Required if users want GET_TABLES requests return correct table "
     "types or comments.");
 
+DEFINE_bool(tolerate_statestore_startup_delay, true, "If set to true, the subscriber "
+    "is able to tolerate the delay of the statestore's availability. The subscriber's "
+    "process will not exit if it cannot register with the specified statestore on "
+    "startup. But instead it enters into Recovery mode, it will loop, sleep and retry "
+    "till it successfully registers with the statestore.");
+
+// Starting flags for CatalogD High Availability
+DEFINE_bool(enable_catalogd_ha, false, "Set to true to enable CatalogD HA");
+DEFINE_bool(force_catalogd_active, false, "Set to true to force this catalogd instance "
+    "to take active role. It's used to perform manual fail over for catalog service.");
+// Use subscriber-id which is built with network address as priority value of catalogd
+// instance when designating active catalogd. The lower subscriber_id (i.e. lower network
+// address) corresponds to a higher priority.
+// This is mainly used in unit-test for predictable results.
+DEFINE_bool(use_subscriber_id_as_catalogd_priority, false, "Subscriber-id is used as "
+    "priority value of catalogd instance if this is set as true. Otherwise, "
+    "registration_id which is generated as random number will be used as priority value "
+    "of catalogd instance.");
+// Waiting period in ms for HA preemption. It should be set with proper value based on the
+// time to take for bringing a catalogd instance in-line in the deployment environment.
+DEFINE_int64(catalogd_ha_preemption_wait_period_ms, 10000, "(Advanced) The time after "
+    "which statestore designates the first registered catalogd as active if statestore "
+    "does not receive registration request from the second catalogd.");
+DEFINE_int64(active_catalogd_designation_monitoring_interval_ms, 100, "(Advanced) "
+    "Interval (in ms) with which the statestore monitors if active catalogd is "
+    "designated.");
+DEFINE_int64(update_catalogd_rpc_resend_interval_ms, 100, "(Advanced) Interval (in ms) "
+    "with which the statestore resends the update catalogd RPC to a subscriber if the "
+    "statestore has failed to send the RPC to the subscriber.");
+
+DEFINE_int32(iceberg_reload_new_files_threshold, 100, "(Advanced) If during a table "
+    "refresh the number of new files are greater than this, catalogd will completely "
+    "reload all file metadata. If number of new files are less or equal to this, "
+    "catalogd will only load the metadata of the newly added files.");
+
+// TGeospatialLibrary's values are mapped here as constants
+static const string geo_lib_none = "NONE";
+static const string geo_lib_hive_esri = "HIVE_ESRI";
+
+static const string geo_lib_help_msg =
+    "Specifies which implementation of "
+    "geospatial functions should be included as builtins. Possible values: [\""
+    + geo_lib_none + "\", \"" + geo_lib_hive_esri + "\"]";
+
+DEFINE_string(geospatial_library, geo_lib_none, geo_lib_help_msg.c_str());
+
 // ++========================++
 // || Startup flag graveyard ||
 // ++========================++
