@@ -18,6 +18,8 @@
 #include "common/init.h"
 
 #include <csignal>
+#include <regex>
+#include <boost/filesystem.hpp>
 #include <gperftools/heap-profiler.h>
 #include <third_party/lss/linux_syscall_support.h>
 
@@ -43,6 +45,7 @@
 #include "util/cpu-info.h"
 #include "util/debug-util.h"
 #include "util/disk-info.h"
+#include "util/filesystem-util.h"
 #include "util/jni-util.h"
 #include "util/logging-support.h"
 #include "util/mem-info.h"
@@ -51,6 +54,7 @@
 #include "util/network-util.h"
 #include "util/openssl-util.h"
 #include "util/os-info.h"
+#include "util/os-util.h"
 #include "util/parse-util.h"
 #include "util/periodic-counter-updater.h"
 #include "util/pretty-printer.h"
@@ -63,6 +67,7 @@
 #include "common/names.h"
 
 using namespace impala;
+namespace filesystem = boost::filesystem;
 
 DECLARE_bool(enable_process_lifetime_heap_profiling);
 DECLARE_string(heap_profile_dir);
@@ -94,6 +99,15 @@ DEFINE_string(local_library_dir, "/tmp",
     "Scratch space for local fs operations. Currently used for copying "
     "UDF binaries locally from HDFS and also for initializing the timezone db");
 
+DEFINE_bool(jvm_automatic_add_opens, true,
+    "Adds necessary --add-opens options for core Java modules necessary to correctly "
+    "calculate catalog metadata cache object sizes.");
+
+DEFINE_string_hidden(java_weigher, "auto",
+    "Choose between 'jamm' (com.github.jbellis:jamm) and 'sizeof' (org.ehcache:sizeof) "
+    "weighers for determining catalog metadata cache entry size. 'auto' uses 'sizeof' "
+    "for Java 8 - 11, and 'jamm' for Java 15+.");
+
 // Defined by glog. This allows users to specify the log level using a glob. For
 // example -vmodule=*scanner*=3 would enable full logging for scanners. If redaction
 // is enabled, this option won't be allowed because some logging dumps table data
@@ -297,6 +311,168 @@ void BlockImpalaShutdownSignal() {
   AbortIfError(pthread_sigmask(SIG_BLOCK, &signals, nullptr), error_msg);
 }
 
+// Returns Java major version, such as 8, 11, or 17.
+static int GetJavaMajorVersion() {
+  string cmd = "java";
+  const char* java_home = getenv("JAVA_HOME");
+  if (java_home != NULL) {
+    cmd = (filesystem::path(java_home) / "bin" / "java").string();
+  }
+  cmd += " -version 2>&1";
+  string msg;
+  if (!RunShellProcess(cmd, &msg, false, {"JAVA_TOOL_OPTIONS"})) {
+    LOG(INFO) << Substitute("Unable to determine Java version (default to 8): $0", msg);
+    return 8;
+  }
+
+  // Find a version string in the first line.
+  string first_line;
+  std::getline(istringstream(msg), first_line);
+  // Need to allow for a wide variety of formats for different JDK implementations.
+  // Example: openjdk version "11.0.19" 2023-04-18
+  std::regex java_version_pattern("\"([0-9]{1,3})\\.[0-9]+\\.[0-9]+[^\"]*\"");
+  std::smatch matches;
+  if (!std::regex_search(first_line, matches, java_version_pattern)) {
+    LOG(INFO) << Substitute("Unable to determine Java version (default to 8): $0", msg);
+    return 8;
+  }
+  DCHECK_EQ(matches.size(), 2);
+  return std::stoi(matches.str(1));
+}
+
+// Append the javaagent arg to JAVA_TOOL_OPTIONS to load jamm.
+static Status JavaAddJammAgent() {
+  stringstream val_out;
+  char* current_val_c = getenv("JAVA_TOOL_OPTIONS");
+  if (current_val_c != NULL) {
+    val_out << current_val_c << " ";
+  }
+
+  istringstream classpath {getenv("CLASSPATH")};
+  string jamm_path, test_path;
+  while (getline(classpath, test_path, ':')) {
+    jamm_path = FileSystemUtil::FindFileInPath(test_path, "jamm-.*.jar");
+    if (!jamm_path.empty()) break;
+  }
+  if (jamm_path.empty()) {
+    return Status("Could not find jamm-*.jar in Java CLASSPATH");
+  }
+  val_out << "-javaagent:" << jamm_path;
+
+  if (setenv("JAVA_TOOL_OPTIONS", val_out.str().c_str(), 1) < 0) {
+    return Status(Substitute("Could not update JAVA_TOOL_OPTIONS: $0", GetStrErrMsg()));
+  }
+  return Status::OK();
+}
+
+// Append add-opens args to JAVA_TOOL_OPTIONS for ehcache and jamm.
+static Status JavaAddOpens(bool useSizeOf) {
+  if (!FLAGS_jvm_automatic_add_opens) return Status::OK();
+
+  stringstream val_out;
+  char* current_val_c = getenv("JAVA_TOOL_OPTIONS");
+  if (current_val_c != NULL) {
+    val_out << current_val_c;
+  }
+
+  for (const string& param : {
+    // Needed for jamm and ehcache
+    "--add-opens=java.base/java.lang=ALL-UNNAMED",
+    "--add-opens=java.base/java.nio=ALL-UNNAMED",
+    "--add-opens=java.base/sun.nio.ch=ALL-UNNAMED"
+  }) {
+    val_out << " " << param;
+  }
+
+  if (useSizeOf) {
+    for (const string& param : {
+      // Only needed for ehcache
+      "--add-opens=java.base/java.io=ALL-UNNAMED",
+      "--add-opens=java.base/java.lang.invoke=ALL-UNNAMED",
+      "--add-opens=java.base/java.lang.module=ALL-UNNAMED",
+      "--add-opens=java.base/java.lang.ref=ALL-UNNAMED",
+      "--add-opens=java.base/java.lang.reflect=ALL-UNNAMED",
+      "--add-opens=java.base/java.net=ALL-UNNAMED",
+      "--add-opens=java.base/java.nio.charset=ALL-UNNAMED",
+      "--add-opens=java.base/java.nio.file.attribute=ALL-UNNAMED",
+      "--add-opens=java.base/java.security=ALL-UNNAMED",
+      "--add-opens=java.base/java.util.concurrent.locks=ALL-UNNAMED",
+      "--add-opens=java.base/java.util.concurrent=ALL-UNNAMED",
+      "--add-opens=java.base/java.util.jar=ALL-UNNAMED",
+      "--add-opens=java.base/java.util.zip=ALL-UNNAMED",
+      "--add-opens=java.base/java.util=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.loader=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.math=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.module=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.perf=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.platform=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.ref=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.reflect=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.util.jar=ALL-UNNAMED",
+      "--add-opens=java.base/sun.net.www.protocol.jar=ALL-UNNAMED",
+      "--add-opens=java.base/sun.nio.fs=ALL-UNNAMED",
+      "--add-opens=jdk.dynalink/jdk.dynalink.beans=ALL-UNNAMED",
+      "--add-opens=jdk.dynalink/jdk.dynalink.linker.support=ALL-UNNAMED",
+      "--add-opens=jdk.dynalink/jdk.dynalink.linker=ALL-UNNAMED",
+      "--add-opens=jdk.dynalink/jdk.dynalink.support=ALL-UNNAMED",
+      "--add-opens=jdk.dynalink/jdk.dynalink=ALL-UNNAMED",
+      "--add-opens=jdk.management.jfr/jdk.management.jfr=ALL-UNNAMED",
+      "--add-opens=jdk.management/com.sun.management.internal=ALL-UNNAMED"
+    }) {
+      val_out << " " << param;
+    }
+  }
+
+  if (setenv("JAVA_TOOL_OPTIONS", val_out.str().c_str(), 1) < 0) {
+    return Status(Substitute("Could not update JAVA_TOOL_OPTIONS: $0", GetStrErrMsg()));
+  }
+  return Status::OK();
+}
+
+static Status InitializeJavaWeigher() {
+  // This is set up so the default if things go wrong is to continue using ehcache.sizeof.
+  int version = GetJavaMajorVersion();
+  if (FLAGS_java_weigher == "auto") {
+    // Update for backend-gflag-util.cc setting use_jamm_weigher.
+    FLAGS_java_weigher = (version >= 15) ? "jamm" : "sizeof";
+  }
+  LOG(INFO) << "Using Java weigher " << FLAGS_java_weigher;
+
+  if (FLAGS_java_weigher == "jamm") {
+    RETURN_IF_ERROR(JavaAddJammAgent());
+  }
+  if (version >= 9) {
+    // add-opens is only supported in Java 9+.
+    RETURN_IF_ERROR(JavaAddOpens(FLAGS_java_weigher != "jamm"));
+  }
+  return Status::OK();
+}
+
+static Status JavaSetProcessName(string name) {
+  string current_val;
+  char* current_val_c = getenv("JAVA_TOOL_OPTIONS");
+  if (current_val_c != NULL) {
+    current_val = current_val_c;
+  }
+
+  if (!current_val.empty() && current_val.find("-Dsun.java.command") != string::npos) {
+    LOG(WARNING) << "Overriding sun.java.command in JAVA_TOOL_OPTIONS to " << name;
+  }
+
+  stringstream val_out;
+  if (!current_val.empty()) {
+    val_out << current_val << " ";
+  }
+  // Set sun.java.command so jps reports the name correctly, and ThreadNameAnnotator can
+  // use the process name for the main thread (and correctly restore the process name).
+  val_out << "-Dsun.java.command=" << name;
+
+  if (setenv("JAVA_TOOL_OPTIONS", val_out.str().c_str(), 1) < 0) {
+    return Status(Substitute("Could not update JAVA_TOOL_OPTIONS: $0", GetStrErrMsg()));
+  }
+  return Status::OK();
+}
+
 void impala::InitCommonRuntime(int argc, char** argv, bool init_jvm,
     TestInfo::Mode test_mode, bool external_fe) {
   srand(time(NULL));
@@ -370,11 +546,6 @@ void impala::InitCommonRuntime(int argc, char** argv, bool init_jvm,
   if (!external_fe) {
     ABORT_IF_ERROR(RegisterMinidump(argv[0]));
   }
-#ifndef THREAD_SANITIZER
-#ifndef __aarch64__
-  AtomicOps_x86CPUFeaturesInit();
-#endif
-#endif
   impala::InitThreading();
   impala::datetime_parse_util::SimpleDateFormatTokenizer::InitCtx();
   impala::SeedOpenSSLRNG();
@@ -452,6 +623,8 @@ void impala::InitCommonRuntime(int argc, char** argv, bool init_jvm,
 
   if (init_jvm) {
     if (!external_fe) {
+      ABORT_IF_ERROR(InitializeJavaWeigher());
+      ABORT_IF_ERROR(JavaSetProcessName(filesystem::path(argv[0]).filename().string()));
       JniUtil::InitLibhdfs();
     }
     ABORT_IF_ERROR(JniUtil::Init());
