@@ -121,7 +121,14 @@ void ExchangePlanNode::Codegen(FragmentState* state) {
   if (IsNodeCodegenDisabled()) return;
 
   if (row_comparator_config_ != nullptr) {
-    AddCodegenStatus(row_comparator_config_->Codegen(state));
+    Status codegen_status;
+    llvm::Function* compare_fn = nullptr;
+    codegen_status = row_comparator_config_->Codegen(state, &compare_fn);
+    if (codegen_status.ok()) {
+      codegen_status =
+          SortedRunMerger::Codegen(state, compare_fn, &codegend_heapify_helper_fn_);
+    }
+    AddCodegenStatus(codegen_status);
   }
 }
 
@@ -131,11 +138,13 @@ Status ExchangeNode::Open(RuntimeState* state) {
   RETURN_IF_ERROR(ExecNode::Open(state));
   RETURN_IF_CANCELLED(state);
   if (is_merging_) {
+    const ExchangePlanNode& pnode = static_cast<const ExchangePlanNode&>(plan_node_);
     // CreateMerger() will populate its merging heap with batches from the stream_recvr_,
     // so it is not necessary to call FillInputRowBatch().
     RETURN_IF_ERROR(
         less_than_->Open(pool_, state, expr_perm_pool(), expr_results_pool()));
-    RETURN_IF_ERROR(stream_recvr_->CreateMerger(*less_than_.get()));
+    RETURN_IF_ERROR(stream_recvr_->CreateMerger(*less_than_.get(),
+        pnode.codegend_heapify_helper_fn_));
   } else {
     RETURN_IF_ERROR(FillInputRowBatch(state));
   }
