@@ -37,6 +37,7 @@
 #include "exec/hbase/hbase-scan-node.h"
 #include "exec/hdfs-scan-node-mt.h"
 #include "exec/hdfs-scan-node.h"
+#include "exec/iceberg-delete-node.h"
 #include "exec/kudu/kudu-scan-node-mt.h"
 #include "exec/kudu/kudu-scan-node.h"
 #include "exec/kudu/kudu-util.h"
@@ -83,6 +84,18 @@ Status PlanNode::Init(const TPlanNode& tnode, FragmentState* state) {
     RETURN_IF_ERROR(
         ScalarExpr::Create(tnode_->conjuncts, *row_descriptor_, state, &conjuncts_));
   }
+
+  if (state->query_options().compute_processing_cost) {
+    DCHECK_GT(state->fragment().effective_instance_count, 0);
+    is_mt_fragment_ = true;
+    num_instances_per_node_ = state->fragment().effective_instance_count;
+  } else if (state->query_options().mt_dop > 0) {
+    is_mt_fragment_ = true;
+    num_instances_per_node_ = state->query_options().mt_dop;
+  } else {
+    is_mt_fragment_ = false;
+    num_instances_per_node_ = 1;
+  }
   return Status::OK();
 }
 
@@ -210,6 +223,9 @@ Status PlanNode::CreatePlanNode(
     case TPlanNodeType::CARDINALITY_CHECK_NODE:
       *node = pool->Add(new CardinalityCheckPlanNode());
       break;
+    case TPlanNodeType::ICEBERG_DELETE_NODE:
+      *node = pool->Add(new IcebergDeletePlanNode());
+      break;
     default:
       map<int, const char*>::const_iterator i =
           _TPlanNodeType_VALUES_TO_NAMES.find(tnode.node_type);
