@@ -28,6 +28,7 @@
 #include "exec/sequence/hdfs-sequence-scanner.h"
 #include "exec/text/hdfs-text-scanner.h"
 #include "exec/text/hdfs-plugin-text-scanner.h"
+#include "exec/json/hdfs-json-scanner.h"
 
 
 #include <avro/errors.h>
@@ -111,6 +112,8 @@ PROFILE_DEFINE_COUNTER(BytesReadShortCircuit, STABLE_LOW, TUnit::BYTES,
     "The total number of bytes read via short circuit read");
 PROFILE_DEFINE_COUNTER(BytesReadDataNodeCache, STABLE_HIGH, TUnit::BYTES,
     "The total number of bytes read from data node cache");
+PROFILE_DEFINE_COUNTER(BytesReadEncrypted, STABLE_LOW, TUnit::BYTES,
+    "The total number of bytes read from encrypted data");
 PROFILE_DEFINE_COUNTER(BytesReadErasureCoded, STABLE_LOW, TUnit::BYTES,
     "The total number of bytes read from erasure-coded data");
 PROFILE_DEFINE_COUNTER(RemoteScanRanges, STABLE_HIGH, TUnit::UNIT,
@@ -241,6 +244,7 @@ Status HdfsScanPlanNode::Init(const TPlanNode& tnode, FragmentState* state) {
 
 Status HdfsScanPlanNode::ProcessScanRangesAndInitSharedState(FragmentState* state) {
   // Initialize the template tuple pool.
+  using namespace org::apache::impala::fb;
   shared_state_.template_pool_.reset(new MemPool(state->query_mem_tracker()));
   auto& template_tuple_map_ = shared_state_.partition_template_tuple_map_;
   ObjectPool* obj_pool = shared_state_.obj_pool();
@@ -300,9 +304,29 @@ Status HdfsScanPlanNode::ProcessScanRangesAndInitSharedState(FragmentState* stat
         file_desc->file_length = split.file_length();
         file_desc->mtime = split.mtime();
         file_desc->file_compression = CompressionTypePBToThrift(split.file_compression());
+        file_desc->is_encrypted = split.is_encrypted();
         file_desc->is_erasure_coded = split.is_erasure_coded();
-        file_desc->file_format = partition_desc->file_format();
         file_desc->file_metadata = file_metadata;
+        if (file_metadata) {
+          DCHECK(file_metadata->iceberg_metadata() != nullptr);
+          switch (file_metadata->iceberg_metadata()->file_format()) {
+            case FbIcebergDataFileFormat::FbIcebergDataFileFormat_PARQUET:
+              file_desc->file_format = THdfsFileFormat::PARQUET;
+              break;
+            case FbIcebergDataFileFormat::FbIcebergDataFileFormat_ORC:
+              file_desc->file_format = THdfsFileFormat::ORC;
+              break;
+            case FbIcebergDataFileFormat::FbIcebergDataFileFormat_AVRO:
+              file_desc->file_format = THdfsFileFormat::AVRO;
+              break;
+            default:
+              return Status(Substitute(
+                  "Unknown Iceberg file format type: $0",
+                  file_metadata->iceberg_metadata()->file_format()));
+          }
+        } else {
+          file_desc->file_format = partition_desc->file_format();
+        }
         RETURN_IF_ERROR(HdfsFsCache::instance()->GetConnection(
             native_file_path, &file_desc->fs, &fs_cache));
         shared_state_.per_type_files_[partition_desc->file_format()].push_back(file_desc);
@@ -427,8 +451,9 @@ HdfsScanNodeBase::HdfsScanNodeBase(ObjectPool* pool, const HdfsScanPlanNode& pno
             hdfs_scan_node.skip_header_line_count :
             0),
     tuple_id_(pnode.tuple_id_),
-    count_star_slot_offset_(hdfs_scan_node.__isset.count_star_slot_offset ?
-            hdfs_scan_node.count_star_slot_offset :
+    parquet_count_star_slot_offset_(
+        hdfs_scan_node.__isset.parquet_count_star_slot_offset ?
+            hdfs_scan_node.parquet_count_star_slot_offset :
             -1),
     is_partition_key_scan_(hdfs_scan_node.is_partition_key_scan),
     tuple_desc_(pnode.tuple_desc_),
@@ -486,7 +511,6 @@ Status HdfsScanNodeBase::Prepare(RuntimeState* state) {
   }
 
   // One-time initialization of state that is constant across scan ranges
-  scan_node_pool_.reset(new MemPool(mem_tracker()));
   iceberg_partition_filtering_pool_.reset(new MemPool(mem_tracker()));
   runtime_profile()->AddInfoString("Table Name", hdfs_table_->fully_qualified_name());
 
@@ -610,6 +634,7 @@ Status HdfsScanNodeBase::Open(RuntimeState* state) {
   bytes_read_short_circuit_ =
       PROFILE_BytesReadShortCircuit.Instantiate(runtime_profile());
   bytes_read_dn_cache_ = PROFILE_BytesReadDataNodeCache.Instantiate(runtime_profile());
+  bytes_read_encrypted_ = PROFILE_BytesReadEncrypted.Instantiate(runtime_profile());
   bytes_read_ec_ = PROFILE_BytesReadErasureCoded.Instantiate(runtime_profile());
   num_remote_ranges_ = PROFILE_RemoteScanRanges.Instantiate(runtime_profile());
   unexpected_remote_bytes_ =
@@ -650,7 +675,6 @@ void HdfsScanNodeBase::Close(RuntimeState* state) {
   // There should be no active hdfs read threads.
   DCHECK_EQ(active_hdfs_read_thread_counter_.value(), 0);
 
-  if (scan_node_pool_.get() != nullptr) scan_node_pool_->FreeAll();
   if (iceberg_partition_filtering_pool_.get() != nullptr) {
     iceberg_partition_filtering_pool_->FreeAll();
   }
@@ -721,6 +745,9 @@ Status HdfsScanNodeBase::IssueInitialScanRanges(RuntimeState* state) {
       case THdfsFileFormat::ORC:
         RETURN_IF_ERROR(HdfsOrcScanner::IssueInitialRanges(this, entry.second));
         break;
+      case THdfsFileFormat::JSON:
+        RETURN_IF_ERROR(HdfsJsonScanner::IssueInitialRanges(this, entry.second));
+        break;
       default:
         DCHECK(false) << "Unexpected file type " << entry.first;
     }
@@ -922,6 +949,13 @@ Status HdfsScanNodeBase::CreateAndOpenScannerHelper(HdfsPartitionDescriptor* par
       case THdfsFileFormat::ORC:
         scanner->reset(new HdfsOrcScanner(this, runtime_state_));
         break;
+      case THdfsFileFormat::JSON:
+        if (HdfsJsonScanner::HasBuiltinSupport(compression)) {
+          scanner->reset(new HdfsJsonScanner(this, runtime_state_));
+        } else {
+          return Status("Scanning compressed Json file is not implemented yet.");
+        }
+        break;
       default:
         return Status(
             Substitute("Unknown Hdfs file format type: $0", partition->file_format()));
@@ -1086,6 +1120,11 @@ bool HdfsScanPlanNode::HasVirtualColumnInTemplateTuple() const {
       // We return false at the end of the function if there are no virtual
       // columns in the template tuple.
       continue;
+    } else if (sd->virtual_column_type() == TVirtualColumnType::PARTITION_SPEC_ID) {
+      return true;
+    } else if (sd->virtual_column_type() ==
+        TVirtualColumnType::ICEBERG_PARTITION_SERIALIZED) {
+      return true;
     } else {
       // Adding DCHECK here so we don't forget to update this when adding new virtual
       // column.
@@ -1095,8 +1134,8 @@ bool HdfsScanPlanNode::HasVirtualColumnInTemplateTuple() const {
   return false;
 }
 
-void HdfsScanNodeBase::TransferToScanNodePool(MemPool* pool) {
-  scan_node_pool_->AcquireData(pool, false);
+void HdfsScanNodeBase::TransferToSharedStatePool(MemPool* pool) {
+  shared_state_->TransferToSharedStatePool(pool);
 }
 
 void HdfsScanNodeBase::UpdateHdfsSplitStats(
@@ -1218,6 +1257,7 @@ void HdfsScanNodeBase::StopAndFinalizeCounters() {
     bytes_read_local_->Set(reader_context_->bytes_read_local());
     bytes_read_short_circuit_->Set(reader_context_->bytes_read_short_circuit());
     bytes_read_dn_cache_->Set(reader_context_->bytes_read_dn_cache());
+    bytes_read_encrypted_->Set(reader_context_->bytes_read_encrypted());
     bytes_read_ec_->Set(reader_context_->bytes_read_ec());
     num_remote_ranges_->Set(reader_context_->num_remote_ranges());
     unexpected_remote_bytes_->Set(reader_context_->unexpected_remote_bytes());
@@ -1243,6 +1283,8 @@ void HdfsScanNodeBase::StopAndFinalizeCounters() {
         bytes_read_short_circuit_->value());
     ImpaladMetrics::IO_MGR_CACHED_BYTES_READ->Increment(
         bytes_read_dn_cache_->value());
+    ImpaladMetrics::IO_MGR_ENCRYPTED_BYTES_READ->Increment(
+        bytes_read_encrypted_->value());
     ImpaladMetrics::IO_MGR_ERASURE_CODED_BYTES_READ->Increment(
         bytes_read_ec_->value());
   }
@@ -1303,6 +1345,11 @@ Tuple* ScanRangeSharedState::GetTemplateTupleForPartitionId(int64_t partition_id
   return partition_template_tuple_map_[partition_id];
 }
 
+void ScanRangeSharedState::TransferToSharedStatePool(MemPool* pool) {
+  unique_lock<mutex> l(metadata_lock_);
+  template_pool_->AcquireData(pool, false);
+}
+
 void ScanRangeSharedState::UpdateRemainingScanRangeSubmissions(int32_t delta) {
   int new_val = remaining_scan_range_submissions_.Add(delta);
   DCHECK_GE(new_val, 0);
