@@ -91,6 +91,12 @@ Status ScalarExprEvaluator::Create(const ScalarExpr& root, RuntimeState* state,
     DCHECK_EQ(root.fn_ctx_idx_, -1);
     DCHECK((*eval)->fn_ctxs_ptr_ == nullptr);
   }
+  if (root.type().IsStructType()) {
+    DCHECK(root.GetNumChildren() > 0);
+    Status status = Create(root.children(), state, pool, expr_perm_pool,
+        expr_results_pool, &((*eval)->childEvaluators_));
+    DCHECK((*eval)->childEvaluators_.size() == root.GetNumChildren());
+  }
   (*eval)->initialized_ = true;
   return Status::OK();
 }
@@ -113,9 +119,15 @@ void ScalarExprEvaluator::CreateFnCtxs(RuntimeState* state, const ScalarExpr& ex
   const int fn_ctx_idx = expr.fn_ctx_idx();
   const bool has_fn_ctx = fn_ctx_idx != -1;
   vector<FunctionContext::TypeDesc> arg_types;
-  for (const ScalarExpr* child : expr.children()) {
-    CreateFnCtxs(state, *child, expr_perm_pool, expr_results_pool);
-    if (has_fn_ctx) arg_types.push_back(AnyValUtil::ColumnTypeToTypeDesc(child->type()));
+  // It's not needed to create contexts for the children of structs here as Create() is
+  // called recursively for each of their children and that will take care of the context
+  // creation as well.
+  if (!expr.type().IsStructType()) {
+    for (const ScalarExpr* child : expr.children()) {
+      CreateFnCtxs(state, *child, expr_perm_pool, expr_results_pool);
+      if (has_fn_ctx) arg_types.push_back(
+          AnyValUtil::ColumnTypeToTypeDesc(child->type()));
+    }
   }
   if (has_fn_ctx) {
     FunctionContext::TypeDesc return_type =
@@ -158,6 +170,7 @@ void ScalarExprEvaluator::Close(RuntimeState* state) {
     delete fn_ctxs_[i];
   }
   fn_ctxs_.clear();
+  for (ScalarExprEvaluator* child : childEvaluators_) child->Close(state);
   // Memory allocated by 'fn_ctx_' is still in the MemPools. It's the responsibility of
   // the owners of those pools to free it.
   closed_ = true;
@@ -355,6 +368,12 @@ void* ScalarExprEvaluator::GetValue(const ScalarExpr& expr, const TupleRow* row)
       result_.collection_val.num_tuples = v.num_tuples;
       return &result_.collection_val;
     }
+    case TYPE_STRUCT: {
+      StructVal v = expr.GetStructVal(this, row);
+      if (v.is_null) return nullptr;
+      result_.struct_val = v;
+      return &result_.struct_val;
+    }
     default:
       DCHECK(false) << "Type not implemented: " << expr.type_.DebugString();
       return nullptr;
@@ -413,6 +432,10 @@ CollectionVal ScalarExprEvaluator::GetCollectionVal(const TupleRow* row) {
   return root_.GetCollectionVal(this, row);
 }
 
+StructVal ScalarExprEvaluator::GetStructVal(const TupleRow* row) {
+  return root_.GetStructVal(this, row);
+}
+
 TimestampVal ScalarExprEvaluator::GetTimestampVal(const TupleRow* row) {
   return root_.GetTimestampVal(this, row);
 }
