@@ -18,6 +18,7 @@
 #ifndef IMPALA_EXPRS_SLOTREF_H
 #define IMPALA_EXPRS_SLOTREF_H
 
+#include "codegen/codegen-anyval.h"
 #include "exprs/scalar-expr.h"
 #include "runtime/descriptors.h"
 
@@ -45,6 +46,10 @@ class SlotRef : public ScalarExpr {
   /// does not generate the appropriate exprs).
   SlotRef(const SlotDescriptor* desc, const ColumnType& type);
 
+  /// Create a SlotRef based on the given SlotDescriptor 'desc' and make sure the type is
+  /// not TYPE_NULL (if so, replaced it with TYPE_BOOLEAN).
+  static SlotRef* TypeSafeCreate(const SlotDescriptor* desc);
+
   /// Used for testing.  GetValue will return tuple + offset interpreted as 'type'
   SlotRef(const ColumnType& type, int offset, const bool nullable = false);
 
@@ -59,6 +64,7 @@ class SlotRef : public ScalarExpr {
   const SlotId& slot_id() const { return slot_id_; }
   static const char* LLVM_CLASS_NAME;
   NullIndicatorOffset GetNullIndicatorOffset() const { return null_indicator_offset_; }
+  const SlotDescriptor* GetSlotDescriptor() const { return slot_desc_; }
   int GetSlotOffset() const { return slot_offset_; }
   virtual const TupleDescriptor* GetCollectionTupleDesc() const override;
 
@@ -78,12 +84,32 @@ class SlotRef : public ScalarExpr {
       ScalarExprEvaluator*, const TupleRow*) const override;
 
  private:
+  CodegenAnyVal CodegenValue(LlvmCodeGen* codegen, LlvmBuilder* builder,
+      llvm::Function* fn, llvm::Value* eval_ptr, llvm::Value* row_ptr,
+      llvm::BasicBlock* entry_block = nullptr);
+  void CodegenNullChecking(LlvmCodeGen* codegen, LlvmBuilder* builder, llvm::Function* fn,
+      llvm::BasicBlock* next_block_if_null, llvm::BasicBlock* next_block_if_not_null,
+      llvm::Value* tuple_ptr);
+
   int tuple_idx_;  // within row
   int slot_offset_;  // within tuple
   NullIndicatorOffset null_indicator_offset_;  // within tuple
   const SlotId slot_id_;
   bool tuple_is_nullable_; // true if the tuple is nullable.
   const SlotDescriptor* slot_desc_ = nullptr;
+
+  // After the function returns, the instruction point of the LlvmBuilder will be reset to
+  // where it was before the call.
+  CodegenAnyValReadWriteInfo CreateCodegenAnyValReadWriteInfo(LlvmCodeGen* codegen,
+      LlvmBuilder* builder,
+      llvm::Function* fn,
+      llvm::Value* eval_ptr,
+      llvm::Value* row_ptr,
+      llvm::BasicBlock* entry_block = nullptr);
+  CodegenAnyValReadWriteInfo CodegenReadSlot(LlvmCodeGen* codegen, LlvmBuilder* builder,
+      llvm::Value* eval_ptr, llvm::Value* row_ptr, llvm::BasicBlock* entry_block,
+      llvm::BasicBlock* null_block, llvm::BasicBlock* read_slot_block,
+      llvm::Value* tuple_ptr, llvm::Value* slot_offset);
 };
 
 }
