@@ -43,7 +43,11 @@
 #include <glog/raw_logging.h>
 #ifdef __linux__
 #define UNW_LOCAL_ONLY
+#ifdef __aarch64__
+#include <libunwind-aarch64.h>
+#else
 #include <libunwind.h>
+#endif //__aarch64__
 #endif
 
 #include "kudu/gutil/basictypes.h"
@@ -163,7 +167,7 @@ class CompletionFlag {
     sys_futex(reinterpret_cast<int32_t*>(&complete_),
               FUTEX_WAKE | FUTEX_PRIVATE_FLAG,
               INT_MAX, // wake all
-	      NULL, NULL,
+              nullptr, nullptr,
               0 /* ignored */);
 #endif
   }
@@ -182,7 +186,7 @@ class CompletionFlag {
       sys_futex(reinterpret_cast<int32_t*>(&complete_),
                 FUTEX_WAIT | FUTEX_PRIVATE_FLAG,
                 0, // wait if value is still 0
-                reinterpret_cast<struct kernel_timespec *>(&ts), NULL, 0);
+                reinterpret_cast<struct kernel_timespec *>(&ts), nullptr, 0);
 #else
       sched_yield();
 #endif
