@@ -23,7 +23,7 @@
 #define IMPALA_RPC_TACCEPTQUEUESERVER_H
 
 #include <thrift/concurrency/Monitor.h>
-#include <thrift/concurrency/Thread.h>
+#include <thrift/concurrency/ThreadFactory.h>
 #include <thrift/server/TServer.h>
 #include <thrift/transport/TServerTransport.h>
 
@@ -87,9 +87,7 @@ class TAcceptQueueServer : public TServer {
   void SetupConnection(std::shared_ptr<TAcceptQueueEntry> entry);
 
   // Helper function to close a client connection in case of server side errors.
-  void CleanupAndClose(const std::string& error,
-      std::shared_ptr<TTransport> input,
-      std::shared_ptr<TTransport> output,
+  void CleanupAndClose(const std::string& error, std::shared_ptr<TTransport> io_transport,
       std::shared_ptr<TTransport> client);
 
   std::shared_ptr<ThreadFactory> threadFactory_;
