@@ -132,6 +132,12 @@ DEFINE_bool(trusted_domain_use_xff_header, false,
     "domain. Only used if '--trusted_domain' is specified. Warning: Only use this if you "
     "trust the incoming connection to have this set correctly.");
 
+DEFINE_bool(trusted_domain_strict_localhost, true,
+    "If set to true and trusted_domain='localhost', this will not use reverse DNS to "
+    "determine if something is from localhost. It will only match 127.0.0.1. This is "
+    "important for security, because reverse DNS can resolve other non-local addresses "
+    "to localhost.");
+
 // This flag must be used with caution to avoid security risks.
 DEFINE_string(trusted_auth_header, "",
     "If set as non empty string, Impala will look for this header in the HTTP headers. "
@@ -166,6 +172,18 @@ DEFINE_string(jwks_file_path, "",
     "File path of the pre-installed JSON Web Key Set (JWKS) for JWT verification");
 // This specifies the URL for JWKS to be downloaded.
 DEFINE_string(jwks_url, "", "URL of the JSON Web Key Set (JWKS) for JWT verification");
+// Enables retrieving the JWKS URL without verifying the presented TLS certificate
+// from the server.
+DEFINE_bool(jwks_verify_server_certificate, true,
+    "Specifies if the TLS certificate of the JWKS server is verified when retrieving "
+    "the JWKS from the specified JWKS URL.  A certificate is considered valid if a "
+    "trust chain can be established for it, and if the certificate has a common name or "
+    "SAN that matches the server's hostname. This should only be set to false for "
+    "development / testing.");
+// Enables defining a custom pem bundle file containing root certificates to trust.
+DEFINE_string(jwks_ca_certificate, "", "File path of a pem bundle of root ca "
+    "certificates that will be trusted when retrieving the JWKS from the "
+    "specified JWKS URL.");
 DEFINE_int32(jwks_update_frequency_s, 60,
     "(Advanced) The time in seconds to wait between downloading JWKS from the specified "
     "URL.");
@@ -182,6 +200,13 @@ DEFINE_bool_hidden(jwt_allow_without_tls, false,
     "When this configuration is set to true, Impala allows JWT authentication on "
     "unsecure channel. This should be only enabled for testing, or development for which "
     "TLS is handled by proxy.");
+DEFINE_bool(enable_group_filter_check_for_authenticated_kerberos_user, false,
+    "If this configuration is set to true, Impala checks the provided "
+    "LDAP group filter, if any, with the authenticated Kerberos user. "
+    "This should be only enabled if both Kerberos and LDAP authentication are enabled "
+    "and the users in KDC and LDAP are synchronized (e.g. when the KDC and the LDAP "
+    "is the same Active Directory server). "
+    "The default value is false, which provides backwards-compatible behavior.");
 
 namespace impala {
 
@@ -256,27 +281,34 @@ static int SaslLogCallback(void* context, int level, const char* message) {
   return SASL_OK;
 }
 
-// Calls into the LDAP utils to check the provided user/pass.
+// Calls into the LDAP utils to check the provided group filters
+bool DoLdapCheckFilters(const char* user) {
+  ImpalaLdap* ldap = AuthManager::GetInstance()->GetLdap();
+  ImpalaServer* server = ExecEnv::GetInstance()->impala_server();
+  if (server == nullptr) {
+    LOG(FATAL) << "Invalid config: SASL LDAP is only supported for client connections "
+               << "to an impalad.";
+  }
+  // If the user is an authorized proxy user, we do not yet know the effective user as
+  // it may be set by 'impala.doas.user', in which case we defer checking LDAP filters
+  // until OpenSession(). Otherwise, we prefer to check the filters as easly as
+  // possible, so check them here.
+  if (server->IsAuthorizedProxyUser(user)) {
+    return true;
+  }
+  return ldap->LdapCheckFilters(user);
+}
+
+// Calls into the LDAP utils to check the provided user/pass,
+// and the provided group filters.
 bool DoLdapCheck(const char* user, const char* pass, unsigned passlen) {
   ImpalaLdap* ldap = AuthManager::GetInstance()->GetLdap();
   bool success = ldap->LdapCheckPass(user, pass, passlen);
-
-  if (success) {
-    ImpalaServer* server = ExecEnv::GetInstance()->impala_server();
-    if (server == nullptr) {
-      LOG(FATAL) << "Invalid config: SASL LDAP is only supported for client connections "
-                 << "to an impalad.";
-    }
-    // If the user is an authorized proxy user, we do not yet know the effective user as
-    // it may be set by 'impala.doas.user', in which case we defer checking LDAP filters
-    // until OpenSession(). Otherwise, we prefer to check the filters as easly as
-    // possible, so check them here.
-    if (!server->IsAuthorizedProxyUser(user)) {
-      success = ldap->LdapCheckFilters(user);
-    }
+  if (!success) {
+    return false;
   }
 
-  return success;
+  return DoLdapCheckFilters(user);
 }
 
 // Wrapper around the function we use to check passwords with LDAP which has the function
@@ -468,8 +500,56 @@ int SaslAuthorizeInternal(sasl_conn_t* conn, void* context,
   return SASL_BADAUTH;
 }
 
+// If Kerberos and LDAP authentications are enabled and
+// enable_group_filter_check_for_authenticated_kerberos_user flag is set,
+// then this callback checks if the authenticated user passes LDAP group
+// filters.
+//
+// conn: Sasl connection - Ignored
+// context: Ignored, always NULL
+// requested_user: The identity/username to authorize
+// rlen: Length of above
+// auth_identity: "The identity associated with the secret"
+// alen: Length of above
+// def_realm: Default user realm
+// urlen: Length of above
+// propctx: Auxiliary properties - Ignored
+// Return: SASL_OK
+static int SaslKerberosAuthorizeExternal(sasl_conn_t* conn, void* context,
+    const char* requested_user, unsigned rlen,
+    const char* auth_identity, unsigned alen,
+    const char* def_realm, unsigned urlen,
+    struct propctx* propctx) {
+  if (FLAGS_enable_ldap_auth &&
+      FLAGS_enable_group_filter_check_for_authenticated_kerberos_user) {
+    DCHECK(IsKerberosEnabled());
+
+    string username = string(requested_user, rlen);
+    string short_user =
+        GetShortUsernameFromKerberosPrincipal(username);
+
+    LOG(INFO) << "Checking LDAP group filters for "
+              << "username \"" << short_user << "\" "
+              << "parsed from user principal \""
+              << username << "\".";
+
+    bool success = DoLdapCheckFilters(short_user.c_str());
+    if (!success) {
+      LOG(WARNING) << "Got authenticated principal but the "
+                   << "user \"" << short_user << "\" "
+                   << "didn't pass the group filters.";
+      return SASL_BADAUTH;
+    }
+  }
+
+  LOG(INFO) << "Successfully authenticated client user \""
+            << string(requested_user, rlen) << "\"";
+  return SASL_OK;
+}
+
 // This callback could be used to authorize or restrict access to certain
-// users.  Currently it is used to log a message that we successfully
+// users when authenticating with LDAP.
+// Currently it is used to log a message that we successfully
 // authenticated with a user on an external connection.
 //
 // conn: Sasl connection - Ignored
@@ -482,7 +562,7 @@ int SaslAuthorizeInternal(sasl_conn_t* conn, void* context,
 // urlen: Length of above
 // propctx: Auxiliary properties - Ignored
 // Return: SASL_OK
-static int SaslAuthorizeExternal(sasl_conn_t* conn, void* context,
+static int SaslLdapAuthorizeExternal(sasl_conn_t* conn, void* context,
     const char* requested_user, unsigned rlen,
     const char* auth_identity, unsigned alen,
     const char* def_realm, unsigned urlen,
@@ -545,7 +625,10 @@ bool GetUsernameFromBasicAuthHeader(
 
 bool TrustedDomainCheck(ThriftServer::ConnectionContext* connection_context,
     const AuthenticationHash& hash, const std::string& origin, string auth_header) {
-  if (!IsTrustedDomain(origin, FLAGS_trusted_domain)) return false;
+  if (!IsTrustedDomain(origin, FLAGS_trusted_domain,
+          FLAGS_trusted_domain_strict_localhost)) {
+    return false;
+  }
 
   if (!GetUsernameFromBasicAuthHeader(connection_context, auth_header)) return false;
   // Create a cookie to return.
@@ -622,11 +705,12 @@ bool JWTTokenAuth(ThriftServer::ConnectionContext* connection_context,
   return true;
 }
 
-// Performs a step of SPNEGO auth for the HTTP transport and sets the username on
-// 'connection_context' if auth is successful. 'header_token' is the value from an
-// 'Authorization: Negotiate" header. Returns true if the step was successful and sets
-// 'is_complete' to indicate if more steps are needed. Returns false if an error was
-// encountered and the connection should be closed.
+// Performs a step of SPNEGO auth for the HTTP transport and sets the username and
+// kerberos_user_principal on 'connection_context' if auth is successful.
+// 'header_token' is the value from an 'Authorization: Negotiate" header.
+// Returns true if the step was successful and sets 'is_complete' to indicate
+// if more steps are needed. Returns false if an error was encountered and the
+// connection should be closed.
 bool NegotiateAuth(ThriftServer::ConnectionContext* connection_context,
     const AuthenticationHash& hash, const std::string& header_token, bool* is_complete) {
   if (header_token.empty()) {
@@ -658,8 +742,31 @@ bool NegotiateAuth(ThriftServer::ConnectionContext* connection_context,
                     << TNetworkAddressToString(connection_context->network_address)
                     << ": " << spnego_status.ToString();
       } else {
+        if (FLAGS_enable_ldap_auth &&
+            FLAGS_enable_group_filter_check_for_authenticated_kerberos_user) {
+
+          string short_user = GetShortUsernameFromKerberosPrincipal(username);
+
+          LOG(INFO) << "Checking LDAP group filters for "
+                    << "username \"" << short_user << "\" "
+                    << "parsed from user principal \""
+                    << username << "\".";
+
+          bool success = DoLdapCheckFilters(short_user.c_str());
+          if (!success) {
+            LOG(WARNING) << "Got authenticated principal for SPNEGO-authenticated "
+                        << "connection from "
+                        << TNetworkAddressToString(connection_context->network_address)
+                        << " but the authenticated user \"" << short_user << "\" "
+                        << "didn't pass the group filters.";
+            return false;
+          }
+        }
+
         // Authentication was successful, so set the username on the connection.
         connection_context->username = username;
+        // Save the username as Kerberos user principal in the connection context.
+        connection_context->kerberos_user_principal = username;
         // Create a cookie to return.
         connection_context->return_headers.push_back(
             Substitute("Set-Cookie: $0", GenerateCookie(username, hash)));
@@ -903,7 +1010,7 @@ Status InitAuth(const string& appname) {
       KERB_EXT_CALLBACKS[0].context = ((void *)"Kerberos (external)");
 
       KERB_EXT_CALLBACKS[1].id = SASL_CB_PROXY_POLICY;
-      KERB_EXT_CALLBACKS[1].proc = (int (*)())&SaslAuthorizeExternal;
+      KERB_EXT_CALLBACKS[1].proc = (int (*)())&SaslKerberosAuthorizeExternal;
       KERB_EXT_CALLBACKS[1].context = NULL;
 
       KERB_EXT_CALLBACKS[2].id = SASL_CB_LIST_END;
@@ -918,7 +1025,7 @@ Status InitAuth(const string& appname) {
       LDAP_EXT_CALLBACKS[0].context = ((void *)"LDAP");
 
       LDAP_EXT_CALLBACKS[1].id = SASL_CB_PROXY_POLICY;
-      LDAP_EXT_CALLBACKS[1].proc = (int (*)())&SaslAuthorizeExternal;
+      LDAP_EXT_CALLBACKS[1].proc = (int (*)())&SaslLdapAuthorizeExternal;
       LDAP_EXT_CALLBACKS[1].context = NULL;
 
       // This last callback is where we take the password and turn around and
@@ -1253,6 +1360,11 @@ void SecureAuthProvider::SetupConnectionContext(
       socket = down_cast<TSocket*>(sasl_transport->getUnderlyingTransport().get());
       // Get the username from the transport.
       connection_ptr->username = sasl_transport->getUsername();
+      if (sasl_transport->getMechanismName() == KERBEROS_MECHANISM) {
+        // Save the username as Kerberos user principal in the connection context
+        // if the actual auth mechanism is Kerberos.
+        connection_ptr->kerberos_user_principal = connection_ptr->username;
+      }
       break;
     }
     case ThriftServer::HTTP: {
@@ -1381,6 +1493,14 @@ Status AuthManager::Init() {
   TSSLSocketFactory::setManualOpenSSLInitialization(true);
   kudu::security::InitializeOpenSSL();
   LOG(INFO) << "Initialized " << OPENSSL_VERSION_TEXT;
+  LOG(INFO) << "Runtime OpenSSL version " << SSLeay_version(SSLEAY_VERSION);
+  unsigned long openssl_version = SSLeay();
+  // Check if we are running against an OpenSSL version that is vulnerable to
+  // CVE-2009-3555
+  if (openssl_version >= 0x10100000L && openssl_version <= 0x1010007fL)  {
+    LOG(WARNING) <<
+        "OpenSSL runtime version detected that is vulnerable to CVE-2009-3555";
+  }
 
   // Could use any other requiered flag for SAML
   bool use_saml = !FLAGS_saml2_sp_callback_url.empty();
