@@ -114,6 +114,9 @@ class ThriftServer {
     std::string saml_relay_state;
     std::unique_ptr<TWrappedHttpRequest> request;
     std::unique_ptr<TWrappedHttpResponse> response;
+    // Used in case of Kerberos authentication only to store the authenticated Kerberos
+    // user principal
+    std::string kerberos_user_principal;
   };
 
   /// Interface class for receiving connection creation / termination events.
@@ -178,6 +181,11 @@ class ThriftServer {
   /// It is also only safe to reference the returned value during an RPC method.
   static const ConnectionContext* GetThreadConnectionContext();
 
+  typedef std::vector<std::shared_ptr<const ConnectionContext>> ConnectionContextList;
+
+  /// Gets connection contexts for the thrift server.
+  void GetConnectionContextList(ConnectionContextList* connection_contexts);
+
  private:
   friend class ThriftServerBuilder;
   friend class apache::thrift::server::TAcceptQueueServer;
