@@ -135,8 +135,9 @@ class Coordinator::BackendState {
 
   /// Update overall execution status, including the instances' exec status/profiles
   /// and the error log, if this backend is not already done. Updates the fragment
-  /// instances' TExecStats in exec_summary (exec_summary->nodes.exec_stats) and updates
-  /// scan_range_progress with any newly-completed scan ranges.
+  /// instances' TExecStats in exec_summary (exec_summary->nodes.exec_stats), updates
+  /// scan_range_progress with any newly-completed scan ranges and updates query_progress
+  /// with any newly-completed fragment instances.
   ///
   /// If any instance reports an error, the overall execution status becomes the first
   /// reported error status. Returns true iff this update changed IsDone() from false
@@ -145,8 +146,8 @@ class Coordinator::BackendState {
   /// FragmentInstanceExecStatusPB in backend_exec_status to the vector aux_error_info.
   bool ApplyExecStatusReport(const ReportExecStatusRequestPB& backend_exec_status,
       const TRuntimeProfileForest& thrift_profiles, ExecSummary* exec_summary,
-      ProgressUpdater* scan_range_progress, DmlExecState* dml_exec_state,
-      std::vector<AuxErrorInfoPB>* aux_error_info,
+      ProgressUpdater* scan_range_progress, ProgressUpdater* query_progress,
+      DmlExecState* dml_exec_state, std::vector<AuxErrorInfoPB>* aux_error_info,
       const std::vector<FragmentStats*>& fragment_stats);
 
   /// Merges the incoming 'thrift_profile' into this backend state's host profile.
