@@ -152,19 +152,19 @@ Status Coordinator::Exec() {
              << " stmt=" << request.query_ctx.client_request.stmt;
   stmt_type_ = request.stmt_type;
 
-  query_profile_ =
-      RuntimeProfile::Create(obj_pool(), "Execution Profile " + PrintId(query_id()));
+  query_profile_ = RuntimeProfile::Create(
+      obj_pool(), "Execution Profile " + PrintId(query_id()), false);
   finalization_timer_ = PROFILE_FinalizationTimer.Instantiate(query_profile_);
   filter_updates_received_ = PROFILE_FiltersReceived.Instantiate(query_profile_);
 
-  host_profiles_ = RuntimeProfile::Create(obj_pool(), "Per Node Profiles");
+  host_profiles_ = RuntimeProfile::Create(obj_pool(), "Per Node Profiles", false);
   query_profile_->AddChild(host_profiles_);
 
   SCOPED_TIMER(query_profile_->total_time_counter());
 
   // initialize progress updater
   const string& str = Substitute("Query $0", PrintId(query_id()));
-  progress_.Init(str, exec_params_.query_schedule().num_scan_ranges());
+  scan_progress_.Init(str, exec_params_.query_schedule().num_scan_ranges());
 
   query_state_ = ExecEnv::GetInstance()->query_exec_mgr()->CreateQueryState(
       query_ctx(), exec_params_.query_schedule().coord_backend_mem_limit());
@@ -177,6 +177,14 @@ Status Coordinator::Exec() {
   InitBackendStates();
   exec_summary_.Init(exec_params_);
 
+  int64_t total_finstances = 0;
+  for (BackendState* backend_state : backend_states_) {
+    total_finstances += backend_state->exec_params().instance_params().size();
+  }
+  const string& query_progress_str =
+      Substitute("Query $0 progress", PrintId(query_id()));
+  query_progress_.Init(query_progress_str, total_finstances);
+
   if (filter_mode_ != TRuntimeFilterMode::OFF) {
     // Populate the runtime filter routing table. This should happen before starting the
     // fragment instances. This code anticipates the indices of the instance states
@@ -654,6 +662,7 @@ string Coordinator::FilterDebugString() {
 
       // Also add the min/max value for the accumulated filter as follows.
       //  'PartialUpdates' - The min and the max are partially updated;
+      //  'LOCAL'          - It is a local filter that is not aggregate in coordinator;
       //  'AlwaysTrue'     - One received filter is AlwaysTrue;
       //  'AlwaysFalse'    - No filter is received or all received filters are empty;
       //  'Real values'    - The final accumulated min/max from all filters received.
@@ -678,9 +687,12 @@ string Coordinator::FilterDebugString() {
             row.push_back(MinMaxFilter::DebugString(minmax_filterPB.max(),
                 ColumnType::FromThrift(state.desc().src_expr.nodes[0].type)));
           }
+        } else if (state.desc().has_remote_targets) {
+          row.push_back("PartialUpdates");
+          row.push_back("PartialUpdates");
         } else {
-          row.push_back("PartialUpdates");
-          row.push_back("PartialUpdates");
+          row.push_back("LOCAL");
+          row.push_back("LOCAL");
         }
       }
       row.push_back("");
@@ -699,8 +711,10 @@ string Coordinator::FilterDebugString() {
         } else {
           row.push_back(std::to_string(in_list_filterPB.value().size()));
         }
-      } else {
+      } else if (state.desc().has_remote_targets) {
         row.push_back("PartialUpdates");
+      } else {
+        row.push_back("LOCAL");
       }
     }
     table_printer.AddRow(row);
@@ -813,7 +827,10 @@ void Coordinator::HandleExecStateTransition(
   }
   ReleaseQueryAdmissionControlResources();
   // Once the query has released its admission control resources, update its end time.
-  parent_request_state_->UpdateEndTime();
+  // However, for non Query statement like DML statement, we still need to update HMS
+  // after the query finishes. So the end time of non Query statement is not set here.
+  // Instead, we set it in ClientRequestState::Wait().
+  if (stmt_type_ == TStmtType::QUERY) parent_request_state_->UpdateEndTime();
   // Can compute summary only after we stop accepting reports from the backends. Both
   // WaitForBackends() and CancelBackends() ensures that.
   // TODO: should move this off of the query execution path?
@@ -850,7 +867,7 @@ Status Coordinator::FinalizeHdfsDml() {
   DCHECK(has_called_wait_.Load());
   DCHECK(finalize_params() != nullptr);
   bool is_hive_acid = finalize_params()->__isset.write_id;
-  bool is_iceberg_table = finalize_params()->__isset.spec_id;
+  bool is_iceberg_table = finalize_params()->__isset.iceberg_params;
 
   VLOG_QUERY << "Finalizing query: " << PrintId(query_id());
   SCOPED_TIMER(finalization_timer_);
@@ -947,7 +964,7 @@ Status Coordinator::Wait() {
     RETURN_IF_ERROR(UpdateExecState(FinalizeResultSink(), nullptr, FLAGS_hostname));
   }
 
-  // DML requests are finished at this point.
+  // DML queries are finished at this point.
   RETURN_IF_ERROR(SetNonErrorTerminalState(ExecState::RETURNED_RESULTS));
   query_profile_->AddInfoString(
       "DML Stats", dml_exec_state_.OutputPartitionStats("\n"));
@@ -1047,7 +1064,8 @@ Status Coordinator::UpdateBackendExecStatus(const ReportExecStatusRequestPB& req
   vector<AuxErrorInfoPB> aux_error_info;
 
   if (backend_state->ApplyExecStatusReport(request, thrift_profiles, &exec_summary_,
-          &progress_, &dml_exec_state_, &aux_error_info, fragment_stats_)) {
+          &scan_progress_, &query_progress_, &dml_exec_state_, &aux_error_info,
+          fragment_stats_)) {
     // This backend execution has completed.
     if (VLOG_QUERY_IS_ON) {
       // Don't log backend completion if the query has already been cancelled.
@@ -1149,7 +1167,7 @@ Status Coordinator::UpdateBlacklistWithAuxErrorInfo(
   // the failed RPC. Only blacklist one node per ReportExecStatusRequestPB to avoid
   // blacklisting nodes too aggressively. Currently, only blacklist the first node
   // that contains a valid RPCErrorInfoPB object.
-  for (auto aux_error : *aux_error_info) {
+  for (const auto& aux_error : *aux_error_info) {
     if (aux_error.has_rpc_error_info()) {
       RPCErrorInfoPB rpc_error_info = aux_error.rpc_error_info();
       DCHECK(rpc_error_info.has_dest_node());
@@ -1239,6 +1257,7 @@ void Coordinator::HandleFailedExecRpcs(vector<BackendState*> failed_backend_stat
 
   // Create an error based on the Exec RPC failure Status
   vector<string> backend_addresses;
+  backend_addresses.reserve(failed_backend_states.size());
   for (BackendState* backend_state : failed_backend_states) {
     backend_addresses.push_back(
         NetworkAddressPBToString(backend_state->krpc_impalad_address()));
@@ -1411,6 +1430,7 @@ void Coordinator::ReleaseBackendAdmissionControlResources(
       parent_request_state_->admission_control_client();
   DCHECK(admission_control_client != nullptr);
   vector<NetworkAddressPB> host_addrs;
+  host_addrs.reserve(backend_states.size());
   for (auto backend_state : backend_states) {
     host_addrs.push_back(backend_state->impalad_address());
   }
