@@ -24,6 +24,7 @@
 #include <gutil/strings/substitute.h>
 
 #include "catalog/catalog-service-client-wrapper.h"
+#include "codegen/llvm-codegen-cache.h"
 #include "common/logging.h"
 #include "common/object-pool.h"
 #include "exec/kudu/kudu-util.h"
@@ -50,6 +51,7 @@
 #include "service/data-stream-service.h"
 #include "service/frontend.h"
 #include "service/impala-server.h"
+#include "statestore/statestore-subscriber-catalog.h"
 #include "statestore/statestore-subscriber.h"
 #include "util/cpu-info.h"
 #include "util/debug-util.h"
@@ -99,6 +101,8 @@ DEFINE_int32(admission_control_slots, 0,
     "this backend. The degree of parallelism of the query determines the number of slots "
     "that it needs. Defaults to number of cores / -num_cores for executors, and 8x that "
     "value for dedicated coordinators).");
+DEFINE_string(codegen_cache_capacity, "1GB",
+    "Specify the capacity of the codegen cache. If set to 0, codegen cache is disabled.");
 
 DEFINE_bool(use_local_catalog, false,
     "Use the on-demand metadata feature in coordinators. If this is set, coordinators "
@@ -136,6 +140,8 @@ DECLARE_int32(webserver_port);
 DECLARE_int64(tcmalloc_max_total_thread_cache_bytes);
 DECLARE_string(admission_service_host);
 DECLARE_int32(admission_service_port);
+DECLARE_string(catalog_service_host);
+DECLARE_int32(catalog_service_port);
 
 DECLARE_string(ssl_client_ca_certificate);
 
@@ -165,6 +171,10 @@ DEFINE_string(metrics_webserver_interface, "",
 
 const static string DEFAULT_FS = "fs.defaultFS";
 
+// The max percentage that the codegen cache can take from the total process memory.
+// The value is set to 10%.
+const double MAX_CODEGEN_CACHE_MEM_PERCENT = 0.1;
+
 // The multiplier for how many queries a dedicated coordinator can run compared to an
 // executor. This is only effective when using non-default settings for executor groups
 // and the absolute value can be overridden by the '--admission_control_slots' flag.
@@ -238,15 +248,34 @@ ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
   TNetworkAddress statestore_address =
       MakeNetworkAddress(statestore_host, statestore_port);
 
+  catalogd_address_ = std::make_shared<const TNetworkAddress>(
+      MakeNetworkAddress(FLAGS_catalog_service_host, FLAGS_catalog_service_port));
+  active_catalogd_version_checker_.reset(new ActiveCatalogdVersionChecker());
+
+  TStatestoreSubscriberType::type subscriber_type = TStatestoreSubscriberType::UNKNOWN;
+  if (FLAGS_is_coordinator) {
+    subscriber_type = FLAGS_is_executor ? TStatestoreSubscriberType::COORDINATOR_EXECUTOR
+                                        : TStatestoreSubscriberType::COORDINATOR;
+  } else if (FLAGS_is_executor) {
+    subscriber_type = TStatestoreSubscriberType::EXECUTOR;
+  }
   // Set StatestoreSubscriber::subscriber_id as hostname + krpc_port.
   statestore_subscriber_.reset(new StatestoreSubscriber(
       Substitute("impalad@$0:$1", FLAGS_hostname, FLAGS_krpc_port), subscriber_address,
-      statestore_address, metrics_.get()));
+      statestore_address, metrics_.get(), subscriber_type));
 
   if (FLAGS_is_coordinator) {
     hdfs_op_thread_pool_.reset(
         CreateHdfsOpThreadPool("hdfs-worker-pool", FLAGS_num_hdfs_worker_threads, 1024));
+
+    StatestoreSubscriber::UpdateCatalogdCallback update_catalogd_cb =
+        bind<void>(mem_fn(&ExecEnv::UpdateActiveCatalogd), this, _1, _2, _3);
+    statestore_subscriber_->AddUpdateCatalogdTopic(update_catalogd_cb);
   }
+  StatestoreSubscriber::CompleteRegistrationCallback complete_registration_cb =
+      bind<void>(mem_fn(&ExecEnv::SetStatestoreRegistrationCompleted), this);
+  statestore_subscriber_->AddCompleteRegistrationTopic(complete_registration_cb);
+
   if (FLAGS_is_coordinator && !AdmissionServiceEnabled()) {
     // We only need a Scheduler if we're performing admission control locally, i.e. if
     // this is a coordinator and there isn't an admissiond.
@@ -266,15 +295,6 @@ ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
         FLAGS_metrics_webserver_port, metrics_.get(), Webserver::AuthMode::NONE));
   }
 
-  if (AdmissionServiceEnabled()) {
-    IpAddr ip;
-    ABORT_IF_ERROR(HostnameToIpAddr(FLAGS_admission_service_host, &ip));
-    // TODO: get BackendId of admissiond in global admission control mode.
-    // Use admissiond's IP address as unique ID for UDS now.
-    admission_service_address_ = MakeNetworkAddressPB(
-        ip, FLAGS_admission_service_port, UdsAddressUniqueIdPB::IP_ADDRESS);
-  }
-
   exec_env_ = this;
 }
 
@@ -325,7 +345,35 @@ Status ExecEnv::Init() {
     }
   }
 
-  bool is_percent;
+  bool is_percent = false;
+  int64_t codegen_cache_capacity =
+      ParseUtil::ParseMemSpec(FLAGS_codegen_cache_capacity, &is_percent, 0);
+  if (codegen_cache_capacity > 0) {
+    // If codegen_cache_capacity is larger than 0, the number should not be a percentage.
+    DCHECK(!is_percent);
+    int64_t codegen_cache_limit = admit_mem_limit_ * MAX_CODEGEN_CACHE_MEM_PERCENT;
+    DCHECK(codegen_cache_limit > 0);
+    if (codegen_cache_capacity > codegen_cache_limit) {
+      LOG(INFO) << "CodeGen Cache capacity changed from "
+                << PrettyPrinter::Print(codegen_cache_capacity, TUnit::BYTES) << " to "
+                << PrettyPrinter::Print(codegen_cache_limit, TUnit::BYTES)
+                << " due to reaching the limit.";
+      codegen_cache_capacity = codegen_cache_limit;
+    }
+    codegen_cache_.reset(new CodeGenCache(metrics_.get()));
+    RETURN_IF_ERROR(codegen_cache_->Init(codegen_cache_capacity));
+    LOG(INFO) << "CodeGen Cache initialized with capacity "
+              << PrettyPrinter::Print(codegen_cache_capacity, TUnit::BYTES);
+    // Preserve the memory for codegen cache.
+    admit_mem_limit_ -= codegen_cache_capacity;
+    DCHECK_GT(admit_mem_limit_, 0);
+  } else {
+    LOG(INFO) << "CodeGen Cache is disabled.";
+  }
+
+  LOG(INFO) << "Admit memory limit: "
+            << PrettyPrinter::Print(admit_mem_limit_, TUnit::BYTES);
+
   int64_t buffer_pool_limit = ParseUtil::ParseMemSpec(FLAGS_buffer_pool_limit,
       &is_percent, admit_mem_limit_);
   if (buffer_pool_limit <= 0) {
@@ -473,6 +521,7 @@ Status ExecEnv::StartStatestoreSubscriberService() {
       status.AddDetail("Statestore subscriber did not start up.");
       return status;
     }
+    if (statestore_subscriber_->IsRegistered()) SetStatestoreRegistrationCompleted();
   }
 
   return Status::OK();
@@ -515,8 +564,7 @@ void ExecEnv::SetImpalaServer(ImpalaServer* server) {
   }
 }
 
-void ExecEnv::InitBufferPool(int64_t min_buffer_size, int64_t capacity,
-    int64_t clean_pages_limit) {
+void ExecEnv::InitTcMallocAggressiveDecommit() {
 #if !defined(ADDRESS_SANITIZER) && !defined(THREAD_SANITIZER)
   // Aggressive decommit is required so that unused pages in the TCMalloc page heap are
   // not backed by physical pages and do not contribute towards memory consumption.
@@ -524,6 +572,11 @@ void ExecEnv::InitBufferPool(int64_t min_buffer_size, int64_t capacity,
   MallocExtension::instance()->SetNumericProperty(
       "tcmalloc.aggressive_memory_decommit", 1);
 #endif
+}
+
+void ExecEnv::InitBufferPool(int64_t min_buffer_size, int64_t capacity,
+    int64_t clean_pages_limit) {
+  InitTcMallocAggressiveDecommit();
   buffer_pool_.reset(
       new BufferPool(metrics_.get(), min_buffer_size, capacity, clean_pages_limit));
   buffer_reservation_.reset(new ReservationTracker());
@@ -601,4 +654,70 @@ bool ExecEnv::AdmissionServiceEnabled() const {
   return !FLAGS_admission_service_host.empty();
 }
 
+Status ExecEnv::GetAdmissionServiceAddress(
+    NetworkAddressPB& admission_service_address) const {
+  if (AdmissionServiceEnabled()) {
+    IpAddr ip;
+    RETURN_IF_ERROR(HostnameToIpAddr(FLAGS_admission_service_host, &ip));
+    // TODO: get BackendId of admissiond in global admission control mode.
+    // Use admissiond's IP address as unique ID for UDS now.
+    admission_service_address = MakeNetworkAddressPB(
+        ip, FLAGS_admission_service_port, UdsAddressUniqueIdPB::IP_ADDRESS);
+  }
+  return Status::OK();
+}
+
+void ExecEnv::UpdateActiveCatalogd(bool is_registration_reply,
+    int64 active_catalogd_version, const TCatalogRegistration& catalogd_registration) {
+  std::lock_guard<std::mutex> l(catalogd_address_lock_);
+  if (!active_catalogd_version_checker_->CheckActiveCatalogdVersion(
+          is_registration_reply, active_catalogd_version)) {
+    return;
+  }
+  if (catalogd_registration.address.hostname.empty()
+      || catalogd_registration.address.port == 0) {
+    return;
+  }
+  if (catalogd_registration.protocol
+      != statestore_subscriber_->GetCatalogProtocolVersion()) {
+    // This should not happen now. Since we bump up catalog and statestore service
+    // versions at same time, coordinators must have same protocol versions as catalogd
+    // if they can join one cluster. But in future, it may happen if the two protocol
+    // versions are not updated at same time for some reason.
+    // Note that Impalad coordinators must wait for their local replica of the catalog to
+    // be initialized from the statestore prior to opening up client ports. If the
+    // coordinator has incompatible protocol version from catalogd, it should not open
+    // client ports.
+    LOG(INFO) << "Protocol version of Catalog service does not match the protocol "
+              << "version of registered catalogd: "
+              << statestore_subscriber_->GetCatalogProtocolVersion()
+              << " vs. " << catalogd_registration.protocol;
+  }
+  DCHECK(catalogd_address_.get() != nullptr);
+  if (!is_catalogd_address_metric_set_) {
+    // At least set the metric once.
+    is_catalogd_address_metric_set_ = true;
+    ImpaladMetrics::ACTIVE_CATALOGD_ADDRESS->SetValue(
+        TNetworkAddressToString(catalogd_registration.address));
+  }
+  bool is_matching = (catalogd_registration.address.port == catalogd_address_->port
+      && catalogd_registration.address.hostname == catalogd_address_->hostname);
+  if (!is_matching) {
+    LOG(INFO) << "The address of Catalog service is changed from "
+              << TNetworkAddressToString(*catalogd_address_.get())
+              << " to " << TNetworkAddressToString(catalogd_registration.address);
+    catalogd_address_ =
+        std::make_shared<const TNetworkAddress>(catalogd_registration.address);
+    ImpaladMetrics::ACTIVE_CATALOGD_ADDRESS->SetValue(
+        TNetworkAddressToString(catalogd_registration.address));
+  }
+}
+
+std::shared_ptr<const TNetworkAddress> ExecEnv::GetCatalogdAddress() const {
+  std::lock_guard<std::mutex> l(catalogd_address_lock_);
+  DCHECK(catalogd_address_.get() != nullptr);
+  std::shared_ptr<const TNetworkAddress> address = catalogd_address_;
+  return address;
+}
+
 } // namespace impala
