@@ -25,6 +25,7 @@
 #include <kudu/client/client.h>
 
 // NOTE: try not to add more headers here: exec-env.h is included in many many files.
+#include "common/atomic.h"
 #include "common/global-types.h"
 #include "common/status.h"
 #include "runtime/client-cache-types.h"
@@ -41,6 +42,7 @@ class KuduClient;
 
 namespace impala {
 
+class ActiveCatalogdVersionChecker;
 class AdmissionController;
 class BufferPool;
 class CallableThreadPool;
@@ -69,6 +71,8 @@ class SystemStateInfo;
 class ThreadResourceMgr;
 class TmpFileMgr;
 class Webserver;
+class CodeGenCache;
+class TCatalogRegistration;
 
 namespace io {
   class DiskIoMgr;
@@ -151,6 +155,8 @@ class ExecEnv {
   Scheduler* scheduler() { return scheduler_.get(); }
   AdmissionController* admission_controller() { return admission_controller_.get(); }
   StatestoreSubscriber* subscriber() { return statestore_subscriber_.get(); }
+  CodeGenCache* codegen_cache() const { return codegen_cache_.get(); }
+  bool codegen_cache_enabled() const { return codegen_cache_ != nullptr; }
 
   const TNetworkAddress& configured_backend_address() const {
     return configured_backend_address_;
@@ -182,13 +188,36 @@ class ExecEnv {
   int64_t admit_mem_limit() const { return admit_mem_limit_; }
   int64_t admission_slots() const { return admission_slots_; }
 
-  const NetworkAddressPB& admission_service_address() const {
-    return admission_service_address_;
-  }
+  /// Gets the resolved IP address and port where the admission control service is
+  /// running, if enabled.
+  Status GetAdmissionServiceAddress(NetworkAddressPB& address) const;
 
   /// Returns true if the admission control service is enabled.
   bool AdmissionServiceEnabled() const;
 
+  /// Returns true if the registration with statestore is completed.
+  bool IsStatestoreRegistrationCompleted() const {
+    return statestore_registration_completed_.Load() != 0;
+  }
+
+  /// Set the flag when the registration with statestore is completed.
+  void SetStatestoreRegistrationCompleted() {
+    statestore_registration_completed_.CompareAndSwap(0, 1);
+  }
+
+  /// Callback function for receiving notification of new active catalogd.
+  /// This function is called when active catalogd is found from registration process,
+  /// or UpdateCatalogd RPC is received. The two kinds of RPCs could be received out of
+  /// sending order.
+  /// Reset 'last_active_catalogd_version_' if 'is_registration_reply' is true and
+  /// 'active_catalogd_version' is negative. In this case, 'catalogd_registration' is
+  /// invalid and should not be used.
+  void UpdateActiveCatalogd(bool is_registration_reply, int64_t active_catalogd_version,
+      const TCatalogRegistration& catalogd_registration);
+
+  /// Return the current address of Catalog service.
+  std::shared_ptr<const TNetworkAddress> GetCatalogdAddress() const;
+
  private:
   // Used to uniquely identify this impalad.
   BackendIdPB backend_id_;
@@ -232,6 +261,9 @@ class ExecEnv {
   /// Tracks system resource usage which we then include in profiles.
   boost::scoped_ptr<SystemStateInfo> system_state_info_;
 
+  /// Singleton cache for codegen functions.
+  boost::scoped_ptr<CodeGenCache> codegen_cache_;
+
   /// Not owned by this class
   ImpalaServer* impala_server_ = nullptr;
   MetricGroup* rpc_metrics_ = nullptr;
@@ -244,7 +276,7 @@ class ExecEnv {
   friend class DataStreamTest;
 
   // For access to InitHadoopConfig().
-  FRIEND_TEST(HdfsUtilTest, CheckFilesystemsMatch);
+  FRIEND_TEST(HdfsUtilTest, CheckFilesystemsAndBucketsMatch);
 
   static ExecEnv* exec_env_;
   bool is_fe_tests_ = false;
@@ -290,13 +322,31 @@ class ExecEnv {
   /// this backend. Queries take up multiple slots only when mt_dop > 1.
   int64_t admission_slots_;
 
-  /// If the admission control service is enabled, the resolved IP address and port where
-  /// the service is running.
-  NetworkAddressPB admission_service_address_;
+  /// Flag that indicate if the registration with statestore is completed.
+  AtomicInt32 statestore_registration_completed_{0};
+
+  /// Current address of Catalog service
+  std::shared_ptr<const TNetworkAddress> catalogd_address_;
+
+  /// Object to track the version of received active catalogd.
+  boost::scoped_ptr<ActiveCatalogdVersionChecker> active_catalogd_version_checker_;
+
+  /// Flag that indicate if the metric for catalogd address has been set.
+  bool is_catalogd_address_metric_set_ = false;
+
+  /// Protects catalogd_address_ and active_catalogd_version_tracker_.
+  mutable std::mutex catalogd_address_lock_;
 
   /// Initialize ExecEnv based on Hadoop config from frontend.
   Status InitHadoopConfig();
 
+  /// Set tcmalloc's aggressive_memory_decommit=1. This needs to be called before
+  /// initializing the buffer pool, because the buffer pool asserts that this
+  /// property is set and newer versions of tcmalloc do not set it by default.
+  /// InitBufferPool() calls this automatically, so this is only used directly by
+  /// TestEnv.
+  void InitTcMallocAggressiveDecommit();
+
   /// Initialise 'buffer_pool_' and 'buffer_reservation_' with given capacity.
   void InitBufferPool(int64_t min_page_len, int64_t capacity, int64_t clean_pages_limit);
 
