@@ -342,6 +342,9 @@ class DiskIoMgr : public CacheLineAligned {
   /// The disk ID (and therefore disk_queues_ index) used for SFS accesses.
   int RemoteSFSDiskId() const { return num_local_disks() + REMOTE_SFS_DISK_OFFSET; }
 
+  /// The disk ID (and therefore disk_queues_ index) used for OBS accesses.
+  int RemoteOBSDiskId() const { return num_local_disks() + REMOTE_OBS_DISK_OFFSET; }
+
   /// Dumps the disk IoMgr queues (for readers and disks)
   std::string DebugString();
 
@@ -399,6 +402,7 @@ class DiskIoMgr : public CacheLineAligned {
     REMOTE_S3_DISK_FILE_OPER_OFFSET,
     REMOTE_SFS_DISK_OFFSET,
     REMOTE_OSS_DISK_OFFSET,
+    REMOTE_OBS_DISK_OFFSET,
     REMOTE_NUM_DISKS
   };
 
@@ -414,6 +418,10 @@ class DiskIoMgr : public CacheLineAligned {
   /// is something invalid about the scan range.
   Status ValidateScanRange(ScanRange* range) WARN_UNUSED_RESULT;
 
+  /// Try to dump the data of remote data cache to disk, so it could be loaded when
+  /// impalad restart.
+  Status DumpDataCache();
+
   DataCache* remote_data_cache() { return remote_data_cache_.get(); }
 
  private:
