@@ -342,8 +342,19 @@ Status ScanRange::ReadSubRanges(
         buffer_desc->buffer_len() - buffer_desc->len());
 
     if (cache_.data != nullptr) {
+      // The cache_.data buffer starts at offset_, so adjust the starting
+      // offset for the copies.
+      int64_t buffer_offset = offset - offset_;
+      DCHECK_LE(buffer_offset + bytes_to_read, cache_.len);
+      // DCHECKs are only effective with test coverage, so also return an error
+      // if this would read past the edge of the cache_.data buffer. We wanted
+      // bytes_to_read, but only cache_.len - buffer_offset bytes were available.
+      if (buffer_offset + bytes_to_read > cache_.len) {
+        return Status(TErrorCode::SCANNER_INCOMPLETE_READ, bytes_to_read,
+            cache_.len - buffer_offset, file(), offset);
+      }
       memcpy(buffer_desc->buffer_ + buffer_desc->len(),
-          cache_.data + offset, bytes_to_read);
+          cache_.data + buffer_offset, bytes_to_read);
     } else {
       int64_t current_bytes_read;
       Status read_status = file_reader->ReadFromPos(queue, offset,
@@ -511,6 +522,7 @@ void ScanRange::Reset(const FileInfo &fi, int64_t len, int64_t offset, int disk_
   bytes_to_read_ = len;
   offset_ = offset;
   disk_id_ = disk_id;
+  is_encrypted_ = fi.is_encrypted;
   is_erasure_coded_ = fi.is_erasure_coded;
   cache_options_ = buffer_opts.cache_options_;
   disk_file_ = disk_file;
@@ -636,6 +648,10 @@ Status ScanRange::ReadFromCache(
   if (cache_.len < len()) {
     VLOG_QUERY << "Error reading file from HDFS cache: " << file_ << ". Expected "
       << len() << " bytes, but read " << cache_.len << ". Switching to disk read path.";
+    // Null out the cache buffer to avoid any interactions when this falls
+    // back to the regular read path.
+    cache_.len = 0;
+    cache_.data = nullptr;
     // Close the scan range. 'read_succeeded' is still false, so the caller will fall back
     // to non-cached read of this scan range.
     file_reader_->Close();
