@@ -19,6 +19,7 @@
 
 #include <mutex>
 
+#include "codegen/llvm-codegen-cache.h"
 #include "codegen/llvm-codegen.h"
 #include "common/thread-debug-info.h"
 #include "exec/kudu/kudu-util.h"
@@ -110,7 +111,7 @@ QueryState::QueryState(
     refcnt_(0),
     is_cancelled_(0),
     query_spilled_(0),
-    host_profile_(RuntimeProfile::Create(obj_pool(), "<track resource usage>")) {
+    host_profile_(RuntimeProfile::Create(obj_pool(), "<track resource usage>", false)) {
   if (query_ctx_.request_pool.empty()) {
     // fix up pool name for tests
     DCHECK(!request_pool.empty());
@@ -175,7 +176,7 @@ Status QueryState::Init(const ExecQueryFInstancesRequestPB* exec_rpc_params,
 
   ExecEnv* exec_env = ExecEnv::GetInstance();
 
-  RuntimeProfile* jvm_host_profile = RuntimeProfile::Create(&obj_pool_, "JVM");
+  RuntimeProfile* jvm_host_profile = RuntimeProfile::Create(&obj_pool_, "JVM", false);
   host_profile_->AddChild(jvm_host_profile);
 
   int64_t gc_count = JvmMemoryCounterMetric::GC_COUNT->GetValue();
@@ -800,6 +801,11 @@ bool QueryState::WaitForFinishOrTimeout(int32_t timeout_ms) {
   return !timed_out;
 }
 
+bool QueryState::codegen_cache_enabled() const {
+  return !query_options().disable_codegen_cache && !disable_codegen_cache_
+      && ExecEnv::GetInstance()->codegen_cache_enabled();
+}
+
 bool QueryState::StartFInstances() {
   VLOG(2) << "StartFInstances(): query_id=" << PrintId(query_id())
           << " #instances=" << fragment_info_.fragment_instance_ctxs.size();
