@@ -51,6 +51,7 @@ class FragmentState;
 class FragmentInstanceState;
 class InitialReservations;
 class LlvmCodeGen;
+class CodeGenCache;
 class MemTracker;
 class PlanNode;
 class PublishFilterParamsPB;
@@ -150,6 +151,7 @@ class QueryState {
   const TQueryOptions& query_options() const {
     return query_ctx_.client_request.query_options;
   }
+  bool codegen_cache_enabled() const;
   MemTracker* query_mem_tracker() const { return query_mem_tracker_; }
   RuntimeProfile* host_profile() const { return host_profile_; }
   UniqueIdPB GetCoordinatorBackendId() const;
@@ -291,6 +293,10 @@ class QueryState {
     return (it != fragment_state_map_.end()) ? it->second : nullptr;
   }
 
+  const std::unordered_map<TFragmentIdx, FragmentState*>& FragmentStateMap() {
+    return fragment_state_map_;
+  }
+
  private:
   friend class QueryExecMgr;
 
@@ -462,6 +468,9 @@ class QueryState {
   /// send a status report so that we can cancel after a configurable timeout.
   int64_t failed_report_time_ms_ = 0;
 
+  /// Indicator of whether to disable the codegen cache for the query.
+  bool disable_codegen_cache_ = false;
+
   /// Create QueryState w/ a refcnt of 0 and a memory limit of 'mem_limit' bytes applied
   /// to the query mem tracker. The query is associated with the resource pool set in
   /// 'query_ctx.request_pool' or from 'request_pool', if the former is not set (needed
