@@ -40,17 +40,6 @@ constexpr float RawValue::CANONICAL_FLOAT_NAN;
 constexpr double RawValue::CANONICAL_DOUBLE_ZERO;
 constexpr float RawValue::CANONICAL_FLOAT_ZERO;
 
-namespace {
-
-// Top level null values are printed as "NULL"; collections and structs are printed in
-// JSON format, which requires "null".
-constexpr const char* NullLiteral(bool top_level) {
-  if (top_level) return "NULL";
-  return "null";
-}
-
-}
-
 void RawValue::PrintValueAsBytes(const void* value, const ColumnType& type,
                                  stringstream* stream) {
   if (value == NULL) return;
@@ -108,11 +97,9 @@ void RawValue::PrintValue(const void* value, const ColumnType& type, int scale,
     return;
   }
 
-  stringstream out;
-  out.precision(ASCII_PRECISION);
   const StringValue* string_val = NULL;
-  string tmp;
   bool val;
+  string tmp;
 
   // Special case types that we can print more efficiently without using a stringstream
   switch (type.type) {
@@ -129,16 +116,28 @@ void RawValue::PrintValue(const void* value, const ColumnType& type, int scale,
     case TYPE_CHAR:
       *str = string(reinterpret_cast<const char*>(value), type.len);
       return;
+    case TYPE_TIMESTAMP:
+      *str = reinterpret_cast<const TimestampValue*>(value)->ToString();
+      return;
+    case TYPE_DATE:
+      *str = reinterpret_cast<const DateValue*>(value)->ToString();
+      return;
     case TYPE_FIXED_UDA_INTERMEDIATE:
       *str = "Intermediate UDA step, no value printed";
       return;
     default:
-      PrintValue(value, type, scale, &out);
+      break;
   }
+
+  stringstream out;
+  out.precision(ASCII_PRECISION);
+
+  PrintValue(value, type, scale, &out);
+
   *str = out.str();
 }
 
-void RawValue::Write(const void* value, void* dst, const ColumnType& type,
+void RawValue::WriteNonNullPrimitive(const void* value, void* dst, const ColumnType& type,
     MemPool* pool) {
   DCHECK(value != NULL);
   switch (type.type) {
@@ -183,9 +182,9 @@ void RawValue::Write(const void* value, void* dst, const ColumnType& type,
       dest->len = src->len;
       if (type.type == TYPE_VARCHAR) DCHECK_LE(dest->len, type.len);
       if (pool != NULL) {
-        // Note: if this changes to TryAllocate(), CodegenAnyVal::WriteToSlot() will need
-        // to reflect this change as well (the codegen'd Allocate() call is actually
-        // generated in CodegenAnyVal::StoreToNativePtr()).
+        // Note: if this changes to TryAllocate(), SlotDescriptor::CodegenWriteToSlot()
+        // will need to reflect this change as well (the codegen'd Allocate() call is
+        // actually generated in SlotDescriptor::CodegenWriteStringOrCollectionToSlot()).
         dest->ptr = reinterpret_cast<char*>(pool->Allocate(dest->len));
         Ubsan::MemCpy(dest->ptr, src->ptr, dest->len);
       } else {
@@ -202,11 +201,8 @@ void RawValue::Write(const void* value, void* dst, const ColumnType& type,
       break;
     case TYPE_ARRAY:
     case TYPE_MAP: {
-      DCHECK(pool == NULL) << "RawValue::Write(): deep copy of CollectionValues NYI";
-      const CollectionValue* src = reinterpret_cast<const CollectionValue*>(value);
-      CollectionValue* dest = reinterpret_cast<CollectionValue*>(dst);
-      dest->num_tuples = src->num_tuples;
-      dest->ptr = src->ptr;
+      // Collections should be handled by a different Write() function within this class.
+      DCHECK(false);
       break;
     }
     case TYPE_STRUCT: {
@@ -214,39 +210,62 @@ void RawValue::Write(const void* value, void* dst, const ColumnType& type,
       DCHECK(false);
     }
     default:
-      DCHECK(false) << "RawValue::Write(): bad type: " << type.DebugString();
+      DCHECK(false) << "RawValue::WriteNonNullPrimitive(): bad type: "
+          << type.DebugString();
   }
 }
 
 void RawValue::Write(const void* value, Tuple* tuple, const SlotDescriptor* slot_desc,
-                     MemPool* pool) {
-  if (value == NULL) {
-    tuple->SetNull(slot_desc->null_indicator_offset());
+    MemPool* pool) {
+  RawValue::Write<false>(value, tuple, slot_desc, pool, nullptr, nullptr);
+}
+
+template <bool COLLECT_VAR_LEN_VALS>
+void RawValue::Write(const void* value, Tuple* tuple, const SlotDescriptor* slot_desc,
+    MemPool* pool, std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values) {
+  if (value == nullptr) {
+    if (slot_desc->type().IsStructType()) {
+      tuple->SetStructToNull(slot_desc);
+    } else {
+      tuple->SetNull(slot_desc->null_indicator_offset());
+    }
   } else {
-    void* slot = tuple->GetSlot(slot_desc->tuple_offset());
-    RawValue::Write(value, slot, slot_desc->type(), pool);
+    RawValue::WriteNonNull<COLLECT_VAR_LEN_VALS>(value, tuple, slot_desc, pool,
+        string_values, collection_values);
   }
 }
 
-template <bool COLLECT_STRING_VALS>
-void RawValue::Write(const void* value, Tuple* tuple,
+template <bool COLLECT_VAR_LEN_VALS>
+void RawValue::WriteNonNull(const void* value, Tuple* tuple,
     const SlotDescriptor* slot_desc, MemPool* pool,
-    vector<StringValue*>* string_values) {
-  DCHECK(value != nullptr && tuple != nullptr && slot_desc != nullptr &&
-      string_values != nullptr);
-  DCHECK(string_values->size() == 0);
+    vector<StringValue*>* string_values,
+    vector<pair<CollectionValue*, int64_t>>* collection_values) {
+  DCHECK(value != nullptr && tuple != nullptr && slot_desc != nullptr);
+
+  if (COLLECT_VAR_LEN_VALS) {
+    DCHECK(string_values != nullptr);
+    DCHECK(string_values->size() == 0);
+    DCHECK(collection_values != nullptr);
+    DCHECK(collection_values->size() == 0);
+  }
 
   if (slot_desc->type().IsStructType()) {
-    WriteStruct<COLLECT_STRING_VALS>(value, tuple, slot_desc, pool, string_values);
+    WriteStruct<COLLECT_VAR_LEN_VALS>(value, tuple, slot_desc, pool,
+        string_values, collection_values);
+  } else if (slot_desc->type().IsCollectionType()) {
+    WriteCollection<COLLECT_VAR_LEN_VALS>(value, tuple, slot_desc, pool,
+        string_values, collection_values);
   } else {
-    WritePrimitive<COLLECT_STRING_VALS>(value, tuple, slot_desc, pool, string_values);
+    WritePrimitiveCollectVarlen<COLLECT_VAR_LEN_VALS>(value, tuple, slot_desc, pool,
+        string_values, collection_values);
   }
 }
 
-template <bool COLLECT_STRING_VALS>
+template <bool COLLECT_VAR_LEN_VALS>
 void RawValue::WriteStruct(const void* value, Tuple* tuple,
-    const SlotDescriptor* slot_desc, MemPool* pool,
-    vector<StringValue*>* string_values) {
+    const SlotDescriptor* slot_desc, MemPool* pool, vector<StringValue*>* string_values,
+    vector<pair<CollectionValue*, int64_t>>* collection_values) {
   DCHECK(tuple != nullptr);
   DCHECK(slot_desc->type().IsStructType());
   DCHECK(slot_desc->children_tuple_descriptor() != nullptr);
@@ -263,30 +282,66 @@ void RawValue::WriteStruct(const void* value, Tuple* tuple,
     uint8_t* src_child = src->ptr[i];
     if (child_slot->type().IsStructType()) {
       // Recursive call in case of nested structs.
-      WriteStruct<COLLECT_STRING_VALS>(src_child, tuple, child_slot, pool,
-          string_values);
+      WriteStruct<COLLECT_VAR_LEN_VALS>(src_child, tuple, child_slot, pool,
+          string_values, collection_values);
       continue;
     }
     if (src_child == nullptr) {
       tuple->SetNull(child_slot->null_indicator_offset());
     } else {
-      WritePrimitive<COLLECT_STRING_VALS>(src_child, tuple, child_slot, pool,
-          string_values);
+      WritePrimitiveCollectVarlen<COLLECT_VAR_LEN_VALS>(src_child, tuple, child_slot,
+          pool, string_values, collection_values);
     }
   }
 }
 
-template <bool COLLECT_STRING_VALS>
-void RawValue::WritePrimitive(const void* value, Tuple* tuple,
-    const SlotDescriptor* slot_desc, MemPool* pool,
-    vector<StringValue*>* string_values) {
-  DCHECK(value != nullptr && tuple != nullptr && slot_desc != nullptr &&
-      string_values != nullptr);
+template <bool COLLECT_VAR_LEN_VALS>
+void RawValue::WriteCollection(const void* value, Tuple* tuple,
+    const SlotDescriptor* slot_desc, MemPool* pool, vector<StringValue*>* string_values,
+    vector<pair<CollectionValue*, int64_t>>* collection_values) {
+  DCHECK(slot_desc->type().IsCollectionType());
 
   void* dst = tuple->GetSlot(slot_desc->tuple_offset());
-  Write(value, dst, slot_desc->type(), pool);
-  if (COLLECT_STRING_VALS && slot_desc->type().IsVarLenStringType()) {
-    string_values->push_back(reinterpret_cast<StringValue*>(dst));
+
+  // TODO IMPALA-10939: Enable recursive collections.
+  const CollectionValue* src = reinterpret_cast<const CollectionValue*>(value);
+  CollectionValue* dest = reinterpret_cast<CollectionValue*>(dst);
+  dest->num_tuples = src->num_tuples;
+
+  int64_t byte_size = dest->ByteSize(*slot_desc->children_tuple_descriptor());
+  if (pool != nullptr) {
+    // Note: if this changes to TryAllocate(), SlotDescriptor::CodegenWriteToSlot() will
+    // need to reflect this change as well (the codegen'd Allocate() call is actually
+    // generated in SlotDescriptor::CodegenWriteStringOrCollectionToSlot()).
+    dest->ptr = reinterpret_cast<uint8_t*>(pool->Allocate(byte_size));
+    Ubsan::MemCpy(dest->ptr, src->ptr, byte_size);
+  } else {
+    dest->ptr = src->ptr;
+  }
+
+  if (COLLECT_VAR_LEN_VALS) {
+    DCHECK(string_values != nullptr);
+    DCHECK(collection_values != nullptr);
+    collection_values->push_back(std::make_pair(dest, byte_size));
+  }
+}
+
+template <bool COLLECT_VAR_LEN_VALS>
+void RawValue::WritePrimitiveCollectVarlen(const void* value, Tuple* tuple,
+    const SlotDescriptor* slot_desc, MemPool* pool, vector<StringValue*>* string_values,
+    vector<pair<CollectionValue*, int64_t>>* collection_values) {
+  DCHECK(value != nullptr && tuple != nullptr && slot_desc != nullptr);
+
+  void* dst = tuple->GetSlot(slot_desc->tuple_offset());
+  WriteNonNullPrimitive(value, dst, slot_desc->type(), pool);
+  if (COLLECT_VAR_LEN_VALS) {
+    DCHECK(string_values != nullptr);
+    DCHECK(collection_values != nullptr);
+    if (slot_desc->type().IsVarLenStringType()) {
+      string_values->push_back(reinterpret_cast<StringValue*>(dst));
+    } else if (slot_desc->type().IsCollectionType()) {
+      DCHECK(false) << "Collections should be handled in WriteCollection.";
+    }
   }
 }
 
@@ -312,7 +367,7 @@ void RawValue::PrintValue(
     case TYPE_BOOLEAN: {
       bool val = *reinterpret_cast<const bool*>(value);
       *stream << (val ? "true" : "false");
-      return;
+      break;
     }
     case TYPE_TINYINT:
       // Extra casting for chars since they should not be interpreted as ASCII.
@@ -403,102 +458,38 @@ void RawValue::PrintValue(
 
 template void RawValue::Write<true>(const void* value, Tuple* tuple,
     const SlotDescriptor* slot_desc, MemPool* pool,
-    std::vector<StringValue*>* string_values);
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
 template void RawValue::Write<false>(const void* value, Tuple* tuple,
     const SlotDescriptor* slot_desc, MemPool* pool,
-    std::vector<StringValue*>* string_values);
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
+
+template void RawValue::WriteNonNull<true>(const void* value, Tuple* tuple,
+    const SlotDescriptor* slot_desc, MemPool* pool,
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
+template void RawValue::WriteNonNull<false>(const void* value, Tuple* tuple,
+    const SlotDescriptor* slot_desc, MemPool* pool,
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
 
 template void RawValue::WriteStruct<true>(const void* value, Tuple* tuple,
-      const SlotDescriptor* slot_desc, MemPool* pool,
-      std::vector<StringValue*>* string_values);
+    const SlotDescriptor* slot_desc, MemPool* pool,
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
 template void RawValue::WriteStruct<false>(const void* value, Tuple* tuple,
-      const SlotDescriptor* slot_desc, MemPool* pool,
-      std::vector<StringValue*>* string_values);
-
-template void RawValue::WritePrimitive<true>(const void* value, Tuple* tuple,
-      const SlotDescriptor* slot_desc, MemPool* pool,
-      std::vector<StringValue*>* string_values);
-template void RawValue::WritePrimitive<false>(const void* value, Tuple* tuple,
-      const SlotDescriptor* slot_desc, MemPool* pool,
-      std::vector<StringValue*>* string_values);
-
-bool PrintNestedValueIfNull(const SlotDescriptor& slot_desc, Tuple* item,
-    stringstream* stream) {
-  bool is_null = item->IsNull(slot_desc.null_indicator_offset());
-  if (is_null) *stream << NullLiteral(false);
-  return is_null;
-}
-
-void PrintNonNullNestedCollection(const SlotDescriptor& slot_desc, Tuple* item, int scale,
-    stringstream* stream) {
-  const CollectionValue* nested_collection_val =
-      item->GetCollectionSlot(slot_desc.tuple_offset());
-  DCHECK(nested_collection_val != nullptr);
-  const TupleDescriptor* child_item_tuple_desc =
-      slot_desc.children_tuple_descriptor();
-  DCHECK(child_item_tuple_desc != nullptr);
-  RawValue::PrintCollectionValue(nested_collection_val, child_item_tuple_desc, scale,
-      stream, slot_desc.type().IsMapType());
-}
-
-void PrintNonNullNestedPrimitive(const SlotDescriptor& slot_desc, Tuple* item, int scale,
-    stringstream* stream) {
-  RawValue::PrintValue(item->GetSlot(slot_desc.tuple_offset()), slot_desc.type(), scale,
-      stream, true);
-}
-
-void PrintNestedValue(const SlotDescriptor& slot_desc, Tuple* item, int scale,
-    stringstream* stream) {
-  bool is_null = PrintNestedValueIfNull(slot_desc, item, stream);
-  if (is_null) return;
-
-  if (slot_desc.type().IsCollectionType()) {
-    // The item is also an array or a map, recurse deeper if not NULL.
-    PrintNonNullNestedCollection(slot_desc, item, scale, stream);
-  } else if (!slot_desc.type().IsComplexType()) {
-    // The item is a scalar, print it with the usual PrintValue.
-    PrintNonNullNestedPrimitive(slot_desc, item, scale, stream);
-  } else {
-    DCHECK(false);
-  }
-}
-
-void RawValue::PrintCollectionValue(const CollectionValue* coll_val,
-    const TupleDescriptor* item_tuple_desc, int scale, stringstream *stream,
-    bool is_map) {
-  DCHECK(item_tuple_desc != nullptr);
-  if (coll_val == nullptr) {
-    // We only reach this code path if this is a top level collection. Otherwise
-    // PrintNestedValue() handles the printing of the NULL literal.
-    *stream << NullLiteral(true);
-    return;
-  }
-  int item_byte_size = item_tuple_desc->byte_size();
-
-  const vector<SlotDescriptor*>& slot_descs = item_tuple_desc->slots();
-  // TODO: This has to be changed once structs are supported too.
-  if (is_map) {
-    DCHECK(slot_descs.size() == 2);
-    DCHECK(slot_descs[0] != nullptr);
-    DCHECK(slot_descs[1] != nullptr);
-  } else {
-    DCHECK(slot_descs.size() == 1);
-    DCHECK(slot_descs[0] != nullptr);
-  }
-
-  *stream << (is_map ? "{" : "[");
-  for (int i = 0; i < coll_val->num_tuples; ++i) {
-    Tuple* item = reinterpret_cast<Tuple*>(coll_val->ptr + i * item_byte_size);
-
-    PrintNestedValue(*slot_descs[0], item, scale, stream);
-    if (is_map) {
-      *stream << ":";
-      PrintNestedValue(*slot_descs[1], item, scale, stream);
-    }
-
-    if (i < coll_val->num_tuples - 1) *stream << ",";
-  }
-  *stream << (is_map ? "}" : "]");
-}
+    const SlotDescriptor* slot_desc, MemPool* pool,
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
 
+template void RawValue::WritePrimitiveCollectVarlen<true>(const void* value,
+    Tuple* tuple, const SlotDescriptor* slot_desc, MemPool* pool,
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
+template void RawValue::WritePrimitiveCollectVarlen<false>(const void* value,
+    Tuple* tuple,
+    const SlotDescriptor* slot_desc, MemPool* pool,
+    std::vector<StringValue*>* string_values,
+    std::vector<std::pair<CollectionValue*, int64_t>>* collection_values);
 }
