@@ -27,7 +27,6 @@
 #include "common/compiler-util.h"
 #include "common/logging.h"
 #include "gen-cpp/row_batch.pb.h"
-#include "kudu/util/slice.h"
 #include "runtime/bufferpool/buffer-pool.h"
 #include "runtime/descriptors.h"
 #include "runtime/mem-pool.h"
@@ -41,65 +40,14 @@ namespace impala {
 
 template <typename K, typename V> class FixedSizeHashTable;
 class MemTracker;
+class OutboundRowBatch;
+class RowBatchHeaderPB;
 class RowBatchSerializeTest;
 class RuntimeState;
 class Tuple;
 class TupleRow;
 class TupleDescriptor;
 
-/// A KRPC outbound row batch which contains the serialized row batch header and buffers
-/// for holding the tuple offsets and tuple data.
-class OutboundRowBatch {
- public:
-  OutboundRowBatch(std::shared_ptr<CharMemTrackerAllocator> allocator)
-    : tuple_data_(*allocator.get()), compression_scratch_(*allocator.get()) {}
-
-  const RowBatchHeaderPB* header() const { return &header_; }
-
-  /// Returns the serialized tuple offsets' vector as a kudu::Slice.
-  /// The tuple offsets vector is sent as KRPC sidecar.
-  kudu::Slice TupleOffsetsAsSlice() const {
-    return kudu::Slice(
-        const_cast<uint8_t*>(reinterpret_cast<const uint8_t*>(tuple_offsets_.data())),
-        tuple_offsets_.size() * sizeof(tuple_offsets_[0]));
-  }
-
-  /// Returns the serialized tuple data's buffer as a kudu::Slice.
-  /// The tuple data is sent as KRPC sidecar.
-  kudu::Slice TupleDataAsSlice() const {
-    return kudu::Slice(
-        const_cast<uint8_t*>(reinterpret_cast<const uint8_t*>(tuple_data_.data())),
-        tuple_data_.length());
-  }
-
-  /// Returns true if the header has been initialized and ready to be sent.
-  /// This entails setting some fields initialized in RowBatch::Serialize().
-  bool IsInitialized() const {
-     return header_.has_num_rows() && header_.has_uncompressed_size() &&
-         header_.has_compression_type();
-  }
-
- private:
-  friend class RowBatch;
-  friend class RowBatchSerializeBaseline;
-
-  /// The serialized header which contains the meta-data of the row batch such as the
-  /// number of rows and compression scheme used etc.
-  RowBatchHeaderPB header_;
-
-  /// Contains offsets into 'tuple_data_' of all tuples in a row batch. -1 refers to
-  /// a NULL tuple.
-  vector<int32_t> tuple_offsets_;
-
-  /// Contains the actual data of all the tuples. The data could be compressed.
-  TrackedString tuple_data_;
-
-  /// Contains the compression scratch for the compressed data in serialization.
-  /// The compression_scratch_ will be swapped with tuple_data_ if the compressed data
-  /// is shorter.
-  TrackedString compression_scratch_;
-};
-
 /// A RowBatch encapsulates a batch of rows, each composed of a number of tuples.
 /// The maximum number of rows is fixed at the time of construction.
 /// The row batch can reference various types of memory.
