@@ -46,6 +46,7 @@ namespace impala {
 class BloomFilter;
 class MemTracker;
 class MinMaxFilter;
+class InListFilter;
 class RuntimeFilter;
 class QueryState;
 class TBloomFilter;
@@ -73,10 +74,11 @@ struct FilterRegistration {
 ///
 /// All producers and consumers of filters must register via RegisterProducer() and
 /// RegisterConsumer(). Local plan fragments update the filters by calling
-/// UpdateFilterFromLocal(), with either a bloom filter or a min-max filter, depending
-/// on the filter's type. The 'bloom_filter' or 'min_max_filter' that is passed into
-/// UpdateFilterFromLocal() must have been allocated by AllocateScratch*Filter(); this
-/// allows RuntimeFilterBank to manage all memory associated with filters.
+/// UpdateFilterFromLocal(), with either a bloom filter, a min-max filter, or an in-list
+/// filter, depending on the filter's type. The 'bloom_filter', 'min_max_filter' or
+/// 'in_list_filter' that is passed into UpdateFilterFromLocal() must have been allocated
+/// by AllocateScratch*Filter(); this allows RuntimeFilterBank to manage all memory
+/// associated with filters.
 ///
 /// Filters are aggregated, first locally in this RuntimeFilterBank, if there are multiple
 /// producers, and then made available to consumers after PublishGlobalFilter() has been
@@ -84,10 +86,11 @@ struct FilterRegistration {
 /// of time so that RuntimeFilterBank knows when the filter is complete.
 ///
 /// After PublishGlobalFilter() has been called (at most once per filter_id), the
-/// RuntimeFilter object associated with filter_id will have a valid bloom_filter or
-/// min_max_filter, and may be used for filter evaluation. This operation occurs
-/// without synchronisation, and neither the thread that calls PublishGlobalFilter()
-/// nor the thread that may call RuntimeFilter::Eval() need to coordinate in any way.
+/// RuntimeFilter object associated with filter_id will have a valid bloom_filter,
+/// min_max_filter or in_list_filter, and may be used for filter evaluation. This
+/// operation occurs without synchronisation, and neither the thread that calls
+/// PublishGlobalFilter() nor the thread that may call RuntimeFilter::Eval() need to
+/// coordinate in any way.
 class RuntimeFilterBank {
  public:
   /// 'filters': contains an entry for every filter produced or consumed on this backend.
@@ -118,14 +121,15 @@ class RuntimeFilterBank {
   /// to check for the filter's arrival.
   RuntimeFilter* RegisterConsumer(const TRuntimeFilterDesc& filter_desc);
 
-  /// Updates a filter's 'bloom_filter' or 'min_max_filter' which has been produced by
-  /// some operator in a local fragment instance. At most one of 'bloom_filter' and
-  /// 'min_max_filter' may be non-NULL, depending on the filter's type. They may both be
-  /// NULL, representing a filter that allows all rows to pass.
-  void UpdateFilterFromLocal(
-      int32_t filter_id, BloomFilter* bloom_filter, MinMaxFilter* min_max_filter);
+  /// Updates a filter's 'bloom_filter', 'min_max_filter' or 'in_list_filter' which has
+  /// been produced by some operator in a local fragment instance. At most one of
+  /// 'bloom_filter', 'min_max_filter' and 'in_list_filter' may be non-NULL, depending on
+  /// the filter's type. They may both be NULL, representing a filter that allows all rows
+  /// to pass.
+  void UpdateFilterFromLocal(int32_t filter_id, BloomFilter* bloom_filter,
+      MinMaxFilter* min_max_filter, InListFilter* in_list_filter);
 
-  /// Makes a bloom_filter (aggregated globally from all producer fragments) available for
+  /// Makes a filter (aggregated globally from all producer fragments) available for
   /// consumption by operators that wish to use it for filtering.
   void PublishGlobalFilter(
       const PublishFilterParamsPB& params, kudu::rpc::RpcContext* context);
@@ -142,6 +146,9 @@ class RuntimeFilterBank {
   /// Returns a new MinMaxFilter. Handles memory the same as AllocateScratchBloomFilter().
   MinMaxFilter* AllocateScratchMinMaxFilter(int32_t filter_id, ColumnType type);
 
+  /// Returns a new InListFilter. Handles memory the same as AllocateScratchBloomFilter().
+  InListFilter* AllocateScratchInListFilter(int32_t filter_id, ColumnType type);
+
   /// Default hash seed to use when computing hashed values to insert into filters.
   static int32_t IR_ALWAYS_INLINE DefaultHashSeed() { return 1234; }
 
@@ -221,6 +228,10 @@ class RuntimeFilterBank {
     /// Contains references to all the min-max filters generated. Used in Close() to
     /// safely release all memory allocated for MinMaxFilters.
     vector<MinMaxFilter*> min_max_filters;
+
+    /// Contains references to all the in-list filters generated. Used in Close() to
+    /// safely release all memory allocated for InListFilters.
+    vector<InListFilter*> in_list_filters;
   } CACHELINE_ALIGNED;
 
   /// Object pool for objects that will be freed in Close(), e.g. allocated filters.
@@ -260,6 +271,9 @@ class RuntimeFilterBank {
   /// Total amount of memory allocated to Bloom Filters
   RuntimeProfile::Counter* const bloom_memory_allocated_;
 
+  /// Total number of items of all in-list filters.
+  RuntimeProfile::Counter* const total_in_list_filter_items_;
+
   /// Total amount of memory required by the bloom filters as calculated by the planner.
   const int64_t total_bloom_filter_mem_required_;
 
