@@ -114,13 +114,22 @@ class Sorter::Page {
 ///
 /// Runs are either "initial runs" constructed from the sorter's input by evaluating
 /// the expressions in 'sort_tuple_exprs_' or "intermediate runs" constructed
-/// by merging already-sorted runs. Initial runs are sorted in-place in memory. Once
-/// sorted, runs can be spilled to disk to free up memory. Sorted runs are merged by
+/// by merging already-sorted runs. Initial runs are sorted in-place in memory.
+/// Once sorted, runs can be spilled to disk to free up memory. Sorted runs are merged by
 /// SortedRunMerger, either to produce the final sorted output or to produce another
 /// sorted run.
+/// By default, the size of initial runs is determined by the available memory: the
+/// sorter tries to add batches to the run until some (memory) limit is reached.
+/// Some query options can also limit the size of an initial (or in-memory) run.
+/// SORT_RUN_BYTES_LIMIT triggers spilling after the size of data in the run exceeds the
+/// given threshold (usually expressed in MB or GB).
+/// MAX_SORT_RUN_SIZE allows constructing runs up to a certain size by limiting the
+/// number of pages in the initial runs. These smaller in-memory runs are also referred
+/// to as 'miniruns'. Miniruns are not spilled immediately, but sorted in-place first,
+/// and collected to be merged in memory before spilling the produced output run to disk.
 ///
 /// The expected calling sequence of functions is as follows:
-/// * Init() to initialize the run and allocate initial pages.
+/// * Init() or TryInit() to initialize the run and allocate initial pages.
 /// * Add*Batch() to add batches of tuples to the run.
 /// * FinalizeInput() to signal that no more batches will be added.
 /// * If the run is unsorted, it must be sorted. After that set_sorted() must be called.
@@ -141,13 +150,23 @@ class Sorter::Run {
   /// var-len data into var_len_copy_page_.
   Status Init();
 
+  /// Similar to Init(), except for the following differences:
+  /// It is only used to initialize miniruns (query option MAX_SORT_RUN_SIZE > 0 cases).
+  /// The first in-memory run is always initialized by calling Init(), because that must
+  /// succeed. The following ones are initialized by TryInit().
+  /// TryInit() allocates one fixed-len page and one var-len page if 'has_var_len_slots_'
+  /// is true. There is no need for var_len_copy_page here. Returns false if
+  /// initialization was successful, returns true, if reservation was not enough.
+  Status TryInit(bool* allocation_failed);
+
   /// Add the rows from 'batch' starting at 'start_index' to the current run. Returns
   /// the number of rows actually added in 'num_processed'. If the run is full (no more
   /// pages can be allocated), 'num_processed' may be less than the number of remaining
   /// rows in the batch. AddInputBatch() materializes the input rows using the
   /// expressions in sorter_->sort_tuple_expr_evals_, while AddIntermediateBatch() just
   /// copies rows.
-  Status AddInputBatch(RowBatch* batch, int start_index, int* num_processed);
+  Status AddInputBatch(
+      RowBatch* batch, int start_index, int* num_processed, bool* allocation_failed);
 
   Status AddIntermediateBatch(RowBatch* batch, int start_index, int* num_processed);
 
@@ -199,6 +218,9 @@ class Sorter::Run {
   bool is_finalized() const { return is_finalized_; }
   bool is_sorted() const { return is_sorted_; }
   void set_sorted() { is_sorted_ = true; }
+  int max_num_of_pages() const { return max_num_of_pages_; }
+  int fixed_len_size() { return fixed_len_pages_.size(); }
+  int run_size() { return fixed_len_pages_.size() + var_len_pages_.size(); }
   int64_t num_tuples() const { return num_tuples_; }
   /// Returns true if we have var-len pages in the run.
   bool HasVarLenPages() const {
@@ -215,17 +237,23 @@ class Sorter::Run {
   /// INITIAL_RUN and HAS_VAR_LEN_SLOTS are template arguments for performance and must
   /// match 'initial_run_' and 'has_var_len_slots_'.
   template <bool HAS_VAR_LEN_SLOTS, bool INITIAL_RUN>
-  Status AddBatchInternal(
-      RowBatch* batch, int start_index, int* num_processed);
+  Status AddBatchInternal(RowBatch* batch, int start_index, int* num_processed,
+        bool* allocation_failed);
 
   /// Finalize the list of pages: delete empty final pages and unpin the previous page
   /// if the run is unpinned.
   Status FinalizePages(vector<Page>* pages);
 
-  /// Collect the non-null var-len (e.g. STRING) slots from 'src' in 'var_len_values' and
-  /// return the total length of all var-len values in 'total_var_len'.
+  void CheckTypeForVarLenCollectionSorting();
+
+  /// Collects the non-null var-len slots (strings and collections) from 'src'. Strings
+  /// are returned in 'string_values' and collections are returned, along with their byte
+  /// size, in 'collection_values'. The total length of all var-len values is returned in
+  /// 'total_var_len'.
   void CollectNonNullVarSlots(
-      Tuple* src, vector<StringValue*>* var_len_values, int* total_var_len);
+      Tuple* src, vector<StringValue*>* string_values,
+      std::vector<CollValueAndSize>* collection_values,
+      int* total_var_len);
 
   enum AddPageMode { KEEP_PREV_PINNED, UNPIN_PREV };
 
@@ -251,15 +279,18 @@ class Sorter::Run {
   /// this function will pin the page at 'page_index' + 1 in 'pages'.
   Status PinNextReadPage(vector<Page>* pages, int page_index);
 
-  /// Copy the StringValues in 'var_values' to 'dest' in order and update the StringValue
-  /// ptrs in 'dest' to point to the copied data.
-  void CopyVarLenData(const vector<StringValue*>& var_values, uint8_t* dest);
+  /// Copy the var len data in 'string_values' and 'collection_values_and_sizes' to 'dest'
+  /// in order and update the pointers to point to the copied data.
+  void CopyVarLenData(const vector<StringValue*>& string_values,
+      const vector<CollValueAndSize>& collection_values_and_sizes, uint8_t* dest);
 
-  /// Copy the StringValues in 'var_values' to 'dest' in order. Update the StringValue
-  /// ptrs in 'dest' to contain a packed offset for the copied data comprising
-  /// page_index and the offset relative to page_start.
-  void CopyVarLenDataConvertOffset(const vector<StringValue*>& var_values, int page_index,
-      const uint8_t* page_start, uint8_t* dest);
+  /// Copy the StringValues in 'var_values' and the CollectionValues referenced in
+  /// 'collection_values_and_sizes' to 'dest' in order. Update the StringValue ptrs in
+  /// 'dest' to contain a packed offset for the copied data comprising page_index and the
+  /// offset relative to page_start.
+  void CopyVarLenDataConvertOffset(const vector<StringValue*>& var_values,
+      const std::vector<CollValueAndSize>& collection_values_and_sizes,
+      int page_index, const uint8_t* page_start, uint8_t* dest);
 
   /// Convert encoded offsets to valid pointers in tuple with layout 'sort_tuple_desc_'.
   /// 'tuple' is modified in-place. Returns true if the pointers refer to the page at
@@ -267,6 +298,13 @@ class Sorter::Run {
   /// data is in the next page, in which case 'tuple' is unmodified.
   bool ConvertOffsetsToPtrs(Tuple* tuple);
 
+  template <class ValueType>
+  bool ConvertValueOffsetsToPtrs(Tuple* tuple, uint8_t* page_start,
+      const vector<SlotDescriptor*>& slots);
+
+  bool ConvertStringValueOffsetsToPtrs(Tuple* tuple, uint8_t* page_start);
+  bool ConvertCollectionValueOffsetsToPtrs(Tuple* tuple, uint8_t* page_start);
+
   int NumOpenPages(const vector<Page>& pages);
 
   /// Close all open pages and clear vector.
@@ -289,7 +327,7 @@ class Sorter::Run {
 
   const bool has_var_len_slots_;
 
-  /// True if this is an initial run. False implies this is an sorted intermediate run
+  /// True if this is an initial run. False implies this is a sorted intermediate run
   /// resulting from merging other runs.
   const bool initial_run_;
 
@@ -336,6 +374,12 @@ class Sorter::Run {
   /// Used to implement GetNextBatch() interface required for the merger.
   boost::scoped_ptr<RowBatch> buffered_batch_;
 
+  /// Max number of fixed-len + var-len pages in an in-memory minirun. It defines the
+  /// length of a minirun.
+  /// The default value is 0 which means that only 1 in-memory run will be created, and
+  /// its size will be determined by other limits eg. memory or sort_run_bytes_limit.
+  int max_num_of_pages_;
+
   /// Members used when a run is read in GetNext().
   /// The index into 'fixed_' and 'var_len_pages_' of the pages being read in GetNext().
   int fixed_len_pages_index_;
