@@ -71,8 +71,6 @@ static const int64_t GIGABYTE = 1024L * MEGABYTE;
 static const int64_t TERABYTE = 1024L * GIGABYTE;
 
 /// For testing spill to remote.
-static const string HDFS_LOCAL_URL = "hdfs://localhost:20500/tmp";
-static const string REMOTE_URL = HDFS_LOCAL_URL;
 static const string LOCAL_BUFFER_PATH = "/tmp/tmp-file-mgr-test-buffer";
 
 /// Read buffer sizes for TestBatchReadingSetMaxBytes().
@@ -99,6 +97,7 @@ class TmpFileMgrTest : public ::testing::Test {
     test_env_.reset(new TestEnv);
     ASSERT_OK(test_env_->Init());
     cb_counter_ = 0;
+    remote_url_ = test_env_->GetDefaultFsPath("/tmp");
   }
 
   virtual void TearDown() {
@@ -186,7 +185,7 @@ class TmpFileMgrTest : public ::testing::Test {
     vector<string> tmp_create_dirs{{LOCAL_BUFFER_PATH}};
     RemoveAndCreateDirs(tmp_create_dirs);
     tmp_dirs->push_back(Substitute(LOCAL_BUFFER_PATH + ":$0", 4096));
-    tmp_dirs->push_back(REMOTE_URL);
+    tmp_dirs->push_back(remote_url_);
   }
 
   // Helper for TestBatchReadingSetMaxBytes() to set the read buffer size and check
@@ -393,6 +392,9 @@ class TmpFileMgrTest : public ::testing::Test {
   mutex cb_cv_lock_;
   ConditionVariable cb_cv_;
   int64_t cb_counter_;
+
+  /// URL for remote spilling.
+  string remote_url_;
 };
 
 /// Regression test for IMPALA-2160. Verify that temporary file manager allocates blocks
@@ -1054,7 +1056,7 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsing) {
   // directories.
   auto& dirs2 = GetTmpDirs(
       CreateTmpFileMgr("/tmp/tmp-file-mgr-test1:foo,/tmp/tmp-file-mgr-test2:?,"
-                       "/tmp/tmp-file-mgr-test3:1.2.3.4,/tmp/tmp-file-mgr-test4: ,"
+                       "/tmp/tmp-file-mgr-test3:1.2.3.4,/tmp/tmp-file-mgr-test4:a,"
                        "/tmp/tmp-file-mgr-test5:5pb,/tmp/tmp-file-mgr-test6:10%,"
                        "/tmp/tmp-file-mgr-test1:100"));
   EXPECT_EQ(1, dirs2.size());
@@ -1088,10 +1090,11 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsingRemotePath) {
   RemoveAndCreateDirs({"/tmp/local-buffer-dir", "/tmp/local-buffer-dir1",
       "/tmp/local-buffer-dir2", "/tmp/local-buffer-dir3"});
 
-  // Successful cases for HDFS paths.
+  // Successful cases for FS paths.
   // Two types of paths, one with directory, one without.
-  vector<string> hdfs_paths{"hdfs://localhost:20500", "hdfs://localhost:20500/tmp"};
-  for (string hdfs_path : hdfs_paths) {
+  vector<string> hdfs_paths{
+      test_env_->GetDefaultFsPath(""), test_env_->GetDefaultFsPath("/tmp")};
+  for (const string& hdfs_path : hdfs_paths) {
     string full_hdfs_path = hdfs_path + "/impala-scratch";
     auto& dirs1 = GetTmpRemoteDir(CreateTmpFileMgr(hdfs_path + ",/tmp/local-buffer-dir"));
     EXPECT_NE(nullptr, dirs1);
@@ -1108,10 +1111,10 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsingRemotePath) {
     EXPECT_EQ(full_hdfs_path, dirs3->path());
     EXPECT_EQ(1024, dirs3->bytes_limit());
 
-    // Multiple local paths with one remote path.
+    // Multiple local paths with one remote path. Trims spaces.
     auto tmp_mgr_4 = CreateTmpFileMgr(hdfs_path
-        + ",/tmp/local-buffer-dir1,"
-          "/tmp/local-buffer-dir2,/tmp/local-buffer-dir3");
+        + " , /tmp/local-buffer-dir1, "
+          "/tmp/local-buffer-dir2 ,/tmp/local-buffer-dir3");
     auto& dirs4_local = GetTmpDirs(tmp_mgr_4);
     auto& dirs4_remote = GetTmpRemoteDir(tmp_mgr_4);
     EXPECT_NE(nullptr, dirs4_remote);
@@ -1120,8 +1123,8 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsingRemotePath) {
     EXPECT_EQ("/tmp/local-buffer-dir2/impala-scratch", dirs4_local[0]->path());
     EXPECT_EQ("/tmp/local-buffer-dir3/impala-scratch", dirs4_local[1]->path());
 
-    // Fails the parsing due to no port number for the HDFS path.
-    auto tmp_mgr_5 = CreateTmpFileMgr("hdfs://localhost/tmp,/tmp/local-buffer-dir");
+    // Fails to parse HDFS URI due to no path element.
+    auto tmp_mgr_5 = CreateTmpFileMgr("hdfs://localhost,/tmp/local-buffer-dir");
     auto& dirs5_local = GetTmpDirs(tmp_mgr_5);
     auto& dirs5_remote = GetTmpRemoteDir(tmp_mgr_5);
     EXPECT_EQ(1, dirs5_local.size());
@@ -1133,34 +1136,27 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsingRemotePath) {
 
     // Parse successfully, but the parsed HDFS path is unable to connect.
     // These cases would fail the initialization of TmpFileMgr.
-    auto& dirs7 = GetTmpRemoteDir(
-        CreateTmpFileMgr("hdfs://localhost:1/tmp::1,/tmp/local-buffer-dir", false));
-    EXPECT_EQ(nullptr, dirs7);
-
-    auto& dirs8 = GetTmpRemoteDir(
-        CreateTmpFileMgr("hdfs://localhost:/tmp::,/tmp/local-buffer-dir", false));
-    EXPECT_EQ(nullptr, dirs8);
-
-    auto& dirs9 = GetTmpRemoteDir(
-        CreateTmpFileMgr("hdfs://localhost/tmp::1,/tmp/local-buffer-dir", false));
-    EXPECT_EQ(nullptr, dirs9);
-
-    auto& dirs10 = GetTmpRemoteDir(
-        CreateTmpFileMgr("hdfs://localhost/tmp:1,/tmp/local-buffer-dir", false));
-    EXPECT_EQ(nullptr, dirs10);
+    for (const string& unreachable_path : {
+      "hdfs://localhost:1/tmp::1", "hdfs://localhost:/tmp::", "hdfs://localhost/tmp::1",
+      "hdfs://localhost/tmp:1", "hdfs://localhost/tmp", "hdfs://localhost/:1",
+      "hdfs://localhost:1 ", "hdfs://localhost/ "
+    }) {
+      auto& dirs = GetTmpRemoteDir(
+          CreateTmpFileMgr(unreachable_path + ",/tmp/local-buffer-dir", false));
+      EXPECT_EQ(nullptr, dirs) << unreachable_path;
+    }
 
     // Multiple remote paths, should support only one.
-    auto& dirs11 = GetTmpRemoteDir(CreateTmpFileMgr(hdfs_path
-        + ",hdfs://localhost:20501/tmp,"
-          "/tmp/local-buffer-dir"));
-    EXPECT_NE(nullptr, dirs11);
-    EXPECT_EQ(full_hdfs_path, dirs11->path());
+    auto& dirs6 = GetTmpRemoteDir(CreateTmpFileMgr(Substitute(
+        "$0,hdfs://localhost:20501/tmp,/tmp/local-buffer-dir", hdfs_path)));
+    EXPECT_NE(nullptr, dirs6);
+    EXPECT_EQ(full_hdfs_path, dirs6->path());
 
     // The order of the buffer and the remote dir should not affect the result.
-    auto& dirs12 = GetTmpRemoteDir(CreateTmpFileMgr(
-        "/tmp/local-buffer-dir, " + hdfs_path + ",hdfs://localhost:20501/tmp"));
-    EXPECT_NE(nullptr, dirs12);
-    EXPECT_EQ(full_hdfs_path, dirs12->path());
+    auto& dirs7 = GetTmpRemoteDir(CreateTmpFileMgr(Substitute(
+        "/tmp/local-buffer-dir,$0,hdfs://localhost:20501/tmp", hdfs_path)));
+    EXPECT_NE(nullptr, dirs7);
+    EXPECT_EQ(full_hdfs_path, dirs7->path());
   }
 
   // Successful cases for parsing S3 paths.
@@ -1170,7 +1166,7 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsingRemotePath) {
   fake_hdfs_conn_map.insert(make_pair("s3a://fake_host/", fake_conn));
   // Two types of paths, one with directory, one without.
   vector<string> s3_paths{"s3a://fake_host", "s3a://fake_host/dir"};
-  for (string s3_path : s3_paths) {
+  for (const string& s3_path : s3_paths) {
     string full_s3_path = s3_path + "/impala-scratch";
     auto& dirs13 = GetTmpRemoteDir(
         CreateTmpFileMgr("/tmp/local-buffer-dir, " + s3_path, true, &fake_hdfs_conn_map));
@@ -1258,8 +1254,8 @@ void TmpFileMgrTest::TestCompressBufferManagement(
   TmpFileMgr tmp_file_mgr;
   if (spill_to_remote) {
     RemoveAndCreateDirs(vector<string>{LOCAL_BUFFER_PATH});
-    ASSERT_OK(tmp_file_mgr.InitCustom(vector<string>{REMOTE_URL, LOCAL_BUFFER_PATH}, true,
-        "lz4", true, metrics_.get()));
+    ASSERT_OK(tmp_file_mgr.InitCustom(vector<string>{remote_url_, LOCAL_BUFFER_PATH},
+        true, "lz4", true, metrics_.get()));
   } else {
     ASSERT_OK(tmp_file_mgr.InitCustom(
         vector<string>{"/tmp/tmp-file-mgr-test.1"}, true, "lz4", true, metrics_.get()));
@@ -1383,7 +1379,7 @@ TEST_F(TmpFileMgrTest, TestDirectoryPriorityParsing) {
   // directories.
   auto& dirs2 = GetTmpDirs(
       CreateTmpFileMgr("/tmp/tmp-file-mgr-test1::foo,/tmp/tmp-file-mgr-test2::?,"
-                       "/tmp/tmp-file-mgr-test3::1.2.3.4,/tmp/tmp-file-mgr-test4:: ,"
+                       "/tmp/tmp-file-mgr-test3::1.2.3.4,/tmp/tmp-file-mgr-test4::a,"
                        "/tmp/tmp-file-mgr-test5::p0,/tmp/tmp-file-mgr-test6::10%,"
                        "/tmp/tmp-file-mgr-test1:100:-1"));
   EXPECT_EQ(1, dirs2.size());
@@ -1512,7 +1508,7 @@ TEST_F(TmpFileMgrTest, TestRemoteOneDir) {
   vector<string> tmp_dirs({LOCAL_BUFFER_PATH});
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
   TUniqueId id;
   TmpFileGroup file_group(&tmp_file_mgr, io_mgr(), profile_, id);
@@ -1541,7 +1537,7 @@ TEST_F(TmpFileMgrTest, TestRemoteDirReportError) {
   vector<string> tmp_dirs({LOCAL_BUFFER_PATH});
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, false, "", false, metrics_.get()));
   TUniqueId id;
   TmpFileGroup file_group(&tmp_file_mgr, io_mgr(), profile_, id);
@@ -1596,7 +1592,7 @@ TEST_F(TmpFileMgrTest, TestRemoteAllocateNonWritable) {
   vector<string> tmp_dirs({LOCAL_BUFFER_PATH});
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
   TUniqueId id;
   TmpFileGroup file_group(&tmp_file_mgr, io_mgr(), profile_, id);
@@ -1629,7 +1625,7 @@ TEST_F(TmpFileMgrTest, TestRemoteScratchLimit) {
   vector<string> tmp_dirs({LOCAL_BUFFER_PATH});
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
 
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
   TUniqueId id;
@@ -1671,7 +1667,7 @@ TEST_F(TmpFileMgrTest, TestRemoteWriteRange) {
   vector<string> tmp_dirs({LOCAL_BUFFER_PATH});
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
 
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
   TUniqueId id;
@@ -1706,7 +1702,7 @@ TEST_F(TmpFileMgrTest, TestRemoteBlockVerification) {
   vector<string> tmp_dirs({LOCAL_BUFFER_PATH});
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
 
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
   TUniqueId id;
@@ -1755,7 +1751,7 @@ TEST_F(TmpFileMgrTest, TestRemoteDirectoryLimits) {
   vector<string> tmp_dirs{{LOCAL_BUFFER_PATH}};
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
   int64_t alloc_size = 1024;
   int64_t file_size = 1024;
   FLAGS_remote_tmp_file_size = "1K";
@@ -1805,7 +1801,7 @@ TEST_F(TmpFileMgrTest, TestMixDirectoryLimits) {
   vector<string> tmp_dirs{{LOCAL_BUFFER_PATH, "/tmp/tmp-file-mgr-test-local:1024"}};
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_create_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
   int64_t alloc_size = 1024;
   int64_t file_size = 1024;
   FLAGS_remote_tmp_file_size = "1K";
@@ -1868,7 +1864,7 @@ TEST_F(TmpFileMgrTest, TestMixTmpFileLimits) {
   vector<string> tmp_dirs{{LOCAL_BUFFER_PATH}};
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_create_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
   int64_t alloc_size = 1024;
   int64_t file_size = 512 * 1024 * 1024;
   int64_t offset;
@@ -1896,7 +1892,7 @@ void TmpFileMgrTest::TestTmpFileBufferPoolHelper(TmpFileMgr& tmp_file_mgr,
   vector<string> tmp_create_dirs{{LOCAL_BUFFER_PATH}};
   vector<string> tmp_dirs{{Substitute(LOCAL_BUFFER_PATH + ":$0", 2048)}};
   RemoveAndCreateDirs(tmp_create_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
   int64_t alloc_size = 1024;
   int64_t file_size = 1024;
   FLAGS_remote_tmp_file_size = "1KB";
@@ -2035,7 +2031,7 @@ TEST_F(TmpFileMgrTest, TestRemoteRemoveBuffer) {
   vector<string> tmp_dirs({LOCAL_BUFFER_PATH});
   TmpFileMgr tmp_file_mgr;
   RemoveAndCreateDirs(tmp_dirs);
-  tmp_dirs.push_back(REMOTE_URL);
+  tmp_dirs.push_back(remote_url_);
 
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
   TUniqueId id;
@@ -2091,7 +2087,7 @@ TEST_F(TmpFileMgrTest, TestRemoteUploadFailed) {
   vector<string> tmp_create_dirs{{LOCAL_BUFFER_PATH}};
   vector<string> tmp_dirs{{Substitute(LOCAL_BUFFER_PATH + ":$0", buffer_limit)}};
   RemoveAndCreateDirs(tmp_create_dirs);
-  tmp_dirs.push_back(HDFS_LOCAL_URL);
+  tmp_dirs.push_back(remote_url_);
 
   ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
   TUniqueId id;
@@ -2234,4 +2230,26 @@ TEST_F(TmpFileMgrTest, TestBatchReadingSetMaxBytes) {
   }
 }
 
+TEST_F(TmpFileMgrTest, TestHdfsScratchParsing) {
+  struct parsed { string path; int64_t bytes_limit; int priority; };
+  constexpr int64_t dbytes = numeric_limits<int64_t>::max();
+  constexpr int dpriority = numeric_limits<int>::max();
+  for (auto& valid : map<string, parsed>{
+    { "hdfs://10.0.0.1:8020", parsed{"hdfs://10.0.0.1:8020", dbytes, dpriority} },
+    { "hdfs://10.0.0.1:8020:1024:", parsed{"hdfs://10.0.0.1:8020", 1024, dpriority} },
+    { "hdfs://10.0.0.1/", parsed{"hdfs://10.0.0.1", dbytes, dpriority} },
+    { "hdfs://10.0.0.1/:1k", parsed{"hdfs://10.0.0.1", 1024, dpriority} },
+    { "hdfs://localhost/tmp::", parsed{"hdfs://localhost/tmp", dbytes, dpriority} },
+    { "hdfs://localhost/tmp/:10k:1", parsed{"hdfs://localhost/tmp", 10240, 1} },
+    { "ofs://ozone1/tmp:10k:1", parsed{"ofs://ozone1/tmp", 10240, 1} },
+    { "ofs://ozone1/tmp::1", parsed{"ofs://ozone1/tmp", dbytes, 1} },
+  }) {
+    TmpDirHdfs fs(valid.first);
+    EXPECT_OK(fs.Parse());
+    EXPECT_EQ(path(valid.second.path) / "impala-scratch", fs.path());
+    EXPECT_EQ(valid.second.bytes_limit, fs.bytes_limit());
+    EXPECT_EQ(valid.second.priority, fs.priority());
+  }
+}
+
 } // namespace impala
