@@ -217,11 +217,12 @@ void Tuple::SetNullIndicators(NullIndicatorOffset offset, int64_t num_tuples,
   }
 }
 
-template <bool COLLECT_STRING_VALS, bool NO_POOL>
+template <bool COLLECT_VAR_LEN_VALS, bool NO_POOL>
 void Tuple::MaterializeExprs(TupleRow* row, const TupleDescriptor& desc,
     ScalarExprEvaluator* const* evals, MemPool* pool,
-    StringValue** non_null_string_values, int* total_string_lengths,
-    int* num_non_null_string_values) {
+    StringValue** non_null_string_values, CollValueAndSize* non_null_collection_values,
+    int* total_varlen_lengths,
+    int* num_non_null_string_values, int* num_non_null_collection_values) {
   ClearNullBits(desc);
   // Evaluate the materialize_expr_evals and place the results in the tuple.
   for (int i = 0; i < desc.slots().size(); ++i) {
@@ -232,22 +233,27 @@ void Tuple::MaterializeExprs(TupleRow* row, const TupleDescriptor& desc,
     DCHECK(slot_desc->type().type == TYPE_NULL ||
         slot_desc->type() == evals[i]->root().type());
     void* src = evals[i]->GetValue(row);
-    if (src != NULL) {
-      vector<StringValue*> string_values;
-      RawValue::Write<COLLECT_STRING_VALS>(src, this, slot_desc, pool, &string_values);
-      if (string_values.size() > 0) {
-        for (StringValue* string_val : string_values) {
-          *(non_null_string_values++) = string_val;
-          *total_string_lengths += string_val->len;
-        }
-        (*num_non_null_string_values) += string_values.size();
+
+    vector<StringValue*> string_values;
+    vector<pair<CollectionValue*, int64_t>> collection_values;
+    RawValue::Write<COLLECT_VAR_LEN_VALS>(src, this, slot_desc, pool,
+        &string_values, &collection_values);
+    if (string_values.size() > 0) {
+      for (StringValue* string_val : string_values) {
+        *(non_null_string_values++) = string_val;
+        *total_varlen_lengths += string_val->len;
       }
-    } else {
-      if (slot_desc->type().IsStructType()) {
-        SetStructToNull(slot_desc);
-      } else {
-        SetNull(slot_desc->null_indicator_offset());
+      (*num_non_null_string_values) += string_values.size();
+    }
+
+    if (collection_values.size() > 0) {
+      for (const pair<CollectionValue*, int64_t>& collection_val_pair
+          : collection_values) {
+        CollValueAndSize cvs(collection_val_pair.first, collection_val_pair.second);
+        *(non_null_collection_values++) = cvs;
+        *total_varlen_lengths += collection_val_pair.second;
       }
+      (*num_non_null_collection_values) += collection_values.size();
     }
   }
 }
@@ -277,66 +283,78 @@ char* Tuple::AllocateStrings(const char* err_ctx, RuntimeState* state,
 // Codegens an unrolled version of MaterializeExprs(). Uses codegen'd exprs and slot
 // writes. If 'pool' is non-NULL, string data is copied into it.
 //
-// Example IR for materializing a string column with non-NULL 'pool':
+// Example IR for materializing a string column with non-NULL 'pool', produced by the
+// following query:
+//   select l_comment
+//   from tpch.lineitem
+//   order by l_comment
+//   limit 10;
 //
-// ; Function Attrs: alwaysinline
 // define void @MaterializeExprs(%"class.impala::Tuple"* %opaque_tuple,
 //     %"class.impala::TupleRow"* %row, %"class.impala::TupleDescriptor"* %desc,
-//     %"class.impala::ScalarExprEvaluator"** %materialize_expr_evals,
+//     %"class.impala::ScalarExprEvaluator"** %slot_materialize_expr_evals,
 //     %"class.impala::MemPool"* %pool,
 //     %"struct.impala::StringValue"** %non_null_string_values,
-//     i32* %total_string_lengths, i32* %num_non_null_string_values) #34 {
+//     %"struct.impala::CollValueAndSize"* %non_null_collection_values,
+//     i32* %total_varlen_lengths, i32* %num_non_null_string_values,
+//     i32* %num_non_null_collection_values) #50 {
 // entry:
-//   %tuple = bitcast %"class.impala::Tuple"* %opaque_tuple to
-//       <{ %"struct.impala::StringValue", i8 }>*
+//   %tuple = bitcast %"class.impala::Tuple"* %opaque_tuple
+//       to <{ %"struct.impala::StringValue", i8 }>*
 //   %int8_ptr = bitcast <{ %"struct.impala::StringValue", i8 }>* %tuple to i8*
-//   %null_bytes_ptr = getelementptr inbounds i8, i8* %int8_ptr, i32 16
+//   %null_bytes_ptr = getelementptr inbounds i8, i8* %int8_ptr, i32 12
 //   call void @llvm.memset.p0i8.i64(i8* %null_bytes_ptr, i8 0, i64 1, i32 0, i1 false)
-//   %0 = getelementptr %"class.impala::ExprContext"*,
-//       %"class.impala::ExprContext"** %materialize_expr_ctxs, i32 0
-//   %expr_ctx = load %"class.impala::ExprContext"*, %"class.impala::ExprContext"** %0
-//   %src = call { i64, i8* } @"impala::StringFunctions::UpperWrapper"(
-//        %"class.impala::ExprContext"* %expr_ctx, %"class.impala::TupleRow"* %row)
+//   %0 = getelementptr %"class.impala::ScalarExprEvaluator"*,
+//       %"class.impala::ScalarExprEvaluator"** %slot_materialize_expr_evals, i32 0
+//   %expr_eval = load %"class.impala::ScalarExprEvaluator"*,
+//       %"class.impala::ScalarExprEvaluator"** %0
+//   %src = call { i64, i8* } @GetSlotRef.4(
+//       %"class.impala::ScalarExprEvaluator"* %expr_eval,
+//       %"class.impala::TupleRow"* %row)
+//   ; -- generated by CodegenAnyVal::ToReadWriteInfo() and SlotDescriptor::WriteToSlot()
+//   br label %entry1
+//
+// entry1:                                           ; preds = %entry
 //   %1 = extractvalue { i64, i8* } %src, 0
-//   ; ----- generated by CodegenAnyVal::WriteToSlot() ----------------------------------
 //   %is_null = trunc i64 %1 to i1
 //   br i1 %is_null, label %null, label %non_null
 //
-// non_null:                                         ; preds = %entry
-//   %slot = getelementptr inbounds <{ %"struct.impala::StringValue", i8 }>,
-//       <{ %"struct.impala::StringValue", i8 }>* %tuple, i32 0, i32 0
+// non_null:                                         ; preds = %entry1
 //   %2 = extractvalue { i64, i8* } %src, 0
 //   %3 = ashr i64 %2, 32
 //   %4 = trunc i64 %3 to i32
+//   %src2 = extractvalue { i64, i8* } %src, 1
+//   %slot = getelementptr inbounds <{ %"struct.impala::StringValue", i8 }>,
+//       <{ %"struct.impala::StringValue", i8 }>* %tuple, i32 0, i32 0
 //   %5 = insertvalue %"struct.impala::StringValue" zeroinitializer, i32 %4, 1
 //   %6 = sext i32 %4 to i64
 //   %new_ptr = call i8* @_ZN6impala7MemPool8AllocateILb0EEEPhli(
 //       %"class.impala::MemPool"* %pool, i64 %6, i32 8)
-//   %src1 = extractvalue { i64, i8* } %src, 1
-//   call void @llvm.memcpy.p0i8.p0i8.i32(
-//       i8* %new_ptr, i8* %src1, i32 %4, i32 0, i1 false)
+//   call void @llvm.memcpy.p0i8.p0i8.i32(i8* %new_ptr, i8* %src2, i32 %4, i32 0,
+//       i1 false)
 //   %7 = insertvalue %"struct.impala::StringValue" %5, i8* %new_ptr, 0
 //   store %"struct.impala::StringValue" %7, %"struct.impala::StringValue"* %slot
 //   br label %end_write
 //
-// null:                                             ; preds = %entry
+// null:                                             ; preds = %entry1
 //   %8 = bitcast <{ %"struct.impala::StringValue", i8 }>* %tuple to i8*
-//   %null_byte_ptr = getelementptr inbounds i8, i8* %8, i32 16
+//   %null_byte_ptr = getelementptr inbounds i8, i8* %8, i32 12
 //   %null_byte = load i8, i8* %null_byte_ptr
 //   %null_bit_set = or i8 %null_byte, 1
 //   store i8 %null_bit_set, i8* %null_byte_ptr
 //   br label %end_write
 //
 // end_write:                                        ; preds = %null, %non_null
-//   ; ----- end CodegenAnyVal::WriteToSlot() -------------------------------------------
+//   ; -- end CodegenAnyVal::ToReadWriteInfo() and SlotDescriptor::WriteToSlot() --------
 //   ret void
 // }
-Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_string_vals,
+Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_varlen_vals,
     const TupleDescriptor& desc, const vector<ScalarExpr*>& slot_materialize_exprs,
     bool use_mem_pool, llvm::Function** fn) {
-  // Only support 'collect_string_vals' == false for now.
-  if (collect_string_vals) {
-    return Status("CodegenMaterializeExprs() collect_string_vals == true NYI");
+  // Only support 'collect_varlen_vals' == false for now.
+  // TODO IMPALA-12068: implement it for 'collect_varlen_vals' == true too.
+  if (collect_varlen_vals) {
+    return Status("CodegenMaterializeExprs() collect_varlen_vals == true NYI");
   }
   llvm::LLVMContext& context = codegen->context();
 
@@ -353,9 +371,12 @@ Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_string_
 
   // Construct function signature (this must exactly match the actual signature since it's
   // used in xcompiled IR). With 'pool':
-  // void MaterializeExprs(Tuple* tuple, TupleRow* row, TupleDescriptor* desc,
-  //     ScalarExprEvaluator** slot_materialize_exprs, MemPool* pool,
-  //     StringValue** non_null_string_values, int* total_string_lengths)
+  // void MaterializeExprs(Tuple* opaque_tuple, TupleRow* row,
+  //     const TupleDescriptor& desc, ScalarExprEvaluator* const* evals, MemPool* pool,
+  //     StringValue** non_null_string_values,
+  //     CollValueAndSize* non_null_collection_values,
+  //     int* total_varlen_lengths, int* num_non_null_string_values,
+  //     int* num_non_null_collection_values);
   llvm::PointerType* opaque_tuple_type = codegen->GetStructPtrType<Tuple>();
   llvm::PointerType* row_type = codegen->GetStructPtrType<TupleRow>();
   llvm::PointerType* desc_type = codegen->GetStructPtrType<TupleDescriptor>();
@@ -363,29 +384,35 @@ Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_string_
       codegen->GetStructPtrPtrType<ScalarExprEvaluator>();
   llvm::PointerType* pool_type = codegen->GetStructPtrType<MemPool>();
   llvm::PointerType* string_values_type = codegen->GetStructPtrPtrType<StringValue>();
+  llvm::PointerType* coll_values_and_sizes_type =
+      codegen->GetStructPtrType<CollValueAndSize>();
   llvm::PointerType* int_ptr_type = codegen->i32_ptr_type();
   LlvmCodeGen::FnPrototype prototype(codegen, "MaterializeExprs", codegen->void_type());
   prototype.AddArgument("opaque_tuple", opaque_tuple_type);
   prototype.AddArgument("row", row_type);
   prototype.AddArgument("desc", desc_type);
-  prototype.AddArgument("slot_materialize_exprs", expr_evals_type);
+  prototype.AddArgument("slot_materialize_expr_evals", expr_evals_type);
   prototype.AddArgument("pool", pool_type);
   prototype.AddArgument("non_null_string_values", string_values_type);
-  prototype.AddArgument("total_string_lengths", int_ptr_type);
+  prototype.AddArgument("non_null_collection_values", coll_values_and_sizes_type);
+  prototype.AddArgument("total_varlen_lengths", int_ptr_type);
   prototype.AddArgument("num_non_null_string_values", int_ptr_type);
+  prototype.AddArgument("num_non_null_collection_values", int_ptr_type);
 
   LlvmBuilder builder(context);
-  llvm::Value* args[8];
+  llvm::Value* args[10];
   *fn = prototype.GeneratePrototype(&builder, args);
   llvm::Value* opaque_tuple_arg = args[0];
   llvm::Value* row_arg = args[1];
   // llvm::Value* desc_arg = args[2]; // unused
   llvm::Value* expr_evals_arg = args[3];
   llvm::Value* pool_arg = args[4];
-  // The followings arguments are unused as 'collect_string_vals' is false.
+  // The followings arguments are unused as 'collect_varlen_vals' is false.
   // llvm::Value* non_null_string_values_arg = args[5]; // unused
-  // llvm::Value* total_string_lengths_arg = args[6]; // unused
-  // llvm::Value* num_non_null_string_values_arg = args[7]; // unused
+  // llvm::Value* non_null_collection_values_arg = args[6]; // unused
+  // llvm::Value* total_varlen_lengths_arg = args[7]; // unused
+  // llvm::Value* num_non_null_string_values_arg = args[8]; // unused
+  // llvm::Value* num_non_null_collection_values_arg = args[9]; // unused
 
   // Cast the opaque Tuple* argument to the generated struct type
   llvm::Type* tuple_struct_type = desc.GetLlvmStruct(codegen);
@@ -412,7 +439,9 @@ Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_string_
         slot_materialize_exprs[i]->type(), materialize_expr_fns[i], expr_args, "src");
 
     // Write expr result 'src' to slot
-    src.WriteToSlot(*slot_desc, tuple, use_mem_pool ? pool_arg : nullptr);
+    CodegenAnyValReadWriteInfo read_write_info = src.ToReadWriteInfo();
+    slot_desc->CodegenWriteToSlot(
+        read_write_info, tuple, use_mem_pool ? pool_arg : nullptr);
   }
   builder.CreateRetVoid();
   // TODO: if pool != NULL, OptimizeFunctionWithExprs() is inlining the Allocate()
@@ -488,11 +517,15 @@ llvm::Constant* SlotOffsets::ToIR(LlvmCodeGen* codegen) const {
 }
 
 template void Tuple::MaterializeExprs<false, false>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
+    int*, int*, int*);
 template void Tuple::MaterializeExprs<false, true>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
+    int*, int*, int*);
 template void Tuple::MaterializeExprs<true, false>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
+    int*, int*, int*);
 template void Tuple::MaterializeExprs<true, true>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
+    int*, int*, int*);
 }
