@@ -249,10 +249,6 @@ struct ColumnType {
   inline bool IsArrayType() const { return type == TYPE_ARRAY; }
   inline bool IsMapType() const { return type == TYPE_MAP; }
 
-  inline bool IsVarLenType() const {
-    return IsVarLenStringType() || IsCollectionType();
-  }
-
   /// Returns the byte size of this type.  Returns 0 for variable length types.
   inline int GetByteSize() const { return GetByteSize(*this); }
 
@@ -290,7 +286,7 @@ struct ColumnType {
     switch (col_type.type) {
       case TYPE_STRUCT: {
         int struct_size = 0;
-        for (ColumnType child_type : col_type.children) {
+        for (const ColumnType& child_type : col_type.children) {
           struct_size += GetSlotSize(child_type);
         }
         return struct_size;
@@ -314,7 +310,7 @@ struct ColumnType {
     switch (col_type.type) {
       case TYPE_STRUCT: {
         int struct_size = 0;
-        for (ColumnType child_type : col_type.children) {
+        for (const ColumnType& child_type : col_type.children) {
           struct_size += GetByteSize(child_type);
         }
         return struct_size;
