@@ -277,8 +277,8 @@ enum class AdmissionOutcome {
 /// FindGroupToAdmitOrReject(). From there we call ComputeGroupScheduleStates() which
 /// calls compute schedules for both executor groups. Then we perform rejection tests and
 /// afterwards call CanAdmitRequest() for each of the schedules. Executor groups are
-/// processed in alphanumerically sorted order, so we attempt admission to group
-/// "default-pool-group-1" first. CanAdmitRequest() calls HasAvailableSlots() to check
+/// processed in a deterministic order, see comments in ComputeGroupScheduleStates() for
+/// details. CanAdmitRequest() calls HasAvailableSlots() to check
 /// whether any of the hosts in the group can fit the new query in their available slots
 /// and since it does fit, admission succeeds. The query is admitted and 'slots_in_use'
 /// is incremented for each host in that group based on the effective parallelism of the
@@ -328,6 +328,7 @@ class AdmissionController {
   // Profile info strings
   static const std::string PROFILE_INFO_KEY_ADMISSION_RESULT;
   static const std::string PROFILE_INFO_VAL_ADMIT_IMMEDIATELY;
+  static const std::string PROFILE_INFO_VAL_ADMIT_TRIVIAL;
   static const std::string PROFILE_INFO_VAL_QUEUED;
   static const std::string PROFILE_INFO_VAL_CANCELLED_IN_QUEUE;
   static const std::string PROFILE_INFO_VAL_ADMIT_QUEUED;
@@ -377,7 +378,8 @@ class AdmissionController {
   /// function returns due to the timeout 'wait_timed_out' will be true.
   Status WaitOnQueued(const UniqueIdPB& query_id,
       std::unique_ptr<QuerySchedulePB>* schedule_result, int64_t timeout_ms = 0,
-      bool* wait_timed_out = nullptr);
+      bool* wait_timed_out = nullptr, int64_t* wait_start_time_ms = nullptr,
+      int64_t* wait_end_time_ms = nullptr);
 
   /// Updates the pool statistics when a query completes (either successfully,
   /// is cancelled or failed). This should be called for all requests that have
@@ -543,26 +545,35 @@ class AdmissionController {
       IntGauge* max_query_mem_limit;
       IntGauge* min_query_mem_limit;
       BooleanProperty* clamp_mem_limit_query_option;
+      IntGauge* max_query_cpu_core_per_node_limit;
+      IntGauge* max_query_cpu_core_coordinator_limit;
     };
 
     PoolStats(AdmissionController* parent, const std::string& name)
-      : name_(name), parent_(parent), agg_num_running_(0), agg_num_queued_(0),
-        agg_mem_reserved_(0), local_mem_admitted_(0), wait_time_ms_ema_(0.0) {
+      : name_(name),
+        parent_(parent),
+        agg_num_running_(0),
+        agg_num_queued_(0),
+        agg_mem_reserved_(0),
+        local_trivial_running_(0),
+        local_mem_admitted_(0),
+        wait_time_ms_ema_(0.0) {
       peak_mem_histogram_.resize(HISTOGRAM_NUM_OF_BINS, 0);
       InitMetrics();
     }
 
     int64_t agg_num_running() const { return agg_num_running_; }
     int64_t agg_num_queued() const { return agg_num_queued_; }
+    int64_t local_trivial_running() const { return local_trivial_running_; }
     int64_t EffectiveMemReserved() const {
       return std::max(agg_mem_reserved_, local_mem_admitted_);
     }
 
     // ADMISSION LIFECYCLE METHODS
     /// Updates the pool stats when the request represented by 'state' is admitted.
-    void AdmitQueryAndMemory(const ScheduleState& state);
+    void AdmitQueryAndMemory(const ScheduleState& state, bool is_trivial);
     /// Updates the pool stats except the memory admitted stat.
-    void ReleaseQuery(int64_t peak_mem_consumption);
+    void ReleaseQuery(int64_t peak_mem_consumption, bool is_trivial);
     /// Releases the specified memory from the pool stats.
     void ReleaseMem(int64_t mem_to_release);
     /// Updates the pool stats when the request represented by 'state' is queued.
@@ -630,6 +641,9 @@ class AdmissionController {
 
     const std::string& name() const { return name_; }
 
+    /// The max number of running trivial queries that can be allowed at the same time.
+    static const int MAX_NUM_TRIVIAL_QUERY_RUNNING;
+
    private:
     const std::string name_;
     AdmissionController* parent_;
@@ -648,6 +662,13 @@ class AdmissionController {
     /// other hosts. Updated only by UpdateAggregates().
     int64_t agg_mem_reserved_;
 
+    /// Number of running trivial queries in this pool that have been admitted by this
+    /// local coordinator. The purpose of it is to control the concurrency of running
+    /// trivial queries in case they may consume too many resources because trivial
+    /// queries bypass the normal admission control procedure.
+    /// Updated only in AdmitQueryAndMemory() and ReleaseQuery().
+    int64_t local_trivial_running_;
+
     /// Memory in this pool (across all nodes) that is needed for requests that have been
     /// admitted by this local coordinator. Updated only on Admit() and Release(). Stored
     /// separately from the other 'local' stats in local_stats_ because it is not sent
@@ -855,6 +876,9 @@ class AdmissionController {
     /// Map from backend addresses to the resouces this query was allocated on them. When
     /// backends are released, they are removed from this map.
     std::unordered_map<NetworkAddressPB, BackendAllocation> per_backend_resources;
+
+    /// Indicate whether the query is admitted as a trivial query.
+    bool is_trivial;
   };
 
   /// Map from host id to a map from query id of currently running queries to information
@@ -933,9 +957,12 @@ class AdmissionController {
   /// true and keeps queue_node->admitted_schedule unset if the query cannot be admitted
   /// now, but also does not need to be rejected. If the query must be rejected, this
   /// method returns false and sets queue_node->not_admitted_reason.
+  /// The is_trivial is set to true when is_trivial is not null if the query is admitted
+  /// as a trivial query.
   bool FindGroupToAdmitOrReject(ClusterMembershipMgr::SnapshotPtr membership_snapshot,
       const TPoolConfig& pool_config, bool admit_from_queue, PoolStats* pool_stats,
-      QueueNode* queue_node, bool& coordinator_resource_limited);
+      QueueNode* queue_node, bool& coordinator_resource_limited,
+      bool* is_trivial = nullptr);
 
   /// Dequeues the queued queries when notified by dequeue_cv_ and admits them if they
   /// have not been cancelled yet.
@@ -952,6 +979,10 @@ class AdmissionController {
       bool admit_from_queue, string* not_admitted_reason, string* not_admitted_details,
       bool& coordinator_resource_limited);
 
+  /// Returns true if the query can be admitted as a trivial query, therefore it can
+  /// bypass the admission control immediately.
+  bool CanAdmitTrivialRequest(const ScheduleState& state);
+
   /// Returns true if all executors can accommodate the largest initial reservation of
   /// any executor and the backend running the coordinator fragment can accommodate its
   /// own initial reservation. Otherwise, returns false with the details about the memory
@@ -988,7 +1019,7 @@ class AdmissionController {
   /// Updates the memory admitted and the num of queries running for each backend in
   /// 'state'. Also updates the stats of its associated resource pool. Used only when
   /// the 'state' is admitted.
-  void UpdateStatsOnAdmission(const ScheduleState& state);
+  void UpdateStatsOnAdmission(const ScheduleState& state, bool is_trivial);
 
   /// Updates the memory admitted and the num of queries running for each backend in
   /// 'state' which have been release/completed. The list of completed backends is
@@ -1057,7 +1088,7 @@ class AdmissionController {
   /// Sets the per host mem limit and mem admitted in the schedule and does the necessary
   /// accounting and logging on successful submission.
   /// Caller must hold 'admission_ctrl_lock_'.
-  void AdmitQuery(QueueNode* node, bool was_queued);
+  void AdmitQuery(QueueNode* node, bool was_queued, bool is_trivial);
 
   /// Same as PoolToJson() but requires 'admission_ctrl_lock_' to be held by the caller.
   /// Is a helper method used by both PoolToJson() and AllPoolsToJson()
@@ -1100,6 +1131,10 @@ class AdmissionController {
   /// Returns the maximum number of requests that can be queued in the pool.
   static int64_t GetMaxQueuedForPool(const TPoolConfig& pool_config);
 
+  /// Returns available memory and slots of the executor group.
+  const std::pair<int64_t, int64_t> GetAvailableMemAndSlots(
+      const ExecutorGroup& group) const;
+
   /// Return all executor groups from 'all_groups' that can be used to run the query. If
   /// the query is a coordinator only query then a reserved empty group is returned
   /// otherwise returns all healthy groups that can be used to run queries for the
