@@ -31,6 +31,7 @@
 #include "gen-cpp/DataSinks_types.h"
 #include "gen-cpp/ErrorCodes_types.h"
 #include "gen-cpp/ImpalaInternalService_types.h"
+#include "gen-cpp/Query_constants.h"
 #include "gen-cpp/Types_types.h"
 #include "gen-cpp/common.pb.h"
 #include "gen-cpp/statestore_service.pb.h"
@@ -60,6 +61,7 @@ namespace impala {
 static const string LOCAL_ASSIGNMENTS_KEY("simple-scheduler.local-assignments.total");
 static const string ASSIGNMENTS_KEY("simple-scheduler.assignments.total");
 static const string SCHEDULER_INIT_KEY("simple-scheduler.initialized");
+static const string SCHEDULER_WARNING_KEY("Scheduler Warning");
 
 static const vector<TPlanNodeType::type> SCAN_NODE_TYPES{TPlanNodeType::HDFS_SCAN_NODE,
     TPlanNodeType::HBASE_SCAN_NODE, TPlanNodeType::DATA_SOURCE_NODE,
@@ -134,6 +136,7 @@ Status Scheduler::GenerateScanRanges(const vector<TFileSplitGeneratorSpec>& spec
       hdfs_scan_range.__set_offset(scan_range_offset);
       hdfs_scan_range.__set_partition_id(spec.partition_id);
       hdfs_scan_range.__set_partition_path_hash(spec.partition_path_hash);
+      hdfs_scan_range.__set_is_encrypted(fb_desc->is_encrypted());
       hdfs_scan_range.__set_is_erasure_coded(fb_desc->is_ec());
       if (fb_desc->absolute_path() != nullptr) {
         hdfs_scan_range.__set_absolute_path(fb_desc->absolute_path()->str());
@@ -207,7 +210,7 @@ Status Scheduler::ComputeScanRangeAssignment(
   return Status::OK();
 }
 
-void Scheduler::ComputeFragmentExecParams(
+Status Scheduler::ComputeFragmentExecParams(
     const ExecutorConfig& executor_config, ScheduleState* state) {
   const TQueryExecRequest& exec_request = state->request();
 
@@ -217,8 +220,8 @@ void Scheduler::ComputeFragmentExecParams(
   // be easily co-located with the plans consuming their output.
   for (const TPlanExecInfo& plan_exec_info : exec_request.plan_exec_info) {
     // set instance_id, host, per_node_scan_ranges
-    ComputeFragmentExecParams(executor_config, plan_exec_info,
-        state->GetFragmentScheduleState(plan_exec_info.fragments[0].idx), state);
+    RETURN_IF_ERROR(ComputeFragmentExecParams(executor_config, plan_exec_info,
+        state->GetFragmentScheduleState(plan_exec_info.fragments[0].idx), state));
 
     // Set destinations, per_exch_num_senders, sender_id.
     for (const TPlanFragment& src_fragment : plan_exec_info.fragments) {
@@ -261,16 +264,86 @@ void Scheduler::ComputeFragmentExecParams(
       }
     }
   }
+  return Status::OK();
 }
 
-void Scheduler::ComputeFragmentExecParams(const ExecutorConfig& executor_config,
+Status Scheduler::CheckEffectiveInstanceCount(
+    const FragmentScheduleState* fragment_state, ScheduleState* state) {
+  // These checks are only intended if COMPUTE_PROCESSING_COST=true.
+  if (!state->query_options().compute_processing_cost) return Status::OK();
+
+  int effective_instance_count = fragment_state->fragment.effective_instance_count;
+  if (effective_instance_count < fragment_state->instance_states.size()) {
+    if (state->summary_profile()->GetInfoString(SCHEDULER_WARNING_KEY) == nullptr) {
+      state->summary_profile()->AddInfoString(SCHEDULER_WARNING_KEY,
+          "Cluster membership might changed between planning and scheduling");
+    }
+
+    string warn_message = Substitute(
+        "$0 scheduled instance count ($1) is higher than its effective count ($2)",
+        fragment_state->fragment.display_name, fragment_state->instance_states.size(),
+        effective_instance_count);
+    state->summary_profile()->AppendInfoString(SCHEDULER_WARNING_KEY, warn_message);
+    LOG(WARNING) << warn_message;
+  }
+
+  DCHECK(!fragment_state->instance_states.empty());
+  // Find host with the largest instance assignment.
+  // Initialize to the first fragment instance.
+  int largest_inst_idx = 0;
+  int largest_inst_per_host = 1;
+  int this_inst_count = 1;
+  int num_host = 1;
+  for (int i = 1; i < fragment_state->instance_states.size(); i++) {
+    if (fragment_state->instance_states[i].host
+        == fragment_state->instance_states[i - 1].host) {
+      this_inst_count++;
+    } else {
+      if (largest_inst_per_host < this_inst_count) {
+        largest_inst_per_host = this_inst_count;
+        largest_inst_idx = i - 1;
+      }
+      this_inst_count = 1;
+      num_host++;
+    }
+  }
+  if (largest_inst_per_host < this_inst_count) {
+    largest_inst_per_host = this_inst_count;
+    largest_inst_idx = fragment_state->instance_states.size() - 1;
+  }
+
+  QueryConstants qc;
+  if (largest_inst_per_host > qc.MAX_FRAGMENT_INSTANCES_PER_NODE) {
+    return Status(Substitute(
+        "$0 scheduled instance count ($1) is higher than maximum instances per node"
+        " ($2), indicating a planner bug. Consider running the query with"
+        " COMPUTE_PROCESSING_COST=false.",
+        fragment_state->fragment.display_name, largest_inst_per_host,
+        qc.MAX_FRAGMENT_INSTANCES_PER_NODE));
+  }
+
+  int planned_inst_per_host = ceil((float)effective_instance_count / num_host);
+  if (largest_inst_per_host > planned_inst_per_host) {
+    LOG(WARNING) << fragment_state->fragment.display_name
+                 << " has imbalance number of instance to host assignment."
+                 << " Consider running the query with COMPUTE_PROCESSING_COST=false."
+                 << " Host " << fragment_state->instance_states[largest_inst_idx].host
+                 << " has " << largest_inst_per_host << " instances assigned."
+                 << " effective_instance_count=" << effective_instance_count
+                 << " planned_inst_per_host=" << planned_inst_per_host
+                 << " num_host=" << num_host;
+  }
+  return Status::OK();
+}
+
+Status Scheduler::ComputeFragmentExecParams(const ExecutorConfig& executor_config,
     const TPlanExecInfo& plan_exec_info, FragmentScheduleState* fragment_state,
     ScheduleState* state) {
   // Create exec params for child fragments connected by an exchange. Instance creation
   // for this fragment depends on where the input fragment instances are scheduled.
   for (FragmentIdx input_fragment_idx : fragment_state->exchange_input_fragments) {
-    ComputeFragmentExecParams(executor_config, plan_exec_info,
-        state->GetFragmentScheduleState(input_fragment_idx), state);
+    RETURN_IF_ERROR(ComputeFragmentExecParams(executor_config, plan_exec_info,
+        state->GetFragmentScheduleState(input_fragment_idx), state));
   }
 
   const TPlanFragment& fragment = fragment_state->fragment;
@@ -337,8 +410,7 @@ void Scheduler::ComputeFragmentExecParams(const ExecutorConfig& executor_config,
         instance_state.AddScanRanges(entry.first, entry.second);
       }
     }
-  } else if (ContainsNode(fragment.plan, TPlanNodeType::UNION_NODE)
-      || ContainsScanNode(fragment.plan)) {
+  } else if (ContainsUnionNode(fragment.plan) || ContainsScanNode(fragment.plan)) {
     VLOG(3) << "Computing exec params for scan and/or union fragment.";
     // case 2: leaf fragment (i.e. no input fragments) with a single scan node.
     // case 3: union fragment, which may have scan nodes and may have input fragments.
@@ -347,9 +419,11 @@ void Scheduler::ComputeFragmentExecParams(const ExecutorConfig& executor_config,
     VLOG(3) << "Computing exec params for interior fragment.";
     // case 4: interior (non-leaf) fragment without a scan or union.
     // We assign the same hosts as those of our leftmost input fragment (so that a
-    // merge aggregation fragment runs on the hosts that provide the input data).
+    // merge aggregation fragment runs on the hosts that provide the input data) OR
+    // follow the effective_instance_count specified by planner.
     CreateInputCollocatedInstances(fragment_state, state);
   }
+  return CheckEffectiveInstanceCount(fragment_state, state);
 }
 
 // Maybe the easiest way to understand the objective of this algorithm is as a
@@ -380,7 +454,7 @@ void Scheduler::ComputeFragmentExecParams(const ExecutorConfig& executor_config,
 void Scheduler::CreateCollocatedAndScanInstances(const ExecutorConfig& executor_config,
     FragmentScheduleState* fragment_state, ScheduleState* state) {
   const TPlanFragment& fragment = fragment_state->fragment;
-  bool has_union = ContainsNode(fragment.plan, TPlanNodeType::UNION_NODE);
+  bool has_union = ContainsUnionNode(fragment.plan);
   DCHECK(has_union || ContainsScanNode(fragment.plan));
   // Build a map of hosts to the num instances this fragment should have, before we take
   // into account scan ranges. If this fragment has input fragments, we always run with
@@ -417,8 +491,9 @@ void Scheduler::CreateCollocatedAndScanInstances(const ExecutorConfig& executor_
       << "for plans with no union and multiple scans per fragment";
   vector<NetworkAddressPB> scan_hosts;
   GetScanHosts(scan_node_ids, *fragment_state, &scan_hosts);
-  if (scan_hosts.empty()) {
-    // None of the scan nodes have any scan ranges; run it on a random executor.
+  if (scan_hosts.empty() && instances_per_host.empty()) {
+    // None of the scan nodes have any scan ranges and there is no input fragment feeding
+    // into this fragment; run it on a random executor.
     // TODO TODO: the TODO below seems partially stale
     // TODO: we'll need to revisit this strategy once we can partition joins
     // (in which case this fragment might be executing a right outer join
@@ -437,11 +512,24 @@ void Scheduler::CreateCollocatedAndScanInstances(const ExecutorConfig& executor_
     int& host_num_instances = instances_per_host[host_addr];
     host_num_instances = max(1, host_num_instances);
   }
-  DCHECK(!instances_per_host.empty()) << "no hosts for fragment " << fragment.idx;
+  DCHECK(!instances_per_host.empty())
+      << "no hosts for fragment " << fragment.idx << " (" << fragment.display_name << ")";
+
+  const TQueryOptions& request_query_option =
+      state->request().query_ctx.client_request.query_options;
+  int max_num_instances = 1;
+  if (request_query_option.compute_processing_cost) {
+    // Numbers should follow 'effective_instance_count' from planner.
+    // 'effective_instance_count' is total instances across all executors.
+    DCHECK(fragment.effective_instance_count > 0);
+    int inst_per_host =
+        ceil((float)fragment.effective_instance_count / instances_per_host.size());
+    max_num_instances = max(1, inst_per_host);
+  } else {
+    // Number of instances should be bounded by mt_dop.
+    max_num_instances = max(1, request_query_option.mt_dop);
+  }
 
-  // Number of instances should be bounded by mt_dop.
-  int max_num_instances =
-      max(1, state->request().query_ctx.client_request.query_options.mt_dop);
   // Track the index of the next instance to be created for this fragment.
   int per_fragment_instance_idx = 0;
   for (const auto& entry : instances_per_host) {
@@ -470,7 +558,12 @@ void Scheduler::CreateCollocatedAndScanInstances(const ExecutorConfig& executor_
     int num_instances = entry.second;
     DCHECK_GE(num_instances, 1);
     DCHECK_LE(num_instances, max_num_instances)
-        << "Input parallelism too high for mt_dop";
+        << "Input parallelism too high. Fragment " << fragment.display_name
+        << " num_host=" << instances_per_host.size()
+        << " scan_host_count=" << scan_hosts.size()
+        << " mt_dop=" << request_query_option.mt_dop
+        << " compute_processing_cost=" << request_query_option.compute_processing_cost
+        << " effective_instance_count=" << fragment.effective_instance_count;
     for (const auto& per_scan_ranges : per_scan_per_instance_ranges) {
       num_instances = max(num_instances, static_cast<int>(per_scan_ranges.size()));
     }
@@ -498,10 +591,7 @@ void Scheduler::CreateCollocatedAndScanInstances(const ExecutorConfig& executor_
       }
     }
   }
-  if (fragment.output_sink.__isset.table_sink
-      && fragment.output_sink.table_sink.__isset.hdfs_table_sink
-      && state->query_options().max_fs_writers > 0
-      && fragment_state->instance_states.size() > state->query_options().max_fs_writers) {
+  if (IsExceedMaxFsWriters(fragment_state, fragment_state, state)) {
     LOG(WARNING) << "Extra table sink instances scheduled, probably due to mismatch of "
                     "cluster state during planning vs scheduling. Expected: "
                  << state->query_options().max_fs_writers
@@ -535,11 +625,14 @@ void Scheduler::CreateInputCollocatedInstances(
       *state->GetFragmentScheduleState(fragment_state->exchange_input_fragments[0]);
   int per_fragment_instance_idx = 0;
 
-  if (fragment.output_sink.__isset.table_sink
-      && fragment.output_sink.table_sink.__isset.hdfs_table_sink
-      && state->query_options().max_fs_writers > 0
-      && input_fragment_state.instance_states.size()
-          > state->query_options().max_fs_writers) {
+  int max_instances = input_fragment_state.instance_states.size();
+  if (fragment.effective_instance_count > 0) {
+    max_instances = fragment.effective_instance_count;
+  } else if (IsExceedMaxFsWriters(fragment_state, &input_fragment_state, state)) {
+    max_instances = state->query_options().max_fs_writers;
+  }
+
+  if (max_instances != input_fragment_state.instance_states.size()) {
     std::unordered_set<std::pair<NetworkAddressPB, NetworkAddressPB>> all_hosts;
     for (const FInstanceScheduleState& input_instance_state :
         input_fragment_state.instance_states) {
@@ -549,7 +642,6 @@ void Scheduler::CreateInputCollocatedInstances(
     // across hosts and ensuring that instances on the same host get consecutive instance
     // indexes.
     int num_hosts = all_hosts.size();
-    int max_instances = state->query_options().max_fs_writers;
     int instances_per_host = max_instances / num_hosts;
     int remainder = max_instances % num_hosts;
     auto host_itr = all_hosts.begin();
@@ -786,6 +878,10 @@ bool Scheduler::ContainsScanNode(const TPlan& plan) {
   return ContainsNode(plan, SCAN_NODE_TYPES);
 }
 
+bool Scheduler::ContainsUnionNode(const TPlan& plan) {
+  return ContainsNode(plan, TPlanNodeType::UNION_NODE);
+}
+
 std::vector<TPlanNodeId> Scheduler::FindNodes(
     const TPlan& plan, const vector<TPlanNodeType::type>& types) {
   vector<TPlanNodeId> results;
@@ -821,7 +917,7 @@ void Scheduler::GetScanHosts(const vector<TPlanNodeId>& scan_ids,
 Status Scheduler::Schedule(const ExecutorConfig& executor_config, ScheduleState* state) {
   RETURN_IF_ERROR(DebugAction(state->query_options(), "SCHEDULER_SCHEDULE"));
   RETURN_IF_ERROR(ComputeScanRangeAssignment(executor_config, state));
-  ComputeFragmentExecParams(executor_config, state);
+  RETURN_IF_ERROR(ComputeFragmentExecParams(executor_config, state));
   ComputeBackendExecParams(executor_config, state);
 #ifndef NDEBUG
   state->Validate();
@@ -1124,6 +1220,7 @@ void TScanRangeToScanRangePB(const TScanRange& tscan_range, ScanRangePB* scan_ra
     hdfs_file_split->set_mtime(tscan_range.hdfs_file_split.mtime);
     hdfs_file_split->set_partition_path_hash(
         tscan_range.hdfs_file_split.partition_path_hash);
+    hdfs_file_split->set_is_encrypted(tscan_range.hdfs_file_split.is_encrypted);
     hdfs_file_split->set_is_erasure_coded(tscan_range.hdfs_file_split.is_erasure_coded);
     if (tscan_range.hdfs_file_split.__isset.absolute_path) {
       hdfs_file_split->set_absolute_path(
