@@ -25,11 +25,9 @@
 #include <rapidjson/rapidjson.h>
 #include <rapidjson/stringbuffer.h>
 #include <rapidjson/writer.h>
-#include <rapidjson/error/en.h>
 
 #include "catalog/catalog-service-client-wrapper.h"
 #include "common/status.h"
-#include "exec/kudu/kudu-util.h"
 #include "exprs/timezone_db.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "rpc/rpc-mgr.inline.h"
@@ -52,15 +50,12 @@
 #include "util/debug-util.h"
 #include "util/impalad-metrics.h"
 #include "util/lineage-util.h"
-#include "util/metrics.h"
 #include "util/pretty-printer.h"
-#include "util/promise.h"
 #include "util/redactor.h"
 #include "util/runtime-profile-counters.h"
 #include "util/time.h"
 #include "util/uid-util.h"
 
-#include "gen-cpp/CatalogService.h"
 #include "gen-cpp/CatalogService_types.h"
 #include "gen-cpp/control_service.pb.h"
 #include "gen-cpp/control_service.proxy.h"
@@ -82,8 +77,6 @@ using namespace strings;
 DECLARE_bool(abort_on_failed_audit_event);
 DECLARE_bool(abort_on_failed_lineage_event);
 DECLARE_int32(krpc_port);
-DECLARE_int32(catalog_service_port);
-DECLARE_string(catalog_service_host);
 DECLARE_int64(max_result_cache_size);
 DECLARE_bool(use_local_catalog);
 
@@ -108,10 +101,10 @@ ClientRequestState::ClientRequestState(const TQueryCtx& query_ctx, Frontend* fro
     session_(session),
     coord_exec_called_(false),
     // Profile is assigned name w/ id after planning
-    profile_(RuntimeProfile::Create(&profile_pool_, "Query")),
-    frontend_profile_(RuntimeProfile::Create(&profile_pool_, "Frontend")),
-    server_profile_(RuntimeProfile::Create(&profile_pool_, "ImpalaServer")),
-    summary_profile_(RuntimeProfile::Create(&profile_pool_, "Summary")),
+    profile_(RuntimeProfile::Create(&profile_pool_, "Query", false)),
+    frontend_profile_(RuntimeProfile::Create(&profile_pool_, "Frontend", false)),
+    server_profile_(RuntimeProfile::Create(&profile_pool_, "ImpalaServer", false)),
+    summary_profile_(RuntimeProfile::Create(&profile_pool_, "Summary", false)),
     exec_request_(exec_request),
     frontend_(frontend),
     parent_server_(server),
@@ -131,6 +124,11 @@ ClientRequestState::ClientRequestState(const TQueryCtx& query_ctx, Frontend* fro
   num_rows_fetched_from_cache_counter_ =
       ADD_COUNTER(server_profile_, "NumRowsFetchedFromCache", TUnit::UNIT);
   client_wait_timer_ = ADD_TIMER(server_profile_, "ClientFetchWaitTimer");
+  client_wait_time_stats_ =
+      ADD_SUMMARY_STATS_TIMER(server_profile_, "ClientFetchWaitTimeStats");
+  rpc_read_timer_ = ADD_TIMER(server_profile_, "RPCReadTimer");
+  rpc_write_timer_ = ADD_TIMER(server_profile_, "RPCWriteTimer");
+  rpc_count_ = ADD_COUNTER(server_profile_, "RPCCount", TUnit::UNIT);
   bool is_external_fe = session_type() == TSessionType::EXTERNAL_FRONTEND;
   // "Impala Backend Timeline" was specifically chosen to exploit the lexicographical
   // ordering defined by the underlying std::map holding the timelines displayed in
@@ -154,6 +152,7 @@ ClientRequestState::ClientRequestState(const TQueryCtx& query_ctx, Frontend* fro
   summary_profile_->AddInfoString("Start Time", ToStringFromUnixMicros(start_time_us(),
       TimePrecision::Nanosecond));
   summary_profile_->AddInfoString("End Time", "");
+  summary_profile_->AddInfoString("Duration", "");
   summary_profile_->AddInfoString("Query Type", "N/A");
   summary_profile_->AddInfoString("Query State", PrintValue(BeeswaxQueryState()));
   summary_profile_->AddInfoString(
@@ -178,6 +177,9 @@ ClientRequestState::ClientRequestState(const TQueryCtx& query_ctx, Frontend* fro
 
 ClientRequestState::~ClientRequestState() {
   DCHECK(wait_thread_.get() == NULL) << "BlockOnWait() needs to be called!";
+  DCHECK(!track_rpcs_);  // Should get set to false in Finalize()
+  DCHECK(pending_rpcs_.empty()); // Should get cleared in Finalize()
+  UnRegisterRemainingRPCs(); // Avoid memory leaks if Finalize() didn't get called
 }
 
 Status ClientRequestState::SetResultCache(QueryResultSet* cache,
@@ -198,11 +200,15 @@ void ClientRequestState::SetRemoteSubmitTime(int64_t remote_submit_time) {
   query_events_->Start(remote_submit_time);
 }
 
-void ClientRequestState::SetFrontendProfile(TRuntimeProfileNode profile) {
+void ClientRequestState::SetFrontendProfile(const TExecRequest& exec_request) {
   // Should we defer creating and adding the child until here? probably.
   TRuntimeProfileTree prof_tree;
-  prof_tree.nodes.emplace_back(std::move(profile));
-  frontend_profile_->Update(prof_tree);
+  prof_tree.nodes.emplace_back(std::move(exec_request.profile));
+  for (auto& child : exec_request.profile_children) {
+    prof_tree.nodes.emplace_back(std::move(child));
+  }
+  prof_tree.nodes.at(0).num_children = prof_tree.nodes.size() - 1;
+  frontend_profile_->Update(prof_tree, false);
 }
 
 void ClientRequestState::AddBlacklistedExecutorAddress(const NetworkAddressPB& addr) {
@@ -302,6 +308,10 @@ Status ClientRequestState::Exec() {
       DCHECK(exec_request_->admin_request.type == TAdminRequestType::SHUTDOWN);
       RETURN_IF_ERROR(ExecShutdownRequest());
       break;
+    case TStmtType::CONVERT:
+      DCHECK(exec_request_->__isset.convert_table_request);
+      LOG_AND_RETURN_IF_ERROR(ExecMigrateRequest());
+      break;
     default:
       stringstream errmsg;
       errmsg << "Unknown exec request stmt type: " << exec_request_->stmt_type;
@@ -580,7 +590,7 @@ void ClientRequestState::FinishExecQueryOrDmlRequest() {
       {query_id_pb, ExecEnv::GetInstance()->backend_id(),
           exec_request_->query_exec_request, exec_request_->query_options,
           summary_profile_, blacklisted_executor_addresses_},
-      query_events_, &schedule_);
+      query_events_, &schedule_, &wait_start_time_ms_, &wait_end_time_ms_);
   {
     lock_guard<mutex> l(lock_);
     if (!UpdateQueryStatus(admit_status).ok()) return;
@@ -695,11 +705,20 @@ void ClientRequestState::ExecDdlRequestImpl(bool exec_in_worker_thread) {
   DebugActionNoFail(exec_request_->query_options, "CRS_DELAY_BEFORE_CATALOG_OP_EXEC");
 
   Status status = catalog_op_executor_->Exec(exec_request_->catalog_op_request);
+  query_events_->MarkEvent("CatalogDdlRequest finished");
   {
     lock_guard<mutex> l(lock_);
     RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
   }
 
+  if (catalog_op_executor_->ddl_exec_response() != nullptr &&
+      catalog_op_executor_->ddl_exec_response()->__isset.profile) {
+    for (const TEventSequence& catalog_timeline :
+        catalog_op_executor_->ddl_exec_response()->profile.event_sequences) {
+      summary_profile_->AddEventSequence(catalog_timeline.name, catalog_timeline);
+    }
+  }
+
   // If this is a CTAS request, there will usually be more work to do
   // after executing the CREATE TABLE statement (the INSERT portion of the operation).
   // The exception is if the user specified IF NOT EXISTS and the table already
@@ -715,7 +734,7 @@ void ClientRequestState::ExecDdlRequestImpl(bool exec_in_worker_thread) {
   // Add newly created table to catalog cache.
   status = parent_server_->ProcessCatalogUpdateResult(
       *catalog_op_executor_->update_catalog_result(),
-      exec_request_->query_options.sync_ddl);
+      exec_request_->query_options.sync_ddl, query_options(), query_events_);
   {
     lock_guard<mutex> l(lock_);
     RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
@@ -789,6 +808,9 @@ void ClientRequestState::ExecLoadDataRequestImpl(bool exec_in_worker_thread) {
 
   TLoadDataResp response;
   Status status = frontend_->LoadData(exec_request_->load_data_request, &response);
+  if (exec_request_->load_data_request.iceberg_tbl) {
+    ExecLoadIcebergDataRequestImpl(response);
+  }
   {
     lock_guard<mutex> l(lock_);
     RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
@@ -812,10 +834,8 @@ void ClientRequestState::ExecLoadDataRequestImpl(bool exec_in_worker_thread) {
   catalog_update.db_name = exec_request_->load_data_request.table_name.db_name;
   catalog_update.is_overwrite = exec_request_->load_data_request.overwrite;
 
-  const TNetworkAddress& address =
-      MakeNetworkAddress(FLAGS_catalog_service_host, FLAGS_catalog_service_port);
-  CatalogServiceConnection client(
-      ExecEnv::GetInstance()->catalogd_client_cache(), address, &status);
+  CatalogServiceConnection client(ExecEnv::GetInstance()->catalogd_client_cache(),
+      *ExecEnv::GetInstance()->GetCatalogdAddress().get(), &status);
   {
     lock_guard<mutex> l(lock_);
     RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
@@ -831,13 +851,83 @@ void ClientRequestState::ExecLoadDataRequestImpl(bool exec_in_worker_thread) {
 
   status = parent_server_->ProcessCatalogUpdateResult(
       resp.result,
-      exec_request_->query_options.sync_ddl);
+      exec_request_->query_options.sync_ddl, query_options(), query_events_);
   {
     lock_guard<mutex> l(lock_);
     RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
   }
 }
 
+void ClientRequestState::ExecLoadIcebergDataRequestImpl(TLoadDataResp response) {
+  TLoadDataReq load_data_req = exec_request_->load_data_request;
+  RuntimeProfile* child_profile =
+      RuntimeProfile::Create(&profile_pool_, "Child Queries");
+  profile_->AddChild(child_profile);
+  // Add child queries for computing table and column stats.
+  vector<ChildQuery> child_queries;
+  // Prepare CREATE
+  RuntimeProfile* create_profile =
+      RuntimeProfile::Create(&profile_pool_, "Create table query");
+  child_profile->AddChild(create_profile);
+  child_queries.emplace_back(response.create_tmp_tbl_query, this, parent_server_,
+      create_profile, &profile_pool_);
+  // Prepare INSERT
+  RuntimeProfile* insert_profile =
+      RuntimeProfile::Create(&profile_pool_, "Insert query");
+  child_profile->AddChild(insert_profile);
+  child_queries.emplace_back(load_data_req.insert_into_dst_tbl_query, this,
+      parent_server_, insert_profile, &profile_pool_);
+  // Prepare DROP
+  RuntimeProfile* drop_profile =
+      RuntimeProfile::Create(&profile_pool_, "Drop table query");
+  child_profile->AddChild(drop_profile);
+  child_queries.emplace_back(load_data_req.drop_tmp_tbl_query, this,
+      parent_server_, drop_profile, &profile_pool_);
+  // Execute queries
+  RETURN_VOID_IF_ERROR(child_query_executor_->ExecAsync(move(child_queries)));
+  vector<ChildQuery*>* completed_queries = new vector<ChildQuery*>();
+  Status query_status = child_query_executor_->WaitForAll(completed_queries);
+  if (query_status.ok()) {
+    const char* path = response.create_location.c_str();
+    string delete_err = "Load was succesful, but failed to remove staging data under '"
+        + response.create_location + "', HDFS error: ";
+    hdfsFS hdfs_conn;
+    Status hdfs_ret = HdfsFsCache::instance()->GetConnection(path, &hdfs_conn);
+    if (!hdfs_ret.ok()) {
+      lock_guard<mutex> l(lock_);
+      RETURN_VOID_IF_ERROR(UpdateQueryStatus(Status(delete_err + hdfs_ret.GetDetail())));
+    }
+    if (hdfsDelete(hdfs_conn, path, 1)) {
+      lock_guard<mutex> l(lock_);
+      RETURN_VOID_IF_ERROR(UpdateQueryStatus(Status(delete_err + strerror(errno))));
+    }
+  } else {
+    const char* dst_path = load_data_req.source_path.c_str();
+    hdfsFS hdfs_dst_conn;
+    string revert_err = "Failed to load data and failed to revert data movement, "
+        "please check source and staging directory under '" + response.create_location
+        + "', Query error: " + query_status.GetDetail() + " HDFS error: ";
+    Status hdfs_ret = HdfsFsCache::instance()->GetConnection(dst_path, &hdfs_dst_conn);
+    if (!hdfs_ret.ok()) {
+      lock_guard<mutex> l(lock_);
+      RETURN_VOID_IF_ERROR(UpdateQueryStatus(Status(revert_err + hdfs_ret.GetDetail())));
+    }
+    for (const string& src_path : response.loaded_files) {
+      hdfsFS hdfs_src_conn;
+      hdfs_ret = HdfsFsCache::instance()->GetConnection(dst_path, &hdfs_src_conn);
+      if (!hdfs_ret.ok()) {
+        lock_guard<mutex> l(lock_);
+        RETURN_VOID_IF_ERROR(UpdateQueryStatus(Status(revert_err
+            + hdfs_ret.GetDetail())));
+      }
+      if (hdfsMove(hdfs_src_conn, src_path.c_str(), hdfs_dst_conn, dst_path)) {
+        lock_guard<mutex> l(lock_);
+        RETURN_VOID_IF_ERROR(UpdateQueryStatus(Status(revert_err + strerror(errno))));
+      }
+    }
+  }
+}
+
 
 Status ClientRequestState::ExecLoadDataRequest() {
   if (exec_request_->query_options.enable_async_load_data_execution) {
@@ -902,6 +992,7 @@ Status ClientRequestState::ExecShutdownRequest() {
     }
   }
   string krpc_error = "RemoteShutdown() RPC failed: Network error";
+  string krpc_error2 = "RemoteShutdown() RPC failed: Timed out";
   NetworkAddressPB krpc_addr = MakeNetworkAddressPB(ip_address, port, backend_id,
       ExecEnv::GetInstance()->rpc_mgr()->GetUdsAddressUniqueId());
   std::unique_ptr<ControlServiceProxy> proxy;
@@ -933,7 +1024,9 @@ Status ClientRequestState::ExecShutdownRequest() {
     string err_string = Substitute(
         "Rpc to $0 failed with error '$1'", NetworkAddressPBToString(krpc_addr), msg);
     // Attempt to detect if the the failure is because of not using a KRPC port.
-    if (backend_port_specified && msg.find(krpc_error) != string::npos) {
+    if (backend_port_specified &&
+           (msg.find(krpc_error) != string::npos ||
+            msg.find(krpc_error2) != string::npos)) {
       // Prior to IMPALA-7985 :shutdown() used the backend port.
       err_string.append(" This may be because the port specified is wrong. You may have"
                         " specified the backend (thrift) port which :shutdown() can no"
@@ -949,6 +1042,7 @@ Status ClientRequestState::ExecShutdownRequest() {
 }
 
 Status ClientRequestState::Finalize(bool check_inflight, const Status* cause) {
+  UnRegisterCompletedRPCs();
   RETURN_IF_ERROR(Cancel(check_inflight, cause, /*wait_until_finalized=*/true));
   MarkActive();
   // Make sure we join on wait_thread_ before we finish (and especially before this object
@@ -1002,6 +1096,7 @@ Status ClientRequestState::Finalize(bool check_inflight, const Status* cause) {
 
   // Update the timeline here so that all of the above work is captured in the timeline.
   query_events_->MarkEvent("Unregister query");
+  UnRegisterRemainingRPCs();
   return Status::OK();
 }
 
@@ -1057,6 +1152,7 @@ void ClientRequestState::Wait() {
       query_events()->MarkEvent("Rows available");
     } else {
       query_events()->MarkEvent("Request finished");
+      UpdateEndTime();
     }
     discard_result(UpdateQueryStatus(status));
   }
@@ -1436,6 +1532,9 @@ Status ClientRequestState::UpdateCatalog() {
     TUpdateCatalogRequest catalog_update;
     catalog_update.__set_sync_ddl(exec_request_->query_options.sync_ddl);
     catalog_update.__set_header(GetCatalogServiceRequestHeader());
+    if (exec_request_->query_options.__isset.debug_action) {
+      catalog_update.__set_debug_action(exec_request_->query_options.debug_action);
+    }
     DmlExecState* dml_exec_state = GetCoordinator()->dml_exec_state();
     if (!dml_exec_state->PrepareCatalogUpdate(&catalog_update)) {
       VLOG_QUERY << "No partitions altered, not updating metastore (query id: "
@@ -1460,21 +1559,28 @@ Status ClientRequestState::UpdateCatalog() {
         catalog_update.__set_transaction_id(finalize_params.transaction_id);
         catalog_update.__set_write_id(finalize_params.write_id);
       }
-      if (finalize_params.__isset.spec_id) {
+      if (finalize_params.__isset.iceberg_params) {
+        const TIcebergDmlFinalizeParams& ice_finalize_params =
+            finalize_params.iceberg_params;
+        TIcebergOperationParam& cat_ice_op = catalog_update.iceberg_operation;
         catalog_update.__isset.iceberg_operation = true;
-        TIcebergOperationParam& ice_op = catalog_update.iceberg_operation;
-        ice_op.__set_spec_id(finalize_params.spec_id);
-        ice_op.__set_iceberg_data_files_fb(
-            dml_exec_state->CreateIcebergDataFilesVector());
-        ice_op.__set_is_overwrite(finalize_params.is_overwrite);
-        ice_op.__set_initial_snapshot_id(finalize_params.initial_snapshot_id);
+        cat_ice_op.__set_operation(ice_finalize_params.operation);
+        cat_ice_op.__set_initial_snapshot_id(
+            ice_finalize_params.initial_snapshot_id);
+        cat_ice_op.__set_spec_id(ice_finalize_params.spec_id);
+        if (ice_finalize_params.operation == TIcebergOperation::INSERT) {
+          cat_ice_op.__set_iceberg_data_files_fb(
+              dml_exec_state->CreateIcebergDataFilesVector());
+          cat_ice_op.__set_is_overwrite(finalize_params.is_overwrite);
+        } else if (ice_finalize_params.operation == TIcebergOperation::DELETE) {
+          cat_ice_op.__set_iceberg_delete_files_fb(
+              dml_exec_state->CreateIcebergDataFilesVector());
+        }
       }
 
       Status cnxn_status;
-      const TNetworkAddress& address =
-          MakeNetworkAddress(FLAGS_catalog_service_host, FLAGS_catalog_service_port);
-      CatalogServiceConnection client(
-          ExecEnv::GetInstance()->catalogd_client_cache(), address, &cnxn_status);
+      CatalogServiceConnection client(ExecEnv::GetInstance()->catalogd_client_cache(),
+          *ExecEnv::GetInstance()->GetCatalogdAddress().get(), &cnxn_status);
       RETURN_IF_ERROR(cnxn_status);
 
       VLOG_QUERY << "Executing FinalizeDml() using CatalogService";
@@ -1500,7 +1606,7 @@ Status ClientRequestState::UpdateCatalog() {
         query_events_->MarkEvent("Transaction committed");
       }
       RETURN_IF_ERROR(parent_server_->ProcessCatalogUpdateResult(resp.result,
-          exec_request_->query_options.sync_ddl));
+          exec_request_->query_options.sync_ddl, query_options(), query_events_));
     }
   } else if (InKuduTransaction()) {
     // Commit the Kudu transaction. Clear transaction state if it's successful.
@@ -1609,7 +1715,24 @@ void ClientRequestState::MarkInactive() {
 void ClientRequestState::MarkActive() {
   client_wait_sw_.Stop();
   int64_t elapsed_time = client_wait_sw_.ElapsedTime();
-  client_wait_timer_->Set(elapsed_time);
+  // If we have reached eos, then the query is already complete,
+  // and we should not accumulate more client wait time. This mostly
+  // impacts the finalization step, where the client is closing the
+  // query and does not need any more rows. Fetching may have already
+  // completed prior to this point, so finalization time should not
+  // count in that case. If the fetch was incomplete, then the client
+  // time should be counted for finalization as well.
+  if (!eos()) {
+    client_wait_timer_->Set(elapsed_time);
+    // The first call is before any MarkInactive() call has run and produces
+    // a zero-length sample. Skip this zero-length sample (but not any later
+    // zero-length samples).
+    if (elapsed_time != 0 || last_client_wait_time_ != 0) {
+      int64_t current_wait_time = elapsed_time - last_client_wait_time_;
+      client_wait_time_stats_->UpdateCounter(current_wait_time);
+    }
+    last_client_wait_time_ = elapsed_time;
+  }
   lock_guard<mutex> l(expiration_data_lock_);
   last_active_time_ms_ = UnixMillis();
   ++ref_count_;
@@ -1643,7 +1766,7 @@ Status ClientRequestState::UpdateTableAndColumnStats(
   }
   RETURN_IF_ERROR(parent_server_->ProcessCatalogUpdateResult(
       *catalog_op_executor_->update_catalog_result(),
-      exec_request_->query_options.sync_ddl));
+      exec_request_->query_options.sync_ddl, query_options(), query_events_));
 
   // Set the results to be reported to the client.
   SetResultSet(catalog_op_executor_->ddl_exec_response());
@@ -1763,6 +1886,9 @@ void ClientRequestState::UpdateEndTime() {
     // of nanosecond precision, so we explicitly specify the precision here.
     summary_profile_->AddInfoString(
         "End Time", ToStringFromUnixMicros(end_time_us(), TimePrecision::Nanosecond));
+    int64_t duration = end_time_us() - start_time_us();
+    summary_profile_->AddInfoString("Duration", Substitute("$0 ($1 us)",
+        PrettyPrinter::Print(duration, TUnit::TIME_US), duration));
   }
 }
 
@@ -2050,6 +2176,171 @@ TCatalogServiceRequestHeader ClientRequestState::GetCatalogServiceRequestHeader(
   header.__set_redacted_sql_stmt(
       query_ctx_.client_request.__isset.redacted_stmt ?
           query_ctx_.client_request.redacted_stmt : query_ctx_.client_request.stmt);
+  header.__set_query_id(query_ctx_.query_id);
   return header;
 }
+
+void ClientRequestState::RegisterRPC() {
+  RpcEventHandler::InvocationContext* rpc_context =
+      RpcEventHandler::GetThreadRPCContext();
+  // The existence of rpc_context means that this is called from an RPC
+  if (rpc_context) {
+    lock_guard<mutex> l(lock_);
+    if (track_rpcs_ && pending_rpcs_.find(rpc_context) == pending_rpcs_.end()) {
+      rpc_context->Register();
+      pending_rpcs_.insert(rpc_context);
+      rpc_count_->Add(1);
+    }
+  }
+}
+
+void ClientRequestState::UnRegisterCompletedRPCs() {
+  lock_guard<mutex> l(lock_);
+  for (auto iter = pending_rpcs_.begin(); iter != pending_rpcs_.end();) {
+    RpcEventHandler::InvocationContext* rpc_context = *iter;
+    uint64_t read_ns = 0, write_ns = 0;
+    if (rpc_context->UnRegisterCompleted(read_ns, write_ns)) {
+      rpc_read_timer_->Add(read_ns);
+      rpc_write_timer_->Add(write_ns);
+      iter = pending_rpcs_.erase(iter);
+    } else {
+      ++iter;
+    }
+  }
+}
+
+void ClientRequestState::UnRegisterRemainingRPCs() {
+  lock_guard<mutex> l(lock_);
+  for (auto rpc_context: pending_rpcs_) {
+    rpc_context->UnRegister();
+  }
+  track_rpcs_ = false;
+  pending_rpcs_.clear();
+}
+
+void ClientRequestState::CopyRPCs(ClientRequestState& from_request) {
+  lock_guard<mutex> l_to(lock_);
+  lock_guard<mutex> l_from(from_request.lock_);
+  rpc_read_timer_->Add(from_request.rpc_read_timer_->value());
+  rpc_write_timer_->Add(from_request.rpc_write_timer_->value());
+  rpc_count_->Add(from_request.rpc_count_->value());
+  for (auto rpc_context: from_request.pending_rpcs_) {
+    rpc_context->Register();
+    pending_rpcs_.insert(rpc_context);
+  }
+}
+
+Status ClientRequestState::ExecMigrateRequest() {
+  ExecMigrateRequestImpl();
+  SetResultSet({"Table has been migrated."});
+  return query_status_;
+}
+
+void ClientRequestState::ExecMigrateRequestImpl() {
+  // A convert table request holds the query strings for the sub-queries. These are
+  // populated by ConvertTableToIcebergStmt in the Frontend during analysis.
+  TConvertTableRequest& params = exec_request_->convert_table_request;
+  {
+    RuntimeProfile* child_profile =
+        RuntimeProfile::Create(&profile_pool_, "Child Queries 1");
+    profile_->AddChild(child_profile);
+    vector<ChildQuery> child_queries;
+
+    // Prepare: SET some table properties for the original table.
+    RuntimeProfile* set_hdfs_table_profile = RuntimeProfile::Create(
+        &profile_pool_, "Set properties for HDFS table query");
+    child_profile->AddChild(set_hdfs_table_profile);
+    child_queries.emplace_back(params.set_hdfs_table_properties_query, this,
+        parent_server_, set_hdfs_table_profile, &profile_pool_);
+
+    // Prepare: RENAME the HDFS table to a temporary HDFS table.
+    RuntimeProfile* rename_hdfs_table_profile = RuntimeProfile::Create(
+        &profile_pool_, "Rename HDFS table query");
+    child_profile->AddChild(rename_hdfs_table_profile);
+    child_queries.emplace_back(params.rename_hdfs_table_to_temporary_query,
+        this, parent_server_, rename_hdfs_table_profile, &profile_pool_);
+
+    // Prepare: REFRESH the temporary HDFS table.
+    RuntimeProfile* refresh_hdfs_table_profile = RuntimeProfile::Create(
+        &profile_pool_, "Refresh temporary HDFS table query");
+    child_profile->AddChild(refresh_hdfs_table_profile);
+    child_queries.emplace_back(params.refresh_temporary_hdfs_table_query, this,
+        parent_server_, refresh_hdfs_table_profile, &profile_pool_);
+
+    // Execute child queries
+    unique_ptr<ChildQueryExecutor> query_executor(new ChildQueryExecutor());
+    RETURN_VOID_IF_ERROR(query_executor->ExecAsync(move(child_queries)));
+    vector<ChildQuery*>* completed_queries = new vector<ChildQuery*>();
+    Status query_status = query_executor->WaitForAll(completed_queries);
+    if (!query_status.ok()) AddTableResetHints(params, &query_status);
+    {
+      lock_guard<mutex> l(lock_);
+      RETURN_VOID_IF_ERROR(UpdateQueryStatus(query_status));
+    }
+  }
+  // Create an external Iceberg table using the data of the HDFS table.
+  Status status = frontend_->Convert(*exec_request_);
+  if (!status.ok()) AddTableResetHints(params, &status);
+  {
+    lock_guard<mutex> l(lock_);
+    RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
+  }
+  {
+    RuntimeProfile* child_profile =
+        RuntimeProfile::Create(&profile_pool_, "Child Queries 2");
+    profile_->AddChild(child_profile);
+    vector<ChildQuery> child_queries;
+
+    if (params.__isset.create_iceberg_table_query) {
+      // Prepare: CREATE the Iceberg table that inherits HDFS table location.
+      RuntimeProfile* create_iceberg_table_profile = RuntimeProfile::Create(
+          &profile_pool_, "Create Iceberg table query");
+      child_profile->AddChild(create_iceberg_table_profile);
+      child_queries.emplace_back(params.create_iceberg_table_query, this,
+          parent_server_, create_iceberg_table_profile, &profile_pool_);
+    } else {
+      // Prepare: Invalidate metadata for tables in Hive catalog to guarantee that it is
+      // propagated instantly to all coordinators.
+      RuntimeProfile* invalidate_metadata_profile = RuntimeProfile::Create(
+          &profile_pool_, "Invalidate metadata Iceberg table query");
+      child_profile->AddChild(invalidate_metadata_profile);
+      child_queries.emplace_back(params.invalidate_metadata_query, this,
+          parent_server_, invalidate_metadata_profile, &profile_pool_);
+    }
+
+    if (params.__isset.post_create_alter_table_query) {
+      // Prepare: ALTER TABLE query after creating the Iceberg table.
+      RuntimeProfile* post_create_alter_table_profile = RuntimeProfile::Create(
+          &profile_pool_, "ALTER TABLE after create Iceberg table query");
+      child_profile->AddChild(post_create_alter_table_profile);
+      child_queries.emplace_back(params.post_create_alter_table_query, this,
+          parent_server_, post_create_alter_table_profile, &profile_pool_);
+    }
+
+    // Prepare: DROP the temporary HDFS table.
+    RuntimeProfile* drop_tmp_hdfs_table_profile = RuntimeProfile::Create(
+        &profile_pool_, "Drop temporary HDFS table query");
+    child_profile->AddChild(drop_tmp_hdfs_table_profile);
+    child_queries.emplace_back(params.drop_temporary_hdfs_table_query, this,
+        parent_server_, drop_tmp_hdfs_table_profile, &profile_pool_);
+
+    // Execute queries
+    unique_ptr<ChildQueryExecutor> query_executor(new ChildQueryExecutor());
+    RETURN_VOID_IF_ERROR(query_executor->ExecAsync(move(child_queries)));
+    vector<ChildQuery*>* completed_queries = new vector<ChildQuery*>();
+    Status query_status = query_executor->WaitForAll(completed_queries);
+    {
+      lock_guard<mutex> l(lock_);
+      RETURN_VOID_IF_ERROR(UpdateQueryStatus(query_status));
+    }
+  }
+}
+
+void ClientRequestState::AddTableResetHints(const TConvertTableRequest& params,
+      Status* status) const {
+  string table_reset_hint("Your table might have been renamed. To reset the name "
+      "try running:\n" + params.reset_table_name_query + ";");
+  status->MergeStatus(Status(table_reset_hint));
+}
+
 }
