@@ -21,9 +21,11 @@
 #include "common/object-pool.h"
 #include "common/status.h"
 #include "exec/catalog-op-executor.h"
+#include "rpc/rpc-trace.h"
 #include "service/child-query.h"
 #include "service/impala-server.h"
 #include "service/query-result-set.h"
+#include "runtime/hdfs-fs-cache.h"
 #include "util/condition-variable.h"
 #include "util/runtime-profile.h"
 #include "gen-cpp/Frontend_types.h"
@@ -88,7 +90,7 @@ class ClientRequestState {
   /// Sets the profile that is produced by the frontend. The frontend creates the
   /// profile during planning and returns it to the backend via TExecRequest,
   /// which then sets the frontend profile.
-  void SetFrontendProfile(TRuntimeProfileNode profile);
+  void SetFrontendProfile(const TExecRequest& exec_request);
 
   /// Sets the coordinator time that the plan request was submitted at so that
   /// the backend timeline starts where the frontend timeline ends
@@ -250,7 +252,14 @@ class ClientRequestState {
   /// control.
   /// Admission control resource pool associated with this query.
   std::string request_pool() const {
-    return query_ctx_.__isset.request_pool ? query_ctx_.request_pool : "";
+    if (is_planning_done_.load() && exec_request_ != nullptr
+        && exec_request_->query_exec_request.query_ctx.__isset.request_pool) {
+      // If the request pool has been set by Planner, return the request pool selected
+      // by Planner.
+      return exec_request_->query_exec_request.query_ctx.request_pool;
+    } else {
+      return query_ctx_.__isset.request_pool ? query_ctx_.request_pool : "";
+    }
   }
 
   int num_rows_fetched() const { return num_rows_fetched_; }
@@ -344,6 +353,18 @@ class ClientRequestState {
   /// Returns the max size of the result_cache_ in number of rows.
   int64_t result_cache_max_size() const { return result_cache_max_size_; }
 
+  /// Returns duration of the query is waiting in the queue.
+  int64_t wait_time_ms() const {
+    // wait_start_time_ms_ is zero iff the query is not require queuing.
+    if (wait_start_time_ms_ == 0) return 0;
+    // wait_end_time_ms_ might still be zero if the query is not yet dequeue.
+    // Use the current Unix time in that case. Note that the duration can be
+    // negative if a system clock reset happened after the query was initiated.
+    int64_t wait_end_time_ms = wait_end_time_ms_ > 0LL ?
+        wait_end_time_ms_ : MonotonicMillis();
+    return wait_end_time_ms - wait_start_time_ms_;
+  }
+
   /// Sets the RetryState to RETRYING. Updates the runtime profile with the retry status
   /// and cause. Must be called while 'lock_' is held. Sets the query_status_. Future
   /// calls to UpdateQueryStatus will not have any effect. This is necessary to prevent
@@ -425,6 +446,22 @@ class ClientRequestState {
   void SetBlacklistedExecutorAddresses(
       std::unordered_set<NetworkAddressPB>& executor_addresses);
 
+  /// Mark planning as done for this request.
+  /// This function should be called after QueryDriver::RunFrontendPlanner() is
+  /// returned without error.
+  void SetPlanningDone() { is_planning_done_.store(true); }
+
+  // Register an RPC context with the query so that RPC metrics can be
+  // accumulated at the query level and shown in the query profile
+  void RegisterRPC();
+  // Unregister any RPCs that have been released by their RPC thread and
+  // accumulate their metrics into the profile timers
+  void UnRegisterCompletedRPCs();
+  // Unregister any remaining RPCs without recording timing in this query.
+  // RPC threads will usually have completed by this point.
+  void UnRegisterRemainingRPCs();
+  // Copy pending RPCs for a retried query
+  void CopyRPCs(ClientRequestState& from_request);
  protected:
   /// Updates the end_time_us_ of this query if it isn't set. The end time is determined
   /// when this function is called for the first time, calling it multiple times does not
@@ -463,6 +500,9 @@ class ClientRequestState {
   /// True if there was a transaction and it got committed or aborted.
   bool transaction_closed_ = false;
 
+  /// Indicates whether the planning is done for the request.
+  std::atomic_bool is_planning_done_{false};
+
   /// Executor for any child queries (e.g. compute stats subqueries). Always non-NULL.
   const boost::scoped_ptr<ChildQueryExecutor> child_query_executor_;
 
@@ -573,9 +613,25 @@ class ClientRequestState {
   RuntimeProfile::Counter* row_materialization_rate_;
 
   /// Tracks how long we are idle waiting for a client to fetch rows.
+  /// Keep summary statistics to get a sense for the number of fetch calls and
+  /// the typical round-trip time.
   RuntimeProfile::Counter* client_wait_timer_;
+  RuntimeProfile::SummaryStatsCounter* client_wait_time_stats_;
   /// Timer to track idle time for the above counter.
   MonotonicStopWatch client_wait_sw_;
+  int64_t last_client_wait_time_ = 0;
+
+  // Tracks time spent by client calls reading RPC arguments
+  RuntimeProfile::Counter* rpc_read_timer_;
+  // Tracks time spent by client calls writing RPC results
+  RuntimeProfile::Counter* rpc_write_timer_;
+  // Tracks number of client RPCs
+  RuntimeProfile::Counter* rpc_count_;
+  // Contexts for RPC calls that have not completed and had their stats collected
+  std::set<RpcEventHandler::InvocationContext*> pending_rpcs_;
+  // Enables tracking of RPCs in pending_rpcs_. Also used to turn off tracking once
+  // Finalize() has been called.
+  bool track_rpcs_ = true;
 
   RuntimeProfile::EventSequence* query_events_;
 
@@ -640,6 +696,11 @@ class ClientRequestState {
   /// the query option FETCH_ROWS_TIMEOUT_MS.
   const int64_t fetch_rows_timeout_us_;
 
+  /// Start/end time of the query queuing, in Unix milliseconds.
+  /// Initialized to 0, indicating that queries do not need to be queued.
+  int64_t wait_start_time_ms_ = 0;
+  int64_t wait_end_time_ms_ = 0;
+
   /// If this ClientRequestState was created as a retry of a previously failed query, the
   /// original_id_ is set to the query id of the original query that failed. The
   /// "original" query is the query that was submitted by the user that failed and had to
@@ -716,6 +777,9 @@ class ClientRequestState {
   /// Core logic of executing a load data statement.
   void ExecLoadDataRequestImpl(bool exec_in_worker_thread);
 
+  /// Executes LOAD DATA related sub-queries for Iceberg tables.
+  void ExecLoadIcebergDataRequestImpl(TLoadDataResp response);
+
   /// Executes a shut down request.
   Status ExecShutdownRequest() WARN_UNUSED_RESULT;
 
@@ -812,6 +876,16 @@ class ClientRequestState {
 
   /// Helper function to get common header
   TCatalogServiceRequestHeader GetCatalogServiceRequestHeader();
-};
 
+  /// The logic of executing a MIGRATE TABLE statement.
+  Status ExecMigrateRequest() WARN_UNUSED_RESULT;
+
+  /// Core logic of executing a MIGRATE TABLE statement.
+  void ExecMigrateRequestImpl();
+
+  /// Used when running into an error during table migration to extend 'status' with some
+  /// hints about how to reset the original table name. 'params' holds the SQL query
+  /// string the user should run.
+  void AddTableResetHints(const TConvertTableRequest& params, Status* status) const;
+};
 }
