@@ -78,7 +78,8 @@ Java_org_apache_impala_service_FeSupport_NativeFeInit(
   // exceptions to the FE.
   InitCommonRuntime(1, &name, true,
       external_fe ? TestInfo::NON_TEST : TestInfo::FE_TEST, external_fe);
-  THROW_IF_ERROR(LlvmCodeGen::InitializeLlvm(true), env, JniUtil::internal_exc_class());
+  THROW_IF_ERROR(
+      LlvmCodeGen::InitializeLlvm(name, true), env, JniUtil::internal_exc_class());
   ExecEnv* exec_env = new ExecEnv(external_fe); // This also caches it from the process.
   THROW_IF_ERROR(exec_env->InitForFeSupport(), env, JniUtil::internal_exc_class());
 }
@@ -180,7 +181,6 @@ Java_org_apache_impala_service_FeSupport_NativeEvalExprsWithoutRow(
   jbyteArray result_bytes = NULL;
   TQueryCtx query_ctx;
   TExprBatch expr_batch;
-  JniLocalFrame jni_frame;
   TResultRow expr_results;
   vector<TColumnValue> results;
   ObjectPool obj_pool;
@@ -211,9 +211,6 @@ Java_org_apache_impala_service_FeSupport_NativeEvalExprsWithoutRow(
     state.ReleaseResources();
   });
 
-  THROW_IF_ERROR_RET(
-      jni_frame.push(env), env, JniUtil::internal_exc_class(), result_bytes);
-
   MemPool expr_mem_pool(state.query_mem_tracker());
 
   // Prepare() the exprs. Always Close() the exprs even in case of errors.
@@ -450,6 +447,7 @@ Java_org_apache_impala_service_FeSupport_NativeLookupSymbol(
       JniUtil::internal_exc_class(), nullptr);
 
   vector<ColumnType> arg_types;
+  arg_types.reserve(lookup.arg_types.size());
   for (int i = 0; i < lookup.arg_types.size(); ++i) {
     arg_types.push_back(ColumnType::FromThrift(lookup.arg_types[i]));
   }
@@ -654,6 +652,14 @@ Java_org_apache_impala_service_FeSupport_MinLogSpaceForBloomFilter(
   return BloomFilter::MinLogSpace(ndv, fpp);
 }
 
+/// Returns the expected false positive rate for the given ndv and log_bufferpool_space.
+extern "C"
+JNIEXPORT jdouble JNICALL
+Java_org_apache_impala_service_FeSupport_FalsePositiveProbForBloomFilter(
+    JNIEnv* env, jclass fe_support_class, jlong ndv, jint log_bufferpool_space) {
+  return BloomFilter::FalsePositiveProb(ndv, log_bufferpool_space);
+}
+
 extern "C"
 JNIEXPORT jbyteArray JNICALL
 Java_org_apache_impala_service_FeSupport_nativeParseDateString(JNIEnv* env,
@@ -755,6 +761,10 @@ static JNINativeMethod native_methods[] = {
     const_cast<char*>("MinLogSpaceForBloomFilter"), const_cast<char*>("(JD)I"),
     (void*)::Java_org_apache_impala_service_FeSupport_MinLogSpaceForBloomFilter
   },
+  {
+    const_cast<char*>("FalsePositiveProbForBloomFilter"), const_cast<char*>("(JI)D"),
+    (void*)::Java_org_apache_impala_service_FeSupport_FalsePositiveProbForBloomFilter
+  },
   {
     const_cast<char*>("nativeParseDateString"),
     const_cast<char*>("(Ljava/lang/String;)[B"),
