@@ -103,6 +103,10 @@ namespace impala {
 
 const string IMPALA_RESULT_CACHING_OPT = "impala.resultset.cache.size";
 
+const string SET_HIVECONF_PREFIX = "set:hiveconf:";
+
+const int SET_HIVECONF_PREFIX_LEN = SET_HIVECONF_PREFIX.length();
+
 void ImpalaServer::ExecuteMetadataOp(const THandleIdentifier& session_handle,
     TMetadataOpRequest* request, TOperationHandle* handle, thrift::TStatus* status) {
   TUniqueId session_id;
@@ -218,8 +222,16 @@ Status ImpalaServer::FetchInternal(TUniqueId query_id, SessionState* session,
   bool is_child_query = query_handle->parent_query_id() != TUniqueId();
   TProtocolVersion::type version = is_child_query ?
       TProtocolVersion::HIVE_CLI_SERVICE_PROTOCOL_V1 : session->hs2_version;
+
+  // In the first fetch, expect 0 results to avoid reserving unnecessarily large result
+  // vectors for small queries. If there are more fetches (so there are more rows than
+  // num_rows_fetched), then expect subsequent fetches to be fully filled.
+  int expected_result_count = query_handle->num_rows_fetched() == 0 ? 0
+      : fetch_size;
+
   scoped_ptr<QueryResultSet> result_set(QueryResultSet::CreateHS2ResultSet(
-      version, *(query_handle->result_metadata()), &(fetch_results->results)));
+      version, *(query_handle->result_metadata()), &(fetch_results->results),
+      query_handle->query_options().stringify_map_keys, expected_result_count));
   RETURN_IF_ERROR(
       query_handle->FetchRows(fetch_size, result_set.get(), block_on_wait_time_us));
   *num_results = result_set->size();
@@ -354,17 +366,24 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
         // If the current user is a valid proxy user, he/she can optionally perform
         // authorization requests on behalf of another user. This is done by setting
         // the 'impala.doas.user' Hive Server 2 configuration property.
+        // Note: The 'impala.doas.user' configuration overrides the user specified
+        // in the 'doAs' request parameter, which can be specified for hs2-http transport.
         state->do_as_user = v.second;
         Status status = AuthorizeProxyUser(state->connected_user, state->do_as_user);
         HS2_RETURN_IF_ERROR(return_val, status, SQLSTATE_GENERAL_ERROR);
       } else if (iequals(v.first, "use:database")) {
         state->database = v.second;
       } else {
+        string key = v.first, value = v.second;
+        // Hive JDBC will add a prefix to the configuration, see IMPALA-11992
+        if (key.rfind(SET_HIVECONF_PREFIX, 0) != string::npos) {
+          key = key.substr(SET_HIVECONF_PREFIX_LEN);
+        }
         // Normal configuration key. Use it to set session default query options.
         // Ignore failure (failures will be logged in SetQueryOption()).
-        Status status = SetQueryOption(v.first, v.second, &state->set_query_options,
+        Status status = SetQueryOption(key, value, &state->set_query_options,
             &state->set_query_options_mask);
-        if (status.ok() && iequals(v.first, "idle_session_timeout")) {
+        if (status.ok() && iequals(key, "idle_session_timeout")) {
           state->session_timeout = state->set_query_options.idle_session_timeout;
           VLOG_QUERY << "OpenSession(): session: " << PrintId(session_id)
                      << " idle_session_timeout="
@@ -377,9 +396,27 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
   // If the connected user is an authorized proxy user, we were not able to check the LDAP
   // filters when the connection was created because we didn't know what the effective
   // user would be yet, so check now.
+  // When no 'doAs' user is specified and Kerberos authentication is enabled, then the
+  // connected user is a Kerberos user principal and that will certainly not pass the
+  // LDAP checks, so in this case the LDAP filters will be checked with the short username
+  // derived from the Kerberos user principal.
   if (FLAGS_enable_ldap_auth && IsAuthorizedProxyUser(state->connected_user)) {
+    string username_for_ldap_filters = GetEffectiveUser(*state);
+    if (state->do_as_user.empty() &&
+        !connection_context->kerberos_user_principal.empty()) {
+
+      string short_username = GetShortUsernameFromKerberosPrincipal(
+        connection_context->kerberos_user_principal);
+      VLOG(1) << "No doAs user is specified and Kerberos authentication is enabled, "
+              << "the authenticated user principal is \""
+              << connection_context->kerberos_user_principal << "\". "
+              << "LDAP filters will be checked using the short username \""
+              << short_username << "\", which is extracted from the user principal.";
+      username_for_ldap_filters = short_username;
+    }
+
     bool success =
-        AuthManager::GetInstance()->GetLdap()->LdapCheckFilters(GetEffectiveUser(*state));
+      AuthManager::GetInstance()->GetLdap()->LdapCheckFilters(username_for_ldap_filters);
     if (!success) {
       HS2_RETURN_ERROR(return_val, "User is not authorized.", SQLSTATE_GENERAL_ERROR);
     }
@@ -412,7 +449,9 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
   return_val.status.__set_statusCode(thrift::TStatusCode::SUCCESS_STATUS);
   return_val.serverProtocolVersion = state->hs2_version;
   VLOG_QUERY << "Opened session: " << PrintId(session_id)
-             << " effective username: " << GetEffectiveUser(*state);
+             << ", effective username: " << GetEffectiveUser(*state)
+             << ", client address: "
+             << "<" << TNetworkAddressToString(state->network_address) << ">.";
 }
 
 void ImpalaServer::CloseSession(TCloseSessionResp& return_val,
@@ -541,7 +580,8 @@ Status ImpalaServer::SetupResultsCacheing(const QueryHandle& query_handle,
   if (cache_num_rows > 0) {
     const TResultSetMetadata* result_set_md = query_handle->result_metadata();
     QueryResultSet* result_set =
-        QueryResultSet::CreateHS2ResultSet(session->hs2_version, *result_set_md, nullptr);
+        QueryResultSet::CreateHS2ResultSet(session->hs2_version, *result_set_md, nullptr,
+            query_handle->query_options().stringify_map_keys, 0);
     RETURN_IF_ERROR(query_handle->SetResultCache(result_set, cache_num_rows));
   }
   return Status::OK();
@@ -986,7 +1026,7 @@ void ImpalaServer::GetLog(TGetLogResp& return_val, const TGetLogReq& request) {
   Coordinator* coord = query_handle->GetCoordinator();
   if (coord != nullptr) {
     // Report progress
-    ss << coord->progress().ToString() << "\n";
+    ss << coord->scan_progress().ToString() << "\n";
   }
   // Report the query status, if the query failed or has been retried.
   if (query_handle->IsRetriedQuery()) {
