@@ -417,27 +417,32 @@ class ImpalaServer : public ImpalaServiceIf,
   /// been updated from a statestore heartbeat that includes this catalog
   /// update's version.
   ///
-  /// If wait_for_all_subscribers is true, this function also
-  /// waits for all other catalog topic subscribers to process this update by checking the
-  /// current min_subscriber_topic_version included in each state store heartbeat.
+  /// If 'wait_for_all_subscribers' is true, this function also waits for all other
+  /// catalog topic subscribers to process this update by checking the current
+  /// min_subscriber_topic_version included in each state store heartbeat.
+  ///
+  /// 'query_options' is used for running debug actions.
   Status ProcessCatalogUpdateResult(const TCatalogUpdateResult& catalog_update_result,
-      bool wait_for_all_subscribers) WARN_UNUSED_RESULT;
+      bool wait_for_all_subscribers, const TQueryOptions& query_options,
+      RuntimeProfile::EventSequence* timeline)
+      WARN_UNUSED_RESULT;
 
   /// Wait until the catalog update with version 'catalog_update_version' is
   /// received and applied in the local catalog cache or until the catalog
   /// service id has changed.
   void WaitForCatalogUpdate(const int64_t catalog_update_version,
-      const TUniqueId& catalog_service_id);
+      const TUniqueId& catalog_service_id, RuntimeProfile::EventSequence* timeline);
 
   /// Wait until the minimum catalog object version in the local cache is
   /// greater than 'min_catalog_update_version' or until the catalog
   /// service id has changed.
   void WaitForMinCatalogUpdate(const int64_t min_catalog_update_version,
-      const TUniqueId& catalog_service_id);
+      const TUniqueId& catalog_service_id, RuntimeProfile::EventSequence* timeline);
 
   /// Wait until the last applied catalog update has been broadcast to
   /// all coordinators or until the catalog service id has changed.
-  void WaitForCatalogUpdateTopicPropagation(const TUniqueId& catalog_service_id);
+  void WaitForCatalogUpdateTopicPropagation(const TUniqueId& catalog_service_id,
+      RuntimeProfile::EventSequence* timeline);
 
   /// Returns true if lineage logging is enabled, false otherwise.
   ///
@@ -483,6 +488,10 @@ class ImpalaServer : public ImpalaServiceIf,
   void CancelQueriesOnFailedBackends(
       const std::unordered_set<BackendIdPB>& current_membership);
 
+  /// Returns true if all services required for the server have been started and the
+  /// registration with statestore is completed.
+  bool AreServicesReady() const;
+
   /// Start the shutdown process. Return an error if it could not be started. Otherwise,
   /// if it was successfully started by this or a previous call, return OK along with
   /// information about the pending shutdown in 'shutdown_status'. 'relative_deadline_s'
@@ -514,6 +523,10 @@ class ImpalaServer : public ImpalaServiceIf,
   /// Returns true if 'user' was configured as an authorized proxy user.
   bool IsAuthorizedProxyUser(const std::string& user) WARN_UNUSED_RESULT;
 
+  /// Gets connection contexts for all types of thrift servers.
+  void GetAllConnectionContexts(
+      ThriftServer::ConnectionContextList* connection_contexts);
+
   // Mapping between query option names and levels
   QueryOptionLevels query_option_levels_;
 
@@ -652,8 +665,11 @@ class ImpalaServer : public ImpalaServiceIf,
   friend class ImpalaServerTest;
   friend class QueryDriver;
 
+  static const string BEESWAX_SERVER_NAME;
+  static const string HS2_SERVER_NAME;
+  static const string HS2_HTTP_SERVER_NAME;
   // Used to identify external frontend RPC calls
-  const string EXTERNAL_FRONTEND_SERVER_NAME = "external-frontend";
+  static const string EXTERNAL_FRONTEND_SERVER_NAME;
 
   boost::scoped_ptr<ImpalaHttpHandler> http_handler_;
 
@@ -925,11 +941,17 @@ class ImpalaServer : public ImpalaServiceIf,
     /// True if the query required a coordinator fragment
     bool has_coord;
 
-    /// The number of fragments that have completed
-    int64_t num_complete_fragments;
+    /// The number of scan ranges that have completed.
+    int64_t num_completed_scan_ranges;
 
-    /// The total number of fragments
-    int64_t total_fragments;
+    /// The total number of scan ranges.
+    int64_t total_scan_ranges;
+
+    /// The number of fragment instances that have completed.
+    int64_t num_completed_fragment_instances;
+
+    /// The total number of fragment instances.
+    int64_t total_fragment_instances;
 
     /// The number of rows fetched by the client
     int64_t num_rows_fetched;
@@ -952,6 +974,26 @@ class ImpalaServer : public ImpalaServiceIf,
     /// ClientRequestState.
     int64_t start_time_us, end_time_us;
 
+    /// The request waited time in ms for queued.
+    int64_t wait_time_ms;
+
+    /// Total peak memory usage by this query at all backends.
+    int64_t total_peak_mem_usage;
+
+    /// The cluster wide estimated memory usage of this query.
+    int64_t cluster_mem_est;
+
+    /// Total bytes read by this query at all backends.
+    int64_t bytes_read;
+
+    /// The total number of bytes sent (across the network) by this query in exchange
+    /// nodes. Does not include remote reads, data written to disk, or data sent to the
+    /// client.
+    int64_t bytes_sent;
+
+    // Query timeline from summary profile.
+    std::string timeline;
+
     /// Summary of execution for this query.
     TExecSummary exec_summary;
 
@@ -1003,6 +1045,7 @@ class ImpalaServer : public ImpalaServiceIf,
 
   /// Returns the active QueryHandle for this query id. The QueryHandle contains the
   /// active ClientRequestState. Returns an error Status if the query id cannot be found.
+  /// If caller is an RPC thread, RPC context will be registered for time tracking
   /// See QueryDriver for a description of active ClientRequestStates.
   Status GetActiveQueryHandle(
       const TUniqueId& query_id, QueryHandle* query_handle);
@@ -1196,25 +1239,34 @@ class ImpalaServer : public ImpalaServiceIf,
 
   /// FIFO list of query records, which are written after the query finishes executing.
   /// Queries may briefly have entries in 'query_log_' and 'query_driver_map_'
-  /// while the query is being unregistered.
-  typedef std::list<std::unique_ptr<QueryStateRecord>> QueryLog;
+  /// while the query is being unregistered. To ensure that the records provided by
+  /// GetQueryRecord function are always valid, this list uses shared_ptr to hold
+  /// QueryStateRecord, preventing its lifecycle from ending prematurely due to removal
+  /// from the list.
+  typedef std::list<std::shared_ptr<QueryStateRecord>> QueryLog;
   QueryLog query_log_;
 
   /// Index that allows lookup via TUniqueId into the query log. The QueryStateRecord
-  /// value is owned by 'query_log_' so the entry in this index must be removed when
+  /// pointer is owned by 'query_log_' so the entry in this index must be removed when
   /// it is removed from 'query_log_'.
-  typedef boost::unordered_map<TUniqueId, QueryStateRecord*> QueryLogIndex;
+  typedef boost::unordered_map<TUniqueId, std::shared_ptr<QueryStateRecord>*>
+      QueryLogIndex;
   QueryLogIndex query_log_index_;
 
-  /// Sets the given QueryLogIndex for the given query_id. Returns an error Status if the
-  /// given query_id cannot be found in the QueryLogIndex.
+  /// Sets the given query_record (and retried_query_record too if given) for the given
+  /// query_id. Returns an error Status if the given query_id cannot be found in the
+  /// QueryLogIndex.
   Status GetQueryRecord(
-      const TUniqueId& query_id, QueryLogIndex::const_iterator* query_record);
+      const TUniqueId& query_id, std::shared_ptr<QueryStateRecord>* query_record,
+      std::shared_ptr<QueryStateRecord>* retried_query_record = nullptr);
 
   /// Decompresses the profile in the given QueryStateRecord into the specified format.
   /// The decompressed profile is added to the given RuntimeProfileOutput.
   Status DecompressToProfile(TRuntimeProfileFormat::type format,
-      QueryLogIndex::const_iterator query_record, RuntimeProfileOutput* profile);
+      std::shared_ptr<QueryStateRecord> query_record, RuntimeProfileOutput* profile);
+
+  void WaitForNewCatalogServiceId(TUniqueId cur_service_id,
+      std::unique_lock<std::mutex>* ver_lock);
 
   /// Logger for writing encoded query profiles, one per line with the following format:
   /// <ms-since-epoch> <query-id> <thrift query profile URL encoded and gzipped>
@@ -1512,6 +1564,7 @@ class ImpalaServer : public ImpalaServiceIf,
       catalog_topic_version(0L),
       catalog_object_version_lower_bound(0L) {
     }
+
     /// Update the metrics to store the current version of catalog, current topic and
     /// current service id used by impalad.
     void  UpdateCatalogVersionMetrics();
@@ -1568,7 +1621,7 @@ class ImpalaServer : public ImpalaServiceIf,
 
   /// Comparator that breaks ties when two queries have identical expiration deadlines.
   struct ExpirationEventComparator {
-    bool operator()(const ExpirationEvent& t1, const ExpirationEvent& t2) {
+    bool operator()(const ExpirationEvent& t1, const ExpirationEvent& t2) const {
       if (t1.deadline < t2.deadline) return true;
       if (t2.deadline < t1.deadline) return false;
       if (t1.query_id < t2.query_id) return true;
