@@ -26,6 +26,7 @@
 
 #include "exprs/timezone_db.h"
 #include "gen-cpp/ImpalaInternalService_types.h"
+#include "gen-cpp/Query_constants.h"
 #include "runtime/runtime-filter.h"
 #include "service/query-option-parser.h"
 #include "util/debug-util.h"
@@ -200,6 +201,7 @@ Status impala::SetQueryOption(const string& key, const string& value,
     return Status(Substitute("Invalid query option: $0", key));
   }
 
+  QueryConstants qc;
   TImpalaQueryOptions::type option = static_cast<TImpalaQueryOptions::type>(option_int);
 
   if (value.empty()) {
@@ -598,6 +600,10 @@ Status impala::SetQueryOption(const string& key, const string& value,
         query_options->__set_max_mem_estimate_for_admission(mem_spec_val.value);
         break;
       }
+      case TImpalaQueryOptions::ENABLE_TRIVIAL_QUERY_FOR_ADMISSION: {
+        query_options->__set_enable_trivial_query_for_admission(IsTrue(value));
+        break;
+      }
       case TImpalaQueryOptions::THREAD_RESERVATION_LIMIT: {
         int32_t int32_t_val = 0;
         RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckInclusiveLowerBound<int32_t>(
@@ -849,6 +855,17 @@ Status impala::SetQueryOption(const string& key, const string& value,
         query_options->__set_async_codegen(IsTrue(value));
         break;
       }
+      case TImpalaQueryOptions::DISABLE_CODEGEN_CACHE: {
+        query_options->__set_disable_codegen_cache(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::CODEGEN_CACHE_MODE: {
+        TCodeGenCacheMode::type enum_type;
+        RETURN_IF_ERROR(GetThriftEnum(
+            value, "CodeGen Cache Mode", _TCodeGenCacheMode_VALUES_TO_NAMES, &enum_type));
+        query_options->__set_codegen_cache_mode(enum_type);
+        break;
+      }
       case TImpalaQueryOptions::ENABLE_DISTINCT_SEMI_JOIN_OPTIMIZATION: {
         query_options->__set_enable_distinct_semi_join_optimization(IsTrue(value));
         break;
@@ -1037,6 +1054,98 @@ Status impala::SetQueryOption(const string& key, const string& value,
         query_options->__set_expand_complex_types(IsTrue(value));
         break;
       }
+      case TImpalaQueryOptions::FALLBACK_DB_FOR_FUNCTIONS: {
+        query_options->__set_fallback_db_for_functions(value);
+        break;
+      }
+      case TImpalaQueryOptions::STRINGIFY_MAP_KEYS: {
+        query_options->__set_stringify_map_keys(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::COMPUTE_PROCESSING_COST: {
+        query_options->__set_compute_processing_cost(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::PROCESSING_COST_MIN_THREADS: {
+        StringParser::ParseResult result;
+        const int32_t min_num =
+            StringParser::StringToInt<int32_t>(value.c_str(), value.length(), &result);
+        if (result != StringParser::PARSE_SUCCESS || min_num < 1
+            || min_num > qc.MAX_FRAGMENT_INSTANCES_PER_NODE) {
+          return Status(Substitute("$0 is not valid for processing_cost_min_threads. "
+                                   "Valid values are in [1, $1].",
+              value, qc.MAX_FRAGMENT_INSTANCES_PER_NODE));
+        }
+        query_options->__set_processing_cost_min_threads(min_num);
+        break;
+      }
+      case TImpalaQueryOptions::JOIN_SELECTIVITY_CORRELATION_FACTOR: {
+        double double_val = 0.0f;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckInclusiveRange<double>(
+            option, value, 0.0, 1.0, &double_val));
+        query_options->__set_join_selectivity_correlation_factor(double_val);
+        break;
+      }
+      case TImpalaQueryOptions::MAX_FRAGMENT_INSTANCES_PER_NODE: {
+        StringParser::ParseResult result;
+        const int32_t max_num =
+            StringParser::StringToInt<int32_t>(value.c_str(), value.length(), &result);
+        if (result != StringParser::PARSE_SUCCESS || max_num < 1
+            || max_num > qc.MAX_FRAGMENT_INSTANCES_PER_NODE) {
+          return Status(Substitute("$0 is not valid for max_fragment_instances_per_node. "
+                                   "Valid values are in [1, $1].",
+              value, qc.MAX_FRAGMENT_INSTANCES_PER_NODE));
+        }
+        query_options->__set_max_fragment_instances_per_node(max_num);
+        break;
+      }
+      case TImpalaQueryOptions::MAX_SORT_RUN_SIZE: {
+        int32_t int32_t_val = 0;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckNonNegative<int32_t>(
+            option, value, &int32_t_val));
+        RETURN_IF_ERROR(
+            QueryOptionValidator<int32_t>::NotEquals(option, int32_t_val, 1));
+        query_options->__set_max_sort_run_size(int32_t_val);
+        break;
+      }
+      case TImpalaQueryOptions::ALLOW_UNSAFE_CASTS: {
+        query_options->__set_allow_unsafe_casts(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::NUM_THREADS_FOR_TABLE_MIGRATION: {
+        int32_t int32_t_val = 0;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckInclusiveRange<int32_t>(
+            option, value, 0, 1024, &int32_t_val));
+        query_options->__set_num_threads_for_table_migration(int32_t_val);
+        break;
+      }
+      case TImpalaQueryOptions::DISABLE_OPTIMIZED_ICEBERG_V2_READ: {
+        query_options->__set_disable_optimized_iceberg_v2_read(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::VALUES_STMT_AVOID_LOSSY_CHAR_PADDING: {
+        query_options->__set_values_stmt_avoid_lossy_char_padding(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::LARGE_AGG_MEM_THRESHOLD: {
+        MemSpec mem_spec_val{};
+        RETURN_IF_ERROR(QueryOptionParser::Parse<MemSpec>(option, value, &mem_spec_val));
+        query_options->__set_large_agg_mem_threshold(mem_spec_val.value);
+        break;
+      }
+      case TImpalaQueryOptions::AGG_MEM_CORRELATION_FACTOR: {
+        double double_val = 0.0f;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckInclusiveRange<double>(
+            option, value, 0.0, 1.0, &double_val));
+        query_options->__set_agg_mem_correlation_factor(double_val);
+        break;
+      }
+      case TImpalaQueryOptions::MEM_LIMIT_COORDINATORS: {
+        MemSpec mem_spec_val{};
+        RETURN_IF_ERROR(QueryOptionParser::Parse<MemSpec>(option, value, &mem_spec_val));
+        query_options->__set_mem_limit_coordinators(mem_spec_val.value);
+        break;
+      }
       default:
         if (IsRemovedQueryOption(key)) {
           LOG(WARNING) << "Ignoring attempt to set removed query option '" << key << "'";
