@@ -20,9 +20,14 @@
 #include <sstream>
 #include <boost/scoped_ptr.hpp>
 
+#include <rapidjson/ostreamwrapper.h>
+#include <rapidjson/rapidjson.h>
+#include <rapidjson/writer.h>
+
 #include "exprs/scalar-expr-evaluator.h"
 #include "exprs/slot-ref.h"
 #include "rpc/thrift-util.h"
+#include "runtime/complex-value-writer.inline.h"
 #include "runtime/descriptors.h"
 #include "runtime/raw-value.h"
 #include "runtime/row-batch.h"
@@ -53,8 +58,14 @@ namespace impala {
 class AsciiQueryResultSet : public QueryResultSet {
  public:
   /// Rows are added into 'rowset'.
-  AsciiQueryResultSet(const TResultSetMetadata& metadata, vector<string>* rowset)
-    : metadata_(metadata), result_set_(rowset) {}
+  AsciiQueryResultSet(const TResultSetMetadata& metadata, vector<string>* rowset,
+      bool stringify_map_keys)
+    : metadata_(metadata), result_set_(rowset), stringify_map_keys_(stringify_map_keys) {
+    types_.reserve(metadata.columns.size());
+    for (int i = 0; i < metadata.columns.size(); ++i) {
+      types_.push_back(ColumnType::FromThrift(metadata_.columns[i].columnType));
+    }
+  }
 
   virtual ~AsciiQueryResultSet() {}
 
@@ -78,13 +89,20 @@ class AsciiQueryResultSet : public QueryResultSet {
 
   /// Points to the result set to be filled. Not owned by this object.
   vector<string>* result_set_;
+
+  // If true, converts map keys to strings; see IMPALA-11778.
+  const bool stringify_map_keys_;
+
+  // De-serialized column metadata
+  vector<ColumnType> types_;
 };
 
 /// Result set container for Hive protocol versions >= V6, where results are returned in
 /// column-orientation.
 class HS2ColumnarResultSet : public QueryResultSet {
  public:
-  HS2ColumnarResultSet(const TResultSetMetadata& metadata, TRowSet* rowset);
+  HS2ColumnarResultSet(const TResultSetMetadata& metadata, TRowSet* rowset,
+      bool stringify_map_keys, int expected_result_count);
 
   virtual ~HS2ColumnarResultSet() {}
 
@@ -117,6 +135,13 @@ class HS2ColumnarResultSet : public QueryResultSet {
 
   int64_t num_rows_;
 
+  // If true, converts map keys to strings; see IMPALA-11778.
+  const bool stringify_map_keys_;
+
+  // Expected number of result rows that will be returned with this
+  // fetch request. Used to reserve results vector memory.
+  const int expected_result_count_;
+
   void InitColumns();
 };
 
@@ -155,16 +180,18 @@ class HS2RowOrientedResultSet : public QueryResultSet {
 };
 
 QueryResultSet* QueryResultSet::CreateAsciiQueryResultSet(
-    const TResultSetMetadata& metadata, vector<string>* rowset) {
-  return new AsciiQueryResultSet(metadata, rowset);
+    const TResultSetMetadata& metadata, vector<string>* rowset, bool stringify_map_keys) {
+  return new AsciiQueryResultSet(metadata, rowset, stringify_map_keys);
 }
 
 QueryResultSet* QueryResultSet::CreateHS2ResultSet(
-    TProtocolVersion::type version, const TResultSetMetadata& metadata, TRowSet* rowset) {
+    TProtocolVersion::type version, const TResultSetMetadata& metadata, TRowSet* rowset,
+    bool stringify_map_keys, int expected_result_count) {
   if (version < TProtocolVersion::HIVE_CLI_SERVICE_PROTOCOL_V6) {
     return new HS2RowOrientedResultSet(metadata, rowset);
   } else {
-    return new HS2ColumnarResultSet(metadata, rowset);
+    return new HS2ColumnarResultSet(
+        metadata, rowset, stringify_map_keys, expected_result_count);
   }
 }
 
@@ -191,17 +218,12 @@ Status AsciiQueryResultSet::AddRows(const vector<ScalarExprEvaluator*>& expr_eva
       // ODBC-187 - ODBC can only take "\t" as the delimiter
       out_stream << (i > 0 ? "\t" : "");
 
-      if (metadata_.columns[i].columnType.types.size() == 1) {
-        RawValue::PrintValue(expr_evals[i]->GetValue(it.Get()),
-            ColumnType::FromThrift(metadata_.columns[i].columnType), scales[i],
-            &out_stream);
-      } else if (metadata_.columns[i].columnType.types.size() > 1) {
-        ColumnType col_type = ColumnType::FromThrift(metadata_.columns[i].columnType);
-        DCHECK(col_type.IsArrayType() || col_type.IsMapType());
-        PrintCollectionValue(expr_evals[i], it.Get(), scales[i], &out_stream,
-            col_type.IsMapType());
+      if (!types_[i].IsComplexType()) {
+        RawValue::PrintValue(expr_evals[i]->GetValue(it.Get()), types_[i],
+            scales[i], &out_stream);
       } else {
-        DCHECK(false);
+        PrintComplexValue(expr_evals[i], it.Get(), &out_stream, types_[i],
+            stringify_map_keys_);
       }
     }
     result_set_->push_back(out_stream.str());
@@ -210,17 +232,44 @@ Status AsciiQueryResultSet::AddRows(const vector<ScalarExprEvaluator*>& expr_eva
   return Status::OK();
 }
 
-void QueryResultSet::PrintCollectionValue(ScalarExprEvaluator* expr_eval,
-    const TupleRow* row, int scale, stringstream *stream, bool is_map) {
+void QueryResultSet::PrintComplexValue(ScalarExprEvaluator* expr_eval,
+    const TupleRow* row, stringstream *stream, const ColumnType& type,
+    bool stringify_map_keys) {
+  DCHECK(type.IsComplexType());
   const ScalarExpr& scalar_expr = expr_eval->root();
-  // Currently scalar_expr can be only a slot ref as no functions return arrays.
+  // Currently scalar_expr can be only a slot ref as no functions return complex types.
   DCHECK(scalar_expr.IsSlotRef());
-  const TupleDescriptor* item_tuple_desc = scalar_expr.GetCollectionTupleDesc();
-  DCHECK(item_tuple_desc != nullptr);
-  const CollectionValue* array_val =
-      static_cast<const CollectionValue*>(expr_eval->GetValue(row));
+  void* value = expr_eval->GetValue(row);
 
-  RawValue::PrintCollectionValue(array_val, item_tuple_desc, scale, stream, is_map);
+  if (value == nullptr) {
+    (*stream) << RawValue::NullLiteral(/*top-level*/ true);
+    return;
+  }
+
+  rapidjson::BasicOStreamWrapper<stringstream> wrapped_stream(*stream);
+  rapidjson::Writer<rapidjson::BasicOStreamWrapper<stringstream>> writer(wrapped_stream);
+
+  if (type.IsCollectionType()) {
+    const CollectionValue* collection_val = static_cast<const CollectionValue*>(value);
+    const TupleDescriptor* item_tuple_desc = scalar_expr.GetCollectionTupleDesc();
+    DCHECK(item_tuple_desc != nullptr);
+
+    ComplexValueWriter<rapidjson::BasicOStreamWrapper<stringstream>>
+        complex_value_writer(&writer, stringify_map_keys);
+    complex_value_writer.CollectionValueToJSON(*collection_val, type.type,
+            item_tuple_desc);
+  } else {
+    DCHECK(type.IsStructType());
+    const StructVal* struct_val = static_cast<const StructVal*>(value);
+    const SlotDescriptor* slot_desc =
+        static_cast<const SlotRef&>(scalar_expr).GetSlotDescriptor();
+    DCHECK(slot_desc != nullptr);
+    DCHECK_EQ(type, slot_desc->type());
+
+    ComplexValueWriter<rapidjson::BasicOStreamWrapper<stringstream>>
+        complex_value_writer(&writer, stringify_map_keys);
+    complex_value_writer.StructValToJSON(*struct_val, *slot_desc);
+  }
 }
 
 Status AsciiQueryResultSet::AddOneRow(const TResultRow& row) {
@@ -302,8 +351,11 @@ uint32_t TColumnByteSize(const ThriftTColumn& col, uint32_t start_idx, uint32_t
 // Result set container for Hive protocol versions >= V6, where results are returned in
 // column-orientation.
 HS2ColumnarResultSet::HS2ColumnarResultSet(
-    const TResultSetMetadata& metadata, TRowSet* rowset)
-  : metadata_(metadata), result_set_(rowset), num_rows_(0) {
+    const TResultSetMetadata& metadata, TRowSet* rowset, bool stringify_map_keys,
+    int expected_result_count)
+  : metadata_(metadata), result_set_(rowset), num_rows_(0),
+    stringify_map_keys_(stringify_map_keys),
+    expected_result_count_(expected_result_count) {
   if (rowset == NULL) {
     owned_result_set_.reset(new TRowSet());
     result_set_ = owned_result_set_.get();
@@ -314,13 +366,15 @@ HS2ColumnarResultSet::HS2ColumnarResultSet(
 Status HS2ColumnarResultSet::AddRows(const vector<ScalarExprEvaluator*>& expr_evals,
     RowBatch* batch, int start_idx, int num_rows) {
   DCHECK_GE(batch->num_rows(), start_idx + num_rows);
+  int expected_result_count =
+      std::max((int64_t) expected_result_count_, num_rows + num_rows_);
   int num_col = expr_evals.size();
   DCHECK_EQ(num_col, metadata_.columns.size());
   for (int i = 0; i < num_col; ++i) {
     const TColumnType& type = metadata_.columns[i].columnType;
     ScalarExprEvaluator* expr_eval = expr_evals[i];
     ExprValuesToHS2TColumn(expr_eval, type, batch, start_idx, num_rows, num_rows_,
-        &(result_set_->columns[i]));
+        expected_result_count, stringify_map_keys_, &(result_set_->columns[i]));
   }
   num_rows_ += num_rows;
   return Status::OK();
