@@ -23,6 +23,7 @@
 
 #include <boost/lexical_cast.hpp>
 #include <thrift/Thrift.h>
+#include <thrift/protocol/TProtocolException.h>
 #include <gutil/strings/substitute.h>
 #include <gutil/strings/util.h>
 
@@ -106,12 +107,23 @@ DEFINE_int32(statestore_update_tcp_timeout_seconds, 300, "(Advanced) The time af
     "badly hung machines that are not able to respond to the update RPC in short "
     "order.");
 
+DEFINE_int32(statestore_update_catalogd_tcp_timeout_seconds, 3, "(Advanced) The "
+    "time after which a UpdateCatalogd RPC to a subscriber will timeout. This setting "
+    "protects against badly hung machines that are not able to respond to the "
+    "UpdateCatalogd RPC in short order");
+
+DECLARE_bool(enable_catalogd_ha);
+DECLARE_int64(active_catalogd_designation_monitoring_interval_ms);
+DECLARE_int64(update_catalogd_rpc_resend_interval_ms);
 DECLARE_string(debug_actions);
 DECLARE_string(ssl_server_certificate);
 DECLARE_string(ssl_private_key);
 DECLARE_string(ssl_private_key_password_cmd);
 DECLARE_string(ssl_cipher_list);
 DECLARE_string(ssl_minimum_version);
+#ifndef NDEBUG
+DECLARE_int32(stress_statestore_startup_delay_ms);
+#endif
 
 // Metric keys
 // TODO: Replace 'backend' with 'subscriber' when we can coordinate a change with CM
@@ -124,6 +136,13 @@ const string STATESTORE_UPDATE_DURATION = "statestore.topic-update-durations";
 const string STATESTORE_PRIORITY_UPDATE_DURATION =
     "statestore.priority-topic-update-durations";
 const string STATESTORE_HEARTBEAT_DURATION = "statestore.heartbeat-durations";
+const string STATESTORE_SUCCESSFUL_UPDATE_CATALOGD_RPC_NUM =
+    "statestore.num-successful-update-catalogd-rpc";
+const string STATESTORE_FAILED_UPDATE_CATALOGD_RPC_NUM =
+    "statestore.num-failed-update-catalogd-rpc";
+const string STATESTORE_CLEAR_TOPIC_ENTRIES_NUM =
+    "statestore.num-clear-topic-entries-requests";
+const string STATESTORE_ACTIVE_CATALOGD_ADDRESS = "statestore.active-catalogd-address";
 
 // Initial version for each Topic registered by a Subscriber. Generally, the Topic will
 // have a Version that is the MAX() of all entries in the Topic, but this initial
@@ -140,6 +159,28 @@ const char* Statestore::IMPALA_REQUEST_QUEUE_TOPIC = "impala-request-queue";
 
 typedef ClientConnection<StatestoreSubscriberClientWrapper> StatestoreSubscriberConn;
 
+namespace impala {
+
+std::string SubscriberTypeToString(TStatestoreSubscriberType::type t) {
+  switch (t) {
+    case TStatestoreSubscriberType::ADMISSIOND:
+      return "ADMISSIOND";
+    case TStatestoreSubscriberType::CATALOGD:
+      return "CATALOGD";
+    case TStatestoreSubscriberType::COORDINATOR:
+      return "COORDINATOR";
+    case TStatestoreSubscriberType::EXECUTOR:
+      return "EXECUTOR";
+    case TStatestoreSubscriberType::COORDINATOR_EXECUTOR:
+      return "COORDINATOR AND EXECUTOR";
+    case TStatestoreSubscriberType::UNKNOWN:
+    default:
+      return "UNKNOWN";
+  };
+}
+
+}
+
 class StatestoreThriftIf : public StatestoreServiceIf {
  public:
   StatestoreThriftIf(Statestore* statestore)
@@ -149,12 +190,65 @@ class StatestoreThriftIf : public StatestoreServiceIf {
 
   virtual void RegisterSubscriber(TRegisterSubscriberResponse& response,
       const TRegisterSubscriberRequest& params) {
+    if (FLAGS_debug_actions == "START_STATESTORE_IN_PROTOCOL_V1"
+        && strncmp(params.subscriber_id.c_str(), "python-test-client", 18) == 0
+        && params.protocol_version > StatestoreServiceVersion::V1) {
+      // Simulate the behaviour of old statestore to throw exception for new version
+      // of subscribers.
+      throw apache::thrift::protocol::TProtocolException(
+          apache::thrift::protocol::TProtocolException::INVALID_DATA, "Invalid data");
+    }
+    if (params.protocol_version < statestore_->GetProtocolVersion()) {
+      // Refuse old version of subscribers
+      response.__set_protocol_version(statestore_->GetProtocolVersion());
+      response.__set_statestore_id(statestore_->GetStateStoreId());
+      Status status = Status(TErrorCode::STATESTORE_INCOMPATIBLE_PROTOCOL,
+          params.subscriber_id, params.protocol_version + 1,
+          statestore_->GetProtocolVersion() + 1);
+      status.ToThrift(&response.status);
+      return;
+    }
+    TStatestoreSubscriberType::type subscriber_type = TStatestoreSubscriberType::UNKNOWN;
+    if (params.__isset.subscriber_type) {
+      subscriber_type = params.subscriber_type;
+    }
+    bool subscribe_catalogd_change = false;
+    if (params.__isset.subscribe_catalogd_change) {
+      subscribe_catalogd_change = params.subscribe_catalogd_change;
+    }
+    TCatalogRegistration catalogd_registration;
+    if (params.__isset.catalogd_registration) {
+      catalogd_registration = params.catalogd_registration;
+    }
+
     RegistrationId registration_id;
+    bool has_active_catalogd;
+    int64_t active_catalogd_version;
+    TCatalogRegistration active_catalogd_registration;
     Status status = statestore_->RegisterSubscriber(params.subscriber_id,
-        params.subscriber_location, params.topic_registrations, &registration_id);
+        params.subscriber_location, params.topic_registrations, subscriber_type,
+        subscribe_catalogd_change, catalogd_registration, &registration_id,
+        &has_active_catalogd, &active_catalogd_version, &active_catalogd_registration);
     status.ToThrift(&response.status);
     response.__set_registration_id(registration_id);
+    response.__set_statestore_id(statestore_->GetStateStoreId());
+    response.__set_protocol_version(statestore_->GetProtocolVersion());
+    if (has_active_catalogd) {
+      response.__set_catalogd_registration(active_catalogd_registration);
+      response.__set_catalogd_version(active_catalogd_version);
+    }
   }
+
+  virtual void GetProtocolVersion(TGetProtocolVersionResponse& response,
+      const TGetProtocolVersionRequest& params) {
+    LOG(INFO) << "Subscriber protocol version: " << params.protocol_version;
+    response.__set_protocol_version(statestore_->GetProtocolVersion());
+    response.__set_statestore_id(statestore_->GetStateStoreId());
+    Status status = Status::OK();
+    status.ToThrift(&response.status);
+    return;
+  }
+
  private:
   Statestore* statestore_;
 };
@@ -318,10 +412,16 @@ void Statestore::Topic::ToJson(Document* document, Value* topic_json) {
 
 Statestore::Subscriber::Subscriber(const SubscriberId& subscriber_id,
     const RegistrationId& registration_id, const TNetworkAddress& network_address,
-    const vector<TTopicRegistration>& subscribed_topics)
+    const vector<TTopicRegistration>& subscribed_topics,
+    TStatestoreSubscriberType::type subscriber_type, bool subscribe_catalogd_change)
   : subscriber_id_(subscriber_id),
     registration_id_(registration_id),
-    network_address_(network_address) {
+    network_address_(network_address),
+    subscriber_type_(subscriber_type),
+    subscribe_catalogd_change_(subscribe_catalogd_change) {
+  LOG(INFO) << "Subscriber '" << subscriber_id_
+            << "' with type " << SubscriberTypeToString(subscriber_type_)
+            << " registered (registration id: " << PrintId(registration_id_) << ")";
   RefreshLastHeartbeatTimestamp();
   for (const TTopicRegistration& topic : subscribed_topics) {
     GetTopicsMapForId(topic.topic_name)
@@ -406,7 +506,9 @@ void Statestore::Subscriber::RefreshLastHeartbeatTimestamp() {
 }
 
 Statestore::Statestore(MetricGroup* metrics)
-  : subscriber_topic_update_threadpool_("statestore-update",
+  : protocol_version_(StatestoreServiceVersion::V2),
+    catalog_manager_(FLAGS_enable_catalogd_ha),
+    subscriber_topic_update_threadpool_("statestore-update",
         "subscriber-update-worker",
         FLAGS_statestore_num_update_threads,
         FLAGS_statestore_max_subscribers,
@@ -432,10 +534,15 @@ Statestore::Statestore(MetricGroup* metrics)
         FLAGS_statestore_heartbeat_tcp_timeout_seconds * 1000,
         FLAGS_statestore_heartbeat_tcp_timeout_seconds * 1000, "",
         IsInternalTlsConfigured())),
+    update_catalogd_client_cache_(new StatestoreSubscriberClientCache(1, 0,
+        FLAGS_statestore_update_catalogd_tcp_timeout_seconds * 1000,
+        FLAGS_statestore_update_catalogd_tcp_timeout_seconds * 1000, "",
+        IsInternalTlsConfigured())),
     thrift_iface_(new StatestoreThriftIf(this)),
     failure_detector_(new MissedHeartbeatFailureDetector(
         FLAGS_statestore_max_missed_heartbeats,
         FLAGS_statestore_max_missed_heartbeats / 2)) {
+  UUIDToTUniqueId(boost::uuids::random_generator()(), &statestore_id_);
   DCHECK(metrics != NULL);
   metrics_ = metrics;
   num_subscribers_metric_ = metrics->AddGauge(STATESTORE_LIVE_SUBSCRIBERS, 0);
@@ -451,9 +558,19 @@ Statestore::Statestore(MetricGroup* metrics)
       metrics, STATESTORE_PRIORITY_UPDATE_DURATION);
   heartbeat_duration_metric_ =
       StatsMetric<double>::CreateAndRegister(metrics, STATESTORE_HEARTBEAT_DURATION);
+  successful_update_catalogd_rpc_metric_ =
+      metrics->AddCounter(STATESTORE_SUCCESSFUL_UPDATE_CATALOGD_RPC_NUM, 0);
+  failed_update_catalogd_rpc_metric_ =
+      metrics->AddCounter(STATESTORE_FAILED_UPDATE_CATALOGD_RPC_NUM, 0);
+  clear_topic_entries_metric_ =
+      metrics->AddCounter(STATESTORE_CLEAR_TOPIC_ENTRIES_NUM, 0);
+  active_catalogd_address_metric_ = metrics->AddProperty<string>(
+      STATESTORE_ACTIVE_CATALOGD_ADDRESS, "");
 
   update_state_client_cache_->InitMetrics(metrics, "subscriber-update-state");
   heartbeat_client_cache_->InitMetrics(metrics, "subscriber-heartbeat");
+  update_catalogd_client_cache_->InitMetrics(
+      metrics, "subscriber-update-catalogd");
 }
 
 Statestore::~Statestore() {
@@ -461,6 +578,13 @@ Statestore::~Statestore() {
 }
 
 Status Statestore::Init(int32_t state_store_port) {
+#ifndef NDEBUG
+  if (FLAGS_stress_statestore_startup_delay_ms > 0) {
+    LOG(INFO) << "Stress statestore startup delay: "
+              << FLAGS_stress_statestore_startup_delay_ms << " ms";
+    SleepForMs(FLAGS_stress_statestore_startup_delay_ms);
+  }
+#endif
   std::shared_ptr<TProcessor> processor(new StatestoreServiceProcessor(thrift_iface()));
   std::shared_ptr<TProcessorEventHandler> event_handler(
       new RpcEventHandler("statestore", metrics_));
@@ -486,17 +610,22 @@ Status Statestore::Init(int32_t state_store_port) {
   RETURN_IF_ERROR(subscriber_heartbeat_threadpool_.Init());
   RETURN_IF_ERROR(Thread::Create("statestore-heartbeat", "heartbeat-monitoring-thread",
       &Statestore::MonitorSubscriberHeartbeat, this, &heartbeat_monitoring_thread_));
+  RETURN_IF_ERROR(Thread::Create("statestore-update-catalogd", "update-catalogd-thread",
+      &Statestore::MonitorUpdateCatalogd, this, &update_catalogd_thread_));
   service_started_ = true;
 
   return Status::OK();
 }
 
-void Statestore::RegisterWebpages(Webserver* webserver) {
+void Statestore::RegisterWebpages(Webserver* webserver, bool metrics_only) {
   Webserver::RawUrlCallback healthz_callback =
       [this](const auto& req, auto* data, auto* response) {
         return this->HealthzHandler(req, data, response);
       };
   webserver->RegisterUrlCallback("/healthz", healthz_callback);
+
+  if (metrics_only) return;
+
   Webserver::UrlCallback topics_callback =
       bind<void>(mem_fn(&Statestore::TopicsHandler), this, _1, _2);
   webserver->RegisterUrlCallback("/topics", "statestore_topics.tmpl",
@@ -601,8 +730,20 @@ Status Statestore::OfferUpdate(const ScheduledSubscriberUpdate& update,
 Status Statestore::RegisterSubscriber(const SubscriberId& subscriber_id,
     const TNetworkAddress& location,
     const vector<TTopicRegistration>& topic_registrations,
-    RegistrationId* registration_id) {
-  if (subscriber_id.empty()) return Status("Subscriber ID cannot be empty string");
+    TStatestoreSubscriberType::type subscriber_type,
+    bool subscribe_catalogd_change,
+    const TCatalogRegistration& catalogd_registration,
+    RegistrationId* registration_id,
+    bool* has_active_catalogd,
+    int64_t* active_catalogd_version,
+    TCatalogRegistration* active_catalogd_registration) {
+  bool is_catalogd = subscriber_type == TStatestoreSubscriberType::CATALOGD;
+  if (subscriber_id.empty()) {
+    return Status("Subscriber ID cannot be empty string");
+  } else if (is_catalogd
+      && FLAGS_enable_catalogd_ha != catalogd_registration.enable_catalogd_ha) {
+    return Status("CalaogD HA enabling flag from catalogd does not match.");
+  }
 
   // Create any new topics first, so that when the subscriber is first sent a topic update
   // by the worker threads its topics are guaranteed to exist.
@@ -625,16 +766,36 @@ Status Statestore::RegisterSubscriber(const SubscriberId& subscriber_id,
     }
   }
   LOG(INFO) << "Registering: " << subscriber_id;
+  bool is_reregistering = false;
   {
     lock_guard<mutex> l(subscribers_lock_);
+    UUIDToTUniqueId(subscriber_uuid_generator_(), registration_id);
     SubscriberMap::iterator subscriber_it = subscribers_.find(subscriber_id);
     if (subscriber_it != subscribers_.end()) {
-      UnregisterSubscriber(subscriber_it->second.get());
+      shared_ptr<Subscriber> subscriber = subscriber_it->second;
+      UnregisterSubscriber(subscriber.get());
+      // Check if the subscriber's network addresses are matching.
+      if (subscriber->network_address().hostname != location.hostname
+          || subscriber->network_address().port != location.port) {
+        LOG(INFO) << "Subscriber " << subscriber_id
+                  << " re-register with different address, old address: "
+                  << TNetworkAddressToString(subscriber->network_address())
+                  << " , new address: " << TNetworkAddressToString(location);
+      }
+      is_reregistering = true;
     }
 
-    UUIDToTUniqueId(subscriber_uuid_generator_(), registration_id);
-    shared_ptr<Subscriber> current_registration(
-        new Subscriber(subscriber_id, *registration_id, location, topic_registrations));
+    if (is_catalogd
+        && catalog_manager_.RegisterCatalogd(is_reregistering, subscriber_id,
+            *registration_id, catalogd_registration)) {
+      LOG(INFO) << "Active catalogd role is designated to "
+                << catalog_manager_.GetActiveCatalogdSubscriberId();
+      update_catalod_cv_.NotifyAll();
+    }
+
+    shared_ptr<Subscriber> current_registration(new Subscriber(
+        subscriber_id, *registration_id, location, topic_registrations,
+        subscriber_type, subscribe_catalogd_change));
     subscribers_.emplace(subscriber_id, current_registration);
     failure_detector_->UpdateHeartbeat(subscriber_id, true);
     num_subscribers_metric_->SetValue(subscribers_.size());
@@ -645,10 +806,11 @@ Status Statestore::RegisterSubscriber(const SubscriberId& subscriber_id,
     RETURN_IF_ERROR(OfferUpdate(update, &subscriber_topic_update_threadpool_));
     RETURN_IF_ERROR(OfferUpdate(update, &subscriber_priority_topic_update_threadpool_));
     RETURN_IF_ERROR(OfferUpdate(update, &subscriber_heartbeat_threadpool_));
+    *active_catalogd_registration =
+        catalog_manager_.GetActiveCatalogRegistration(
+            has_active_catalogd, active_catalogd_version);
   }
 
-  LOG(INFO) << "Subscriber '" << subscriber_id << "' registered (registration id: "
-            << PrintId(*registration_id) << ")";
   return Status::OK();
 }
 
@@ -665,6 +827,13 @@ bool Statestore::FindSubscriber(const SubscriberId& subscriber_id,
 
 Status Statestore::SendTopicUpdate(Subscriber* subscriber, UpdateKind update_kind,
     bool* update_skipped) {
+  // Don't send topic update to inactive catalogd.
+  if (FLAGS_enable_catalogd_ha && subscriber->IsCatalogd()
+      && !catalog_manager_.IsActiveCatalogd(subscriber->id())) {
+    VLOG(3) << "Skip sending topic update to inactive catalogd";
+    return Status::OK();
+  }
+
   // Time any successful RPCs (i.e. those for which UpdateState() completed, even though
   // it may have returned an error.)
   MonotonicStopWatch sw;
@@ -682,6 +851,7 @@ Status Statestore::SendTopicUpdate(Subscriber* subscriber, UpdateKind update_kin
   // Set the expected registration ID, so that the subscriber can reject this update if
   // they have moved on to a new registration instance.
   update_state_request.__set_registration_id(subscriber->registration_id());
+  update_state_request.__set_statestore_id(statestore_id_);
 
   // Second: try and send it
   Status status;
@@ -749,6 +919,7 @@ Status Statestore::SendTopicUpdate(Subscriber* subscriber, UpdateKind update_kin
         DCHECK(!update.__isset.from_version);
         LOG(INFO) << "Received request for clearing the entries of topic: "
                   << update.topic_name << " from: " << subscriber->id();
+        clear_topic_entries_metric_->Increment(1);
         topic.ClearAllEntries();
       }
 
@@ -817,6 +988,12 @@ Statestore::TopicEntry::Version Statestore::GetMinSubscriberTopicVersion(
   bool found = false;
   // Find the minimum version processed for this topic across all topic subscribers.
   for (const SubscriberMap::value_type& subscriber: subscribers_) {
+    if (FLAGS_enable_catalogd_ha && subscriber.second->IsCatalogd()
+        && !catalog_manager_.IsActiveCatalogd(subscriber.second->id())) {
+      // Skip inactive catalogd since it does not apply catalog updates from the active
+      // catalogd.
+      continue;
+    }
     auto subscribed_topics = subscriber.second->GetTopicsMapForId(topic_id);
     if (subscribed_topics->find(topic_id) != subscribed_topics->end()) {
       found = true;
@@ -874,6 +1051,7 @@ Status Statestore::SendHeartbeat(Subscriber* subscriber) {
   THeartbeatRequest request;
   THeartbeatResponse response;
   request.__set_registration_id(subscriber->registration_id());
+  request.__set_statestore_id(statestore_id_);
   RETURN_IF_ERROR(
       client.DoRpc(&StatestoreSubscriberClientWrapper::Heartbeat, request, &response));
 
@@ -991,6 +1169,11 @@ void Statestore::DoSubscriberUpdate(UpdateKind update_kind, int thread_id,
                   << "or re-registered (last known registration ID: "
                   << PrintId(update.registration_id) << ")";
         UnregisterSubscriber(subscriber.get());
+        if (subscriber->IsCatalogd()) {
+          if (catalog_manager_.UnregisterCatalogd(subscriber->id())) {
+            update_catalod_cv_.NotifyAll();
+          }
+        }
       } else {
         LOG(INFO) << "Failure was already detected for subscriber '" << subscriber->id()
                   << "'. Won't send another " << update_kind_str;
@@ -1039,6 +1222,141 @@ void Statestore::DoSubscriberUpdate(UpdateKind update_kind, int thread_id,
   }
 }
 
+[[noreturn]] void Statestore::MonitorUpdateCatalogd() {
+  int64_t last_active_catalogd_version = 0;
+  // rpc_receivers is used to track subscribers to which statestore need to send RPCs
+  // when there is a change in the elected active catalogd. It is updated from
+  // subscribers_, and the subscribers will be removed from this list if the RPCs are
+  // successfully sent to them.
+  vector<std::shared_ptr<Subscriber>> rpc_receivers;
+  int64_t timeout_us =
+      FLAGS_active_catalogd_designation_monitoring_interval_ms * MICROS_PER_MILLI;
+  // Check if the first registered one should be designated with active role.
+  while (!catalog_manager_.CheckActiveCatalog()) {
+    unique_lock<mutex> l(*catalog_manager_.GetLock());
+    update_catalod_cv_.WaitFor(l, timeout_us);
+  }
+  SendUpdateCatalogdNotification(&last_active_catalogd_version, rpc_receivers);
+
+  // Wait for notification. If new leader is elected due to catalogd is registered or
+  // unregistered, send notification to all coordinators and catalogds.
+  timeout_us = FLAGS_update_catalogd_rpc_resend_interval_ms * MICROS_PER_MILLI;
+  while (1) {
+    {
+      unique_lock<mutex> l(*catalog_manager_.GetLock());
+      update_catalod_cv_.WaitFor(l, timeout_us);
+    }
+    SendUpdateCatalogdNotification(&last_active_catalogd_version, rpc_receivers);
+  }
+}
+
+void Statestore::SendUpdateCatalogdNotification(int64_t* last_active_catalogd_version,
+    vector<std::shared_ptr<Subscriber>>& rpc_receivers) {
+  bool has_active_catalogd;
+  int64_t active_catalogd_version = 0;
+  TCatalogRegistration catalogd_registration =
+      catalog_manager_.GetActiveCatalogRegistration(
+          &has_active_catalogd, &active_catalogd_version);
+  if (!has_active_catalogd ||
+      (active_catalogd_version == *last_active_catalogd_version
+          && rpc_receivers.empty())) {
+    // Don't resend RPCs if there is no change in Active Catalogd and no RPC failure in
+    // last round.
+    return;
+  }
+
+  bool resend_rpc = false;
+  if (active_catalogd_version > *last_active_catalogd_version) {
+    // Send notification for the latest elected active catalogd.
+    LOG(INFO) << "Send notification for active catalogd version: "
+              << active_catalogd_version;
+    active_catalogd_address_metric_->SetValue(
+        TNetworkAddressToString(catalogd_registration.address));
+    rpc_receivers.clear();
+    {
+      lock_guard<mutex> l(subscribers_lock_);
+      for (const auto& subscriber : subscribers_) {
+        if (subscriber.second->IsSubscribedCatalogdChange()) {
+          rpc_receivers.push_back(subscriber.second);
+        }
+      }
+    }
+    *last_active_catalogd_version = active_catalogd_version;
+  } else {
+    DCHECK(!rpc_receivers.empty());
+    lock_guard<mutex> l(subscribers_lock_);
+    for (std::vector<std::shared_ptr<Subscriber>>::iterator it = rpc_receivers.begin();
+         it != rpc_receivers.end();) {
+      // Don't resend RPC to subscribers which have been removed from subscriber list.
+      std::shared_ptr<Subscriber> subscriber = *it;
+      if (subscribers_.find(subscriber->id()) == subscribers_.end()) {
+        it = rpc_receivers.erase(it);
+      } else {
+        ++it;
+      }
+    }
+    if (rpc_receivers.empty()) return;
+    resend_rpc = true;
+  }
+
+  for (std::vector<std::shared_ptr<Subscriber>>::iterator it = rpc_receivers.begin();
+       it != rpc_receivers.end();) {
+    std::shared_ptr<Subscriber> subscriber = *it;
+    bool update_skipped = false;
+    Status status;
+    if (!resend_rpc) {
+      status = DebugAction(
+          FLAGS_debug_actions, "SEND_UPDATE_CATALOGD_RPC_FIRST_ATTEMPT");
+    }
+    if (status.ok()) {
+      StatestoreSubscriberConn client(update_catalogd_client_cache_.get(),
+          subscriber->network_address(), &status);
+      if (status.ok()) {
+        TUpdateCatalogdRequest request;
+        TUpdateCatalogdResponse response;
+        request.__set_registration_id(subscriber->registration_id());
+        request.__set_statestore_id(statestore_id_);
+        request.__set_catalogd_version(active_catalogd_version);
+        request.__set_catalogd_registration(catalogd_registration);
+        status = client.DoRpc(
+            &StatestoreSubscriberClientWrapper::UpdateCatalogd, request, &response);
+        if (!status.ok()) {
+          if (status.code() == TErrorCode::RPC_RECV_TIMEOUT) {
+            // Add details to status to make it more useful, while preserving the stack
+            status.AddDetail(Substitute(
+                "Subscriber $0 timed-out during update catalogd RPC. Timeout is $1s.",
+                subscriber->id(), FLAGS_statestore_update_catalogd_tcp_timeout_seconds));
+          }
+        } else {
+          update_skipped = (response.__isset.skipped && response.skipped);
+        }
+      }
+    }
+    if (status.ok()) {
+      if (update_skipped) {
+        // The subscriber skipped processing this update. It's not considered as a failure
+        // since subscribers can decide what they do with any update. The subscriber is
+        // left in the receiver list so that RPC will be resent to it in next round.
+        ++it;
+      } else {
+        successful_update_catalogd_rpc_metric_->Increment(1);
+        // Remove the subscriber from the receiver list so that Statestore will not resend
+        // RPC to it in next round.
+        it = rpc_receivers.erase(it);
+      }
+    } else {
+      LOG(ERROR) << "Couldn't send UpdateCatalogd RPC,  " << status.GetDetail();
+      failed_update_catalogd_rpc_metric_->Increment(1);
+      // Leave the subscriber in the receiver list. Statestore will resend RPC to it in
+      // next round.
+      ++it;
+    }
+  }
+  if (rpc_receivers.empty()) {
+    LOG(INFO) << "Successfully sent UpdateCatalogd RPCs to all subscribers";
+  }
+}
+
 void Statestore::UnregisterSubscriber(Subscriber* subscriber) {
   SubscriberMap::const_iterator it = subscribers_.find(subscriber->id());
   if (it == subscribers_.end() ||
@@ -1050,6 +1368,7 @@ void Statestore::UnregisterSubscriber(Subscriber* subscriber) {
   // Close all active clients so that the next attempt to use them causes a Reopen()
   update_state_client_cache_->CloseConnections(subscriber->network_address());
   heartbeat_client_cache_->CloseConnections(subscriber->network_address());
+  update_catalogd_client_cache_->CloseConnections(subscriber->network_address());
 
   // Prevent the failure detector from growing without bound
   failure_detector_->EvictPeer(subscriber->id());
