@@ -32,13 +32,16 @@
 #include "common/atomic.h"
 #include "common/status.h"
 #include "gen-cpp/StatestoreService.h"
+#include "gen-cpp/StatestoreService_types.h"
 #include "gen-cpp/StatestoreSubscriber.h"
 #include "gen-cpp/Types_types.h"
 #include "rpc/thrift-client.h"
 #include "runtime/client-cache.h"
 #include "statestore/failure-detector.h"
+#include "statestore/statestore-catalogd-mgr.h"
 #include "statestore/statestore-subscriber-client-wrapper.h"
 #include "util/aligned-new.h"
+#include "util/condition-variable.h"
 #include "util/metrics-fwd.h"
 #include "util/thread-pool.h"
 #include "util/webserver.h"
@@ -50,7 +53,8 @@ namespace impala {
 class Status;
 
 typedef ClientCache<StatestoreSubscriberClientWrapper> StatestoreSubscriberClientCache;
-typedef TUniqueId RegistrationId;
+
+std::string SubscriberTypeToString(TStatestoreSubscriberType::type t);
 
 /// The Statestore is a soft-state key-value store that maintains a set of Topics, which
 /// are maps from string keys to byte array values.
@@ -104,6 +108,13 @@ typedef TUniqueId RegistrationId;
 /// These empty updates are important so that subscribers can keep track of the current
 /// version number and report back their progress in receiving the topic contents.
 ///
+/// The Statestore is the central manager of the cluster. Subscribers join in the cluster
+/// through registration. To avoid incompatibility issues between the different components
+/// in the cluster, statestore uses protocol version to isolate cluster components, and
+/// only accepts the registration requests from the subscribers with compatible protocol
+/// versions. This ensure all of components in a cluster could communicate with compatible
+/// APIs and message format in the wire.
+///
 /// +================+
 /// | Implementation |
 /// +================+
@@ -117,10 +128,6 @@ typedef TUniqueId RegistrationId;
 /// 4. Topic::lock_ (terminal)
 class Statestore : public CacheLineAligned {
  public:
-  /// A SubscriberId uniquely identifies a single subscriber, and is
-  /// provided by the subscriber at registration time.
-  typedef std::string SubscriberId;
-
   /// A TopicId uniquely identifies a single topic
   typedef std::string TopicId;
 
@@ -153,9 +160,17 @@ class Statestore : public CacheLineAligned {
   Status RegisterSubscriber(const SubscriberId& subscriber_id,
       const TNetworkAddress& location,
       const std::vector<TTopicRegistration>& topic_registrations,
-      RegistrationId* registration_id) WARN_UNUSED_RESULT;
+      TStatestoreSubscriberType::type subscriber_type,
+      bool subscribe_catalogd_change,
+      const TCatalogRegistration& catalogd_registration,
+      RegistrationId* registration_id,
+      bool* has_active_catalogd,
+      int64_t* active_catalogd_version,
+      TCatalogRegistration* active_catalogd_registration);
 
-  void RegisterWebpages(Webserver* webserver);
+  /// Registers webpages for the input webserver. If metrics_only is set then only
+  /// '/healthz' page is registered.
+  void RegisterWebpages(Webserver* webserver, bool metrics_only);
 
   /// The main processing loop. Runs infinitely.
   void MainLoop();
@@ -185,6 +200,12 @@ class Statestore : public CacheLineAligned {
   /// after it stops responding to heartbeats.
   static int64_t FailedExecutorDetectionTimeMs();
 
+  const TUniqueId& GetStateStoreId() const { return statestore_id_; }
+
+  StatestoreServiceVersion::type GetProtocolVersion() const {
+    return protocol_version_;
+  }
+
  private:
   /// A TopicEntry is a single entry in a topic, and logically is a <string, byte string>
   /// pair.
@@ -356,7 +377,9 @@ class Statestore : public CacheLineAligned {
    public:
     Subscriber(const SubscriberId& subscriber_id, const RegistrationId& registration_id,
         const TNetworkAddress& network_address,
-        const std::vector<TTopicRegistration>& subscribed_topics);
+        const std::vector<TTopicRegistration>& subscribed_topics,
+        TStatestoreSubscriberType::type subscriber_type,
+        bool subscribe_catalogd_change);
 
     /// Information about a subscriber's subscription to a specific topic.
     struct TopicSubscription {
@@ -452,6 +475,16 @@ class Statestore : public CacheLineAligned {
     /// Refresh the subscriber's last heartbeat timestamp to the current monotonic time.
     void RefreshLastHeartbeatTimestamp();
 
+    /// Check if the subscriber is Catalog daemon.
+    bool IsCatalogd() const {
+      return subscriber_type_ == TStatestoreSubscriberType::CATALOGD;
+    }
+
+    /// Check if the subscriber wants to receive the notification of catalogd change.
+    bool IsSubscribedCatalogdChange() const {
+      return subscribe_catalogd_change_;
+    }
+
    private:
     /// Unique human-readable identifier for this subscriber, set by the subscriber itself
     /// on a Register call.
@@ -466,6 +499,12 @@ class Statestore : public CacheLineAligned {
     /// The location of the subscriber service that this subscriber runs.
     const TNetworkAddress network_address_;
 
+    /// Subscriber type
+    TStatestoreSubscriberType::type subscriber_type_;
+
+    /// Indicate if the subscriber subscribe to the notification of updating catalogd.
+    bool subscribe_catalogd_change_;
+
     /// Maps of topic subscriptions to current TopicSubscription, with separate maps for
     /// priority and non-priority topics. The state describes whether updates on the
     /// topic are 'transient' (i.e., to be deleted upon subscriber failure) or not
@@ -487,6 +526,12 @@ class Statestore : public CacheLineAligned {
     bool unregistered_ = false;
   };
 
+  /// Unique identifier for this statestore instance.
+  TUniqueId statestore_id_;
+
+  /// Protocol version of the statestore.
+  StatestoreServiceVersion::type protocol_version_;
+
   /// Protects access to subscribers_ and subscriber_uuid_generator_. See the class
   /// comment for the lock acquisition order.
   std::mutex subscribers_lock_;
@@ -499,10 +544,15 @@ class Statestore : public CacheLineAligned {
   /// Subscribers are held in shared_ptrs so that RegisterSubscriber() may overwrite their
   /// entry in this map while UpdateSubscriber() tries to update an existing registration
   /// without risk of use-after-free.
-  typedef boost::unordered_map<SubscriberId, std::shared_ptr<Subscriber>>
-    SubscriberMap;
+  typedef boost::unordered_map<SubscriberId, std::shared_ptr<Subscriber>> SubscriberMap;
   SubscriberMap subscribers_;
 
+  /// CatalogD Manager
+  StatestoreCatalogdMgr catalog_manager_;
+
+  /// Condition variable for sending the notifications of updating catalogd.
+  ConditionVariable update_catalod_cv_;
+
   /// Used to generated unique IDs for each new registration.
   boost::uuids::random_generator subscriber_uuid_generator_;
 
@@ -566,6 +616,9 @@ class Statestore : public CacheLineAligned {
   /// Thread that monitors the heartbeats of all subscribers.
   std::unique_ptr<Thread> heartbeat_monitoring_thread_;
 
+  /// Thread to send notification of updating catalogd.
+  std::unique_ptr<Thread> update_catalogd_thread_;
+
   /// Indicates whether the statestore has been initialized and the service is ready.
   std::atomic_bool service_started_{false};
 
@@ -579,6 +632,11 @@ class Statestore : public CacheLineAligned {
   /// of time.
   boost::scoped_ptr<StatestoreSubscriberClientCache> heartbeat_client_cache_;
 
+  /// Cache of subscriber clients used for UpdateCatalogd() RPCs. Only one client per
+  /// subscriber should be used, but the cache helps with the client lifecycle on
+  /// failure.
+  boost::scoped_ptr<StatestoreSubscriberClientCache> update_catalogd_client_cache_;
+
   /// Container for the internal statestore service.
   boost::scoped_ptr<ThriftServer> thrift_server_;
 
@@ -615,6 +673,21 @@ class Statestore : public CacheLineAligned {
   /// Same as above, but for SendHeartbeat() RPCs.
   StatsMetric<double>* heartbeat_duration_metric_;
 
+  /// Metric to count the total number of successful UpdateCatalogd RPCs sent by
+  /// statestore.
+  IntCounter* successful_update_catalogd_rpc_metric_;
+
+  /// Metric to count the total number of failed UpdateCatalogd RPCs sent by statestore.
+  IntCounter* failed_update_catalogd_rpc_metric_;
+
+  /// Metric to count the total number of requests for clearing topic entries from
+  /// catalogd. Catalogd indicates to clear topic entries when it is restarted or its
+  /// role has been changed from standby to active.
+  IntCounter* clear_topic_entries_metric_;
+
+  /// Metric that tracks the address of active catalogd for catalogd HA
+  StringProperty* active_catalogd_address_metric_;
+
   /// Utility method to add an update to the given thread pool, and to fail if the thread
   /// pool is already at capacity. Assumes that subscribers_lock_ is held by the caller.
   Status OfferUpdate(const ScheduledSubscriberUpdate& update,
@@ -724,6 +797,13 @@ class Statestore : public CacheLineAligned {
   /// id.
   [[noreturn]] void MonitorSubscriberHeartbeat();
 
+  /// Monitors the notification of updating catalogd.
+  [[noreturn]] void MonitorUpdateCatalogd();
+
+  /// Send notification of updating catalogd to the coordinators.
+  void SendUpdateCatalogdNotification(int64_t* last_active_catalogd_version,
+      vector<std::shared_ptr<Subscriber>>& receivers);
+
   /// Raw callback to indicate whether the service is ready.
   void HealthzHandler(const Webserver::WebRequest& req, std::stringstream* data,
       HttpStatusCode* response);
