@@ -163,52 +163,4 @@ void BloomFilter::Or(const BloomFilterPB& in, const uint8_t* directory_in,
   kudu::BlockBloomFilter::OrEqualArray(directory_size, directory_in, directory_out);
 }
 
-ImpalaBloomFilterBufferAllocator::ImpalaBloomFilterBufferAllocator()
-  : buffer_pool_client_(nullptr), is_allocated_(false) {
-  // Default constructor, which is defined to support the virtual function Clone().
-  // Impala code should not hit this function.
-  LOG(DFATAL) << "Unsupported code path.";
-}
-
-ImpalaBloomFilterBufferAllocator::ImpalaBloomFilterBufferAllocator(
-    BufferPool::ClientHandle* client)
-  : buffer_pool_client_(DCHECK_NOTNULL(client)), is_allocated_(false) {}
-
-ImpalaBloomFilterBufferAllocator::~ImpalaBloomFilterBufferAllocator() {
-  if (is_allocated_) {
-    LOG(DFATAL) << "Close() should have been called before the object is destroyed.";
-    Close();
-  }
-}
-
-void ImpalaBloomFilterBufferAllocator::Close() {
-  if (!is_allocated_) return;
-  DCHECK(buffer_pool_client_ != nullptr);
-  BufferPool* buffer_pool = ExecEnv::GetInstance()->buffer_pool();
-  buffer_pool->FreeBuffer(buffer_pool_client_, &buffer_handle_);
-  is_allocated_ = false;
-}
-
-kudu::Status ImpalaBloomFilterBufferAllocator::AllocateBuffer(size_t bytes, void** ptr) {
-  Close(); // Ensure that any previously allocated memory is released.
-
-  BufferPool* buffer_pool = ExecEnv::GetInstance()->buffer_pool();
-  DCHECK(buffer_pool_client_ != nullptr);
-  impala::Status status =
-      buffer_pool->AllocateBuffer(buffer_pool_client_, bytes, &buffer_handle_);
-  if (!status.ok()) {
-    return kudu::Status::RuntimeError(
-        strings::Substitute("BufferPool bad_alloc, bytes: $0", bytes));
-  }
-  *ptr = reinterpret_cast<void*>(buffer_handle_.data());
-  is_allocated_ = true;
-  return kudu::Status::OK();
-}
-
-void ImpalaBloomFilterBufferAllocator::FreeBuffer(void* ptr) {
-  if (ptr == nullptr) return;
-  DCHECK_EQ(ptr, buffer_handle_.data());
-  Close();
-}
-
 } // namespace impala
