@@ -205,6 +205,7 @@ class CacheShard {
   virtual void Release(HandleBase* handle) = 0;
   virtual void Erase(const Slice& key, uint32_t hash) = 0;
   virtual size_t Invalidate(const Cache::InvalidationControl& ctl) = 0;
+  virtual vector<HandleBase*> Dump() = 0;
 };
 
 // Function to build a cache shard using the given eviction algorithm.
@@ -271,6 +272,10 @@ class ShardedCache : public Cache {
     shards_[Shard(hash)]->Erase(key, hash);
   }
 
+  Slice Key(const UniqueHandle& handle) const override {
+    return reinterpret_cast<HandleBase*>(handle.get())->key();
+  }
+
   Slice Value(const UniqueHandle& handle) const override {
     return reinterpret_cast<HandleBase*>(handle.get())->value();
   }
@@ -304,6 +309,17 @@ class ShardedCache : public Cache {
     return invalidated_count;
   }
 
+  vector<UniqueHandle> Dump() override {
+    vector<UniqueHandle> handles;
+    for (auto& shard : shards_) {
+      for (HandleBase* handle : shard->Dump()) {
+        handles.emplace_back(
+            reinterpret_cast<Cache::Handle*>(handle), Cache::HandleDeleter(this));
+      }
+    }
+    return handles;
+  }
+
  protected:
   void Release(Handle* handle) override {
     HandleBase* h = reinterpret_cast<HandleBase*>(handle);
