@@ -24,7 +24,6 @@
 // Codec libraries
 #include <zlib.h>
 #include <bzlib.h>
-#undef DISALLOW_COPY_AND_ASSIGN // Snappy redefines this.
 #include <lz4.h>
 #include <snappy.h>
 #include <zstd.h>
@@ -32,6 +31,7 @@
 
 #include "common/logging.h"
 #include "exec/read-write-util.h"
+#include "gutil/strings/substitute.h"
 #include "runtime/mem-pool.h"
 #include "runtime/mem-tracker.h"
 
