@@ -36,8 +36,9 @@ class ObjectPool;
 /// runtime filters.
 ///
 /// Filters are constructed using MinMaxFilter::Create() which returns a MinMaxFilter of
-/// the appropriate type. Values can then be added using Insert(), and the min and max can
-/// be retrieved using GetMin()/GetMax().
+/// the appropriate type. Values can then be added using Insert(), InsertForLE(),
+/// InsertForLT(), InsertForGE() or InsertForGT, and the min and max can be retrieved
+/// using GetMin()/GetMax().
 ///
 /// MinMaxFilters ignore NULL values, and so are only appropriate to use as a runtime
 /// filter if the join predicate is '=' and not 'is not distinct from'.
@@ -58,19 +59,36 @@ class MinMaxFilter {
   /// from the calculated min to max is outside the range for 'type'. May only be called
   /// for integer-typed filters.
   virtual bool GetCastIntMinMax(
-      const ColumnType& type, int64_t* out_min, int64_t* out_max);
+      const ColumnType& type, int64_t* out_min, int64_t* out_max) const;
 
   /// Determine whether two ranges: [data_min, data_max] and [filter_min, filter_max]
   /// overlap. Return true if the two overlaps and false otherwise. Both data_min and
   /// data_max are of type 'type'. Since the overlap result can be used to set the
   /// the always_true_ flag, the actual overlap computation ignores the flag.
-  virtual bool EvalOverlap(const ColumnType& type, void* data_min, void* data_max) = 0;
+  virtual bool EvalOverlap(
+      const ColumnType& type, void* data_min, void* data_max) const = 0;
 
-  virtual PrimitiveType type() = 0;
+  virtual PrimitiveType type() const = 0;
 
   /// Add a new value, updating the current min/max.
   virtual void Insert(const void* val) = 0;
 
+  /// Add a new value, updating the current min/max when always false is true, or
+  /// only the current max otherwise.
+  virtual void InsertForLE(const void* val) = 0;
+
+  /// Add a new value which is 'val'-1, updating the current min/max when always false is
+  /// true, or only the current max otherwise.
+  virtual void InsertForLT(const void* val) = 0;
+
+  /// Add a new value, updating the current min/max when always false is true, or
+  /// only the current min otherwise.
+  virtual void InsertForGE(const void* val) = 0;
+
+  /// Add a new value which is 'val'-1, updating the current min/max when always false is
+  /// true, or only the current min otherwise.
+  virtual void InsertForGT(const void* val) = 0;
+
   /// If true, this filter doesn't allow any rows to pass.
   virtual bool AlwaysFalse() const = 0;
 
@@ -105,10 +123,16 @@ class MinMaxFilter {
   /// Returns the LLVM_CLASS_NAME for the given type.
   static std::string GetLlvmClassName(PrimitiveType type);
 
+  /// Returns the LLVM_CLASS_NAME for this base class 'MinMaxFilter'.
+  static const char* LLVM_CLASS_NAME;
+
   /// Returns the IRFunction::Type for Insert() for the given type.
   static IRFunction::Type GetInsertIRFunctionType(ColumnType col_type);
 
-  bool AlwaysTrue() const { return always_true_; }
+  /// Returns the IRFunction::Type for AlwaysTrue() for the given type.
+  static IRFunction::Type GetAlwaysTrueIRFunctionType(ColumnType col_type);
+
+  virtual bool AlwaysTrue() const = 0;
 
   /// Compute and return the ratio of the filter overlapping with a range defined by
   /// [data_min, data_max] of type 'type' as follows.
@@ -121,11 +145,26 @@ class MinMaxFilter {
   virtual float ComputeOverlapRatio(
       const ColumnType& type, void* data_min, void* data_max) = 0;
 
+  /// Compute and return the ratio of the filter overlapping with a range defined by
+  /// [data_min, data_max] of type 'type' where the min and max are of TColumnValues.
+  /// This version of the method calls ComputeOverlapRatio(const ColumnType&, void*,
+  /// void*) with the proper fields in TColumnValues cast to void*.
+  ///
+  /// This method is used mainly by hash join builder to evaluate the usefulness of
+  /// a min/max filter against the min/max column stats, and implemented only for a
+  /// subset of primitive column types (INTEGER, FLAOT, DOUBLE, DATE and DECIMAL) for
+  /// which the column stats can be stored in HMS.
+  ///
+  /// The default implementation returns an overlap ratio of 0.0.
+  virtual float ComputeOverlapRatio(const ColumnType& type, const TColumnValue& data_min,
+      const TColumnValue& data_max) { return 0.0; }
+
   /// Makes this filter always return true.
   virtual void SetAlwaysTrue() { always_true_ = true; }
 
   /// Return a debug string for 'filter'
-  static std::string DebugString(const MinMaxFilterPB& filter);
+  static std::string DebugString(
+      const MinMaxFilterPB& filter, const ColumnType& col_type);
 
   /// Test whether 'filter' is always true field is set and the field value is 'true'
   static bool AlwaysTrue(const MinMaxFilterPB& filter);
@@ -134,47 +173,55 @@ class MinMaxFilter {
   static bool AlwaysFalse(const MinMaxFilterPB& filter);
 
   /// Return a debug string for 'value'
-  static std::string DebugString(const ColumnValuePB& value);
+  static std::string DebugString(const ColumnValuePB& value, const ColumnType& col_type);
 
  protected:
   bool always_true_;
 };
 
-#define NUMERIC_MIN_MAX_FILTER(NAME, TYPE)                                             \
-  class NAME##MinMaxFilter : public MinMaxFilter {                                     \
-   public:                                                                             \
-    NAME##MinMaxFilter() {                                                             \
-      min_ = std::numeric_limits<TYPE>::max();                                         \
-      max_ = std::numeric_limits<TYPE>::lowest();                                      \
-    }                                                                                  \
-    NAME##MinMaxFilter(const MinMaxFilterPB& protobuf);                                \
-    virtual ~NAME##MinMaxFilter() {}                                                   \
-    virtual const void* GetMin() const override { return &min_; }                      \
-    virtual const void* GetMax() const override { return &max_; }                      \
-    virtual bool GetCastIntMinMax(                                                     \
-        const ColumnType& type, int64_t* out_min, int64_t* out_max) override;          \
-    bool EvalOverlap(const ColumnType& type, void* data_min, void* data_max) override; \
-    float ComputeOverlapRatio(                                                         \
-        const ColumnType& type, void* data_min, void* data_max) override;              \
-    virtual PrimitiveType type() override;                                             \
-    virtual void Insert(const void* val) override;                                     \
-    virtual bool AlwaysFalse() const override {                                        \
-      return min_ == std::numeric_limits<TYPE>::max()                                  \
-          && max_ == std::numeric_limits<TYPE>::lowest();                              \
-    }                                                                                  \
-    virtual void ToProtobuf(MinMaxFilterPB* protobuf) const override;                  \
-    virtual std::string DebugString() const override;                                  \
-    static void Or(const MinMaxFilterPB& in, MinMaxFilterPB* out);                     \
-    static void Copy(const MinMaxFilterPB& in, MinMaxFilterPB* out);                   \
-    static const char* LLVM_CLASS_NAME;                                                \
-                                                                                       \
-   private:                                                                            \
-    bool EvalOverlap(const ColumnType& type, void* data_min, void* data_max,           \
-        int64_t* filter_min64, int64_t* filter_max64);                                 \
-                                                                                       \
-   private:                                                                            \
-    TYPE min_;                                                                         \
-    TYPE max_;                                                                         \
+#define NUMERIC_MIN_MAX_FILTER(NAME, TYPE)                                          \
+  class NAME##MinMaxFilter : public MinMaxFilter {                                  \
+   public:                                                                          \
+    NAME##MinMaxFilter() {                                                          \
+      min_ = std::numeric_limits<TYPE>::max();                                      \
+      max_ = std::numeric_limits<TYPE>::lowest();                                   \
+    }                                                                               \
+    NAME##MinMaxFilter(const MinMaxFilterPB& protobuf);                             \
+    virtual ~NAME##MinMaxFilter() {}                                                \
+    virtual const void* GetMin() const override { return &min_; }                   \
+    virtual const void* GetMax() const override { return &max_; }                   \
+    virtual bool GetCastIntMinMax(                                                  \
+        const ColumnType& type, int64_t* out_min, int64_t* out_max) const override; \
+    bool EvalOverlap(                                                               \
+        const ColumnType& type, void* data_min, void* data_max) const override;     \
+    float ComputeOverlapRatio(                                                      \
+        const ColumnType& type, void* data_min, void* data_max) override;           \
+    float ComputeOverlapRatio(const ColumnType& type, const TColumnValue& data_min, \
+        const TColumnValue& data_max) override;                                     \
+    virtual PrimitiveType type() const override;                                    \
+    virtual void Insert(const void* val) override;                                  \
+    virtual void InsertForLE(const void* val) override;                             \
+    virtual void InsertForLT(const void* val) override;                             \
+    virtual void InsertForGE(const void* val) override;                             \
+    virtual void InsertForGT(const void* val) override;                             \
+    bool AlwaysTrue() const override;                                               \
+    virtual bool AlwaysFalse() const override {                                     \
+      return min_ == std::numeric_limits<TYPE>::max()                               \
+          && max_ == std::numeric_limits<TYPE>::lowest();                           \
+    }                                                                               \
+    virtual void ToProtobuf(MinMaxFilterPB* protobuf) const override;               \
+    virtual std::string DebugString() const override;                               \
+    static void Or(const MinMaxFilterPB& in, MinMaxFilterPB* out);                  \
+    static void Copy(const MinMaxFilterPB& in, MinMaxFilterPB* out);                \
+    static const char* LLVM_CLASS_NAME;                                             \
+                                                                                    \
+   private:                                                                         \
+    bool EvalOverlap(const ColumnType& type, void* data_min, void* data_max,        \
+        int64_t* filter_min64, int64_t* filter_max64) const;                        \
+                                                                                    \
+   private:                                                                         \
+    TYPE min_;                                                                      \
+    TYPE max_;                                                                      \
   };
 
 NUMERIC_MIN_MAX_FILTER(Bool, bool);
@@ -197,17 +244,32 @@ class StringMinMaxFilter : public MinMaxFilter {
       always_false_(true) {}
   StringMinMaxFilter(const MinMaxFilterPB& protobuf, MemTracker* mem_tracker);
   virtual ~StringMinMaxFilter() {}
+
+  static const std::string min_string; // a string of 1 byte of 0x0.
+  static const std::string max_string; // a string of MAX_BOUND_LENGTH bytes of 0xff.
+  static const StringValue MIN_BOUND_STRING;
+  static const StringValue MAX_BOUND_STRING;
+
   virtual void Close() override { mem_pool_.FreeAll(); }
 
   virtual const void* GetMin() const override { return &min_; }
   virtual const void* GetMax() const override { return &max_; }
-  virtual PrimitiveType type() override;
-
+  virtual PrimitiveType type() const override;
 
   virtual void Insert(const void* val) override;
+
+  // These four version of insert methods materialize the min_ and max_
+  // by making them pointing at min_buffer/max_buffer_.
+  virtual void InsertForLE(const void* val) override;
+  virtual void InsertForLT(const void* val) override;
+  virtual void InsertForGE(const void* val) override;
+  virtual void InsertForGT(const void* val) override;
+
+
+  bool AlwaysTrue() const override;
   virtual bool AlwaysFalse() const override { return always_false_; }
   bool EvalOverlap(
-      const ColumnType& type, void* data_min, void* data_max) override;
+      const ColumnType& type, void* data_min, void* data_max) const override;
   virtual float ComputeOverlapRatio(
       const ColumnType& type, void* data_min, void* data_max) override;
 
@@ -227,6 +289,18 @@ class StringMinMaxFilter : public MinMaxFilter {
  protected:
   virtual void SetAlwaysTrue() override;
 
+  // Update min_ and max_ to [value, MAX_BOUND_STRING]
+  void UpdateMin(const StringValue& value);
+
+  // Update min_ and max_ to [MIN_BOUND_STRING, value]
+  void UpdateMax(const StringValue& value);
+
+  // Perform the real work to materialize the min value to min_buffer_;
+  void MaterializeMinValue();
+
+  // Perform the real work to materialize the max value to max_buffer_;
+  void MaterializeMaxValue();
+
  private:
   /// Copies the contents of 'value' into 'buffer', up to 'len', and reassignes 'value' to
   /// point to 'buffer'. If an oom is hit, disables the filter by setting 'always_true_'
@@ -254,31 +328,43 @@ class StringMinMaxFilter : public MinMaxFilter {
   bool always_false_;
 };
 
-#define DATE_TIME_MIN_MAX_FILTER(NAME, TYPE)                                           \
-  class NAME##MinMaxFilter : public MinMaxFilter {                                     \
-   public:                                                                             \
-    NAME##MinMaxFilter() { always_false_ = true; }                                     \
-    NAME##MinMaxFilter(const MinMaxFilterPB& protobuf);                                \
-    virtual ~NAME##MinMaxFilter() {}                                                   \
-    virtual const void* GetMin() const override { return &min_; }                      \
-    virtual const void* GetMax() const override { return &max_; }                      \
-    virtual PrimitiveType type() override;                                             \
-    virtual void Insert(const void* val) override;                                     \
-    virtual bool AlwaysFalse() const override { return always_false_; }                \
-    bool EvalOverlap(const ColumnType& type, void* data_min, void* data_max) override; \
-    virtual float ComputeOverlapRatio(                                                 \
-        const ColumnType& type, void* data_min, void* data_max) override;              \
-    virtual void ToProtobuf(MinMaxFilterPB* protobuf) const override;                  \
-    virtual std::string DebugString() const override;                                  \
-    static void Or(const MinMaxFilterPB& in, MinMaxFilterPB* out);                     \
-    static void Copy(const MinMaxFilterPB& in, MinMaxFilterPB* out);                   \
-    static const char* LLVM_CLASS_NAME;                                                \
-                                                                                       \
-   private:                                                                            \
-    TYPE min_;                                                                         \
-    TYPE max_;                                                                         \
-    /* True if no rows have been inserted. */                                          \
-    bool always_false_;                                                                \
+#define DATE_TIME_MIN_MAX_FILTER(NAME, TYPE)                                    \
+  class NAME##MinMaxFilter : public MinMaxFilter {                              \
+   public:                                                                      \
+    NAME##MinMaxFilter() { always_false_ = true; }                              \
+    NAME##MinMaxFilter(const MinMaxFilterPB& protobuf);                         \
+    virtual ~NAME##MinMaxFilter() {}                                            \
+    virtual const void* GetMin() const override { return &min_; }               \
+    virtual const void* GetMax() const override { return &max_; }               \
+    virtual PrimitiveType type() const override;                                \
+    virtual void Insert(const void* val) override;                              \
+    virtual void InsertForLE(const void* val) override;                         \
+    virtual void InsertForLT(const void* val) override;                         \
+    virtual void InsertForGE(const void* val) override;                         \
+    virtual void InsertForGT(const void* val) override;                         \
+    bool AlwaysTrue() const override;                                           \
+    virtual bool AlwaysFalse() const override { return always_false_; }         \
+    bool EvalOverlap(                                                           \
+        const ColumnType& type, void* data_min, void* data_max) const override; \
+    virtual float ComputeOverlapRatio(                                          \
+        const ColumnType& type, void* data_min, void* data_max) override;       \
+    virtual float ComputeOverlapRatio(const ColumnType& type,                   \
+        const TColumnValue& data_min, const TColumnValue& data_max) override;   \
+    virtual void ToProtobuf(MinMaxFilterPB* protobuf) const override;           \
+    virtual std::string DebugString() const override;                           \
+    static void Or(const MinMaxFilterPB& in, MinMaxFilterPB* out);              \
+    static void Copy(const MinMaxFilterPB& in, MinMaxFilterPB* out);            \
+    static const char* LLVM_CLASS_NAME;                                         \
+                                                                                \
+   private:                                                                     \
+    void UpdateMin(const TYPE& val);                                            \
+    void UpdateMax(const TYPE& val);                                            \
+                                                                                \
+   private:                                                                     \
+    TYPE min_;                                                                  \
+    TYPE max_;                                                                  \
+    /* True if no rows have been inserted. */                                   \
+    bool always_false_;                                                         \
   };
 
 DATE_TIME_MIN_MAX_FILTER(Timestamp, TimestampValue);
@@ -332,22 +418,54 @@ class DecimalMinMaxFilter : public MinMaxFilter {
   }
 
   virtual void Insert(const void* val) override;
-  virtual PrimitiveType type() override;
+  virtual void InsertForLE(const void* val) override;
+  virtual void InsertForLT(const void* val) override;
+  virtual void InsertForGE(const void* val) override;
+  virtual void InsertForGT(const void* val) override;
+  virtual PrimitiveType type() const override;
+  bool AlwaysTrue() const override;
   virtual bool AlwaysFalse() const override { return always_false_; }
+  virtual void SetAlwaysFalse() { always_false_ = true; }
   virtual void ToProtobuf(MinMaxFilterPB* protobuf) const override;
   virtual std::string DebugString() const override;
 
   static void Or(const MinMaxFilterPB& in, MinMaxFilterPB* out, int precision);
   static void Copy(const MinMaxFilterPB& in, MinMaxFilterPB* out);
   bool EvalOverlap(
-      const ColumnType& type, void* data_min, void* data_max) override;
+      const ColumnType& type, void* data_min, void* data_max) const override;
   virtual float ComputeOverlapRatio(
       const ColumnType& type, void* data_min, void* data_max) override;
+  virtual float ComputeOverlapRatio(const ColumnType& type, const TColumnValue& data_min,
+      const TColumnValue& data_max) override;
 
   void Insert4(const void* val);
   void Insert8(const void* val);
   void Insert16(const void* val);
 
+  void Insert4ForLE(const void* val);
+  void Insert8ForLE(const void* val);
+  void Insert16ForLE(const void* val);
+
+  void Insert4ForGE(const void* val);
+  void Insert8ForGE(const void* val);
+  void Insert16ForGE(const void* val);
+
+  void Insert4ForLT(const void* val);
+  void Insert8ForLT(const void* val);
+  void Insert16ForLT(const void* val);
+
+  void Insert4ForGT(const void* val);
+  void Insert8ForGT(const void* val);
+  void Insert16ForGT(const void* val);
+
+  void UpdateMin(const Decimal4Value&);
+  void UpdateMin(const Decimal8Value&);
+  void UpdateMin(const Decimal16Value&);
+
+  void UpdateMax(const Decimal4Value&);
+  void UpdateMax(const Decimal8Value&);
+  void UpdateMax(const Decimal16Value&);
+
   /// Struct name in LLVM IR.
   static const char* LLVM_CLASS_NAME;
 
