@@ -23,6 +23,7 @@
 #include "codegen/llvm-codegen.h"
 #include "exprs/scalar-expr.h"
 #include "exprs/scalar-expr-evaluator.h"
+#include "runtime/fragment-state.h"
 #include "runtime/runtime-state.h"
 #include "runtime/multi-precision.h"
 #include "util/runtime-profile-counters.h"
@@ -31,18 +32,30 @@
 using namespace impala;
 using namespace strings;
 
+const char* TupleRowComparator::COMPARE_SYMBOL =
+    "_ZNK6impala18TupleRowComparator7CompareEPKPNS_19ScalarExprEvaluatorES4_PKNS_"
+    "8TupleRowES7_";
+
+const char* TupleRowComparator::LLVM_CLASS_NAME = "class.impala::TupleRowComparator";
+
 TupleRowComparatorConfig::TupleRowComparatorConfig(
     const TSortInfo& tsort_info, const std::vector<ScalarExpr*>& ordering_exprs)
   : sorting_order_(tsort_info.sorting_order),
     ordering_exprs_(ordering_exprs),
-    is_asc_(tsort_info.is_asc_order),
-    codegend_compare_fn_(nullptr) {
+    num_lexical_keys_(tsort_info.num_lexical_keys_in_zorder),
+    is_asc_(tsort_info.is_asc_order) {
   if (sorting_order_ == TSortingOrder::ZORDER) return;
   for (bool null_first : tsort_info.nulls_first) {
     nulls_first_.push_back(null_first ? -1 : 1);
   }
 }
 
+int TupleRowComparator::Compare(ScalarExprEvaluator* const* evaluator_lhs,
+    ScalarExprEvaluator* const* evaluator_rhs, const TupleRow* lhs,
+    const TupleRow* rhs) const {
+  return CompareInterpreted(lhs, rhs);
+}
+
 Status TupleRowComparator::Open(ObjectPool* pool, RuntimeState* state,
     MemPool* expr_perm_pool, MemPool* expr_results_pool) {
   if (ordering_expr_evals_lhs_.empty()) {
@@ -64,15 +77,14 @@ void TupleRowComparator::Close(RuntimeState* state) {
   ScalarExprEvaluator::Close(ordering_expr_evals_lhs_, state);
 }
 
-Status TupleRowComparatorConfig::Codegen(RuntimeState* state) {
+Status TupleRowComparatorConfig::Codegen(FragmentState* state, llvm::Function** fn) {
   if (sorting_order_ == TSortingOrder::ZORDER) {
     return Status("Codegen not yet implemented for sorting order: ZORDER");
   }
-  llvm::Function* fn;
   LlvmCodeGen* codegen = state->codegen();
-  DCHECK(codegen != NULL);
-  RETURN_IF_ERROR(CodegenLexicalCompare(codegen, &fn));
-  codegen->AddFunctionToJit(fn, reinterpret_cast<void**>(&codegend_compare_fn_));
+  DCHECK(codegen != nullptr);
+  RETURN_IF_ERROR(CodegenLexicalCompare(codegen, fn));
+  codegen->AddFunctionToJit(*fn, &codegend_compare_fn_);
   return Status::OK();
 }
 
@@ -103,7 +115,8 @@ int TupleRowLexicalComparator::CompareInterpreted(
 // Example IR for comparing an int column then a float column:
 //
 // ; Function Attrs: alwaysinline
-// define i32 @Compare(%"class.impala::ScalarExprEvaluator"**
+// define i32 @Compare(%"class.impala::TupleRowComparator"*,
+//                     %"class.impala::ScalarExprEvaluator"**
 //                         %ordering_expr_evals_lhs,
 //                     %"class.impala::ScalarExprEvaluator"**
 //                         %ordering_expr_evals_rhs,
@@ -231,27 +244,38 @@ Status TupleRowComparatorConfig::CodegenLexicalCompare(
     }
   }
 
-  // Construct function signature (note that this is different than the interpreted
-  // Compare() function signature):
+  // Construct function signature as follows.
+  //
+  // int Compare(TupleRowComparator*, ScalarExprEvaluator** ordering_expr_evals_lhs,
+  //     ScalarExprEvaluator** ordering_expr_evals_rhs,
+  //     TupleRow* lhs, TupleRow* rhs)
+  //
+  // Note that this is different than the interpreted Compare() function signature:
+  //
   // int Compare(ScalarExprEvaluator** ordering_expr_evals_lhs,
   //     ScalarExprEvaluator** ordering_expr_evals_rhs,
   //     TupleRow* lhs, TupleRow* rhs)
+  //
+  llvm::PointerType* tuple_row_comparator_type =
+      codegen->GetStructPtrType<TupleRowComparator>();
   llvm::PointerType* expr_evals_type =
       codegen->GetStructPtrPtrType<ScalarExprEvaluator>();
   llvm::PointerType* tuple_row_type = codegen->GetStructPtrType<TupleRow>();
   LlvmCodeGen::FnPrototype prototype(codegen, "Compare", codegen->i32_type());
+  prototype.AddArgument("tuple_row_comparator_type", tuple_row_comparator_type);
   prototype.AddArgument("ordering_expr_evals_lhs", expr_evals_type);
   prototype.AddArgument("ordering_expr_evals_rhs", expr_evals_type);
   prototype.AddArgument("lhs", tuple_row_type);
   prototype.AddArgument("rhs", tuple_row_type);
 
   LlvmBuilder builder(context);
-  llvm::Value* args[4];
+  llvm::Value* args[5];
   *fn = prototype.GeneratePrototype(&builder, args);
-  llvm::Value* lhs_evals_arg = args[0];
-  llvm::Value* rhs_evals_arg = args[1];
-  llvm::Value* lhs_arg = args[2];
-  llvm::Value* rhs_arg = args[3];
+  args[0] = nullptr;
+  llvm::Value* lhs_evals_arg = args[1];
+  llvm::Value* rhs_evals_arg = args[2];
+  llvm::Value* lhs_arg = args[3];
+  llvm::Value* rhs_arg = args[4];
 
   // Unrolled loop over each key expr
   for (int i = 0; i < ordering_exprs.size(); ++i) {
@@ -330,17 +354,26 @@ int TupleRowZOrderComparator::CompareInterpreted(const TupleRow* lhs,
   DCHECK_EQ(ordering_exprs_.size(), ordering_expr_evals_lhs_.size());
   DCHECK_EQ(ordering_expr_evals_lhs_.size(), ordering_expr_evals_rhs_.size());
 
-  // The algorithm requires all values having a common type, without loss of data.
-  // This means we have to find the biggest type.
-  int max_size = ordering_exprs_[0]->type().GetByteSize();
-  for (int i = 1; i < ordering_exprs_.size(); ++i) {
-    if (ordering_exprs_[i]->type().GetByteSize() > max_size) {
-      max_size = ordering_exprs_[i]->type().GetByteSize();
-    }
+  // Sort the partition keys lexically. The PreInsert SortNode uses ASC order and NULLS
+  // LAST. See Planner.createPreInsertSort() in FE.
+  for (int i = 0; i < num_lexical_keys_; ++i) {
+    void* lhs_value = ordering_expr_evals_lhs_[i]->GetValue(lhs);
+    void* rhs_value = ordering_expr_evals_rhs_[i]->GetValue(rhs);
+
+    // The sort order of NULLs is independent of asc/desc.
+    if (lhs_value == NULL && rhs_value == NULL) continue;
+    if (lhs_value == NULL) return 1;
+    if (rhs_value == NULL) return -1;
+
+    int result = RawValue::Compare(lhs_value, rhs_value, ordering_exprs_[i]->type());
+    if (result != 0) return result;
+    // Otherwise, try the next Expr
   }
-  if (max_size <= 4) {
+
+  // Sort the remaining keys in Z-order.
+  if (max_col_size_ <= 4) {
     return CompareBasedOnSize<uint32_t>(lhs, rhs);
-  } else if (max_size <= 8) {
+  } else if (max_col_size_ <= 8) {
     return CompareBasedOnSize<uint64_t>(lhs, rhs);
   } else {
     return CompareBasedOnSize<uint128_t>(lhs, rhs);
@@ -351,13 +384,13 @@ template<typename U>
 int TupleRowZOrderComparator::CompareBasedOnSize(const TupleRow* lhs,
     const TupleRow* rhs) const {
   auto less_msb = [](U x, U y) { return x < y && x < (x ^ y); };
-  ColumnType type = ordering_exprs_[0]->type();
+  ColumnType type = ordering_exprs_[num_lexical_keys_]->type();
   // Values of the most significant dimension from both sides.
-  U msd_lhs = GetSharedRepresentation<U>(ordering_expr_evals_lhs_[0]->GetValue(lhs),
-      type);
-  U msd_rhs = GetSharedRepresentation<U>(ordering_expr_evals_rhs_[0]->GetValue(rhs),
-      type);
-  for (int i = 1; i < ordering_exprs_.size(); ++i) {
+  U msd_lhs = GetSharedRepresentation<U>(
+      ordering_expr_evals_lhs_[num_lexical_keys_]->GetValue(lhs), type);
+  U msd_rhs = GetSharedRepresentation<U>(
+      ordering_expr_evals_rhs_[num_lexical_keys_]->GetValue(rhs), type);
+  for (int i = num_lexical_keys_ + 1; i < ordering_exprs_.size(); ++i) {
     type = ordering_exprs_[i]->type();
     void* lhs_v = ordering_expr_evals_lhs_[i]->GetValue(lhs);
     void* rhs_v = ordering_expr_evals_rhs_[i]->GetValue(rhs);
@@ -438,8 +471,9 @@ U TupleRowZOrderComparator::GetSharedRepresentation(void* val, ColumnType type)
 
 template <typename U, typename T>
 U inline TupleRowZOrderComparator::GetSharedIntRepresentation(const T val, U mask) const {
-  return (static_cast<U>(val) <<
-      std::max((sizeof(U) - sizeof(T)) * 8, (uint64_t)0)) ^ mask;
+  uint64_t shift_size = static_cast<uint64_t>(
+      std::max(static_cast<int64_t>((sizeof(U) - sizeof(T)) * 8), (int64_t) 0));
+  return (static_cast<U>(val) << shift_size) ^ mask;
 }
 
 template <typename U, typename T>
@@ -448,13 +482,14 @@ U inline TupleRowZOrderComparator::GetSharedFloatRepresentation(void* val, U mas
   T floating_value = *reinterpret_cast<const T*>(val);
   memcpy(&tmp, &floating_value, sizeof(T));
   if (UNLIKELY(std::isnan(floating_value))) return 0;
+  uint64_t shift_size = static_cast<uint64_t>(
+      std::max(static_cast<int64_t>((sizeof(U) - sizeof(T)) * 8), (int64_t) 0));
   if (floating_value < 0.0) {
     // Flipping all bits for negative values.
-    return static_cast<U>(~tmp) << std::max((sizeof(U) - sizeof(T)) * 8, (uint64_t)0);
+    return static_cast<U>(~tmp) << shift_size;
   } else {
     // Flipping only first bit.
-    return (static_cast<U>(tmp) << std::max((sizeof(U) - sizeof(T)) * 8, (uint64_t)0)) ^
-        mask;
+    return (static_cast<U>(tmp) << shift_size) ^ mask;
   }
 }
 
