@@ -53,13 +53,13 @@
 #
 # The script is directly executable, and it takes no parameters:
 #     ./bootstrap_toolchain.py
+
+from __future__ import absolute_import, division, print_function
 import logging
-import glob
 import multiprocessing.pool
 import os
 import platform
 import random
-import re
 import shutil
 import subprocess
 import sys
@@ -69,50 +69,29 @@ import time
 from collections import namedtuple
 from string import Template
 
-# Maps return values from 'lsb_release -irs' to the corresponding OS labels for both the
-# toolchain and the CDP components.
-OsMapping = namedtuple('OsMapping', ['lsb_release', 'toolchain', 'cdh'])
+# Maps the ID + major version of the VERSION_ID from /etc/os-release to the corresponding
+# OS label for the toolchain. See https://github.com/chef/os_release for a database of
+# /etc/os-release files.
+OsMapping = namedtuple('OsMapping', ['release', 'toolchain'])
 OS_MAPPING = [
-  OsMapping("centos5", "ec2-package-centos-5", None),
-  OsMapping("centos6", "ec2-package-centos-6", "redhat6"),
-  OsMapping("centos7", "ec2-package-centos-7", "redhat7"),
-  OsMapping("centos8", "ec2-package-centos-8", "redhat8"),
-  OsMapping("rocky8", "ec2-package-centos-8", "redhat8"),
-  OsMapping("almalinux8", "ec2-package-centos-8", "redhat8"),
-  OsMapping("redhatenterpriseserver5", "ec2-package-centos-5", None),
-  OsMapping("redhatenterpriseserver6", "ec2-package-centos-6", "redhat6"),
-  OsMapping("redhatenterpriseserver7", "ec2-package-centos-7", "redhat7"),
-  OsMapping("redhatenterprise8", "ec2-package-centos-8", "redhat8"),
-  OsMapping("redhatenterpriseserver8", "ec2-package-centos-8", "redhat8"),
-  OsMapping("debian6", "ec2-package-debian-6", None),
-  OsMapping("debian7", "ec2-package-debian-7", None),
-  OsMapping("debian8", "ec2-package-debian-8", "debian8"),
-  OsMapping("suselinux11", "ec2-package-sles-11", None),
-  OsMapping("suselinux12", "ec2-package-sles-12", "sles12"),
-  OsMapping("suse12", "ec2-package-sles-12", "sles12"),
-  OsMapping("ubuntu12.04", "ec2-package-ubuntu-12-04", None),
-  OsMapping("ubuntu14.04", "ec2-package-ubuntu-14-04", None),
-  OsMapping("ubuntu15.04", "ec2-package-ubuntu-14-04", None),
-  OsMapping("ubuntu15.10", "ec2-package-ubuntu-14-04", None),
-  OsMapping('ubuntu16.04', "ec2-package-ubuntu-16-04", "ubuntu1604"),
-  OsMapping('ubuntu18.04', "ec2-package-ubuntu-18-04", "ubuntu1804"),
-  OsMapping('ubuntu20.04', "ec2-package-ubuntu-20-04", "ubuntu2004")
+  OsMapping("rhel7", "ec2-package-centos-7"),
+  OsMapping("centos7", "ec2-package-centos-7"),
+  OsMapping("rhel8", "ec2-package-centos-8"),
+  OsMapping("centos8", "ec2-package-centos-8"),
+  OsMapping("rocky8", "ec2-package-centos-8"),
+  OsMapping("almalinux8", "ec2-package-centos-8"),
+  OsMapping("rhel9", "ec2-package-rocky-9"),
+  OsMapping("rocky9", "ec2-package-rocky-9"),
+  OsMapping("almalinux9", "ec2-package-rocky-9"),
+  OsMapping("sles12", "ec2-package-sles-12"),
+  OsMapping("sles15", "ec2-package-sles-15"),
+  OsMapping('ubuntu16', "ec2-package-ubuntu-16-04"),
+  OsMapping('ubuntu18', "ec2-package-ubuntu-18-04"),
+  OsMapping('ubuntu20', "ec2-package-ubuntu-20-04"),
+  OsMapping('ubuntu22', "ec2-package-ubuntu-22-04")
 ]
 
 
-def check_output(cmd_args):
-  """Run the command and return the output. Raise an exception if the command returns
-     a non-zero return code. Similar to subprocess.check_output() which is only provided
-     in python 2.7.
-  """
-  process = subprocess.Popen(cmd_args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-  stdout, _ = process.communicate()
-  if process.wait() != 0:
-    raise Exception("Command with args '%s' failed with exit code %s:\n%s"
-        % (cmd_args, process.returncode, stdout))
-  return stdout
-
-
 def get_toolchain_compiler():
   """Return the <name>-<version> string for the compiler package to use for the
   toolchain."""
@@ -132,20 +111,20 @@ def wget_and_unpack_package(download_path, file_name, destination, wget_no_clobb
       download_path, destination, file_name, attempt))
     # --no-clobber avoids downloading the file if a file with the name already exists
     try:
-      cmd = ["wget", download_path,
+      cmd = ["wget", "-q", download_path,
              "--output-document={0}/{1}".format(destination, file_name)]
       if wget_no_clobber:
         cmd.append("--no-clobber")
-      check_output(cmd)
+      subprocess.check_call(cmd)
       break
-    except Exception, e:
+    except subprocess.CalledProcessError as e:
       if attempt == NUM_ATTEMPTS:
         raise
       logging.error("Download failed; retrying after sleep: " + str(e))
       time.sleep(10 + random.random() * 5)  # Sleep between 10 and 15 seconds.
   logging.info("Extracting {0}".format(file_name))
-  check_output(["tar", "xzf", os.path.join(destination, file_name),
-                "--directory={0}".format(destination)])
+  subprocess.check_call(["tar", "xzf", os.path.join(destination, file_name),
+                         "--directory={0}".format(destination)])
   os.unlink(os.path.join(destination, file_name))
 
 
@@ -359,6 +338,7 @@ class ApacheComponent(EnvVersionedPackage):
                                        unpack_directory_tmpl=unpack_directory_tmpl,
                                        makedir=makedir, template_subs_in=template_subs)
 
+
 class ToolchainKudu(ToolchainPackage):
   def __init__(self, platform_label=None):
     super(ToolchainKudu, self).__init__('kudu', platform_release=platform_label)
@@ -386,48 +366,49 @@ def try_get_platform_release_label():
     return None
 
 
-# Cache "lsb_release -irs" to avoid excessive logging from sh, and
-# to shave a little bit of time.
-lsb_release_cache = None
+# Cache the /etc/os-release calculation to shave a little bit of time.
+os_release_cache = None
 
 
 def get_platform_release_label(release=None):
   """Gets the right package label from the OS version. Raise exception if not found.
-     'release' can be provided to override the underlying OS version.
+     'release' can be provided to override the underlying OS version. This uses
+     ID and VERSION_ID from /etc/os-release to identify a distribution. Specifically,
+     this returns the concatenation of the ID and the major version component
+     of VERSION_ID. i.e. ID=ubuntu VERSION_ID=16.04 => ubuntu16
   """
-  global lsb_release_cache
+  global os_release_cache
   if not release:
-    if lsb_release_cache:
-      release = lsb_release_cache
+    if os_release_cache:
+      release = os_release_cache
     else:
-      lsb_release = check_output(["lsb_release", "-irs"])
-      release = "".join(map(lambda x: x.lower(), lsb_release.split()))
-      # Only need to check against the major release if RHEL, CentOS or Suse
-      for distro in ['centos', 'rocky', 'almalinux', 'redhatenterprise',
-                     'redhatenterpriseserver', 'suse']:
-        if distro in release:
-          release = release.split('.')[0]
-          break
-      lsb_release_cache = release
+      os_id = None
+      os_major_version = None
+      with open("/etc/os-release") as f:
+        for line in f:
+          # We assume that ID and VERSION_ID are present and don't contain '=' inside
+          # the actual value. This is true for all distributions we currently support.
+          if line.startswith("ID="):
+            os_id = line.split("=")[1].strip().strip('"')
+          elif line.startswith("VERSION_ID="):
+            os_version_id = line.split("=")[1].strip().strip('"')
+            # Some distributions have a major version that doesn't change (e.g. 3.12.0
+            # and 3.12.0). The distributions that we support don't do this. This
+            # calculation would need to change for that circumstance.
+            os_major_version = os_version_id.split(".")[0]
+
+      if os_id is None or os_major_version is None:
+        raise Exception("Error parsing /etc/os-release: "
+            "os_id={0} os_major_version={1}".format(os_id, os_major_version))
+
+      release = "{0}{1}".format(os_id, os_major_version)
+      os_release_cache = release
   for mapping in OS_MAPPING:
-    if re.search(mapping.lsb_release, release):
+    if mapping.release == release:
       return mapping
   raise Exception("Could not find package label for OS version: {0}.".format(release))
 
 
-def check_output(cmd_args):
-  """Run the command and return the output. Raise an exception if the command returns
-     a non-zero return code. Similar to subprocess.check_output() which is only provided
-     in python 2.7.
-  """
-  process = subprocess.Popen(cmd_args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-  stdout, _ = process.communicate()
-  if process.wait() != 0:
-    raise Exception("Command with args '%s' failed with exit code %s:\n%s"
-        % (cmd_args, process.returncode, stdout))
-  return stdout
-
-
 def check_custom_toolchain(toolchain_packages_home, packages):
   missing = []
   for p in packages:
@@ -466,7 +447,7 @@ def create_directory_from_env_var(env_var):
 
 
 def get_unique_toolchain_downloads(packages):
-  toolchain_packages = map(ToolchainPackage, packages)
+  toolchain_packages = [ToolchainPackage(p) for p in packages]
   unique_pkg_directories = set()
   unique_packages = []
   for p in toolchain_packages:
@@ -485,11 +466,15 @@ def get_toolchain_downloads():
       "llvm", explicit_version=os.environ.get("IMPALA_LLVM_DEBUG_VERSION"))
   gcc_package = ToolchainPackage("gcc")
   toolchain_packages += [llvm_package, llvm_package_asserts, gcc_package]
-  toolchain_packages += map(ToolchainPackage,
-      ["avro", "binutils", "boost", "breakpad", "bzip2", "calloncehack", "cctz", "cmake",
-       "crcutil", "curl", "flatbuffers", "gdb", "gflags", "glog", "gperftools", "gtest",
-       "jwt-cpp", "libev", "libunwind", "lz4", "openldap", "openssl", "orc", "protobuf",
-       "python", "rapidjson", "re2", "snappy", "tpc-h", "tpc-ds", "zlib", "zstd"])
+  toolchain_packages += [ToolchainPackage(p) for p in
+      ["avro", "binutils", "boost", "breakpad", "bzip2", "calloncehack", "cctz",
+       "cloudflarezlib", "cmake", "crcutil", "curl", "flatbuffers", "gdb", "gflags",
+       "glog", "gperftools", "gtest", "jwt-cpp", "libev", "libunwind", "lz4", "openldap",
+       "orc", "protobuf", "python", "rapidjson", "re2", "snappy", "tpc-h", "tpc-ds",
+       "zlib", "zstd"]]
+  python3_package = ToolchainPackage(
+      "python", explicit_version=os.environ.get("IMPALA_PYTHON3_VERSION"))
+  toolchain_packages += [python3_package]
   toolchain_packages += get_unique_toolchain_downloads(
       ["thrift:cpp", "thrift:java", "thrift:py"])
   protobuf_package_clang = ToolchainPackage(
@@ -580,11 +565,11 @@ def main():
   downloads = []
   if os.getenv("SKIP_TOOLCHAIN_BOOTSTRAP", "false") != "true":
     downloads += get_toolchain_downloads()
-  kudu_download = None
   if os.getenv("DOWNLOAD_CDH_COMPONENTS", "false") == "true":
     create_directory_from_env_var("CDP_COMPONENTS_HOME")
     create_directory_from_env_var("APACHE_COMPONENTS_HOME")
-    if platform.processor() != "aarch64":
+    if os.getenv("SKIP_TOOLCHAIN_BOOTSTRAP", "false") != "true":
+      # Kudu is currently sourced from native-toolchain
       downloads += get_kudu_downloads()
     downloads += get_hadoop_downloads()
 
