@@ -70,17 +70,34 @@ fi
 # WARNING: If changing this value, also run these commands:
 # cd ${IMPALA_HOME}/java
 # mvn versions:set -DnewVersion=YOUR_NEW_VERSION
-export IMPALA_VERSION=4.2.0-RELEASE
+export IMPALA_VERSION=4.3.0-RELEASE
+
+# Whether to build on Apache Hive (or CDP Hive). Versions of some toolchain dependencies
+# (e.g. thrift) will also depend on this.
+export USE_APACHE_HIVE=${USE_APACHE_HIVE-false}
+
+# Whether to build the backend on Avro C++ library or C.
+# This is added temporarily to help transitioning from Avro C to C++ library.
+export USE_AVRO_CPP=${USE_AVRO_CPP:=false}
 
 # The unique build id of the toolchain to use if bootstrapping. This is generated by the
 # native-toolchain build when publishing its build artifacts. This should be changed when
 # moving to a different build of the toolchain, e.g. when a version is bumped or a
 # compile option is changed. The build id can be found in the output of the toolchain
 # build jobs, it is constructed from the build number and toolchain git hash prefix.
-export IMPALA_TOOLCHAIN_BUILD_ID=223-7cf7e75bc8
+export IMPALA_TOOLCHAIN_BUILD_ID=358-e7cfab15d3
+export IMPALA_TOOLCHAIN_REPO=\
+${IMPALA_TOOLCHAIN_REPO:-https://github.com/cloudera/native-toolchain.git}
+export IMPALA_TOOLCHAIN_BRANCH=${IMPALA_TOOLCHAIN_BRANCH:-master}
+export IMPALA_TOOLCHAIN_COMMIT_HASH=\
+${IMPALA_TOOLCHAIN_COMMIT_HASH-e7cfab15d36ae051747252b676f0a11a9c58fe05}
 # Versions of toolchain dependencies.
 # -----------------------------------
-export IMPALA_AVRO_VERSION=1.7.4-p5
+if $USE_AVRO_CPP; then
+  export IMPALA_AVRO_VERSION=1.11.1-p1
+else
+  export IMPALA_AVRO_VERSION=1.7.4-p5
+fi
 unset IMPALA_AVRO_URL
 export IMPALA_BINUTILS_VERSION=2.35.1
 unset IMPALA_BINUTILS_URL
@@ -110,7 +127,7 @@ export IMPALA_GFLAGS_VERSION=2.2.0-p2
 unset IMPALA_GFLAGS_URL
 export IMPALA_GLOG_VERSION=0.3.5-p3
 unset IMPALA_GLOG_URL
-export IMPALA_GPERFTOOLS_VERSION=2.5-p4
+export IMPALA_GPERFTOOLS_VERSION=2.10
 unset IMPALA_GPERFTOOLS_URL
 export IMPALA_GTEST_VERSION=1.6.0
 unset IMPALA_GTEST_URL
@@ -120,13 +137,14 @@ export IMPALA_LIBEV_VERSION=4.20-p1
 unset IMPALA_LIBEV_URL
 export IMPALA_LIBUNWIND_VERSION=1.5.0-p1
 unset IMPALA_LIBUNWIND_URL
-export IMPALA_LLVM_VERSION=5.0.1-p5
+export IMPALA_LLVM_VERSION=5.0.1-p7
 unset IMPALA_LLVM_URL
-export IMPALA_LLVM_ASAN_VERSION=5.0.1-p5
+export IMPALA_LLVM_ASAN_VERSION=5.0.1-p7
 unset IMPALA_LLVM_ASAN_URL
 
-# Maximum memory available for mini-cluster and CDH cluster
-export IMPALA_CLUSTER_MAX_MEM_GB
+# To limit maximum memory available for the mini-cluster and CDH cluster, add the
+# following in $IMPALA_HOME/bin/impala-config-local.sh
+#   export IMPALA_CLUSTER_MAX_MEM_GB=<value>
 
 # LLVM stores some files in subdirectories that are named after what
 # version it thinks it is. We might think it is 5.0.1-p1, based on a
@@ -135,7 +153,7 @@ export IMPALA_LLVM_UBSAN_BASE_VERSION=5.0.1
 
 # Debug builds should use the release+asserts build to get additional coverage.
 # Don't use the LLVM debug build because the binaries are too large to distribute.
-export IMPALA_LLVM_DEBUG_VERSION=5.0.1-asserts-p5
+export IMPALA_LLVM_DEBUG_VERSION=5.0.1-asserts-p7
 unset IMPALA_LLVM_DEBUG_URL
 export IMPALA_LZ4_VERSION=1.9.3
 unset IMPALA_LZ4_URL
@@ -143,18 +161,17 @@ export IMPALA_ZSTD_VERSION=1.5.2
 unset IMPALA_ZSTD_URL
 export IMPALA_OPENLDAP_VERSION=2.4.47
 unset IMPALA_OPENLDAP_URL
-export IMPALA_OPENSSL_VERSION=1.0.2l
-unset IMPALA_OPENSSL_URL
-export IMPALA_ORC_VERSION=1.7.0-p14
+export IMPALA_ORC_VERSION=1.7.9-p10
 unset IMPALA_ORC_URL
 export IMPALA_PROTOBUF_VERSION=3.14.0
 unset IMPALA_PROTOBUF_URL
 export IMPALA_PROTOBUF_CLANG_VERSION=3.14.0-clangcompat-p2
 unset IMPALA_PROTOBUF_CLANG_URL
-export IMPALA_POSTGRES_JDBC_DRIVER_VERSION=42.4.1
+export IMPALA_POSTGRES_JDBC_DRIVER_VERSION=42.5.1
 unset IMPALA_POSTGRES_JDBC_DRIVER_URL
 export IMPALA_PYTHON_VERSION=2.7.16
 unset IMPALA_PYTHON_URL
+export IMPALA_PYTHON3_VERSION=3.7.16
 export IMPALA_RAPIDJSON_VERSION=1.1.0
 unset IMPALA_RAPIDJSON_URL
 export IMPALA_RE2_VERSION=20190301
@@ -168,8 +185,10 @@ export IMPALA_TPC_DS_VERSION=2.1.0-p1
 unset IMPALA_TPC_DS_URL
 export IMPALA_TPC_H_VERSION=2.17.0
 unset IMPALA_TPC_H_URL
-export IMPALA_ZLIB_VERSION=1.2.12
+export IMPALA_ZLIB_VERSION=1.2.13
 unset IMPALA_ZLIB_URL
+export IMPALA_CLOUDFLAREZLIB_VERSION=9e601a3f37
+unset IMPALA_CLOUDFLAREZLIB_URL
 export IMPALA_CALLONCEHACK_VERSION=1.0.0
 unset IMPALA_CALLONCEHACK_URL
 # Thrift related environment variables.
@@ -178,46 +197,54 @@ unset IMPALA_CALLONCEHACK_URL
 # If upgrading IMPALA_THRIFT_PY_VERSION, remember to also upgrade thrift version in
 # shell/ext-py and shell/packaging/requirements.txt. IMPALA_THRIFT_PY_VERSION is used
 # with Impyla and for the thrift compiler.
-export IMPALA_THRIFT_CPP_VERSION=0.16.0-p3
+export IMPALA_THRIFT_CPP_VERSION=0.16.0-p5
 unset IMPALA_THRIFT_CPP_URL
-export IMPALA_THRIFT_POM_VERSION=0.16.0
-export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p3
+if $USE_APACHE_HIVE; then
+  # Apache Hive 3 clients can't run on thrift versions >= 0.14 (IMPALA-11801)
+  export IMPALA_THRIFT_POM_VERSION=0.11.0
+  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p5
+else
+  export IMPALA_THRIFT_POM_VERSION=0.16.0
+  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p5
+fi
 unset IMPALA_THRIFT_JAVA_URL
-export IMPALA_THRIFT_PY_VERSION=0.11.0-p5
+export IMPALA_THRIFT_PY_VERSION=0.16.0-p5
 unset IMPALA_THRIFT_PY_URL
 
+# Find system python versions for testing
+export IMPALA_SYSTEM_PYTHON2="${IMPALA_SYSTEM_PYTHON2_OVERRIDE-$(command -v python2)}"
+export IMPALA_SYSTEM_PYTHON3="${IMPALA_SYSTEM_PYTHON3_OVERRIDE-$(command -v python3)}"
+
 if [[ $OSTYPE == "darwin"* ]]; then
   IMPALA_CYRUS_SASL_VERSION=2.1.26
   unset IMPALA_CYRUS_SASL_URL
   IMPALA_GPERFTOOLS_VERSION=2.3
   unset IMPALA_GPERFTOOLS_URL
-  IMPALA_OPENSSL_VERSION=1.0.1p
-  unset IMPALA_OPENSSL_URL
 fi
 
 : ${IMPALA_TOOLCHAIN_HOST:=native-toolchain.s3.amazonaws.com}
 export IMPALA_TOOLCHAIN_HOST
 
-export CDP_BUILD_NUMBER=33375775
+export CDP_BUILD_NUMBER=44206393
 export CDP_MAVEN_REPOSITORY=\
 "https://${IMPALA_TOOLCHAIN_HOST}/build/cdp_components/${CDP_BUILD_NUMBER}/maven"
-export CDP_AVRO_JAVA_VERSION=1.8.2.7.2.16.0-233
-export CDP_HADOOP_VERSION=3.1.1.7.2.16.0-233
-export CDP_HBASE_VERSION=2.4.6.7.2.16.0-233
-export CDP_HIVE_VERSION=3.1.3000.7.2.16.0-233
-export CDP_ICEBERG_VERSION=0.14.1.7.2.16.0-233
-export CDP_KNOX_VERSION=1.3.0.7.2.16.0-233
-export CDP_OZONE_VERSION=1.3.0.7.2.16.0-233
-export CDP_PARQUET_VERSION=1.10.99.7.2.16.0-233
-export CDP_RANGER_VERSION=2.3.0.7.2.16.0-233
-export CDP_TEZ_VERSION=0.9.1.7.2.16.0-233
+export CDP_AVRO_JAVA_VERSION=1.8.2.7.2.18.0-273
+export CDP_HADOOP_VERSION=3.1.1.7.2.18.0-273
+export CDP_HBASE_VERSION=2.4.17.7.2.18.0-273
+export CDP_HIVE_VERSION=3.1.3000.7.2.18.0-273
+export CDP_ICEBERG_VERSION=1.1.0.7.2.18.0-273
+export CDP_KNOX_VERSION=1.3.0.7.2.18.0-273
+export CDP_OZONE_VERSION=1.3.0.7.2.18.0-273
+export CDP_PARQUET_VERSION=1.10.99.7.2.18.0-273
+export CDP_RANGER_VERSION=2.4.0.7.2.18.0-273
+export CDP_TEZ_VERSION=0.9.1.7.2.18.0-273
 
 # Ref: https://infra.apache.org/release-download-pages.html#closer
 : ${APACHE_MIRROR:="https://www.apache.org/dyn/closer.cgi"}
 export APACHE_MIRROR
 export APACHE_HIVE_VERSION=3.1.3
 export APACHE_HIVE_STORAGE_API_VERSION=2.7.0
-export APACHE_OZONE_VERSION=1.2.1
+export APACHE_OZONE_VERSION=1.3.0
 
 export ARCH_NAME=$(uname -p)
 
@@ -231,15 +258,16 @@ export IMPALA_GUAVA_VERSION=31.1-jre
 export IMPALA_HUDI_VERSION=0.5.0-incubating
 export IMPALA_HTTP_CORE_VERSION=4.4.14
 export IMPALA_JACKSON_DATABIND_VERSION=2.13.4.2
-export IMPALA_JSON_SMART_VERSION=2.4.7
+export IMPALA_JSON_SMART_VERSION=2.4.11
 export IMPALA_JUNIT_VERSION=4.12
 export IMPALA_KITE_VERSION=1.1.0
 export IMPALA_ORC_JAVA_VERSION=1.7.6
 export IMPALA_PAC4J_VERSION=4.5.5
 export IMPALA_RELOAD4j_VERSION=1.2.22
 export IMPALA_SLF4J_VERSION=2.0.3
-export IMPALA_SPRINGFRAMEWORK_VERSION=5.3.20
+export IMPALA_SPRINGFRAMEWORK_VERSION=5.3.27
 export IMPALA_XMLSEC_VERSION=2.2.3
+export IMPALA_OBS_VERSION=3.1.1-hw-42
 
 # When Impala is building docker images on Redhat-based distributions,
 # it is useful to be able to customize the base image. Some users will
@@ -250,12 +278,23 @@ export IMPALA_XMLSEC_VERSION=2.2.3
 # free distributions, but Redhat UBI images are known to work.
 export IMPALA_REDHAT7_DOCKER_BASE=${IMPALA_REDHAT7_DOCKER_BASE:-"centos:centos7.9.2009"}
 export IMPALA_REDHAT8_DOCKER_BASE=${IMPALA_REDHAT8_DOCKER_BASE:-"rockylinux:8.5"}
+export IMPALA_REDHAT9_DOCKER_BASE=${IMPALA_REDHAT9_DOCKER_BASE:-"rockylinux:9.2"}
 
-# When set to true, this modifies the Docker image build logic to install Java 11
-# rather than the default of Java 8. This is strictly about what is shipped in the
-# Docker image, and it has no impact on what version of Java is used to compile
-# Impala's Java code.
-export IMPALA_DOCKER_USE_JAVA11=${IMPALA_DOCKER_USE_JAVA11:-"false"}
+# Selects the version of Java to use when start-impala-cluster.py starts with container
+# images (created via e.g. 'make docker_debug_java11_images'). The Java version used in
+# these images is independent of the Java version used to compile Impala.
+# Accepts 8, 11, 17.
+export IMPALA_DOCKER_JAVA=${IMPALA_DOCKER_JAVA:-"8"}
+if [ "${IMPALA_DOCKER_USE_JAVA11:-}" = "true" ]; then
+  export IMPALA_DOCKER_JAVA=11
+fi
+
+# There are multiple compatible implementations of zlib. Cloudflare Zlib is an
+# implementation with optimizations to use platform-specific CPU features that are not
+# in the standard Zlib implementation. When set to true, this builds and links against
+# Cloudflare Zlib. When false, the build uses the regular Madler Zlib. This defaults
+# to true due to the large performance benefits.
+export IMPALA_USE_CLOUDFLARE_ZLIB=${IMPALA_USE_CLOUDFLARE_ZLIB:-"true"}
 
 # When IMPALA_(CDP_COMPONENT)_URL are overridden, they may contain '$(platform_label)'
 # which will be substituted for the CDP platform label in bootstrap_toolchain.py
@@ -282,8 +321,13 @@ fi
 # IMPALA_TOOLCHAIN_PACKAGES_HOME is the location inside IMPALA_TOOLCHAIN where native
 # toolchain packages are placed. This uses a subdirectory that contains the information
 # about the compiler to allow using different compiler versions.
-export IMPALA_TOOLCHAIN_PACKAGES_HOME=\
+IMPALA_TOOLCHAIN_PACKAGES_HOME=\
 ${IMPALA_TOOLCHAIN}/toolchain-packages-gcc${IMPALA_GCC_VERSION}
+if ! [ -z ${NATIVE_TOOLCHAIN_HOME-} ]; then
+  IMPALA_TOOLCHAIN_PACKAGES_HOME=$(realpath ${NATIVE_TOOLCHAIN_HOME})/build
+  export SKIP_TOOLCHAIN_BOOTSTRAP=true
+fi
+export IMPALA_TOOLCHAIN_PACKAGES_HOME
 
 export CDP_HADOOP_URL=${CDP_HADOOP_URL-}
 export CDP_HBASE_URL=${CDP_HBASE_URL-}
@@ -316,7 +360,6 @@ export IMPALA_TEZ_VERSION=${CDP_TEZ_VERSION}
 export IMPALA_TEZ_URL=${CDP_TEZ_URL-}
 
 export APACHE_COMPONENTS_HOME="$IMPALA_TOOLCHAIN/apache_components"
-export USE_APACHE_HIVE=${USE_APACHE_HIVE-false}
 if $USE_APACHE_HIVE; then
   # When USE_APACHE_HIVE is set we use the apache hive version to build as well as deploy
   # in the minicluster
@@ -360,20 +403,71 @@ fi
 
 # It is important to have a coherent view of the JAVA_HOME and JAVA executable.
 # The JAVA_HOME should be determined first, then the JAVA executable should be
-# derived from JAVA_HOME. bin/bootstrap_development.sh adds code to
-# bin/impala-config-local.sh to set JAVA_HOME, so it is important to pick up that
-# setting before deciding what JAVA_HOME to use.
+# derived from JAVA_HOME. For development, it is useful to be able to specify
+# the JDK version as part of bin/impala-config-local.sh
 
-# Try to detect the system's JAVA_HOME
-# If javac exists, then the system has a Java SDK (JRE does not have javac).
-# Follow the symbolic links and use this to determine the system's JAVA_HOME.
-SYSTEM_JAVA_HOME="/usr/java/default"
-if [ -n "$(which javac)" ]; then
-  SYSTEM_JAVA_HOME=$(which javac | xargs readlink -f | sed "s:/bin/javac::")
+# Decision tree:
+# if IMPALA_JAVA_HOME_OVERRIDE is set, respect it
+# else if IMPALA_JDK_VERSION == system, look for system JDK
+# else if IMPALA_JDK_VERSION == 8+, look for Java 8+ JDK
+
+# Initialize IMPALA_JDK_VERSION and set package variables for Docker builds
+. "$IMPALA_HOME/bin/impala-config-java.sh"
+
+if [[ -n "${IMPALA_JAVA_HOME_OVERRIDE-}" ]]; then
+  IMPALA_JDK_VERSION=override
+fi
+
+DETECTED_JAVA_HOME=Invalid
+if [[ "${IMPALA_JDK_VERSION}" == "system" ]]; then
+  # Try to detect the system's JAVA_HOME
+  # If javac exists, then the system has a Java SDK (JRE does not have javac).
+  # Follow the symbolic links and use this to determine the system's JAVA_HOME.
+  DETECTED_JAVA_HOME="/usr/java/default"
+  if [ -n "$(which javac)" ]; then
+    DETECTED_JAVA_HOME=$(dirname $(dirname $(readlink -f $(which javac))))
+  fi
+elif [[ "${IMPALA_JDK_VERSION}" != "override" ]]; then
+  # Now, we are looking for a specific version, and that will depend on the
+  # distribution. Currently, this is implemented for Redhat and Ubuntu.
+  DISTRIBUTION=Unknown
+  if [[ -f /etc/redhat-release ]]; then
+    echo "Identified Redhat image."
+    DISTRIBUTION=Redhat
+  else
+    source /etc/lsb-release
+    if [[ $DISTRIB_ID == Ubuntu ]]; then
+      echo "Identified Ubuntu image."
+      DISTRIBUTION=Ubuntu
+    fi
+  fi
+  if [[ "${DISTRIBUTION}" == "Unknown" ]]; then
+    echo "ERROR: auto-detection of JAVA_HOME only supported for Ubuntu and RedHat."
+    echo "Use IMPALA_JAVA_HOME_OVERRIDE to configure JAVA_HOME."
+    return 1
+  fi
+
+  JVMS_PATH=/usr/lib/jvm
+  if [[ "${DISTRIBUTION}" == "Ubuntu" ]]; then
+    JAVA_PACKAGE_NAME="java-${IMPALA_JDK_VERSION}-openjdk-${UBUNTU_PACKAGE_ARCH}"
+    DETECTED_JAVA_HOME="${JVMS_PATH}/${JAVA_PACKAGE_NAME}"
+  elif [[ "${DISTRIBUTION}" == "Redhat" ]]; then
+    if [[ "${IMPALA_JDK_VERSION}" == "8" ]]; then
+      DETECTED_JAVA_HOME="${JVMS_PATH}/java-1.8.0"
+    else
+      DETECTED_JAVA_HOME="${JVMS_PATH}/java-${IMPALA_JDK_VERSION}"
+    fi
+  fi
+
+  if [[ ! -d "${DETECTED_JAVA_HOME}" ]]; then
+    echo "ERROR: Could not detect Java ${IMPALA_JDK_VERSION}."\
+         "${DETECTED_JAVA_HOME} is not a directory."
+    return 1
+  fi
 fi
 
 # Prefer the JAVA_HOME set in the environment, but use the system's JAVA_HOME otherwise
-export JAVA_HOME="${JAVA_HOME:-${SYSTEM_JAVA_HOME}}"
+export JAVA_HOME="${IMPALA_JAVA_HOME_OVERRIDE:-${DETECTED_JAVA_HOME}}"
 if [ ! -d "$JAVA_HOME" ]; then
   echo "JAVA_HOME must be set to the location of your JDK!"
   return 1
@@ -690,6 +784,14 @@ elif [ "${TARGET_FILESYSTEM}" = "oss" ]; then
   fi
   DEFAULT_FS="oss://${OSS_BUCKET}"
   export DEFAULT_FS
+elif [ "${TARGET_FILESYSTEM}" = "obs" ]; then
+  # Basic error checking
+  OBS_ACCESS_KEY="${OBS_ACCESS_KEY:?OBS_ACCESS_KEY cannot be an empty string for OBS}"
+  OBS_SECRET_KEY="${OBS_SECRET_KEY:?OBS_SECRET_KEY cannot be an empty string for OBS}"
+  OBS_ENDPOINT="${OBS_ENDPOINT:?OBS_ENDPOINT cannot be an empty string for OBS}"
+  OBS_BUCKET="${OBS_BUCKET:?OBS_BUCKET cannot be an empty string for OBS}"
+  DEFAULT_FS="obs://${OBS_BUCKET}"
+  export OBS_ACCESS_KEY OBS_SECRET_KEY OBS_ENDPOINT DEFAULT_FS ENABLE_OBS_FILESYSTEM=true
 elif [ "${TARGET_FILESYSTEM}" = "isilon" ]; then
   if [ "${ISILON_NAMENODE}" = "" ]; then
     echo "In order to access the Isilon filesystem, ISILON_NAMENODE"
@@ -719,6 +821,7 @@ elif [ "${TARGET_FILESYSTEM}" = "local" ]; then
 elif [ "${TARGET_FILESYSTEM}" = "hdfs" ]; then
   if [[ "${ERASURE_CODING}" = true ]]; then
     export HDFS_ERASURECODE_POLICY="RS-3-2-1024k"
+    export ERASURECODE_POLICY="$HDFS_ERASURECODE_POLICY"
     export HDFS_ERASURECODE_PATH="/test-warehouse"
   fi
 elif [ "${TARGET_FILESYSTEM}" = "ozone" ]; then
@@ -727,6 +830,11 @@ elif [ "${TARGET_FILESYSTEM}" = "ozone" ]; then
   export DEFAULT_FS="ofs://${INTERNAL_LISTEN_HOST}:9862"
   export FILESYSTEM_PREFIX="${DEFAULT_FS}/${OZONE_VOLUME}"
   export WAREHOUSE_LOCATION_PREFIX="/${OZONE_VOLUME}"
+  if [[ "${ERASURE_CODING}" = true ]]; then
+    export OZONE_ERASURECODE_POLICY="RS-3-2-1024k"
+    # Ozone normalizes the policy for internal storage. Use this string for tests.
+    export ERASURECODE_POLICY="rs-3-2-1024k"
+  fi
 else
   echo "Unsupported filesystem '$TARGET_FILESYSTEM'"
   echo "Valid values are: hdfs, isilon, s3, abfs, adls, gs, local, ozone"
@@ -861,9 +969,7 @@ fi
 # overall build type) and does not apply when using a local Kudu build.
 export USE_KUDU_DEBUG_BUILD=${USE_KUDU_DEBUG_BUILD-false}
 
-# IMPALA-11441: This githash is a custom Kudu that is equivalent to the upstream
-# githash dc4031f693 plus a revert of KUDU-1644, which avoids IMPALA-11441.
-export IMPALA_KUDU_VERSION=${IMPALA_KUDU_VERSION-"956093dd9d"}
+export IMPALA_KUDU_VERSION=${IMPALA_KUDU_VERSION-"1.17.0"}
 export IMPALA_KUDU_HOME=${IMPALA_TOOLCHAIN_PACKAGES_HOME}/kudu-$IMPALA_KUDU_VERSION
 export IMPALA_KUDU_JAVA_HOME=\
 ${IMPALA_TOOLCHAIN_PACKAGES_HOME}/kudu-${IMPALA_KUDU_VERSION}/java
@@ -912,6 +1018,9 @@ CLASSPATH="$IMPALA_FE_DIR/src/test/resources:$CLASSPATH"
 # A marker in the environment to prove that we really did source this file
 export IMPALA_CONFIG_SOURCED=1
 
+# Whether to strip the impalad binary when generating deployment package
+export STRIP_DEPLOYMENT_IMPALAD=true
+
 echo "IMPALA_VERSION          = $IMPALA_VERSION"
 echo "IMPALA_HOME             = $IMPALA_HOME"
 echo "HADOOP_HOME             = $HADOOP_HOME"
@@ -959,6 +1068,9 @@ echo "IMPALA_RANGER_VERSION   = $IMPALA_RANGER_VERSION"
 echo "IMPALA_ICEBERG_VERSION  = $IMPALA_ICEBERG_VERSION"
 echo "IMPALA_COS_VERSION      = $IMPALA_COS_VERSION"
 echo "IMPALA_OSS_VERSION      = $IMPALA_OSS_VERSION"
+echo "IMPALA_OBS_VERSION      = $IMPALA_OBS_VERSION"
+echo "IMPALA_SYSTEM_PYTHON2   = $IMPALA_SYSTEM_PYTHON2"
+echo "IMPALA_SYSTEM_PYTHON3   = $IMPALA_SYSTEM_PYTHON3"
 
 # Kerberos things.  If the cluster exists and is kerberized, source
 # the required environment.  This is required for any hadoop tool to
