@@ -30,7 +30,7 @@ setup_report_build_error
 
 export IMPALA_MAVEN_OPTIONS="-U"
 
-. bin/impala-config.sh
+. bin/impala-config.sh > /dev/null 2>&1
 
 : ${GENERATE_M2_ARCHIVE:=false}
 
@@ -39,6 +39,7 @@ CONFIGS=(
   # Test gcc builds with and without -so:
   "-skiptests -noclean"
   "-skiptests -noclean -release"
+  "-skiptests -noclean -release -package"
   "-skiptests -noclean -release -so -ninja"
   # clang sanitizer builds:
   "-skiptests -noclean -asan"
@@ -46,6 +47,7 @@ CONFIGS=(
   "-skiptests -noclean -ubsan -so -ninja"
   # USE_APACHE_HIVE=true build:
   "-skiptests -noclean -use_apache_hive"
+  "-skiptests -noclean -use_apache_hive -package"
 )
 
 FAILED=""
@@ -116,7 +118,7 @@ else
       -i impala-config.sh
   popd
   pushd java
-  mvn versions:set -DnewVersion=${NEW_MAVEN_VERSION}
+  mvn org.codehaus.mojo:versions-maven-plugin:2.13.0:set -DnewVersion=${NEW_MAVEN_VERSION}
   popd
 
   if ! ./bin/clean.sh; then
