@@ -32,12 +32,16 @@ fi
 
 rm -rf "${IMPALA_HOME}"/logs_system
 mkdir -p "${IMPALA_HOME}"/logs_system
-dmesg > "${IMPALA_HOME}"/logs_system/dmesg
+# Tolerate dmesg failures. dmesg can fail if there are insufficient permissions.
+if dmesg > "${IMPALA_HOME}"/logs_system/dmesg ; then
 
-# Check dmesg for OOMs and generate a symptom if present.
-if [[ $(grep "Out of memory" "${IMPALA_HOME}"/logs_system/dmesg) ]]; then
-  "${IMPALA_HOME}"/bin/generate_junitxml.py --phase finalize --step dmesg \
-      --stdout "${IMPALA_HOME}"/logs_system/dmesg --error "Process was OOM killed."
+  # Check dmesg for OOMs and generate a symptom if present.
+  if [[ $(grep "Out of memory" "${IMPALA_HOME}"/logs_system/dmesg) ]]; then
+    "${IMPALA_HOME}"/bin/generate_junitxml.py --phase finalize --step dmesg \
+        --stdout "${IMPALA_HOME}"/logs_system/dmesg --error "Process was OOM killed."
+  fi
+else
+  echo "Failed to run dmesg, not checking for OOMs"
 fi
 
 # Check for any minidumps and symbolize and dump them.
@@ -55,6 +59,25 @@ if [[ $(find $LOGS_DIR -path "*minidumps*" -name "*dmp") ]]; then
   rm -rf $SYM_DIR
 fi
 
+# Do a second pass over the minidumps with the resolve_minidump.py script.
+# This means that we now generate two JUnitXMLs per minidump. This should
+# be temporary.
+# TODO: Once we verify everything works, we can remove the first loop.
+if [[ $(find $LOGS_DIR -path "*minidumps*" -name "*dmp") ]]; then
+  for minidump in $(find $LOGS_DIR -path "*minidumps*" -name "*dmp"); do
+    # Since this is experimental, use it inside an if so that any error code doesn't
+    # abort this script.
+    if ! bin/resolve_minidumps.py --minidump_file ${minidump} \
+        --output_file ${minidump}_dumpedv2 ; then
+      echo "bin/resolve_minidumps.py failed!"
+    else
+      "${IMPALA_HOME}"/bin/generate_junitxml.py --phase finalize --step minidumpsv2 \
+          --error "Minidump generated: $minidump" \
+          --stderr "resolve_minidumps.py output:\n$(head -n 100 ${minidump}_dumpedv2)"
+    fi
+  done
+fi
+
 function check_for_asan_error {
   ERROR_LOG=${1}
   if grep -q "ERROR: AddressSanitizer:" ${ERROR_LOG} ; then
