@@ -20,6 +20,7 @@
 # This script is used to load the proper datasets for the specified workloads. It loads
 # all data via Hive except for parquet data which needs to be loaded via Impala.
 # Most ddl commands are executed by Impala.
+from __future__ import absolute_import, division, print_function
 import collections
 import getpass
 import logging
@@ -102,7 +103,7 @@ HIVE_CMD = os.path.join(os.environ['HIVE_HOME'], 'bin/beeline')
 hive_auth = "auth=none"
 if options.use_kerberos:
   if not options.principal:
-    print "--principal is required when --use_kerberos is specified"
+    print("--principal is required when --use_kerberos is specified")
     exit(1)
   hive_auth = "principal=" + options.principal
 
@@ -426,7 +427,7 @@ def main():
     def log_file_list(header, file_list):
       if (len(file_list) == 0): return
       LOG.debug(header)
-      map(LOG.debug, map(os.path.basename, file_list))
+      list(map(LOG.debug, list(map(os.path.basename, file_list))))
       LOG.debug("\n")
 
     log_file_list("Impala Create Files:", impala_create_files)
@@ -482,7 +483,7 @@ def main():
   total_time = 0.0
   thread_pool.close()
   thread_pool.join()
-  for workload, load_time in loading_time_map.iteritems():
+  for workload, load_time in loading_time_map.items():
     total_time += load_time
     LOG.info('Data loading for workload \'%s\' completed in: %.2fs'\
         % (workload, load_time))
