@@ -20,6 +20,8 @@
 # Starts up an Impala cluster (ImpalaD + State Store) with the specified number of
 # ImpalaD instances. Each ImpalaD runs on a different port allowing this to be run
 # on a single machine.
+from __future__ import absolute_import, division, print_function
+from builtins import range
 import getpass
 import itertools
 import json
@@ -32,14 +34,14 @@ from datetime import datetime
 from getpass import getuser
 from time import sleep, time
 from optparse import OptionParser, SUPPRESS_HELP
-from subprocess import call, check_call
-from testdata.common import cgroups
+from subprocess import call, check_call, check_output
 from tests.common.environ import build_flavor_timeout
 from tests.common.impala_cluster import (ImpalaCluster, DEFAULT_BEESWAX_PORT,
     DEFAULT_HS2_PORT, DEFAULT_KRPC_PORT, DEFAULT_HS2_HTTP_PORT,
     DEFAULT_STATE_STORE_SUBSCRIBER_PORT, DEFAULT_IMPALAD_WEBSERVER_PORT,
     DEFAULT_STATESTORED_WEBSERVER_PORT, DEFAULT_CATALOGD_WEBSERVER_PORT,
     DEFAULT_ADMISSIOND_WEBSERVER_PORT, DEFAULT_CATALOGD_JVM_DEBUG_PORT,
+    DEFAULT_CATALOG_SERVICE_PORT, DEFAULT_CATALOGD_STATE_STORE_SUBSCRIBER_PORT,
     DEFAULT_EXTERNAL_FE_PORT, DEFAULT_IMPALAD_JVM_DEBUG_PORT,
     find_user_processes, run_daemon)
 
@@ -130,6 +132,10 @@ parser.add_option("--data_cache_size", dest="data_cache_size", default=0,
 parser.add_option("--data_cache_eviction_policy", dest="data_cache_eviction_policy",
                   default="LRU", help="This specifies the cache eviction policy to use "
                   "for the data cache")
+parser.add_option("--data_cache_num_async_write_threads",
+                  dest="data_cache_num_async_write_threads", default=0,
+                  help="This specifies the number of asynchronous write threads for the "
+                  "data cache, with 0 set means synchronous writes.")
 parser.add_option("--data_cache_enable_tracing", dest="data_cache_enable_tracing",
                   action="store_true", default=False,
                   help="If the data cache is enabled, this enables tracing accesses.")
@@ -141,6 +147,17 @@ parser.add_option("--enable_admission_service", dest="enable_admission_service",
 parser.add_option("--enable_external_fe_support", dest="enable_external_fe_support",
                   action="store_true", default=False,
                   help="If true, impalads will start with the external_fe_port defined.")
+parser.add_option("--geospatial_library", dest="geospatial_library",
+                  action="store", default="HIVE_ESRI",
+                  help="Sets which implementation of geospatial libraries should be "
+                  "initialized")
+parser.add_option("--enable_catalogd_ha", dest="enable_catalogd_ha",
+                  action="store_true", default=False,
+                  help="If true, enables CatalogD HA - the cluster will be launched "
+                  "with two catalogd instances as Active-Passive HA pair.")
+parser.add_option("--jni_frontend_class", dest="jni_frontend_class",
+                  action="store", default="org/apache/impala/service/JniFrontend",
+                  help="Use a custom java frontend interface.")
 
 # For testing: list of comma-separated delays, in milliseconds, that delay impalad catalog
 # replica initialization. The ith delay is applied to the ith impalad.
@@ -275,6 +292,32 @@ def impalad_service_name(i):
     return "impalad_node{node_num}".format(node_num=i)
 
 
+def choose_catalogd_ports(instance_num):
+  """Compute the ports for catalogd instance num 'instance_num', returning as a map
+  from the argument name to the port number."""
+  return {'catalog_service_port': DEFAULT_CATALOG_SERVICE_PORT + instance_num,
+          'state_store_subscriber_port':
+              DEFAULT_CATALOGD_STATE_STORE_SUBSCRIBER_PORT + instance_num,
+          'webserver_port': DEFAULT_CATALOGD_WEBSERVER_PORT + instance_num}
+
+
+def build_catalogd_port_args(instance_num):
+  CATALOGD_PORTS = (
+      "-catalog_service_port={catalog_service_port} "
+      "-state_store_subscriber_port={state_store_subscriber_port} "
+      "-webserver_port={webserver_port}")
+  return CATALOGD_PORTS.format(**choose_catalogd_ports(instance_num))
+
+
+def catalogd_service_name(i):
+  """Return the name to use for the ith catalog daemon in the cluster."""
+  if i == 0:
+    # The first catalogd always logs to catalogd.INFO
+    return "catalogd"
+  else:
+    return "catalogd_node{node_num}".format(node_num=i)
+
+
 def combine_arg_list_opts(opt_args):
   """Helper for processing arguments like impalad_args. The input is a list of strings,
   each of which is the string passed into one instance of the argument, e.g. for
@@ -285,18 +328,32 @@ def combine_arg_list_opts(opt_args):
   return list(itertools.chain(*[shlex.split(arg) for arg in opt_args]))
 
 
-def build_statestored_arg_list():
+def build_statestored_arg_list(enable_catalogd_ha):
   """Build a list of command line arguments to pass to the statestored."""
-  return (build_logging_args("statestored") + build_kerberos_args("statestored") +
-      combine_arg_list_opts(options.state_store_args))
+  args = (build_logging_args("statestored") + build_kerberos_args("statestored")
+      + combine_arg_list_opts(options.state_store_args))
+  if (enable_catalogd_ha):
+    args.extend(["-enable_catalogd_ha=true"])
+  return args
 
 
-def build_catalogd_arg_list():
-  """Build a list of command line arguments to pass to the catalogd."""
-  return (build_logging_args("catalogd") +
-      ["-kudu_master_hosts", options.kudu_master_hosts] +
-      build_kerberos_args("catalogd") +
-      combine_arg_list_opts(options.catalogd_args))
+def build_catalogd_arg_list(num_catalogd, enable_catalogd_ha, remap_ports):
+  """Build a list of command line arguments to pass to the catalogd.
+  Build args for two catalogd instances if catalogd HA is enabled."""
+  catalogd_arg_list = []
+  for i in range(num_catalogd):
+    service_name = catalogd_service_name(i)
+    args = (build_logging_args(service_name)
+        + ["-kudu_master_hosts", options.kudu_master_hosts]
+        + build_kerberos_args("catalogd")
+        + combine_arg_list_opts(options.catalogd_args))
+    if remap_ports:
+      catalogd_port_args = build_catalogd_port_args(i)
+      args.extend(shlex.split(catalogd_port_args))
+    if enable_catalogd_ha:
+      args.extend(["-enable_catalogd_ha=true"])
+    catalogd_arg_list.append(args)
+  return catalogd_arg_list
 
 
 def build_admissiond_arg_list():
@@ -403,6 +460,10 @@ def build_impalad_arg_lists(cluster_size, num_coordinators, use_exclusive_coordi
       args = "-data_cache_eviction_policy={policy} {args}".format(
           policy=options.data_cache_eviction_policy, args=args)
 
+      # Add the number of async write threads.
+      args = "-data_cache_num_async_write_threads={num_threads} {args}".format(
+          num_threads=options.data_cache_num_async_write_threads, args=args)
+
       # Add access tracing arguments if requested
       if options.data_cache_enable_tracing:
         tracing_args = ""
@@ -423,6 +484,14 @@ def build_impalad_arg_lists(cluster_size, num_coordinators, use_exclusive_coordi
       args = "{args} -admission_service_host={host}".format(
           args=args, host=admissiond_host)
 
+    if "geospatial_library" not in args:
+      args = "{args} -geospatial_library={geospatial_library}".format(
+          args=args, geospatial_library=options.geospatial_library)
+
+    if options.jni_frontend_class:
+      args = "-jni_frontend_class={jni_frontend_class} {args}".format(
+          jni_frontend_class=options.jni_frontend_class, args=args)
+
     # Appended at the end so they can override previous args.
     if i < len(per_impalad_args):
       args = "{args} {per_impalad_args}".format(
@@ -465,7 +534,7 @@ def compute_impalad_mem_limit(cluster_size):
   # memory choice here to max out at 12GB. This should be sufficient for tests.
   #
   # Beware that ASAN builds use more memory than regular builds.
-  physical_mem_gb = psutil.virtual_memory().total / 1024 / 1024 / 1024
+  physical_mem_gb = psutil.virtual_memory().total // 1024 // 1024 // 1024
   available_mem = int(os.getenv("IMPALA_CLUSTER_MAX_MEM_GB", str(physical_mem_gb)))
   mem_limit = int(0.7 * available_mem * 1024 * 1024 * 1024 / cluster_size)
   return min(12 * 1024 * 1024 * 1024, mem_limit)
@@ -485,7 +554,7 @@ class MiniClusterOperations(object):
   def kill_all_impalads(self, force=False):
     kill_matching_processes(["impalad"], force=force)
 
-  def kill_catalogd(self, force=False):
+  def kill_all_catalogds(self, force=False):
     kill_matching_processes(["catalogd"], force=force)
 
   def kill_statestored(self, force=False):
@@ -498,20 +567,31 @@ class MiniClusterOperations(object):
     LOG.info("Starting State Store logging to {log_dir}/statestored.INFO".format(
         log_dir=options.log_dir))
     output_file = os.path.join(options.log_dir, "statestore-out.log")
-    run_daemon_with_options("statestored", build_statestored_arg_list(), output_file)
+    run_daemon_with_options("statestored",
+        build_statestored_arg_list(options.enable_catalogd_ha), output_file)
     if not check_process_exists("statestored", 10):
       raise RuntimeError("Unable to start statestored. Check log or file permissions"
                          " for more details.")
 
   def start_catalogd(self):
-    LOG.info("Starting Catalog Service logging to {log_dir}/catalogd.INFO".format(
-        log_dir=options.log_dir))
-    output_file = os.path.join(options.log_dir, "catalogd-out.log")
-    run_daemon_with_options("catalogd", build_catalogd_arg_list(), output_file,
-        jvm_debug_port=DEFAULT_CATALOGD_JVM_DEBUG_PORT)
-    if not check_process_exists("catalogd", 10):
-      raise RuntimeError("Unable to start catalogd. Check log or file permissions"
-                         " for more details.")
+    if options.enable_catalogd_ha:
+      num_catalogd = 2
+    else:
+      num_catalogd = 1
+    catalogd_arg_lists = build_catalogd_arg_list(
+        num_catalogd, options.enable_catalogd_ha, remap_ports=True)
+    for i in range(num_catalogd):
+      service_name = catalogd_service_name(i)
+      LOG.info(
+          "Starting Catalog Service logging to {log_dir}/{service_name}.INFO".format(
+              log_dir=options.log_dir, service_name=service_name))
+      output_file = os.path.join(
+          options.log_dir, "{service_name}-out.log".format(service_name=service_name))
+      run_daemon_with_options("catalogd", catalogd_arg_lists[i], output_file,
+          jvm_debug_port=DEFAULT_CATALOGD_JVM_DEBUG_PORT + i)
+      if not check_process_exists("catalogd", 10):
+        raise RuntimeError("Unable to start catalogd. Check log or file permissions"
+                           " for more details.")
 
   def start_admissiond(self):
     LOG.info("Starting Admission Control Service logging to {log_dir}/admissiond.INFO"
@@ -539,7 +619,7 @@ class MiniClusterOperations(object):
         cluster_size, num_coordinators, use_exclusive_coordinators, remap_ports=True,
         start_idx=start_idx)
     assert cluster_size == len(impalad_arg_lists)
-    for i in xrange(start_idx, start_idx + cluster_size):
+    for i in range(start_idx, start_idx + cluster_size):
       service_name = impalad_service_name(i)
       LOG.info("Starting Impala Daemon logging to {log_dir}/{service_name}.INFO".format(
           log_dir=options.log_dir, service_name=service_name))
@@ -575,7 +655,7 @@ class DockerMiniClusterOperations(object):
 
   def kill_all_daemons(self, force=False):
     self.kill_statestored(force=force)
-    self.kill_catalogd(force=force)
+    self.kill_all_catalogds(force=force)
     self.kill_admissiond(force=force)
     self.kill_all_impalads(force=force)
 
@@ -583,14 +663,21 @@ class DockerMiniClusterOperations(object):
     # List all running containers on the network and kill those with the impalad name
     # prefix to make sure that no running container are left over from previous clusters.
     container_name_prefix = self.__gen_container_name__("impalad")
-    for container_id, info in self.__get_network_info__()["Containers"].iteritems():
+    for container_id, info in self.__get_network_info__()["Containers"].items():
       container_name = info["Name"]
       if container_name.startswith(container_name_prefix):
         LOG.info("Stopping container {0}".format(container_name))
         check_call(["docker", "stop", container_name])
 
-  def kill_catalogd(self, force=False):
-    self.__stop_container__("catalogd")
+  def kill_all_catalogds(self, force=False):
+    # List all running containers on the network and kill those with the catalogd name
+    # prefix to make sure that no running container are left over from previous clusters.
+    container_name_prefix = self.__gen_container_name__("catalogd")
+    for container_id, info in self.__get_network_info__()["Containers"].items():
+      container_name = info["Name"]
+      if container_name.startswith(container_name_prefix):
+        LOG.info("Stopping container {0}".format(container_name))
+        check_call(["docker", "stop", container_name])
 
   def kill_statestored(self, force=False):
     self.__stop_container__("statestored")
@@ -599,12 +686,22 @@ class DockerMiniClusterOperations(object):
     self.__stop_container__("admissiond")
 
   def start_statestore(self):
-    self.__run_container__("statestored", build_statestored_arg_list(),
+    self.__run_container__("statestored",
+        build_statestored_arg_list(options.enable_catalogd_ha),
         {DEFAULT_STATESTORED_WEBSERVER_PORT: DEFAULT_STATESTORED_WEBSERVER_PORT})
 
   def start_catalogd(self):
-    self.__run_container__("catalogd", build_catalogd_arg_list(),
-          {DEFAULT_CATALOGD_WEBSERVER_PORT: DEFAULT_CATALOGD_WEBSERVER_PORT})
+    if options.enable_catalogd_ha:
+      num_catalogd = 2
+    else:
+      num_catalogd = 1
+    catalogd_arg_lists = build_catalogd_arg_list(
+        num_catalogd, options.enable_catalogd_ha, remap_ports=False)
+    for i in range(num_catalogd):
+      chosen_ports = choose_catalogd_ports(i)
+      port_map = {DEFAULT_CATALOG_SERVICE_PORT: chosen_ports['catalog_service_port'],
+                  DEFAULT_CATALOGD_WEBSERVER_PORT: chosen_ports['webserver_port']}
+      self.__run_container__("catalogd", catalogd_arg_lists[i], port_map, i)
 
   def start_admissiond(self):
     self.__run_container__("admissiond", build_admissiond_arg_list(),
@@ -615,7 +712,7 @@ class DockerMiniClusterOperations(object):
         use_exclusive_coordinators, remap_ports=False, admissiond_host="admissiond")
     assert cluster_size == len(impalad_arg_lists)
     mem_limit = compute_impalad_mem_limit(cluster_size)
-    for i in xrange(cluster_size):
+    for i in range(cluster_size):
       chosen_ports = choose_impalad_ports(i)
       port_map = {DEFAULT_BEESWAX_PORT: chosen_ports['beeswax_port'],
                   DEFAULT_HS2_PORT: chosen_ports['hs2_port'],
@@ -655,7 +752,7 @@ class DockerMiniClusterOperations(object):
       port_args = ["-P"]
     else:
       port_args = ["-p{dst}:{src}".format(src=src, dst=dst)
-                   for src, dst in port_map.iteritems()]
+                   for src, dst in port_map.items()]
     # Impersonate the current user for operations against the minicluster. This is
     # necessary because the user name inside the container is "root".
     # TODO: pass in the actual options
@@ -669,6 +766,8 @@ class DockerMiniClusterOperations(object):
       image_tag = daemon + "_debug"
     else:
       image_tag = daemon
+    java_versions = {"8": "", "11": "_java11", "17": "_java17"}
+    image_tag += java_versions[os.getenv('IMPALA_DOCKER_JAVA', '8')]
     host_name = self.__gen_host_name__(daemon, instance)
     container_name = self.__gen_container_name__(daemon, instance)
     # Mount configuration into container so that we don't need to rebuild container
@@ -704,7 +803,8 @@ class DockerMiniClusterOperations(object):
       mount_args + mem_limit_args + [image_tag] + args)
     LOG.info("Running command {0}".format(run_cmd))
     check_call(run_cmd)
-    port_mapping = check_output(["docker", "port", container_name])
+    port_mapping = check_output(["docker", "port", container_name],
+                                universal_newlines=True)
     LOG.info("Launched container {0} with port mapping:\n{1}".format(
         container_name, port_mapping))
 
@@ -722,7 +822,8 @@ class DockerMiniClusterOperations(object):
 
   def __get_network_info__(self):
     """Get the output of "docker network inspect" as a python data structure."""
-    output = check_output(["docker", "network", "inspect", self.network_name])
+    output = check_output(["docker", "network", "inspect", self.network_name],
+                          universal_newlines=True)
     # Only one network should be present in the top level array.
     return json.loads(output)[0]
 
@@ -769,10 +870,6 @@ if __name__ == "__main__":
   if options.docker_network is None:
     cluster_ops = MiniClusterOperations()
   else:
-    if sys.version_info < (2, 7):
-      raise Exception("Docker minicluster only supported on Python 2.7+")
-    # We use some functions in the docker code that don't exist in Python 2.6.
-    from subprocess import check_output
     cluster_ops = DockerMiniClusterOperations(options.docker_network)
 
   # If core-site.xml is missing, it likely means that we are missing config
@@ -785,7 +882,7 @@ if __name__ == "__main__":
   if options.restart_impalad_only:
     cluster_ops.kill_all_impalads(force=options.force_kill)
   elif options.restart_catalogd_only:
-    cluster_ops.kill_catalogd(force=options.force_kill)
+    cluster_ops.kill_all_catalogds(force=options.force_kill)
   elif options.restart_statestored_only:
     cluster_ops.kill_statestored(force=options.force_kill)
   elif options.add_executors:
@@ -838,7 +935,7 @@ if __name__ == "__main__":
     # Check for the cluster to be ready.
     impala_cluster.wait_until_ready(expected_cluster_size,
         expected_cluster_size - expected_catalog_delays)
-  except Exception, e:
+  except Exception as e:
     LOG.exception("Error starting cluster")
     sys.exit(1)
 
