@@ -88,6 +88,8 @@ import org.apache.thrift.TException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.impala.util.HiveMetadataFormatUtils.formatOutput;
+
 /**
  * A wrapper around some of Hive's Metastore API's to abstract away differences
  * between major versions of different Hive publishers. This implements the shimmed
@@ -423,6 +425,32 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     return Collections.EMPTY_LIST;
   }
 
+  /**
+   *   CDP Hive-3 only function.
+   */
+  @VisibleForTesting
+  public static List<Long> fireReloadEventHelper(MetaStoreClient msClient,
+      boolean isRefresh, List<String> partVals, String dbName, String tableName,
+      Map<String, String> selfEventParams) throws TException {
+    throw new UnsupportedOperationException("Reload event is not supported.");
+  }
+
+  /**
+   *   CDP Hive-3 only function.
+   */
+  public static Map<String, Object> getFieldsFromReloadEvent(NotificationEvent event)
+      throws MetastoreNotificationException {
+    throw new UnsupportedOperationException("Reload event is not supported.");
+  }
+
+  /**
+   *   CDP Hive-3 only function.
+   */
+  public static String getPartitionNameFromCommitCompactionEvent(
+      NotificationEvent event) {
+    throw new UnsupportedOperationException("CommitCompaction event is not supported.");
+  }
+
   /**
    * Use thrift API directly instead of HiveMetastoreClient#getNextNotification because
    * the HMS client can throw an IllegalStateException when there is a gap between the
@@ -902,4 +930,14 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     }
     return reads;
   }
+
+  public static void getMaterializedViewInfo(StringBuilder tableInfo,
+      Table tbl, boolean isOutputPadded) {
+    formatOutput("View Original Text:", tbl.getViewOriginalText(), tableInfo);
+    formatOutput("View Expanded Text:", tbl.getViewExpandedText(), tableInfo);
+    formatOutput("Rewrite Enabled:", tbl.isRewriteEnabled() ? "Yes" : "No", tableInfo);
+    // TODO: IMPALA-11815: hive metastore doesn't privide api to judge whether the
+    //  materialized view is outdated for rewriting, so set the flag to "Unknown"
+    formatOutput("Outdated for Rewriting:", "Unknown", tableInfo);
+  }
 }
