@@ -21,6 +21,8 @@ import static org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.ACCES
 import static org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.ACCESSTYPE_READONLY;
 import static org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.ACCESSTYPE_READWRITE;
 import static org.apache.hadoop.hive.metastore.api.hive_metastoreConstants.ACCESSTYPE_WRITEONLY;
+import static org.apache.impala.util.HiveMetadataFormatUtils.LINE_DELIM;
+import static org.apache.impala.util.HiveMetadataFormatUtils.formatOutput;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
@@ -31,6 +33,7 @@ import java.net.InetAddress;
 import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
@@ -65,12 +68,14 @@ import org.apache.hadoop.hive.metastore.api.NotificationEventResponse;
 import org.apache.hadoop.hive.metastore.api.Partition;
 import org.apache.hadoop.hive.metastore.api.PartitionsRequest;
 import org.apache.hadoop.hive.metastore.api.SetPartitionsStatsRequest;
+import org.apache.hadoop.hive.metastore.api.SourceTable;
 import org.apache.hadoop.hive.metastore.api.Table;
 import org.apache.hadoop.hive.metastore.api.WriteEventInfo;
 import org.apache.hadoop.hive.metastore.api.WriteNotificationLogRequest;
 import org.apache.hadoop.hive.metastore.conf.MetastoreConf;
 import org.apache.hadoop.hive.metastore.messaging.AlterTableMessage;
 import org.apache.hadoop.hive.metastore.messaging.CommitTxnMessage;
+import org.apache.hadoop.hive.metastore.messaging.CommitCompactionMessage;
 import org.apache.hadoop.hive.metastore.messaging.EventMessage;
 import org.apache.hadoop.hive.metastore.messaging.MessageBuilder;
 import org.apache.hadoop.hive.metastore.messaging.MessageDeserializer;
@@ -78,7 +83,9 @@ import org.apache.hadoop.hive.metastore.messaging.MessageEncoder;
 import org.apache.hadoop.hive.metastore.messaging.MessageFactory;
 import org.apache.hadoop.hive.metastore.messaging.MessageSerializer;
 import org.apache.hadoop.hive.metastore.messaging.json.JSONDropDatabaseMessage;
+import org.apache.hadoop.hive.metastore.messaging.ReloadMessage;
 import org.apache.hadoop.hive.metastore.utils.MetaStoreUtils;
+import org.apache.hadoop.hive.ql.metadata.formatting.TextMetaDataTable;
 import org.apache.impala.analysis.TableName;
 import org.apache.impala.catalog.CatalogException;
 import org.apache.impala.catalog.CatalogServiceCatalog;
@@ -495,6 +502,84 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     return response.getEventIds();
   }
 
+  /**
+   *  Fires a reload event to HMS notification log. In Hive-3 the relaod event
+   *  in HMS corresponds to refresh table or invalidate metadata of table in impala.
+   *
+   * @param msClient Metastore client,
+   * @param isRefresh if this flag is set to true then it is a refresh query, else it
+   *                  is an invalidate metadata query.
+   * @param partVals The partition list corresponding to
+   *                                 the table, used by Apache Hive 3
+   * @param dbName
+   * @param tableName
+   * @return a list of eventIds for the reload events
+   */
+  @VisibleForTesting
+  public static List<Long> fireReloadEventHelper(MetaStoreClient msClient,
+      boolean isRefresh, List<String> partVals, String dbName, String tableName,
+      Map<String, String> selfEventParams) throws TException {
+    Preconditions.checkNotNull(msClient);
+    Preconditions.checkNotNull(dbName);
+    Preconditions.checkNotNull(tableName);
+    FireEventRequestData data = new FireEventRequestData();
+    data.setRefreshEvent(isRefresh);
+    FireEventRequest rqst = new FireEventRequest(true, data);
+    rqst.setDbName(dbName);
+    rqst.setTableName(tableName);
+    rqst.setPartitionVals(partVals);
+    rqst.setTblParams(selfEventParams);
+    FireEventResponse response = msClient.getHiveClient().fireListenerEvent(rqst);
+    if (!response.isSetEventIds()) {
+      LOG.error("FireEventResponse does not have event ids set for table {}.{}. This "
+              + "may cause the table to unnecessarily be refreshed when the " +
+              "refresh/invalidate event is received.", dbName, tableName);
+      return Collections.EMPTY_LIST;
+    }
+    return response.getEventIds();
+  }
+
+  /**
+   *  This method extracts the table, partition, and isRefresh fields from the
+   *  notification event and returns them in a map.
+   *
+   * @param event Metastore notification event,
+   * @return a Map of fields required for the reload event.
+   */
+  public static Map<String, Object> getFieldsFromReloadEvent(NotificationEvent event)
+      throws MetastoreNotificationException{
+    ReloadMessage reloadMessage =
+        MetastoreEventsProcessor.getMessageDeserializer()
+            .getReloadMessage(event.getMessage());
+    Map<String, Object> updatedFields = new HashMap<>();
+    try {
+      org.apache.hadoop.hive.metastore.api.Table msTbl = Preconditions.checkNotNull(
+          reloadMessage.getTableObj());
+      Partition reloadPartition = reloadMessage.getPtnObj();
+      boolean isRefresh = reloadMessage.isRefreshEvent();
+      updatedFields.put("table", msTbl);
+      updatedFields.put("partition", reloadPartition);
+      updatedFields.put("isRefresh", isRefresh);
+    } catch (Exception e) {
+      throw new MetastoreNotificationException(e);
+    }
+    return updatedFields;
+  }
+
+  /**
+   *  This method extracts the partition name field from the
+   *  notification event and returns it in the form of string.
+   *
+   * @param event Metastore notification event,
+   * @return the partition name, required for the commit compaction event.
+   */
+  public static String getPartitionNameFromCommitCompactionEvent(
+      NotificationEvent event) {
+    CommitCompactionMessage commitCompactionMessage = MetastoreEventsProcessor.
+        getMessageDeserializer().getCommitCompactionMessage(event.getMessage());
+    return commitCompactionMessage.getPartName();
+  }
+
   /**
    * Wrapper around IMetaStoreClient.getThriftClient().get_next_notification() to deal
    * with added arguments.
@@ -605,8 +690,15 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     Map<String, Long> partNameToCompactionId = new HashMap<>();
     if (hdfsTable.isPartitioned()) {
       for (CompactionInfoStruct ci : response.getCompactions()) {
-        partNameToCompactionId.put(
-            Preconditions.checkNotNull(ci.getPartitionname()), ci.getId());
+        // It is possible for a partitioned table to have null partitionname in case of
+        // an aborted dynamic partition insert.
+        if (ci.getPartitionname() != null) {
+          partNameToCompactionId.put(ci.getPartitionname(), ci.getId());
+        } else {
+          LOG.warn(
+              "Partitioned table {} has null partitionname in CompactionInfoStruct: {}",
+              hdfsTable.getFullName(), ci.toString());
+        }
       }
     } else {
       CompactionInfoStruct ci = Iterables.getOnlyElement(response.getCompactions(), null);
@@ -664,8 +756,13 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     Map<String, Long> partNameToCompactionId = new HashMap<>();
     if (table.isPartitioned()) {
       for (CompactionInfoStruct ci : response.getCompactions()) {
-        partNameToCompactionId.put(
-            Preconditions.checkNotNull(ci.getPartitionname()), ci.getId());
+        if (ci.getPartitionname() != null) {
+          partNameToCompactionId.put(ci.getPartitionname(), ci.getId());
+        } else {
+          LOG.warn(
+              "Partitioned table {} has null partitionname in CompactionInfoStruct: {}",
+              tableName, ci.toString());
+        }
       }
     } else {
       CompactionInfoStruct ci = Iterables.getOnlyElement(response.getCompactions(),
@@ -796,7 +893,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
           }
         } else {
           catalog_.reloadTableIfExists(entry.getKey().getDb(), entry.getKey().getTbl(),
-               "CommitTxnEvent", getEventId());
+               "CommitTxnEvent", getEventId(), /*isSkipFileMetadataReload*/false);
         }
       }
     }
@@ -821,4 +918,38 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
       return false;
     }
   }
+
+  public static void getMaterializedViewInfo(StringBuilder tableInfo, Table tbl,
+       boolean isOutputPadded) {
+    formatOutput("View Original Text:", tbl.getViewOriginalText(), tableInfo);
+    formatOutput("View Expanded Text:", tbl.getViewExpandedText(), tableInfo);
+    formatOutput("Rewrite Enabled:", tbl.isRewriteEnabled() ? "Yes" : "No", tableInfo);
+    // TODO: IMPALA-11815: hive metastore doesn't privide api to judge whether the
+    //  materialized view is outdated for rewriting, so set the flag to "Unknown"
+    formatOutput("Outdated for Rewriting:", "Unknown", tableInfo);
+
+    tableInfo.append(LINE_DELIM)
+        .append("# Materialized View Source table information")
+        .append(LINE_DELIM);
+    TextMetaDataTable metaDataTable = new TextMetaDataTable();
+    metaDataTable.addRow("Table name", "I/U/D since last rebuild");
+    List<SourceTable> sourceTableList =
+        new ArrayList<>(tbl.getCreationMetadata().getSourceTables());
+    sourceTableList.sort(
+        Comparator
+            .<SourceTable, String>comparing(
+                sourceTable -> sourceTable.getTable().getDbName())
+            .thenComparing(sourceTable -> sourceTable.getTable().getTableName()));
+    for (SourceTable sourceTable : sourceTableList) {
+      String qualifiedTableName = org.apache.hadoop.hive.common.TableName.getQualified(
+          sourceTable.getTable().getCatName(),
+          sourceTable.getTable().getDbName(),
+          sourceTable.getTable().getTableName());
+      metaDataTable.addRow(qualifiedTableName,
+          String.format("%d/%d/%d",
+              sourceTable.getInsertedCount(), sourceTable.getUpdatedCount(),
+              sourceTable.getDeletedCount()));
+      tableInfo.append(metaDataTable.renderTable(isOutputPadded));
+    }
+  }
 }
