@@ -25,6 +25,8 @@ import org.apache.impala.catalog.AggregateFunction;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.InternalException;
+import org.apache.impala.planner.ProcessingCost;
+import org.apache.impala.util.ExprUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -717,4 +719,11 @@ public class AggregateInfo extends AggregateInfoBase {
 
   @Override
   public AggregateInfo clone() { return new AggregateInfo(this); }
+
+  public ProcessingCost computeProcessingCost(String label, long inputCardinality) {
+    float weight = ExprUtil.computeExprsTotalCost(getGroupingExprs())
+        + ExprUtil.computeExprsTotalCost(getAggregateExprs());
+
+    return ProcessingCost.basicCost(label, inputCardinality, weight);
+  }
 }
