@@ -26,7 +26,9 @@ import org.apache.impala.catalog.FeTable;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.thrift.TAlterTableAddPartitionParams;
 import org.apache.impala.thrift.TAlterTableParams;
+import org.apache.impala.thrift.TAlterTableSetFileFormatParams;
 import org.apache.impala.thrift.TAlterTableType;
+import org.apache.impala.thrift.THdfsFileFormat;
 
 import java.util.HashSet;
 import java.util.List;
@@ -39,6 +41,8 @@ public class AlterTableAddPartitionStmt extends AlterTableStmt {
   private final boolean ifNotExists_;
   private final List<PartitionDef> partitions_;
 
+  private THdfsFileFormat fileFormat;
+
   public AlterTableAddPartitionStmt(TableName tableName,
       boolean ifNotExists, List<PartitionDef> partitions) {
     super(tableName);
@@ -55,6 +59,12 @@ public class AlterTableAddPartitionStmt extends AlterTableStmt {
     }
   }
 
+  public AlterTableAddPartitionStmt(TableName tableName,
+      boolean ifNotExists, List<PartitionDef> partitions, THdfsFileFormat fileFormat) {
+    this(tableName, ifNotExists, partitions);
+    this.fileFormat = fileFormat;
+  }
+
   public boolean getIfNotExists() { return ifNotExists_; }
 
   @Override
@@ -64,6 +74,12 @@ public class AlterTableAddPartitionStmt extends AlterTableStmt {
     sb.append(getTbl()).append(" ADD");
     if (ifNotExists_) sb.append(" IF NOT EXISTS");
     for (PartitionDef p : partitions_) sb.append(" " + p.toSql(options));
+
+    if (fileFormat != null) {
+      sb.append(" SET FILEFORMAT ");
+      sb.append(fileFormat);
+    }
+
     return sb.toString();
   }
 
@@ -75,6 +91,14 @@ public class AlterTableAddPartitionStmt extends AlterTableStmt {
     TAlterTableParams params = super.toThrift();
     params.setAlter_type(TAlterTableType.ADD_PARTITION);
     params.setAdd_partition_params(addPartParams);
+
+    if (fileFormat != null) {
+      TAlterTableSetFileFormatParams fileFormatParams =
+          new TAlterTableSetFileFormatParams();
+      fileFormatParams.setFile_format(fileFormat);
+      params.setSet_file_format_params(fileFormatParams);
+    }
+
     return params;
   }
 
