@@ -25,6 +25,8 @@ import org.apache.avro.SchemaParseException;
 import org.apache.hadoop.hive.metastore.api.hive_metastoreConstants;
 import org.apache.hadoop.hive.serde2.avro.AvroSerdeUtils;
 import org.apache.iceberg.DataFile;
+import org.apache.iceberg.mr.Catalogs;
+import org.apache.iceberg.mr.InputFormatConfig;
 import org.apache.impala.authorization.AuthorizationConfig;
 import org.apache.impala.catalog.FeFsTable;
 import org.apache.impala.catalog.FeHBaseTable;
@@ -155,6 +157,8 @@ public class AlterTableSetTblProperties extends AlterTableSetStmt {
     icebergPropertyCheck(IcebergTable.ICEBERG_CATALOG);
     icebergPropertyCheck(IcebergTable.ICEBERG_CATALOG_LOCATION);
     icebergPropertyCheck(IcebergTable.ICEBERG_TABLE_IDENTIFIER);
+    icebergPropertyCheck(Catalogs.NAME);
+    icebergPropertyCheck(InputFormatConfig.TABLE_IDENTIFIER);
     icebergPropertyCheck(IcebergTable.METADATA_LOCATION);
     if (tblProperties_.containsKey(IcebergTable.ICEBERG_FILE_FORMAT)) {
       icebergTableFormatCheck(tblProperties_.get(IcebergTable.ICEBERG_FILE_FORMAT));
@@ -179,28 +183,6 @@ public class AlterTableSetTblProperties extends AlterTableSetStmt {
     if (IcebergUtil.getIcebergFileFormat(fileformat) == null) {
       throw new AnalysisException("Invalid fileformat for Iceberg table: " + fileformat);
     }
-    try {
-      FeIcebergTable iceTable = (FeIcebergTable)getTargetTable();
-      List<DataFile> dataFiles = IcebergUtil.getIcebergFiles(iceTable,
-          new ArrayList<>(), /*timeTravelSpec=*/null).first;
-      if (dataFiles.isEmpty()) return;
-      DataFile firstFile = dataFiles.get(0);
-      String errorMsg = "Attempt to set Iceberg data file format to %s, but found data " +
-          "file %s with file format %s.";
-      if (!firstFile.format().name().equalsIgnoreCase(fileformat)) {
-        throw new AnalysisException(String.format(errorMsg, fileformat, firstFile.path(),
-        firstFile.format().name()));
-      }
-      //TODO(IMPALA-10610): Iceberg tables with mixed file formats are not readable.
-      for (DataFile df : dataFiles) {
-        if (df.format() != firstFile.format()) {
-          throw new AnalysisException(String.format(errorMsg, fileformat, df.path(),
-              df.format().name()));
-        }
-      }
-    } catch (TableLoadingException e) {
-      throw new AnalysisException(e);
-    }
   }
 
   private void icebergParquetCompressionCodecCheck() throws AnalysisException {
