@@ -212,6 +212,10 @@ public class AnalysisContext {
           || isShowTablesStmt() || isAlterTableStmt() || isShowFunctionsStmt();
     }
 
+    public boolean isConvertTableToIcebergStmt() {
+      return stmt_ instanceof ConvertTableToIcebergStmt;
+    }
+
     public AlterTableStmt getAlterTableStmt() {
       Preconditions.checkState(isAlterTableStmt());
       return (AlterTableStmt) stmt_;
@@ -381,6 +385,11 @@ public class AnalysisContext {
       return (AdminFnStmt) stmt_;
     }
 
+    public ConvertTableToIcebergStmt getConvertTableToIcebergStmt() {
+      Preconditions.checkState(isConvertTableToIcebergStmt());
+      return (ConvertTableToIcebergStmt) stmt_;
+    }
+
     public StatementBase getStmt() { return stmt_; }
     public Analyzer getAnalyzer() { return analyzer_; }
     public Set<TAccessEvent> getAccessEvents() { return analyzer_.getAccessEvents(); }
